/**
 * Created by janis dähne.
 */

import * as genericLayer from './genericLayer'
import {GroupRoleWithPermissionFullBase} from "../types/groupRoles";
import {
  ExerciseReleaseCreateTupleForBackend, ExerciseReleaseForBackend,
  ExerciseReleaseFromBackend
} from "../types/exerciseRelease";
import {ExercisePreviewFromBackend} from "../types/exercisePreview";
import {PaginatedData, PaginationPostData} from '../types/pagination'

const callingName = 'exerciseReleasesLayer'
const controllerPrefix = 'exercise/releases'

export async function getForExercise(exerciseId: number, paginationPostData: PaginationPostData): Promise<PaginatedData<ExerciseReleaseFromBackend>> {
  return genericLayer.update<PaginationPostData,PaginatedData<ExerciseReleaseFromBackend>>(
    callingName, `${controllerPrefix}/for/exercise/${exerciseId}`, paginationPostData
  )
}


export async function getSingleReleaseForExercise(exerciseId: number, releaseId: number): Promise<ExerciseReleaseFromBackend> {
  return genericLayer.get<ExerciseReleaseFromBackend>(
    callingName, `${controllerPrefix}/for/exercise/${exerciseId}/${releaseId}`
  )
}

export async function createExerciseRelease(tuple: ExerciseReleaseCreateTupleForBackend): Promise<ExerciseReleaseFromBackend> {

  return genericLayer.create<ExerciseReleaseCreateTupleForBackend, ExerciseReleaseFromBackend>(callingName,
    `${controllerPrefix}/create`, tuple)
}


export async function updateExerciseRelease(release: ExerciseReleaseForBackend): Promise<ExerciseReleaseFromBackend> {

  return genericLayer.update<ExerciseReleaseForBackend, ExerciseReleaseFromBackend>(callingName,
    `${controllerPrefix}/update`, release)
}


export async function deleteExerciseRelease(releaseId: number): Promise<void> {
  return genericLayer.del<void>(callingName,
    `${controllerPrefix}/delete/${releaseId}`)
}


/**
 * returns a exercise preview from a release code (used when entering a release code)
 *
 * if ResponseCodes.alreadyExists is returned the user already has access to the release
 *
 * @param releaseCode the release code from an exercise release
 * @param allowReaccess true: it's ok if the user already has access to the release, false: some error
 * @returns {Promise<ExercisePreviewFromBackend | null>}
 * ExercisePreviewFromBackend if all is ok and allowReaccess = false, a new release was accessed (not accessed before)
 * null if all is ok and allowReaccess = true, so an old release was accessed
 * else error
 */
export async function accessExerciseWithReleaseCode(releaseCode: string, allowReaccess = false): Promise<ExercisePreviewFromBackend | null> {
  return genericLayer.get<ExercisePreviewFromBackend>(callingName,
    `${controllerPrefix}/access/${releaseCode}/${allowReaccess}`)
}
