/**
 * Created by janis dähne.
 */

import * as genericLayer from './genericLayer'
import {GroupRoleWithPermissionFullBase} from "../types/groupRoles";

const callingName = 'groupRolesLayer'
const controllerPrefix = 'grouproles'

/**
 * returns all group roles, checks system permission (create/update/delete roles)
 */
export function getGroupRoles(): Promise<ReadonlyArray<GroupRoleWithPermissionFullBase>> {
  return genericLayer.get<ReadonlyArray<GroupRoleWithPermissionFullBase>>(
    callingName, `${controllerPrefix}`
  )
}

/**
 * returns all group roles but checks the group role permission
 */
export function getGroupRolesForGroup(groupId: number): Promise<ReadonlyArray<GroupRoleWithPermissionFullBase>> {
  return genericLayer.get<ReadonlyArray<GroupRoleWithPermissionFullBase>>(
    callingName, `${controllerPrefix}/${groupId}/roles`
  )
}

/**
 * creates a new group role
 * @param groupRole the group role
 * @returns {Promise<TagFullBase>}
 */
export function create(groupRole: GroupRoleWithPermissionFullBase): Promise<GroupRoleWithPermissionFullBase> {
  return genericLayer.create(callingName,
    `${controllerPrefix}/create`, groupRole)
}

/**
 * changes a group role
 * @param groupRole the group role to change
 * @returns {Promise<GroupRoleWithPermissionFullBase>}
 */
export function update(groupRole: GroupRoleWithPermissionFullBase): Promise<GroupRoleWithPermissionFullBase> {
  return genericLayer.update(callingName,
    `${controllerPrefix}/update`, groupRole)
}


/**
 * deletes a group role
 * @param groupRole the group role to delete
 * @returns {Promise<void>}
 */
export function deleteEntity(groupRole: GroupRoleWithPermissionFullBase): Promise<void> {
  return genericLayer.del(callingName,
    `${controllerPrefix}/delete/${groupRole.id}`)
}