/**
 * Created by janis dähne.
 */
import {PLangFullBase, PLangWithExtrasFromBackend} from "../types/pLangs";
import * as genericLayer from './genericLayer'


const callingName = 'pLangs'
const controllerPrefix = 'plangs'


/**
 * gets all known plangs
 */
export function getPLangs(): Promise<ReadonlyArray<PLangFullBase>> {
  return genericLayer.get<ReadonlyArray<PLangFullBase>>(callingName,
                                                        `${controllerPrefix}`
  )
}


export function getPLangsWithExtraData(): Promise<ReadonlyArray<PLangWithExtrasFromBackend>> {
  return genericLayer.get<ReadonlyArray<PLangWithExtrasFromBackend>>(callingName,
                                                        `${controllerPrefix}/get/extra`
  )
}


/**
 *
 * @param plang
 * @returns not PLangWithExtrasFromBackend because new plang cannot have any exercises / releases ...
 */
export function createPLang(plang: PLangFullBase): Promise<PLangFullBase> {
  return genericLayer.update<PLangFullBase, PLangFullBase>(callingName,
                                                                 `${controllerPrefix}/create`, plang
  )
}

export function updatePLang(plang: PLangFullBase): Promise<PLangWithExtrasFromBackend> {
  return genericLayer.update<PLangFullBase, PLangWithExtrasFromBackend>(callingName,
                                                                 `${controllerPrefix}/update`, plang
  )
}

export function deletePLang(pLangsId: number): Promise<void> {
  return genericLayer.del<void>(callingName, `${controllerPrefix}/delete/${pLangsId}`)
}
