/**
 * Created by janis dähne.
 */

import * as genericLayer from './genericLayer'
import {AssessmentFullBase} from "../types/submissions";
import {PaginatedData, PaginationPostData} from '../types/pagination'
import {
  AdditionalCsvColumn,
  CsvExportAssessmentOption,
  CsvExportAssessmentOptionsMap, CsvExportAssessmentOptionsObj
} from '../types/csvExportAssessment'
import {AxiosResponse} from 'axios'

const callingName = 'submissionsLayer'
const controllerPrefix = 'submissions'


export async function getFromRelease(releaseId: number, paginationPostData: PaginationPostData, getPreAssessmentResults: boolean | null = null): Promise<PaginatedData<AssessmentFullBase>> {

  return genericLayer.update<PaginationPostData, PaginatedData<AssessmentFullBase>>(
    callingName, `${controllerPrefix}/get/${releaseId}/${getPreAssessmentResults}`, paginationPostData
  )
}

//TODO maybe use release id??
/**
 * rereuns all assessment (for a a release)
 * @param {string} releaseCode
 * @returns {Promise<boolean>}
 */
export async function rerunAutomaticAssessment(releaseCode: string): Promise<boolean> {
  return genericLayer.update<string, boolean>(callingName,
    `${controllerPrefix}/rerun/${releaseCode}`, '')
}

/**
 * rereuns a single assessment (for a single solution)
 * @param {string} releaseCode
 * @param {number} userId
 * @param {number} pLangId
 * @returns {Promise<boolean>}
 */
export async function rerunSingleAutomaticAssessment(releaseCode: string, userId: number, pLangId: number): Promise<boolean> {
  return genericLayer.update<string, boolean>(callingName,
    `${controllerPrefix}/rereunsingle/${releaseCode}/${userId}/${pLangId}`,
    '')
}

export async function importCsvAssessments(releaseCode: string, content: ReadonlyArray<number>): Promise<void> {
  return genericLayer.update<ReadonlyArray<number>, void>(callingName,
                                                          `${controllerPrefix}/import/csv/${releaseCode}`,
                                                          content)
}

export async function downloadAssessmentsAsCsv(releaseCode: string, sortKey: keyof AssessmentFullBase | null, IsOrderByDesc: boolean, csvDelimiter: string, csvExportAssessmentOptions: CsvExportAssessmentOptionsMap, additionalColumns: ReadonlyArray<AdditionalCsvColumn>): Promise<AxiosResponse> {

  const obj: CsvExportAssessmentOptionsObj = {
    options: csvExportAssessmentOptions,
    additionalCsvColumns: additionalColumns
  }

  return genericLayer.rawResponsePost<CsvExportAssessmentOptionsObj>(callingName,
                                              `${controllerPrefix}/export/csv/${releaseCode}/${sortKey}/${IsOrderByDesc}/${csvDelimiter}`,
                                                                     obj)
}