/**
 * Created by janis dähne.
 */

import * as genericLayer from './genericLayer'
import {ExercisePreviewFromBackend} from "../types/exercisePreview";
import {SystemRoleFullBase} from "../types/userData";
import {delay} from "../../debug";

const callingName = 'systemRolesLayer'
const controllerPrefix = 'systemroles'


export async function getSystemRoles(): Promise<ReadonlyArray<SystemRoleFullBase>> {
  return genericLayer.get<ReadonlyArray<SystemRoleFullBase>>(
    callingName, `${controllerPrefix}/all`
  )
}

/**
 * returns the system role ids that cannot be changed or deleted
 */
export async function getNotLockedSystemRoleIds(): Promise<ReadonlyArray<number>> {
  return genericLayer.get<ReadonlyArray<number>>(
    callingName, `${controllerPrefix}/locked/ids`
  )
}


/**
 * creates a new system role
 * @param systemRole the system role to create
 * @returns {Promise<SystemRoleFullBase>}
 */
export function create(systemRole: SystemRoleFullBase): Promise<SystemRoleFullBase> {

  //TODO ensure all ids are unset (in case we want to copy something we just call create instead of update)

  return genericLayer.update<SystemRoleFullBase, SystemRoleFullBase>(callingName,
    `${controllerPrefix}/create`, systemRole)
}

/**
 * updates the given system role
 * @param systemRole the system role
 * @returns {Promise<SystemRoleFullBase>}
 */
export function update(systemRole: SystemRoleFullBase): Promise<SystemRoleFullBase> {

  return genericLayer.update<SystemRoleFullBase, SystemRoleFullBase>(callingName,
    `${controllerPrefix}/update`, systemRole)

}

/**
 * deletes the given system role
 * @param systemRole the system role to delete
 * @returns {Promise<void>}
 */
export async function deleteEntity(systemRole: SystemRoleFullBase): Promise<void> {
  return genericLayer.del<void>(callingName,
    `${controllerPrefix}/delete/${systemRole.id}`)
}
