/**
 * Created by janis dähne.
 */

import * as genericLayer from './genericLayer'
import {TagFullBase} from "../types/tag";

const callingName = 'tagsLayer'
const controllerPrefix = 'tags'

export function getTags(): Promise<ReadonlyArray<TagFullBase>> {
  return genericLayer.get<ReadonlyArray<TagFullBase>>(
    callingName, `${controllerPrefix}`
  )
}


/**
 * creates a new tag
 * @param tag the tag
 * @returns {Promise<TagFullBase>} the created tag from server
 */
export function create(tag: TagFullBase): Promise<TagFullBase> {
  return genericLayer.create(callingName, `${controllerPrefix}/create`, tag)
}

/**
 * updates the given tag
 * @param tag the tag to update
 * @returns {Promise<TagFullBase>} the updated tag from server
 */
export function update(tag: TagFullBase): Promise<TagFullBase> {
  return genericLayer.update(callingName, `${controllerPrefix}/update`, tag)
}

/**
 * deletes the given tag
 * @param tag the tag to delete
 * @returns {Promise<void>}
 */
export function deleteEntity(tag: TagFullBase): Promise<void> {
  return genericLayer.del(callingName, `${controllerPrefix}/${tag.id}`)
}

/**
 * merges the given dependent tags into the base tag
 * @param baseTagId the base tag
 * @param dependentTags the dependent tags that will be merged into the base tag
 * @returns {Promise<void>}
 */
export function merge(baseTagId: number, dependentTags: ReadonlyArray<number>): Promise<void> {
  return genericLayer.update(callingName, `${controllerPrefix}/merge/${baseTagId}`,
    {
      idsToMerge: dependentTags
    })
}


