/**
 * Created by janis dähne.
 */


import * as genericLayer from './genericLayer'
import {
  JustRunProgramCommandForBackend,
  JustRunProgramTestCustomProjectCommandForBackend,
  RunTutorTestCommandForBackend,
  RunCompileSingleFileCommandCustomProjectForBackend,
  RunCompileSingleFileCommandForBackend,
  RunTestCommandCustomProjectForBackend,
  RunTestCommandForBackend,
  TestAnswerFromBackend, JustRunProgramTestTutorViewCommandForBackend
} from "../types/testsAndTestProtocol";
import {testingLayerCallerName} from '../constants'

export const callingName = testingLayerCallerName
const controllerPrefix = 'testing'


/**
 * runs a test on the test server (through the backend)
 */
export async function runTest(runNormalTestCommand: RunTestCommandForBackend): Promise<ReadonlyArray<TestAnswerFromBackend>> {

  return await genericLayer.update<RunTestCommandForBackend, ReadonlyArray<TestAnswerFromBackend>>(
    callingName,
    `${controllerPrefix}/run`, runNormalTestCommand
  )
}

export async function runCustomTestCustomProject(runNormalTestCommand: RunTestCommandCustomProjectForBackend): Promise<ReadonlyArray<TestAnswerFromBackend>> {

  return await genericLayer.update<RunTestCommandCustomProjectForBackend, ReadonlyArray<TestAnswerFromBackend>>(
    callingName,
    `${controllerPrefix}/run/custom/project/custom`, runNormalTestCommand
  )
}

/**
 * runs a sinngle compile file command on the backend and returns the test result
 * @param runCompileSingleFileCommand the command
 * @returns {Promise<TestAnswerFromBackend>}
 */
export async function compileSingleFile(runCompileSingleFileCommand: RunCompileSingleFileCommandForBackend): Promise<TestAnswerFromBackend> {

  return await genericLayer.update<RunCompileSingleFileCommandForBackend,
    TestAnswerFromBackend>(callingName, `${controllerPrefix}/compile/do/exercise`, runCompileSingleFileCommand
  )
}

export async function compileSingleFileCustomProject(runCompileSingleFileCommand: RunCompileSingleFileCommandCustomProjectForBackend): Promise<TestAnswerFromBackend> {

  return await genericLayer.update<RunCompileSingleFileCommandCustomProjectForBackend, TestAnswerFromBackend>(
    callingName,
    `${controllerPrefix}/compile/custom/project`,
    runCompileSingleFileCommand
  )
}

export async function compileSingleFileTutorView(runCompileSingleFileCommand: RunCompileSingleFileCommandForBackend): Promise<TestAnswerFromBackend> {

  return await genericLayer.update<RunCompileSingleFileCommandForBackend,
    TestAnswerFromBackend>(callingName, `${controllerPrefix}/compile/tutor/view`, runCompileSingleFileCommand
  )
}


/**
 * runs a test as a tutor (with a temp frontend only solution)
 */
export function runTestAsTutor(runTestAsTutorCommand: RunTutorTestCommandForBackend): Promise<ReadonlyArray<TestAnswerFromBackend>> {
  return genericLayer.update<RunTutorTestCommandForBackend, ReadonlyArray<TestAnswerFromBackend>>(callingName,
                                                                                                  `${controllerPrefix}/run/as/tutor`,
                                                                                                  runTestAsTutorCommand
  )
}

export function runAfterTest(runAfterTestCommand: RunTestCommandForBackend): Promise<ReadonlyArray<TestAnswerFromBackend>> {
  return genericLayer.update<RunTestCommandForBackend, ReadonlyArray<TestAnswerFromBackend>>(callingName,
                                                                                             `${controllerPrefix}/run/after`,
                                                                                             runAfterTestCommand
  )
}


export async function justRunProgramTest(justRunProgramCommand: JustRunProgramCommandForBackend): Promise<TestAnswerFromBackend> {

  return await genericLayer.update<JustRunProgramCommandForBackend, TestAnswerFromBackend>(callingName,
                                                                                           `${controllerPrefix}/justrunprogram/normal`,
                                                                                           justRunProgramCommand
  )
}


export async function justRunProgramTestCustomProject(justRunProgramCommand: JustRunProgramTestCustomProjectCommandForBackend): Promise<TestAnswerFromBackend> {

  return await genericLayer.update<JustRunProgramTestCustomProjectCommandForBackend, TestAnswerFromBackend>(callingName,
                                                                                                            `${controllerPrefix}/justrunprogram/custom/project`,
                                                                                                            justRunProgramCommand
  )
}


export async function justRunProgramTestTutorView(justRunProgramCommand: JustRunProgramTestTutorViewCommandForBackend): Promise<TestAnswerFromBackend> {

  return await genericLayer.update<JustRunProgramCommandForBackend, TestAnswerFromBackend>(callingName,
                                                                                           `${controllerPrefix}/justrunprogram/tutor/view`,
                                                                                           justRunProgramCommand
  )
}
