/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import Logger from "../../../helpers/logger";

import ReactResizeDetector from "react-resize-detector";
import {getEditorInternalThemeName} from "../../../helpers/themes";

import {initial as initialCodeEditorSettings} from "../../../state/reducers/editorSettingsDialog/editorSettingsReducer"
import {CodeEditorSettingsFullBase} from "../../../types/userData";
import {SimpleCodeEditorProps} from "../simpleCodeEditorWrapper";
import AceEditor from "react-ace";
import throttle from 'lodash-es/throttle'
import {Editor} from 'brace';



//markdown is needed to create new exercises this is the only usage for the single file editor...
//as long as the single file editor is not async loaded we should be ok to include the markdown here
//currently we have not implemented single/simple file editor language async loading/importing
import 'brace/mode/markdown';
import 'brace/mode/text'; //used as default session language type
//import 'brace/theme/monokai';
import '../../../../externalLibsForBundling/brace/themes/monokai_changed';

import '../../../../externalLibsForBundling/brace/modes/regex_test'
import '../../../../externalLibsForBundling/brace/modes/blackbox_test'
import '../../../../externalLibsForBundling/brace/modes/blackbox_compareFiles_test'

import 'brace/theme/sqlserver';
import {AceEditorsStorageHelper} from "./aceEditorsStorageHelper";
import {debounce} from 'lodash-es'
import {setQuickCodeEditorFontSizeInPx} from '../../../state/actions/globalUiActions'
import {Icon} from 'semantic-ui-react'
import {MyPopup} from '../../helpers/myPopup'
import {getI18n} from '../../../../i18n/i18nRoot'

//const css = require('./styles.styl');


/**
 * see webpack config there we copied all modes into this dir
 */
// @ts-ignore
ace.config.set('modePath', '/static/ace-modes')



const mapStateToProps = (rootState: RootState, props: SimpleCodeEditorProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props,
    editorSettings: (rootState.userDataSettingsSate.userData.settings && rootState.userDataSettingsSate.userData.settings.codeEditorSettings
                     ? rootState.userDataSettingsSate.userData.settings.codeEditorSettings
                     : initialCodeEditorSettings) as CodeEditorSettingsFullBase,

    codeEditorFontSizeInPx: rootState.globalUiState.quickCodeEditorFontSizeInPx,
    langId: rootState.i18nState.langId,
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setQuickCodeEditorFontSizeInPx
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

interface AceWrapper {
  readonly editor: Editor
}


//TODO when using e.g. json ace craetes a new web worker to validate the text... but the worker is never cleared??
//or just a chrome display bug? dev tools -> sources -> left panel -> service workers #1, #2 ...

//basically we need to do the same thing as in multi tab editor (maybe the comments work...)
//but no need to support sessions yet ;)
class SimpleAceEditor extends React.Component<Props, any> {

  aceEditorWrapper: AceWrapper

  // initialSession: IEditSession = createEditSession('', `ace/mode/text` as any)
  // editSession: IEditSession = null //this.initialSession
  forceRerenderThrottled: () => void = null
  deferredOnChange: () => void = null

  applyEditorSettingsToTabs(): void {

    const storedTabs = AceEditorsStorageHelper.getEditor(this.props.id)

    if (!storedTabs || storedTabs.length === 0) return

    const storedTab = storedTabs[0]

    storedTab.session.setUseWrapMode(this.props.editorSettings.useWrapping)
    storedTab.session.setTabSize(this.props.editorSettings.tabSize)
  }

  componentWillMount(): void {
    // noinspection TsLint
    this.forceRerenderThrottled = throttle(this.forceRerender, 200)

    if (this.props.debounceTimeInMs !== undefined) {
      this.deferredOnChange = throttle(this.onChangeDeferred, this.props.debounceTimeInMs)
    }

  }

  componentDidMount(): void {

    if (!AceEditorsStorageHelper.getEditor(this.props.id)) {
      //create initial tab
      AceEditorsStorageHelper.addEditor(this.props.id, this.props.editorTab)
    }

    if (this.aceEditorWrapper && this.aceEditorWrapper.editor) {

      const editorTabs = AceEditorsStorageHelper.getEditor(this.props.id)

      const wrapperTab = editorTabs.find(p => p.tab.id === this.props.editorTab.id)

      if (!wrapperTab) {
        Logger.error('could not find editor tab!')
        return
      }

      //not needed any longer, we now use unsetSession which should properly unsubscribe all events
      // wrapperTab.session.setMode(`ace/mode/markdown` as any)
      // wrapperTab.session.setMode(`ace/mode/text` as any)
      // wrapperTab.session.setMode(`ace/mode/${wrapperTab.tab.language}` as any)

      this.aceEditorWrapper.editor.setSession(wrapperTab.session)
      this.applyEditorSettingsToTabs()
    }
  }

  zoomEditorIn(e: ExtendedKeyboardEvent | null): void {

    if (e) {
      e.preventDefault()
    }

    if (this.props.codeEditorFontSizeInPx === null) {
      this.props.setQuickCodeEditorFontSizeInPx(this.props.editorSettings.fontSize + 1)
    }
    else {
      this.props.setQuickCodeEditorFontSizeInPx(this.props.codeEditorFontSizeInPx + 1)
    }
  }

  zoomEditorOut(e: ExtendedKeyboardEvent | null): void {

    if (e) {
      e.preventDefault()
    }

    if (this.props.codeEditorFontSizeInPx === null) {
      this.props.setQuickCodeEditorFontSizeInPx(this.props.editorSettings.fontSize - 1)
    }
    else {
      this.props.setQuickCodeEditorFontSizeInPx(this.props.codeEditorFontSizeInPx - 1)
    }
  }

  zoomEditorReset(e: ExtendedKeyboardEvent | null): void {

    if (e) {
      e.preventDefault()
    }

    this.props.setQuickCodeEditorFontSizeInPx(null)
  }

  componentDidUpdate(): void {

    //we only need this in case e.g. we reset the ace storage and then set the content and e.g.
    //want to clear the undo stack... then we recreate the ace storage and need to
    //apply the new session... after updating the content the simple editor will rerender (update)
    //and apply the ace session we just created (used e.g. in editor exercise description editor
    if (this.aceEditorWrapper && this.aceEditorWrapper.editor) {
      this.aceEditorWrapper.editor.$blockScrolling = Infinity
      const editorTabs = AceEditorsStorageHelper.getEditor(this.props.id)

      if (!editorTabs || editorTabs.length === 0) return

      const wrapperTab = editorTabs[0]

      if (!wrapperTab) {
        //this can happen in the time when we update files (get new ids)
        //and we already got the new ids and update this component but not yet refreshed the ace storage
        //the warning gets then generated between the pending and fulfilled actions
        Logger.warn('maybe could not find editor tab but could be on update!')
        return
      }

      if (wrapperTab.session === this.aceEditorWrapper.editor.getSession()) {
        return
      }

      //the session could have changed e.g. when we load the edit exercise site
      //we first display the initial content (session)
      //after we loaded the exercise data we create a new session where the settings are not applied yet...
      this.aceEditorWrapper.editor.setSession(wrapperTab.session)
      this.applyEditorSettingsToTabs()
    }
  }

  componentWillUnmount(): void {
  }

  /**
   * we need to unset the session when the editor is unmounted/destroyed because else
   * we get errors (when mounting a session again) lik "Uncaught TypeError: Cannot read property 'getTokens' of null" from ace
   * because some editor internal components refer to disposed dom elements
   */
  unsetSession(): void {
    if (this.aceEditorWrapper && this.aceEditorWrapper.editor) {
      const initialSession = AceEditorsStorageHelper.createNewSession('', `text` as any)
      //setting a new session will clear the handlers of the old session
      this.aceEditorWrapper.editor.setSession(initialSession)
    }
  }

  onBlur(): void {

    //make sure the editor storage is in sync

    const editorTabs = AceEditorsStorageHelper.getEditor(this.props.id)

    //can happen if we cleared all AceEditorsStorageHelper
    if (!editorTabs) return

    const wrapperTab = editorTabs.find(p => p.tab.id === this.props.editorTab.id)
    if (!wrapperTab) {
      Logger.error('could not find editor tab!')
      return
    }

    if (!this.aceEditorWrapper.editor) return

    this.aceEditorWrapper.editor.$blockScrolling = Infinity
    const newVal = this.aceEditorWrapper.editor.getValue()

    if (newVal === wrapperTab.tab.content) {
      return //lost focus but nothing to update
    }

    // @ts-ignore
    wrapperTab.tab.content = newVal //this NOT updates the redux store!
    //this only updated the AceEditorStorage content we then need to call update solution files
    //and apply the new content to the real solution files

    if (!this.props.onBlur) return
    this.props.onBlur(newVal, wrapperTab.tab.id)
  }

  onChangeDeferred(): void {


    //if we make changes then fast unmount the editor then this gets called but editor is null
    if ( this.aceEditorWrapper.editor === null) return

    const editorTabs = AceEditorsStorageHelper.getEditor(this.props.id)

    if (!editorTabs) return //can happen if we already unmounted

    const wrapperTab = editorTabs.find(p => p.tab.id === this.props.editorTab.id)
    if (!wrapperTab) {
      Logger.error('could not find editor tab!')
      return
    }

    this.aceEditorWrapper.editor.$blockScrolling = Infinity
    const newVal = this.aceEditorWrapper.editor.getValue()

    if (newVal === wrapperTab.tab.content) {
      return //lost focus but nothing to update
    }

    // @ts-ignore
    wrapperTab.tab.content = newVal //this NOT updates the redux store!
    //this only updated the AceEditorStorage content we then need to call update solution files
    //and apply the new content to the real solution files

    if (!this.deferredOnChange || !this.props.onBlur) return
    this.props.onBlur(newVal, wrapperTab.tab.id)
  }


  /**
   * updates the editor ref and tries to set the active session
   * @param {AceWrapper} wrapper
   */
  updateEditorRef(wrapper: AceWrapper): void {
    this.aceEditorWrapper = wrapper

    if (this.aceEditorWrapper.editor['zoomIn']) {
      //we already added the commands
      return
    }

    /*
    we now use a custom version of brace (ace editor) where we fixed the shortcut ctrl++/cmd++
     see https://github.com/ajaxorg/ace/blob/51f53efa96ca2507bafb1c980d022953df6f35c6/lib/ace/lib/keys.js
     and
     https://github.com/ajaxorg/ace/issues/2566
    */

    const self = this

    this.aceEditorWrapper.editor.commands.addCommand(
      {
        name: 'zoomIn',
        bindKey: {
          win: 'Ctrl-+',
          mac: "Command-+"
        },
        exec: function (): void {
          self.zoomEditorIn(null)
        },
        readOnly: true //also working on readonly files
      }
    )

    this.aceEditorWrapper.editor.commands.addCommand(
      {
        name: 'zoomOut',
        bindKey: {
          win: 'Ctrl--',
          mac: "Command--"
        },
        exec: function (): void {
          console.log(arguments)
          self.zoomEditorOut(null)
        },
        readOnly: true
      }
    )

    this.aceEditorWrapper.editor.commands.addCommand(
      {
        name: 'zoomReset',
        bindKey: {
          win: 'Ctrl-0',
          mac: "Command-0"
        },
        exec: function (): void {
          self.zoomEditorReset(null)
        },
        readOnly: true
      }
    )
  }

  forceRerender(): void {
    if (this.aceEditorWrapper && this.aceEditorWrapper.editor) {
      //console.log("resized")
      this.aceEditorWrapper.editor.resize(false)
    }
  }

  render(): JSX.Element {

    return (<div className="fh flexed-h flex-stretch">

      {//resize for div is not implemented in js ... just for window
      }
      <ReactResizeDetector handleHeight handleWidth onResize={(w, h) => {
        if (this.forceRerenderThrottled) {
          this.forceRerenderThrottled()
        }
      }}/>

      {
        this.props.codeEditorFontSizeInPx !== null
        && this.props.codeEditorFontSizeInPx !== this.props.editorSettings.fontSize &&
        <MyPopup
          trigger={
            <div className="zoomed-indicator">
              <div className="clickable" onClick={() => {
                this.zoomEditorReset(null)
              }}>
                {/*<Icon name="font" />*/}
                <Icon.Group size="large">
                  <Icon name="undo" className="zoomed-indicator-colored"/>
                  <Icon corner name="font"/>
                </Icon.Group>
              </div>
            </div>
          }
          mouseEnterDelay={1000}
          content={getI18n(this.props.langId,
                           "Shortcut: ctrl+0/cmd+0. Resets the quick-zoom to the zoom in the editor settings. Note that the quick-zoom is not persistent"
          )}
        />
      }


      <AceEditor
        className="flex-stretch single-tab-editor"
        onBeforeUmount={() => {
          this.unsetSession()
        }}
        width="100%"
        height={this.props.height
                ? this.props.height
                : '100%'}
        name={this.props.id}
        mode={this.props.editorTab.language}
        theme={getEditorInternalThemeName(this.props.editorSettings.theme)}
        focus={true}
        ref={(editor: any) => {
          if (editor) {
            this.updateEditorRef(editor)
          }
        }}
        value={this.props.editorTab.content} //set this as initial value because after redux update we rerender the new value
        onBlur={() => this.onBlur()}
        onChange={this.deferredOnChange
                  ? () => this.deferredOnChange()
                  : undefined}
        readOnly={this.props.editorTab.isReadonly}

        fontSize={this.props.codeEditorFontSizeInPx !== null
                  ? this.props.codeEditorFontSizeInPx
                  : this.props.editorSettings.fontSize}
        wrapEnabled={this.props.editorSettings.useWrapping}
        tabSize={this.props.editorSettings.tabSize}
        showPrintMargin={false}
        highlightActiveLine={true}
        editorProps={{$blockScrolling: true}}
      />
    </div>)
  }
}

const _default = connect(mapStateToProps, mapDispatchToProps)(SimpleAceEditor)
export default _default
