/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../state/reducers";
import {Checkbox, Form, Radio} from "semantic-ui-react";
import {
  CheckboxOnChangeEventData, InputOnChangeEvent, RadioOnChangeEventData, SelectOnChangeEventData
} from "../../types/reactEvents";
import MaterialInput from '../material/materialInput'
import MaterialSelectWrapper from '../material/materialSelectWrapper'
import {validationRules} from "../../state/reducers/editorSettingsDialog/editorSettingsReducer";
import {validationMessageKeys} from "../../state/reducers/editorSettingsDialog/editorSettingsReducerValidation";
import {
  setFontSize,
  setHighlightCurrentLine,
  setShowInvisibles,
  setShowLineIndentions,
  setShowLineNumbers,
  setTabSize,
  setTheme,
  setUseWrapping
} from "../../state/actions/editorSettingsDialog/editorSettingsActions";
import {parseNotNegativeOrNullNumber} from "../../helpers/convertersAndTransformers";
import {InternalCodeEditorThemes} from "../../helpers/themes";
import {errorDialog} from "../../helpers/dialogHelper";
import {getI18n} from "../../../i18n/i18nRoot";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    editorSettings: rootState.editorSettingsDialogState.editorSettings,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  setFontSize,
  setHighlightCurrentLine,
  setShowInvisibles,
  setShowLineNumbers,
  setTabSize,
  setUseWrapping,
  setShowLineIndentions,
  setTheme
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class editorSettingsView extends React.Component<Props, any> {
  render(): JSX.Element {

    // const isThemeValid = validationRules.theme.every(validator => validator(this.props.editorSettings.theme))

    return (
      <div>

        <div className="ui form divided-checkboxes">
          <Form.Group widths='equal'>

            <Form.Field required>
              <label>
                {
                  getI18n(this.props.langId, 'Theme')
                }
              </label>

              <MaterialSelectWrapper
                value={this.props.editorSettings.theme}
                validations={validationRules.theme}
                validationMessageKey={validationMessageKeys.theme}
              >
                <select
                  value={this.props.editorSettings.theme}
                  onChange={(e: SelectOnChangeEventData) => {
                    this.props.setTheme(e.currentTarget.value)
                  }}
                >
                  <optgroup label={getI18n(this.props.langId,'Light')}>

                    <option key={InternalCodeEditorThemes.light} value={InternalCodeEditorThemes.light}>
                      {
                        getI18n(this.props.langId,'Default light')
                      }
                    </option>

                  </optgroup>

                  <optgroup label={getI18n(this.props.langId,'Dark')}>

                    <option key={InternalCodeEditorThemes.dark} value={InternalCodeEditorThemes.dark}>
                      {
                        getI18n(this.props.langId,'Default dark')
                      }
                    </option>

                  </optgroup>

                </select>
              </MaterialSelectWrapper>
            </Form.Field>


            <Form.Field required>
              <label>{getI18n(this.props.langId,'Font size')}</label>
              <MaterialInput type="number"
                             value={this.props.editorSettings.fontSize}
                             onChange={(e: InputOnChangeEvent) => {

                               const fontSize = parseNotNegativeOrNullNumber(e.currentTarget.value)

                               if (fontSize === null) {

                                 errorDialog(getI18n(this.props.langId,'Error'),
                                   getI18n(this.props.langId,'Could not convert string to int'),
                                             this.props.langId
                                 )

                               } else {
                                 this.props.setFontSize(fontSize)
                               }
                             }}
                             validations={validationRules.fontSize}
                             validationMessageKey={validationMessageKeys.fontSize}
              />
            </Form.Field>

            <Form.Field required>
              <label>{getI18n(this.props.langId,'Tab size')}</label>
              <div className="material-padding">

                <Radio className="mar-right"
                       checked={this.props.editorSettings.tabSize === 2}
                       label="2"
                       value={2}
                       onChange={(e, data: RadioOnChangeEventData) => {
                         this.props.setTabSize(2)
                       }}
                />

                <Radio
                  checked={this.props.editorSettings.tabSize === 4}
                  label="4"
                  value={4}
                  onChange={(e, data: RadioOnChangeEventData) => {
                    this.props.setTabSize(4)
                  }}
                />
              </div>

            </Form.Field>

          </Form.Group>

          {
            /*
           cannot get this to work with ace editor props (method call could work...)... (it should work somehow though...)
            <Form.Field>
            <Checkbox toggle
                      checked={this.props.editorSettings.showLineNumbers}
                      label={getI18n(this.props.langId,(TranslateIds.showLineNumbers)}
                      onChange={(e, data: CheckboxOnChangeEventData) => {
                        this.props.setShowLineNumbers(data.checked)
                      }}
            />
          </Form.Field>
           */
          }


          {
            /* no editor support
            <Form.Field>
            <Checkbox toggle
                      checked={this.props.editorSettings.highlightCurrentLine}
                      label="Aktuelle Zeile hervorheben"
                      onChange={(e, data: CheckboxOnChangeEventData) => {
                        this.props.setHighlightCurrentLine(data.checked)
                      }}
            />
          </Form.Field>
             */
          }

          <Form.Field>
            <Checkbox toggle
                      checked={this.props.editorSettings.useWrapping}
                      label={`${getI18n(this.props.langId, 'Use wrapping')} (${getI18n(this.props.langId, 'visual line break')})`}
                      onChange={(e, data: CheckboxOnChangeEventData) => {
                        this.props.setUseWrapping(data.checked)
                      }}
            />
          </Form.Field>

          {
            /*
            cannot get this to work with ace editor props (method call could work...)... (it should work somehow though...)
            <Form.Field>
            <Checkbox toggle
                      checked={this.props.editorSettings.showLineIndentions}
                      label={getI18n(this.props.langId,(TranslateIds.showLineIndentations)}
                      onChange={(e, data: CheckboxOnChangeEventData) => {
                        this.props.setShowLineIndentions(data.checked)
                      }}
            />
          </Form.Field>
             */
          }


          {
            /* no editor support
            <Form.Field>
            <Checkbox toggle
                      checked={this.props.editorSettings.showInvisibles}
                      label="Nicht druckbare Zeichen anzeigen"
                      onChange={(e, data: CheckboxOnChangeEventData) => {
                        this.props.setShowInvisibles(data.checked)
                      }}
            />
          </Form.Field>
             */
          }


        </div>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(editorSettingsView)