import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from '../../state/reducers'
import PaginationPageSizeSelect from './paginationPageSizeSelect'
import {Icon, Table} from 'semantic-ui-react'
import {Pagination} from 'semantic-ui-react'
import {PaginationStateData} from '../../types/pagination'


export interface MyProps {
  readonly pagination: PaginationStateData<any>
  readonly setPaginationPageSize: (pageSize: number) => void
  readonly setActivePaginationPage: (activePage: number) => void
}

const mapStateToProps = (rootState: RootState , props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
                                                                             //imported reducer funcs here

                                                                           }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class PaginationTableFooter extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <Table.Footer>
        <Table.Row>

          {
            //we need to span all cols
          }
          <Table.HeaderCell colSpan='42'>

            <div className="flexed" style={{float: 'right'}}>

              <PaginationPageSizeSelect
                langId={this.props.langId}
                pageSize={this.props.pagination.pageSize}
                totalItems={this.props.pagination.totalItems}
                setPaginationPageSize={this.props.setPaginationPageSize}
              />


              <Pagination className="material-paging"
                totalPages={this.props.pagination.totalPages} activePage={this.props.pagination.currentPage}
                onPageChange={(event1, data) => {
                  this.props.setActivePaginationPage(data.activePage as number)
                }}
                firstItem={{ content: <Icon name='angle double left' />, icon: true }}
                lastItem={{ content: <Icon name='angle double right' />, icon: true }}
                prevItem={{ content: <Icon name='angle left' />, icon: true }}
                nextItem={{ content: <Icon name='angle right' />, icon: true }}
              />
            </div>


          </Table.HeaderCell>

        </Table.Row>
      </Table.Footer>
    )
  }
}

const _comp = connect(mapStateToProps, mapDispatchToProps)(PaginationTableFooter)
export default _comp
