/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../state/reducers";
import {List} from "semantic-ui-react";

//const css = require('./styles.styl');

export interface SelectableOption {
  readonly id: number | string
  readonly displayName: string
}

export interface MyProps {
  //readonly test: string
  readonly options: ReadonlyArray<SelectableOption>
  readonly defaultOption?: SelectableOption | null

  readonly optionClicked: (option: SelectableOption) => void
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

/**
 * this should be wrapped inside a div (why??)
 */
class SelectOptionList extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div>
        <List divided relaxed>
          {
            this.props.defaultOption &&
            <List.Item className="clickable" onClick={() => this.props.optionClicked(this.props.defaultOption)}>
              <List.Content key={this.props.defaultOption.id}>
              <span>
                {
                  this.props.defaultOption.displayName
                }
              </span>
              </List.Content>
            </List.Item>
          }
          {
            this.props.options.map((option) => {
              return (
                <List.Item className="clickable" key={option.id} onClick={() => this.props.optionClicked(option)}>
                  <List.Content>
                    <span>
                      {
                        option.displayName
                      }
                    </span>
                  </List.Content>
                </List.Item>
              )
            })
          }
        </List>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(SelectOptionList)