/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";

import {Button, Form, Header, Icon} from 'semantic-ui-react'
import {returntypeof} from "react-redux-typescript";
import {RootState} from "../../state/reducers";
import {bindActionCreators, Dispatch} from "redux";
import MaterialInput from '../material/materialInput'
import {
  reset as resetLoginData,
  setIsRegisterFormDisplayed,
  setIsRegistering,
  setRegisterEmail,
  setRegisterFirstName,
  setRegisterLastName,
  setRegisterPw,
  setRegisterPwAgain,
  setRegisterToken
} from "../../state/actions/loginActions";
import {InputOnChangeEvent} from "../../types/reactEvents";
import {notifyError} from "../../helpers/notificationHelper";
import Spinner from "../helpers/spinner";
import * as loginLayer from '../../communicationLayer/loginLayer'
import {infoDialog} from "../../helpers/dialogHelper";
import * as cryptHelper from '../../helpers/cryptoHelper'
import {formSpinnerOpacity} from "./loginForm";
import {getI18n, getRawI18n} from "../../../i18n/i18nRoot";

export interface MyProps {

}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {

    isRegistering: rootState.loginDataState.isRegistering,
    registerToken: rootState.loginDataState.registerToken,
    registerPw: rootState.loginDataState.registerPw,
    registerPwAgain: rootState.loginDataState.registerPwAgain,
    registerFirstName: rootState.loginDataState.registerFirstName,
    registerLastName: rootState.loginDataState.registerLastName,
    registerEmail: rootState.loginDataState.registerEmail,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  setIsRegisterFormDisplayed,
  setIsRegistering,
  setRegisterToken,
  setRegisterPw,
  setRegisterPwAgain,
  setRegisterFirstName,
  setRegisterLastName,
  setRegisterEmail,

  resetLoginData,
}, dispatch)

const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class RegisterForm extends React.Component<Props, any> {


  async registerClicked(): Promise<void> {

    if (this.props.registerPw !== this.props.registerPwAgain) {
      await infoDialog(getI18n(this.props.langId, 'Password'),
        getI18n(this.props.langId, 'The password and the again password don\'t match.'),
                       this.props.langId)
      return
    }

    this.props.setIsRegistering(true)

    try {


      await loginLayer.registerUser({
        token: this.props.registerToken,
        firstName: this.props.registerFirstName,
        lastName: this.props.registerLastName,
        email: this.props.registerEmail,
        password: cryptHelper.hash(this.props.registerPw)
      })

      this.props.resetLoginData()

      await infoDialog(getI18n(this.props.langId, 'Registered'),
        getI18n(this.props.langId, 'Register was successful but the Admin needs to activate your account.'),
                       this.props.langId)

    } catch (err) {

      //TODO maybe display some more specific errors
      //e.g. something was empty
      //token is already used...
      //see server auth controller > register function
      notifyError(getI18n(this.props.langId,'Server error'))
    }
    finally {
      this.props.setIsRegistering(false)
    }

  }

  render(): JSX.Element {
    return (

      <div className="frosted-glass-register fh">
        <div className="frosted-glass-inner absolute-sized-centered-login">

          <div className="corner-div">

            {
              this.props.isRegistering &&
              <Spinner opacity={formSpinnerOpacity}/>
            }

            {
              !this.props.isRegistering &&
              <div className="ui form frosted-glass-form">

                <div className="form-header">
                  <Header as='h1'>
                    {getI18n(this.props.langId, 'YAPEX')}  - {getI18n(this.props.langId, 'Register')}
                  </Header>
                </div>

                <Form.Field required>
                  <label>
                    {getI18n(this.props.langId, 'Token')}
                  </label>
                  <MaterialInput
                    placeholder="e.g. abcef"
                    value={this.props.registerToken}
                    onChange={(e: InputOnChangeEvent) => {
                      this.props.setRegisterToken(e.currentTarget.value)
                    }}
                  />
                </Form.Field>

                <Form.Group widths='equal'>
                  <Form.Field required>
                    <label>
                      {
                        getI18n(this.props.langId, 'Password')
                      }
                    </label>
                    <MaterialInput type="password"
                                   value={this.props.registerPw}
                                   onChange={(e: InputOnChangeEvent) => {
                                     this.props.setRegisterPw(e.currentTarget.value)
                                   }}
                    />
                  </Form.Field>

                  <Form.Field required>
                    <label>
                      {
                        getI18n(this.props.langId, 'Password again')
                      }
                    </label>
                    <MaterialInput type="password"
                                   value={this.props.registerPwAgain}
                                   onChange={(e: InputOnChangeEvent) => {
                                     this.props.setRegisterPwAgain(e.currentTarget.value)
                                   }}

                    />
                  </Form.Field>
                </Form.Group>

                <Form.Group widths='equal'>
                  <Form.Field required>
                    <label>
                      {
                        getI18n(this.props.langId, 'First name')
                      }
                    </label>
                    <MaterialInput
                      value={this.props.registerFirstName}
                      onChange={(e: InputOnChangeEvent) => {
                        this.props.setRegisterFirstName(e.currentTarget.value)
                      }}
                    />
                  </Form.Field>

                  <Form.Field required>
                    <label>
                      {
                        getI18n(this.props.langId, 'Last name')
                      }
                    </label>
                    <MaterialInput
                      value={this.props.registerLastName}
                      onChange={(e: InputOnChangeEvent) => {
                        this.props.setRegisterLastName(e.currentTarget.value)
                      }}
                    />
                  </Form.Field>
                </Form.Group>

                <Form.Field required>
                  <label>
                    {
                      getI18n(this.props.langId, 'Email')
                    }
                  </label>
                  <MaterialInput type="email"
                                 value={this.props.registerEmail}
                                 onChange={(e: InputOnChangeEvent) => {
                                   this.props.setRegisterEmail(e.currentTarget.value)
                                 }}
                  />
                </Form.Field>


                <div>
                  <Button
                    onClick={() => {
                      this.props.setIsRegisterFormDisplayed(false)
                    }}
                  >
                    <Icon name="chevron left"/>
                    <span>
                      {
                        getI18n(this.props.langId, 'Sign In')
                      }
                    </span>

                  </Button>
                  <Button color="blue"
                          onClick={() => {
                            this.registerClicked()
                          }}
                          content={getI18n(this.props.langId, 'Register')}
                  >
                  </Button>
                </div>

              </div>
            }

          </div>
        </div>

        <div className="absolute-sized-centered-below-register cookie-info-wrapper">
          <div>
            <span>
              {
                getI18n(this.props.langId, "Note that an admin needs to activate your account manually after registration")
              }
            </span>
          </div>
        </div>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(RegisterForm)
