/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../state/reducers";
import MaterialInput from './materialInput'
import {MyProps} from './materialInput'
import {HTMLAttributes} from "react";
import {getI18n} from "../../../i18n/i18nRoot";

import {InputOnChangeEvent} from '../../types/reactEvents'

//const css = require('./styles.styl');

const mapStateToProps = (rootState: RootState , props: MyProps & HTMLAttributes<HTMLInputElement>) => {
  return {
    ...props,
    placeholder: props.placeholder,
    disabled: props.disabled,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

//HTMLAttributes
/**
 * usage:
 * <MaterialSearchInput
 * value={this.props.pagination.searchText}
 * onChange={(e: InputOnChangeEvent) => {
 *             this.props.setSearchText(e.currentTarget.value)
 *          }}
 * />
 */
class materialSearchInput extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <MaterialInput {...this.props}
                     icon="search"
                     placeholder={`${getI18n(this.props.langId, 'Search')}...`}
                     disabled={this.props.disabled}
                     value={this.props.value}
      />
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(materialSearchInput)