/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../state/reducers";
import {ValidationFunc} from "../../validation/validations";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string

  //use the wrapped select disabled property
  //readonly disabled?: boolean


  // noinspection TsLint
  /**
   * only used for validation
   */
  value: string | string[] | number;
  /**
   * the validations for the input
   *
   *  usage:
   *
   * <MaterialInput
   *    validations={validationRules.displayName}
   *    validationMessageKey={validationMessageKeys.displayName}
   *  />
   *
   *  validationRules must be specified in the reducer!!
   */
  readonly validations?: ReadonlyArray<ValidationFunc<any>>

  /**
   * the key of the message when the validation fails
   *
   *
   * usage:
   *
   * <MaterialInput
   *    validations={validationRules.displayName}
   *    validationMessageKey={validationMessageKeys.displayName}
   *  />
   *
   * validationMessageKeys must be specified in the reducer!!
   */
  readonly validationMessageKey?: string

}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


/**
 *
 *
 * usage:
 * MaterialSelectWrapper
 *   value={this.props.codeTemplate.mainFileId}
 * >
 *   <select
 *     value={this.props.codeTemplate.mainFileId}
 *     onChange={(e: SelectOnChangeEventData) => {
 *      ...
 *     }}
 *   >
 *     {
 *       this.props.allFiles.map(file => {
 *         return (
 *           <option key={file.id} value={file.id}>
 *             {
 *               file.fileNameWithExtension
 *             }
 *           </option>
 *         )
 *       })
 *     }
 *   </select>
 * </MaterialSelectWrapper>
 *
 */
class MaterialSelectWrapper extends React.Component<Props, any> {
  render(): JSX.Element {
    return (

      <div>
        <div className="ui input material-drop-down">
          {
            this.props.children
          }
          <span
            className={
              this.props.validations &&
              !this.props.validations.every((validation) => validation(this.props.value)) ? 'bar error-bar' :
                'bar'
            }
          >
          </span>
        </div>
        {
          this.props.validations &&
          <div className="error-colored brake-words">
            {
              !this.props.validations.every((validation) => validation(this.props.value)) &&
              this.props.validationMessageKey
            }
          </div>
        }

      </div>
    )
  }
}

const _default = connect(mapStateToProps, mapDispatchToProps)(MaterialSelectWrapper)
export default _default
