/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../state/reducers";
import {MyProps} from './renderWrapper'
import {contentWithAssetsToContentWithUrlsFilter} from "../../helpers/assetHelper";
import Logger from "../../helpers/logger";


//because we need mathjax support support in markdown we changed the parser a bit...
import markdown from '../../helpers/markdownHelper'

//const css = require('./styles.styl');

declare var MathJax: any;

const mapStateToProps = (rootState: RootState , props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here

                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class markdownRenderer extends React.Component<Props, any> {

  componentDidMount(): void {
    //if we enable this there might be the small chance that
    //both Typeset calls run on the same data and some parts are rendered twice.. e.g. 1 --> 11
    //this.refreshMath();

    //i think the following can happen:
    //dummy is initial true -> rendered --> Typeset (not finished yet...)
    //the right props are set (dummy is set to false because assets are created) (the task description from the exercise)
    //update/mount is called with the new props --> Typeset (other Typeset is not yet finished)
    //--> both Typeset runs and some parts are rendered too quickly and we get some parts doubled

    if (this.props.dummy === false) {
      //this can happen if the user opens a dialog --> this will be unmounted
      //but dummy won't change anymore (only set to false once when everything is loaded & assets created)
      this.refreshMath();
    }

  }

  componentDidUpdate(prevProps: Props, prevState: any): void {
    //see componentDidMount for explanation
    if (this.props.content && this.props.dummy === false) {
      this.refreshMath()
    }
  }

  refreshMath(): void {
    Logger.debug("markdownRenderer", "rendering math...")
    const markdownBodies = document.getElementsByClassName('markdown-body')
    // noinspection TsLint
    for(let i = 0; i < markdownBodies.length;i++) {
      MathJax.Hub.Typeset(markdownBodies.item(i), null)
    }
  }

  render(): JSX.Element {


    Logger.debug("markdownRenderer", "rendering...")

    // noinspection TsLint
    let content = 'invalid syntax, please see log'

    try {
      // content = contentWithAssetsToContentWithUrlsFilter(marked(this.props.content))
      content = contentWithAssetsToContentWithUrlsFilter(markdown.render(this.props.content))
    } catch(err) {
      Logger.warn(`could not parse markdown/question: ${err.message}`)
    }

    // const wrapperDiv = `<div class="markdown-body">${content}</div>`


    return (
      <div id={this.props.printId} className="fh full-w">
        {/*<SafetyWrapper htmlContent={wrapperDiv}/>*/}
        <div className="markdown-body" dangerouslySetInnerHTML={{__html: content}} style={{fontSize: this.props.fontSizeInPx}}></div>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(markdownRenderer)
