/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../state/reducers";
import {Divider, Icon, List, Popup} from "semantic-ui-react";
import {setIsLoggerPanelOpen} from "../../state/actions/loggerPanelActions";
import * as constants from "../../constants";
import {
  createCustomProjectSiteToLinkPath,
  exerciseEditorChangeLogoutHelp,
  exerciseEditorCreateNewLogoutHint,
  getCustomProjectLinkPath,
  leftIconPopupOffset,
  versionString
} from "../../constants";
import * as loginLayer from '../../communicationLayer/loginLayer'
import history from "../../routerHistory";
import {Link} from "react-router-dom";
import {notifyError} from "../../helpers/notificationHelper";
import {setIsLoggingOut, setIsUserMenuOpen} from "../../state/actions/siteHeaderBarActions";
import {MyPopup} from "../helpers/myPopup";
import {getI18n} from "../../../i18n/i18nRoot";
import {askDialog, htmlDialog, infoDialog} from '../../helpers/dialogHelper'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    id: rootState.userDataSettingsSate.userData.id,
    firstName: rootState.userDataSettingsSate.userData.firstName,
    lastName: rootState.userDataSettingsSate.userData.lastName,
    systemRole: rootState.userDataSettingsSate.userData.systemRole,
    systemRoleId: rootState.userDataSettingsSate.userData.systemRole
                  ? rootState.userDataSettingsSate.userData.systemRole.id
                  : -1,
    isLoggerPanelOpen: rootState.loggerPanelState.isOpen,
    isUserMenuOpen: rootState.siteHeaderBarState.isUserMenuOpen,

    //for these sites it's easy to detect changes
    someSiteHasUnsavedChanges: rootState.doExerciseSiteState.doEditorState.hasUnsavedChanges
    ,


    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setIsLoggerPanelOpen,

                                                                        setIsUserMenuOpen,
                                                                        setIsLoggingOut

                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

const userMenuOffset = 5

class SiteHeaderUserMenuArea extends React.Component<Props, any> {

  userMenuEntryClicked(): void {
    this.props.setIsUserMenuOpen(false)
  }

  render(): JSX.Element {

    const systemRoleDisplayName = this.props.systemRole
                                  ? this.props.systemRole.displayName
                                  : getI18n(this.props.langId, 'None')

    return (
      <div className="flexed">
        <div className="mar-right">
          <span className="mar-right-half">
            {this.props.firstName}
          </span>
          <span>
            {this.props.lastName}
          </span>
        </div>

        <div className="clickable">
          <MyPopup
            className="site-user-menu"
            trigger={<Icon size="large" name="user circle"></Icon>}
            position="bottom right"
            on="click"
            horizontalOffset={leftIconPopupOffset - userMenuOffset}
            open={this.props.isUserMenuOpen}
            onOpen={() => this.props.setIsUserMenuOpen(true)}
            onClose={() => this.props.setIsUserMenuOpen(false)}
          >
            <Popup.Content>
              <List>
                {/*<List.Item icon="user circle" content={this.props.firstName + ' ' + this.props.lastName}/>*/}
                <List.Item>
                  <List.Icon name='user circle' onClick={()=> {
                    //small egg
                    htmlDialog(`Osterei gefunden!`,
                               `<div style="text-align: center;white-space: pre;">5d0ji41mhkfj-6779

Wirst du entkommen der Nacht,
oder bleibt allein der Traum?
Löse die Rätsel mit Bedacht,
und geh von Raum zu Raum.
Doch verlässt du diese Welt:
gibt es kein zurück,
außer du kennst den Trick
mit der URL im Blick!

</div>`, this.props.langId)
                  }} />
                  <List.Content>{this.props.firstName + ' ' + this.props.lastName}</List.Content>
                </List.Item>


                <List.Item>
                  <List.Icon name="id card"/>
                  <List.Content>
                    <span className="mar-right-half">
                      {
                        `${getI18n(this.props.langId, 'System role')}:`
                      }
                    </span>
                    <span>{systemRoleDisplayName}</span>
                  </List.Content>
                </List.Item>

                <List.Item>
                  <Link to={constants.assessmentStatisticsSiteToLinkPath + "/"}
                        onClick={() => this.userMenuEntryClicked()}
                  >
                    <Icon name='area chart' className="mar-right-half"/>
                    <span className="mar-left-half">
                        {
                          getI18n(this.props.langId, 'My assessment statistic')
                        }
                        </span>
                  </Link>
                </List.Item>
                <Divider/>

                <List.Item>
                  <Link to={constants.ownSettingsLinkPath} onClick={() => this.userMenuEntryClicked()}>
                    <Icon name="settings"/>
                    <span className="mar-left-half">
                      {
                        getI18n(this.props.langId, 'Settings')
                      }
                    </span>
                  </Link>
                </List.Item>

                <List.Item className={'clickable' + (this.props.isLoggerPanelOpen
                                                     ? ' accent-colored'
                                                     : '')}
                           onClick={() => {
                             this.props.setIsLoggerPanelOpen(!this.props.isLoggerPanelOpen)
                             this.userMenuEntryClicked()
                           }}
                >
                  {
                    //we need the forced padding because .item > .list but the list class is only for the icon
                  }
                  <List.Icon name="unordered list" style={{paddingLeft: 0}}/>
                  <List.Content>
                    <span>
                      {
                        getI18n(this.props.langId, 'Log')
                      }
                    </span>
                  </List.Content>
                </List.Item>

                <List.Item>
                  <List.Content>
                    <span className="mar-right-half">
                      {
                        getI18n(this.props.langId, 'Version')
                      }
                      :
                      </span>
                    <span>
                       {
                         versionString
                       }
                    </span>
                  </List.Content>
                </List.Item>

                <List.Item>
                  <List.Content>
                    <a href={constants.privacyPolicyPageLinkPath} onClick={() => this.userMenuEntryClicked()}
                       target="_blank">
                      <Icon name="shield"/>
                      <span className="mar-left-half">
                      {
                        getI18n(this.props.langId, 'Privacy policy')
                      }
                    </span>
                      <Icon className="mar-left-half" name="external alternate"/>
                    </a>
                  </List.Content>
                </List.Item>

                <List.Item>
                  <List.Content>
                    <a href={constants.copyrightPageLinkPath} onClick={() => this.userMenuEntryClicked()}
                       target="_blank">
                      <Icon name="law"/>
                      <span className="mar-left-half">
                      {
                        getI18n(this.props.langId, 'Copyright page')
                      }
                    </span>
                      <Icon className="mar-left-half" name="external alternate"/>
                    </a>
                  </List.Content>
                </List.Item>

                <List.Item>
                  <List.Content>
                    <a href={constants.aboutPageLinkPath} onClick={() => this.userMenuEntryClicked()} target="_blank">
                      <Icon name="question circle outline"/>
                      <span className="mar-left-half">
                      {
                        getI18n(this.props.langId, 'About page')
                      }
                    </span>
                      <Icon className="mar-left-half" name="external alternate"/>
                    </a>
                  </List.Content>
                </List.Item>

                <Divider/>

                <List.Item className="clickable"
                           onClick={async () => {

                             this.userMenuEntryClicked()

                             console.log(history.location)

                             //we suppress the prompts in the sites and show the dialog here...
                             if (this.props.someSiteHasUnsavedChanges) {
                               const shouldLogout = await askDialog(getI18n(this.props.langId, "Logout"),
                                                                    getI18n(this.props.langId,
                                                                            "It seems like you have some unsaved changes. Are you really sure you want to logout?"
                                                                    ), this.props.langId
                               )
                               if (!shouldLogout) return
                             }

                             //for these sites it's hard to check if they have changes... because they have deep nested types...
                             //so we expect always changes
                             if (history.location.pathname.startsWith(getCustomProjectLinkPath)
                               || history.location.pathname.startsWith(createCustomProjectSiteToLinkPath)
                               || history.location.pathname.startsWith(exerciseEditorChangeLogoutHelp)
                               || history.location.pathname.startsWith(exerciseEditorCreateNewLogoutHint)
                             ) {
                               const shouldLogout = await askDialog(getI18n(this.props.langId, "Logout"),
                                                                    getI18n(this.props.langId,
                                                                            "You might have some unsaved changes. Are you sure you want to logout?"
                                                                    ), this.props.langId
                               )
                               if (!shouldLogout) return
                             }

                             this.props.setIsLoggingOut(true)


                             try {
                               await loginLayer.logout()

                               //don't clear the frontend settings here...
                               //when we do a login and the user is different then we clear the frontend settings

                             } catch (err) {
                               this.props.setIsLoggingOut(false)
                               notifyError(getI18n(this.props.langId, 'Logout error'))
                               return
                             }


                             //we need to change the url or the next user will start at the last site the user logged out...
                             history.replace(constants.homeLinkPath)

                             // await wait(1000)
                             // window.location.reload(true) //forces to ignore the cache...
                             window.location.reload() //reload so we clear everything (state, memory ...)
                             // this.props.setIsLoggingOut(false) //this is done via the reload

                           }}
                >
                  <List.Icon name="sign out"/>
                  <List.Content>
                    <span>
                      {
                        getI18n(this.props.langId, 'Logout')
                      }
                    </span>
                  </List.Content>
                </List.Item>

              </List>
            </Popup.Content>
          </MyPopup>
        </div>
        {
          /*
          not yet used
          <div className="mar-right clickable">
          <Icon size="large" name="alarm"></Icon>
        </div>
           */
        }


      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(SiteHeaderUserMenuArea)
