/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";

import ListView from './listView'
import SinglePanelSiteWrapper from '../../../singlePanelSiteWrapper'
import MaterialSearchInput from '../../../material/materialSearchInput'
import {
  getClosedExercisesViaCode,
  loadClosedExerciseViaCodeAsync,
  setActivePaginationPage,
  setPaginationPageSize,
  setSearchText,
  setSortByKey
} from "../../../../state/actions/allAccessibleExercises/closedExercisesSite/closedExercisesSiteActions";
import Spinner from "../../../helpers/spinner";
import TagFilterPanel from '../../../tagComponents/tagsFilterPanel'
import {getI18n} from "../../../../../i18n/i18nRoot";
import {InputOnChangeEvent} from '../../../../types/reactEvents'
import {ErrorHelper} from '../../../../helpers/errorHelper'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string

  /**
   * true: load all data,
   * false: not only exercises
   */
  readonly shouldLoadData: boolean

  /**
   * true: can display, false not (probably because parent does)
   */
  readonly canDisplayTagFilterPanel: boolean

}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    closedExercisePreviews: rootState.closedExercisesState.closedExercisePreviewsViaCode,
    isLoading: rootState.closedExercisesState.isLoading  || rootState.closedExercisesState.isLoadingViaCode
      || rootState.manageTagsState.isLoading,
    shouldLoadData: props.shouldLoadData,

    sortByKey: rootState.closedExercisesState.paginationViaCode.orderByProp,
    sortDirection: rootState.closedExercisesState.paginationViaCode.isOrderByDesc,
    pagination: rootState.closedExercisesState.paginationViaCode,

    selectedFilterTagsIds: rootState.tagsFilterState.selectedFilterTagsIds,
    selectedNegativeFilterTagsIds: rootState.tagsFilterState.selectedNegativeFilterTagsIds,

    canDisplayTagFilterPanel: props.canDisplayTagFilterPanel,
    isTagsFilterDisplayed: rootState.closedExercisesState.isTagsFilterDisplayed,
    tags: rootState.manageTagsState.tags,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        loadClosedExerciseViaCodeAsync,
                                                                        getClosedExercisesViaCode,
                                                                        setSortByKey,
                                                                        setSearchText,
                                                                        setActivePaginationPage,
                                                                        setPaginationPageSize
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class ClosedExercisesSite extends React.Component<Props, any> {


  async componentWillMount(): Promise<void> {

    try {
      if (this.props.shouldLoadData) {
        await this.props.loadClosedExerciseViaCodeAsync()
        return
      }

      //only exercises
      await this.props.getClosedExercisesViaCode()
    } catch(err) {

      ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Error', 'Could not load site data')
    }
  }

  render(): JSX.Element {


    return (

      <div className="multiple-single-panels">

        {
          this.props.isTagsFilterDisplayed && this.props.canDisplayTagFilterPanel &&
          <TagFilterPanel
            isLoading={this.props.isLoading}
            allTags={this.props.tags}
            onTagsUpdated={this.props.getClosedExercisesViaCode}
          />
        }

        <SinglePanelSiteWrapper>

          {
            this.props.isLoading &&
            <Spinner/>
          }

          <div className="view-padding">
            <div className="view-head-line">
              <div className="view-caption">
                <h1>{getI18n(this.props.langId, 'Closed exercises via code')}</h1>
              </div>
              <div className="view-options">
                <MaterialSearchInput
                  value={this.props.pagination.searchText}
                  onChange={(e: InputOnChangeEvent) => {
                    this.props.setSearchText(e.currentTarget.value, true)
                  }}
                />
              </div>
            </div>

            <div className="view-content">
                <ListView
                  pagination={this.props.pagination}
                  setActivePaginationPage={this.props.setActivePaginationPage}
                  setPaginationPageSize={this.props.setPaginationPageSize}
                  exercisePreviews={this.props.closedExercisePreviews}
                  displayAsViaCode={true}
                  sortByKey={this.props.sortByKey}
                  sortDirection={this.props.sortDirection}
                  setSortByKey={this.props.setSortByKey}
                />
            </div>
          </div>

        </SinglePanelSiteWrapper>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ClosedExercisesSite)
