/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {Checkbox, Icon, Table} from "semantic-ui-react";
import {ReleaseDurationType} from "../../../../types/ReleaseDurationType";
import {ReleaseStartType} from "../../../../types/ReleaseStartType";
import {ExercisePreviewFromBackend} from "../../../../types/exercisePreview";
import {isCurrentSortingColumnPaginated, rotateSorting} from "../../../../helpers/tablesHelper";
import {copyToClipboard} from "../../../../helpers/clipboard";
import {getDoExerciseSiteToLink} from "../../../../constants";
import {Link} from "react-router-dom";
import {displayDateAndTime} from "../../../../helpers/convertersAndTransformers";
import Tag from '../../../tagComponents/tag'
import {HelpPopup} from "../../../helpers/helpPopup";
import {getI18n} from "../../../../../i18n/i18nRoot";
import PaginationTableFooter from '../../../helpers/paginationTableFooter'
import {PaginationStateData} from '../../../../types/pagination'
import {getLeftTimeString} from '../../../../helpers/editorHelper'

//const css = require('../styles.styl');

export interface MyProps {
  //readonly test: string
  readonly exercisePreviews: ReadonlyArray<ExercisePreviewFromBackend>

  readonly pagination: PaginationStateData<ExercisePreviewFromBackend>

  /**
   * true: display the releases as they were accessed via code,
   * false: display the releases as they are always accessible/accessed (release is visible to all setting)
   */
  readonly displayAsViaCode: boolean

  readonly sortByKey: keyof ExercisePreviewFromBackend
  readonly sortDirection: boolean | null
  readonly setSortByKey: (sortByKey: keyof ExercisePreviewFromBackend, sortDirection: boolean | null,
                          viaCode: boolean
  ) => void

  readonly setPaginationPageSize: (pageSize: number, viaCode: boolean) => void
  readonly setActivePaginationPage: (activePage: number, viaCode: boolean) => void
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here

                                                                      }, dispatch)

const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class AllAccessibleExercisesList extends React.Component<Props, any> {


  render(): JSX.Element {

    const list: ReadonlyArray<ExercisePreviewFromBackend> = this.props.exercisePreviews

    return (
      <div>

        <Table sortable selectable padded striped singleLine className="material-table">
          <Table.Header>
            <Table.Row>
              <Table.HeaderCell>
                <span>
                  {
                    getI18n(this.props.langId, 'Open')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<ExercisePreviewFromBackend>(this.props.sortByKey, 'displayName',
                                                                              this.props.sortDirection
                  )
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('displayName',
                                                rotateSorting<ExercisePreviewFromBackend>(this.props.sortDirection,
                                                                                          this.props.sortByKey,
                                                                                          'displayName',
                                                ),
                                                this.props.displayAsViaCode
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Name')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<ExercisePreviewFromBackend>(this.props.sortByKey, 'shortDescription',
                                                                              this.props.sortDirection
                  )
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('shortDescription',
                                                rotateSorting<ExercisePreviewFromBackend>(this.props.sortDirection,
                                                                                          this.props.sortByKey,
                                                                                          'shortDescription'
                                                ),
                                                this.props.displayAsViaCode
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Description')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span>
                  <HelpPopup icon="flag outline"
                             defaultText={getI18n(this.props.langId, 'Should not count')}/>
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span>
                 <HelpPopup icon="lock"
                            defaultText={getI18n(this.props.langId, 'Locked permanently')}/>
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span>
                  {
                    getI18n(this.props.langId, 'Tags')
                  }
                </span>
              </Table.HeaderCell>
              {
                this.props.displayAsViaCode &&
                <Table.HeaderCell>
                  <span>
                    {
                      getI18n(this.props.langId, 'Release code')
                    }
                  </span>
                </Table.HeaderCell>
              }
              <Table.HeaderCell>
                <span>
                  {
                    getI18n(this.props.langId, 'Working time')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span>
                  {
                    getI18n(this.props.langId, 'End')
                  }
                </span>
              </Table.HeaderCell>
            </Table.Row>
          </Table.Header>

          <Table.Body>
            {
              list.map((exercisePreview, index) => {
                //we can't use the id for the key because one exercise can have multiple releases...
                return (
                  <Table.Row key={exercisePreview.generatedCode}>
                    <Table.Cell>
                      <Link to={getDoExerciseSiteToLink(exercisePreview.generatedCode)} className="clickable">
                        <Icon name="share"/>
                      </Link>
                    </Table.Cell>
                    <Table.Cell>
                      <div className="flexed">
                        <HelpPopup className="mar-right-half" defaultText={exercisePreview.displayName}/>
                        <span>{exercisePreview.displayName}</span>
                      </div>
                    </Table.Cell>
                    <Table.Cell>
                      {exercisePreview.shortDescription &&
                      <HelpPopup defaultText={exercisePreview.shortDescription}/>
                      }
                    </Table.Cell>
                    <Table.Cell>
                      <div>
                        <Checkbox checked={exercisePreview.shouldNotCount} disabled/>
                      </div>
                    </Table.Cell>
                    <Table.Cell>
                      <div>
                        <Checkbox checked={exercisePreview.isPermanentlyLocked} disabled/>
                      </div>
                    </Table.Cell>
                    <Table.Cell>
                      {
                        (exercisePreview.tags && exercisePreview.tags.length > 0) &&
                        <HelpPopup
                          wide="very"
                          defaultText={
                            <div className="exercise-tag-wrapper">
                              {
                                exercisePreview.tags.map((tag, index2) => {
                                  return (
                                    <Tag key={tag.id} tag={tag}/>
                                  )
                                })
                              }
                            </div>
                          }
                        />
                      }
                    </Table.Cell>
                    {
                      this.props.displayAsViaCode &&
                      <Table.Cell>
                        <Icon name="copy" className="clickable mar-right-half"
                              onClick={() => {
                                copyToClipboard(exercisePreview.generatedCode)
                              }}
                        />
                        <span>{exercisePreview.generatedCode}</span>
                      </Table.Cell>
                    }

                    <Table.Cell>
                      {
                        (exercisePreview.hasLimitedWorkingTime
                         ?
                         <span>{getLeftTimeString(exercisePreview.availableWorkingTimeInMinutes * 60,
                                                  this.props.langId
                         )}</span>
                         :
                         <span>&infin;</span>
                        )
                      }
                    </Table.Cell>

                    <Table.Cell>
                      {
                        exercisePreview.releaseStartType === ReleaseStartType.Manual
                        ? <span>{getI18n(this.props.langId, 'Manual')}</span>
                        : null
                      }
                      {
                        exercisePreview.releaseStartType === ReleaseStartType.Automatic
                        && exercisePreview.releaseDurationType === ReleaseDurationType.Limited
                        ?
                        <span>{displayDateAndTime(exercisePreview.automaticEndAt, -1)}</span>
                        : null
                      }
                    </Table.Cell>

                  </Table.Row>
                )
              })
            }

          </Table.Body>
          <PaginationTableFooter
            pagination={this.props.pagination}
            setPaginationPageSize={(pageSize) => {
              this.props.setPaginationPageSize(pageSize, this.props.displayAsViaCode)
            }}
            setActivePaginationPage={(activePage) => {
              this.props.setActivePaginationPage(activePage, this.props.displayAsViaCode)
            }}
          />

        </Table>


      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(AllAccessibleExercisesList)
