/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {SingleAssessmentStatistic} from "../../../types/assessmentStatistics";
import {RootState} from "../../../state/reducers";
import {Table, Icon, Checkbox} from "semantic-ui-react";
import {HelpPopup} from "../../helpers/helpPopup";
import {isNotEmptyOrSpaces} from "../../../validation/validations";
import {displayDate, displayTime} from "../../../helpers/convertersAndTransformers";
import {getDoExerciseSiteToLink, getTutorViewForSubmission} from "../../../constants";
import {Link} from "react-router-dom";
import {getI18n} from "../../../../i18n/i18nRoot";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly singleAssessmentStatistic: SingleAssessmentStatistic
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    assessmentStatistics: rootState.assessmentStatisticsSiteState.assessmentStatistics,
    singleAssessmentStatistic: props.singleAssessmentStatistic,
    pLangs: rootState.pLangsState.pLangs,

    currentUserId: rootState.userDataSettingsSate.userData
      ? rootState.userDataSettingsSate.userData.id
      : -1,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class listViewItem extends React.Component<Props, any> {
  render(): JSX.Element {
    return (

      <Table.Row>
        <Table.Cell>
          {
            this.props.singleAssessmentStatistic.exerciseDisplayName
          }
        </Table.Cell>
        <Table.Cell>
          {
            isNotEmptyOrSpaces(this.props.singleAssessmentStatistic.exerciseShortDescription) &&
            <HelpPopup defaultText={this.props.singleAssessmentStatistic.exerciseShortDescription}/>
          }
        </Table.Cell>
        <Table.Cell>

          {
            this.props.pLangs.find(p => p.id === this.props.singleAssessmentStatistic.pLangId) &&
            this.props.pLangs.find(p => p.id === this.props.singleAssessmentStatistic.pLangId).displayName
          }
        </Table.Cell>
        <Table.Cell>
          <div>
            <Checkbox checked={this.props.singleAssessmentStatistic.exerciseReleaseIsReleased} disabled/>
          </div>
        </Table.Cell>
        <Table.Cell>
          <div>
            <Checkbox checked={this.props.singleAssessmentStatistic.shouldNotCount} disabled/>
          </div>
        </Table.Cell>
        <Table.Cell>
          <span>
            {
              this.props.singleAssessmentStatistic.participationCount === -1 ? getI18n(this.props.langId,
                'Error') : this.props.singleAssessmentStatistic.participationCount
            }
          </span>
        </Table.Cell>
        <Table.Cell>
          {
            this.props.singleAssessmentStatistic.normalTestPoints !== null &&
            `${this.props.singleAssessmentStatistic.normalTestPoints} / ${this.props.singleAssessmentStatistic.maxNormalTestPoints}`
          }
        </Table.Cell>

        <Table.Cell>
          {
            this.props.singleAssessmentStatistic.submitTestPoints !== null &&
            `${this.props.singleAssessmentStatistic.submitTestPoints} / ${this.props.singleAssessmentStatistic.maxSubmitTestPoints}`
          }
        </Table.Cell>

        <Table.Cell>
          {
            this.props.singleAssessmentStatistic.manualPoints !== null &&
            <span>
              {
                this.props.singleAssessmentStatistic.manualPoints
              }
              {
                this.props.singleAssessmentStatistic.maxManualPoints !== null &&
                <span>
                  {
                    ` / ${this.props.singleAssessmentStatistic.maxManualPoints}`
                  }
                </span>
              }
            </span>
          }
        </Table.Cell>

        <Table.Cell>
          {
            isNotEmptyOrSpaces(this.props.singleAssessmentStatistic.feedbackForStudent) &&
            <HelpPopup defaultText={this.props.singleAssessmentStatistic.feedbackForStudent}
                       wide='very'
            />
          }
        </Table.Cell>

        <Table.Cell>
          {
            `${displayTime(this.props.singleAssessmentStatistic.lastUpdatedAt, -1)} - ${displayDate(
              this.props.singleAssessmentStatistic.lastUpdatedAt, -1)}`
          }
        </Table.Cell>

        {
          this.props.currentUserId !== this.props.assessmentStatistics.userId &&
          <Table.Cell>
            <div>

              <Link to={getTutorViewForSubmission(null,this.props.singleAssessmentStatistic.exerciseReleaseId,
                this.props.assessmentStatistics.userId,
                this.props.singleAssessmentStatistic.pLangId)}>
                <Icon name="life ring"/>
              </Link>
            </div>
          </Table.Cell>
        }

        {
          this.props.currentUserId === this.props.assessmentStatistics.userId &&
          <Table.Cell>
            <div>
              <Link to={getDoExerciseSiteToLink(this.props.singleAssessmentStatistic.releaseCode)}>
                <Icon name="share"/>
              </Link>
            </div>
          </Table.Cell>
        }


      </Table.Row>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(listViewItem)
