/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import Spinner from "../../helpers/spinner";
import SinglePanelSiteWrapper from '../../singlePanelSiteWrapper'
import {Area, AreaChart, CartesianGrid, ResponsiveContainer, XAxis, YAxis} from "recharts";
import {convertSizeToHumanReadableSize} from "../../../helpers/convertersAndTransformers";
import {dashboardRefreshIntervalInS, leftIconPopupOffset} from "../../../constants";
import {MyPopup} from "../../helpers/myPopup";
import {getI18n} from "../../../../i18n/i18nRoot";
import {Icon} from 'semantic-ui-react'
import {HelpPopup} from '../../helpers/helpPopup'
import {setIsHardwareUsageStatisticRefreshing} from '../../../state/actions/dashboardSite/dashboardSiteActions'
import {getHardwareUsageAsync} from '../../../state/actions/dashboardSite/dashboardCrudActions'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    hardwareUsage: rootState.dashboardSiteState.hardwareUsageStatistic,
    isHardwareUsageStatisticRefreshing: rootState.dashboardSiteState.isHardwareUsageStatisticRefreshing,
    isLoading: rootState.dashboardSiteState.isLoading,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setIsHardwareUsageStatisticRefreshing,
                                                                        getHardwareUsageAsync,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

interface UsageDataPoint {
  readonly time: string
  readonly value: number
}

// noinspection TsLint
let intervalHandle: number = null

class hardwareUsageView extends React.Component<Props, any> {

  async componentWillMount(): Promise<void> {

    intervalHandle = setInterval(async () => {

      if (this.props.isHardwareUsageStatisticRefreshing) {
        await this.props.getHardwareUsageAsync()
      }

    }, dashboardRefreshIntervalInS * 1000)

  }

  componentWillUnmount(): void {

    if (intervalHandle) {
      clearInterval(intervalHandle)
    }
  }

  render(): JSX.Element {


    const cpuUsageData: ReadonlyArray<UsageDataPoint> = this.props.hardwareUsage.map(p => {
      const temp: UsageDataPoint = {
        value: p.cpuUsageInPercentage,
        time: p.time
      }
      return temp
    })


    const ramUsageData: ReadonlyArray<UsageDataPoint> = this.props.hardwareUsage.map(p => {
      // noinspection TsLint
      const temp: UsageDataPoint = {
        value: (p.totalRamInKb - p.freeRamInKb) / 1024, //mb
        time: p.time
      }
      return temp
    })

    // console.log(this.props.hardwareUsage)

    // noinspection TsLint
    return (
      <SinglePanelSiteWrapper>

        {
          this.props.isLoading &&
          <Spinner/>
        }


        <div>
          <div className="view-head-line">
            <div className="view-caption">
              <h1>
                {
                  getI18n(this.props.langId, 'Hardware charts')
                }
              </h1>

            </div>
            <div className="view-options">

              {
                this.props.isHardwareUsageStatisticRefreshing &&
                <MyPopup

                  trigger={
                    <div className="live-sonar clickable"
                         onClick={() => this.props.setIsHardwareUsageStatisticRefreshing(false)}>
                      <div>
                  <span>
                    <span/>
                  </span>
                      </div>
                    </div>
                  }
                  content={
                    getI18n(this.props.langId,
                            'This view/panel is live and will automatically refresh (not the whole site). Click to stop automatic refreshing'
                    )
                  }
                />
              }

              {
                !this.props.isHardwareUsageStatisticRefreshing &&
                <div>
                  <MyPopup
                    trigger={
                      <div className="clickable" onClick={() => {
                        this.props.setIsHardwareUsageStatisticRefreshing(true)
                      }}>
                        <Icon name="refresh"/>
                      </div>
                    }
                    content={getI18n(this.props.langId, "Click to make the view automatically refresh")}
                  />
                </div>
              }


            </div>
          </div>

          <div className="view-content">


            <div className="flexed">

              {
                //cpu
                cpuUsageData.length > 0 &&

                <div style={{width: '45%'}}>

                  <div className="flexed-h">
                    <h2>
                      {
                        getI18n(this.props.langId, 'CPU')
                      }

                      {
                        ` (${this.props.hardwareUsage[this.props.hardwareUsage.length - 1].cpuUsageInPercentage})`
                      }
                    </h2>

                    <ResponsiveContainer
                      width={'100%'}
                      height={200}
                      debounce={1000}
                    >

                      <AreaChart width={600} height={400} data={cpuUsageData as any[]}
                                 margin={{
                                   top: 10,
                                   right: 30,
                                   left: 0,
                                   bottom: 0
                                 }}>

                        <XAxis dataKey="time"/>
                        <YAxis domain={[0, 100]}/>
                        <Area type='monotone' dataKey='value' stroke='#8884d8' fill='#8884d8'/>

                        <CartesianGrid stroke="#eee" strokeDasharray="5 5"/>
                      </AreaChart>

                    </ResponsiveContainer>


                  </div>

                </div>
              }

              {
                //ram
                ramUsageData.length > 0 &&
                <div style={{width: '45%'}}>

                  <div className="flexed-h">

                    <h2>
                      {
                        getI18n(this.props.langId, 'RAM') + ' '
                      }
                      (
                      {
                        `${convertSizeToHumanReadableSize(
                          this.props.hardwareUsage[this.props.hardwareUsage.length - 1].freeRamInKb * 1024,
                          2
                        )} ${getI18n(this.props.langId, 'free')}/${convertSizeToHumanReadableSize(
                          this.props.hardwareUsage[this.props.hardwareUsage.length - 1].totalRamInKb * 1024,
                          2
                        )}`
                      }
                      )
                    </h2>

                    <ResponsiveContainer
                      width={'100%'}
                      height={200}
                      debounce={1000}
                    >

                      <AreaChart width={600} height={400} data={ramUsageData as any[]}
                                 margin={{
                                   top: 10,
                                   right: 30,
                                   left: 0,
                                   bottom: 0
                                 }}>

                        <XAxis dataKey="time"/>
                        <YAxis domain={[0, this.props.hardwareUsage[0].totalRamInKb / 1024]}/>
                        <Area type='monotone' dataKey='value' stroke='#8884d8' fill='#8884d8'/>

                        <CartesianGrid stroke="#eee" strokeDasharray="5 5"/>
                      </AreaChart>

                    </ResponsiveContainer>

                  </div>

                </div>
              }


            </div>


          </div>
        </div>

      </SinglePanelSiteWrapper>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(hardwareUsageView)
