/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import MultiTabCodeEditorWrapper from '../../../codeEditors/multiTabCodeEditorWrapper'
import {
  closeTabChangeAfterEditor,
  openTabChangeAfterEditor,
  setAfterEditor_hasFileChangedMap,
  setAfterEditor_hasUnsavedChanges,
  setAfterEditor_isFileExplorerDisplayedDoEditor,
  setAfterEditor_isRenamingFileDoEditor,
  setAfterEditor_newFileNameDoEditor,
  setAfterEditor_renamingFileIdDoEditor,
  setAfterEditor_renamingFileNameDoEditor,
  setAfterEditor_selectedTabIdDoEditor
} from "../../../../state/actions/doExerciseSite/editorActions/afterEditorActions";
import {SolutionFileDoExerciseFullBase} from "../../../../types/exerciseSolution";
import Logger from "../../../../helpers/logger";
import {errorDialog} from "../../../../helpers/dialogHelper";
import {setAfterSolutionFiles} from "../../../../state/actions/doExerciseSite/doExerciseSiteActions";
import {EditorTab} from "../../../codeEditors/multiTabCodeEditorProps";
import {getTabModeFromExtension, setSolutionFiles_AndRefreshEditor} from "../../../../helpers/editorHelper";
import {getI18n} from "../../../../../i18n/i18nRoot";
import {doEditorAfterEditorPanelViewEditorId, multiFileEditorDebounceInMs} from '../../../../constants'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    doExercise: rootState.doExerciseSiteState.exercise,
    afterSolution: rootState.doExerciseSiteState.afterSolution,

    pLangs: rootState.pLangsState.pLangs,

    selectedTabIdDoEditor: rootState.doExerciseSiteState.afterEditorState.selectedTabIdDoEditor,
    isFileExplorerDisplayedDoEditor: rootState.doExerciseSiteState.afterEditorState.isFileExplorerDisplayedDoEditor,
    newFileNameDoEditor: rootState.doExerciseSiteState.afterEditorState.newFileNameDoEditor,
    renamingFileIdDoEditor: rootState.doExerciseSiteState.afterEditorState.renamingFileIdDoEditor,
    isRenamingFileDoEditor: rootState.doExerciseSiteState.afterEditorState.isRenamingFileDoEditor,
    renamingFileNameDoEditor: rootState.doExerciseSiteState.afterEditorState.renamingFileNameDoEditor,
    hasUnsavedChanges: rootState.doExerciseSiteState.afterEditorState.hasUnsavedChanges,
    hasFileChangedMap: rootState.doExerciseSiteState.afterEditorState.hasFileChangedMap,

    //release should be not active and assessment finished then the after tests can be run
    releaseFinishedAndAutomaticAssessmentHasFinished: !rootState.doExerciseSiteState.exercise.exerciseRelease.isReleased && rootState.doExerciseSiteState.exercise.exerciseRelease.hasAutomaticAssessmentFinished,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  setAfterEditor_hasUnsavedChanges,
  setAfterEditor_hasFileChangedMap,
  openTabChangeAfterEditor,
  closeTabChangeAfterEditor,
  setAfterEditor_selectedTabIdDoEditor,
  setAfterEditor_isFileExplorerDisplayedDoEditor,
  setAfterEditor_newFileNameDoEditor,
  setAfterEditor_renamingFileIdDoEditor,
  setAfterEditor_isRenamingFileDoEditor,
  setAfterEditor_renamingFileNameDoEditor,
  setAfterSolutionFiles,

  setSolutionFiles_AndRefreshEditor,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class afterEditorPanelView extends React.Component<Props, any> {
  render(): JSX.Element {

    const pLang = this.props.pLangs.find(p => p.id === this.props.afterSolution.pLangId)

    if (!pLang) {
      //TODO only error if really not found
      //on first render we have no plang but thats no and error!!
      return null
    }

    const editorTabs = this.props.afterSolution.solutionFiles.map<EditorTab>(p => {
      return {
        id: p.id,
        fileNameWithExtensions: p.fileNameWithExtension,
        content: p.content,
        isMainFile: p.id === this.props.afterSolution.mainFileId,
        isReadonly: !p.isEditableByUser,
        displayIndex: p.displayIndex,
        language: getTabModeFromExtension(p.fileNameWithExtension, this.props.pLangs),
        isDisplayed: p.uiIsDisplayed,
        isContentVisibleForUser: p.isContentVisibleForUser
      }
    })
      .sort((a, b) => a.fileNameWithExtensions.localeCompare(b.fileNameWithExtensions))

    return (<div className="fh hidden-overflow-i">
        <MultiTabCodeEditorWrapper
          id={doEditorAfterEditorPanelViewEditorId}
          height="100%"
          debounceTimeInMs={multiFileEditorDebounceInMs}
          selectedTabId={this.props.selectedTabIdDoEditor}
          canPrint={true}
          onSelectedTabIdChanged={(tabId: number) => {
            this.props.setAfterEditor_selectedTabIdDoEditor(tabId)
          }}

          isFileExplorerDisplayed={this.props.isFileExplorerDisplayedDoEditor}
          onChangeIsFileExplorerDisplayed={(isDisplayed: boolean) => {
            this.props.setAfterEditor_isFileExplorerDisplayedDoEditor(isDisplayed)
          }}

          newFileName={this.props.newFileNameDoEditor}
          onNewFileNameChanged={(name: string) => {
            this.props.setAfterEditor_newFileNameDoEditor(name)
          }}

          renamingFileId={this.props.renamingFileIdDoEditor}
          onRenamingFileIdChanged={(fileId: number) => {
            this.props.setAfterEditor_renamingFileIdDoEditor(fileId)
          }}

          isRenamingFile={this.props.isRenamingFileDoEditor}
          onIsRenamingFileChanged={(isRenamingFileDoEditor: boolean) => {
            this.props.setAfterEditor_isRenamingFileDoEditor(isRenamingFileDoEditor)
          }}

          renamingFileName={this.props.renamingFileNameDoEditor}
          onRenamingFileNameChanged={(newName: string) => {
            this.props.setAfterEditor_renamingFileNameDoEditor(newName)
          }}

          hasUnsavedChanges={this.props.hasUnsavedChanges}
          onHasUnsavedChangesChanged={(hasChanged: boolean) => {
            if (hasChanged === this.props.hasUnsavedChanges) return
            this.props.setAfterEditor_hasUnsavedChanges(true)
          }}

          userCanCreateNewFiles={this.props.doExercise.canUserCreateFiles}

          onCreateFileClicked={(editorTab: EditorTab) => {

            const newFile: SolutionFileDoExerciseFullBase = {
              id: editorTab.id,
              content: editorTab.content,
              fileNameWithExtension: editorTab.fileNameWithExtensions,
              uiIsDisplayed: editorTab.displayIndex >= 0,
              displayIndex: editorTab.displayIndex,
              isEditableByUser: true,
              templateFileId: null,
              isContentVisibleForUser: true,
            }

            // this.props.setAfterSolutionFiles(
            //   [...this.props.afterSolution.solutionFiles, newFile]
            // )

            this.props.setSolutionFiles_AndRefreshEditor([...this.props.afterSolution.solutionFiles, newFile],
              this.props.setAfterSolutionFiles
            )

          }}

          onOpenTabClicked={(editorTab: EditorTab) => {
            const file = this.props.afterSolution.solutionFiles.find(p => p.id === editorTab.id)

            if (!file) {
              Logger.error("file not found (open tab clicked)")
              errorDialog(getI18n(this.props.langId, 'Error'), getI18n(this.props.langId, 'Could not find editor file'),this.props.langId)
              return
            }

            const mappedFiles = this.props.afterSolution.solutionFiles.map<SolutionFileDoExerciseFullBase>(p => {
              return p.id === file.id
                     ? {
                  ...file,
                  uiIsDisplayed: true,
                }
                     : p
            })

            this.props.openTabChangeAfterEditor(file, mappedFiles, this.props.selectedTabIdDoEditor)
          }}

          onCloseTabClicked={(editorTab: EditorTab) => {

            const file = this.props.afterSolution.solutionFiles.find(p => p.id === editorTab.id)

            if (!file) {
              Logger.error("file not found (closed tab clicked)")
              errorDialog(getI18n(this.props.langId, 'Error'), getI18n(this.props.langId, 'Could not find editor file'),this.props.langId)
              return
            }

            const mappedFiles = this.props.afterSolution.solutionFiles.map<SolutionFileDoExerciseFullBase>(p => {
              return p.id === file.id
                     ? {
                  ...file,
                  uiIsDisplayed: false,
                }
                     : p
            })

            this.props.closeTabChangeAfterEditor(file, mappedFiles, this.props.selectedTabIdDoEditor)
          }}

          onFileRename={(editorTab: EditorTab, newName: string) => {

            const currentFile = this.props.afterSolution.solutionFiles.find(p => p.id === editorTab.id)

            if (!currentFile) {
              Logger.error("file not found (onFileRename)")
              errorDialog(getI18n(this.props.langId, 'Error'), getI18n(this.props.langId, 'Could not find editor file'),this.props.langId)
              return
            }

            this.props.setAfterSolutionFiles(
              this.props.afterSolution.solutionFiles.map<SolutionFileDoExerciseFullBase>(file => {
                return file.id === currentFile.id
                       ? {
                    id: file.id,
                    displayIndex: file.displayIndex,
                    fileNameWithExtension: newName,
                    isEditableByUser: file.isEditableByUser,
                    content: file.content,
                    templateFileId: file.templateFileId,
                    uiIsDisplayed: file.uiIsDisplayed,
                    isContentVisibleForUser: file.isContentVisibleForUser
                  }
                       : file
              }))
          }}

          onRemoveFile={(editorTab: EditorTab) => {

            // this.props.setAfterSolutionFiles(
            //   this.props.afterSolution.solutionFiles.filter(p => p.id !== editorTab.id)
            // )

            this.props.setSolutionFiles_AndRefreshEditor(
              this.props.afterSolution.solutionFiles.filter(p => p.id !== editorTab.id),
              this.props.setAfterSolutionFiles,
            )

          }}

          editorTabs={editorTabs}
          onBlur={(value, editorTab) => {
            const currentFile = this.props.afterSolution.solutionFiles.find(p => p.id === editorTab.id)

            if (!currentFile) {
              Logger.error("file not found (onChange)")
              errorDialog(getI18n(this.props.langId, 'Error'), getI18n(this.props.langId, 'Could not find editor file'),this.props.langId)
              return
            }

            this.props.setAfterSolutionFiles(
              this.props.afterSolution.solutionFiles.map<SolutionFileDoExerciseFullBase>(file => {
                return file.id === currentFile.id
                       ? {
                          id: file.id,
                          displayIndex: file.displayIndex,
                          fileNameWithExtension: file.fileNameWithExtension,
                          isEditableByUser: file.isEditableByUser,
                          content: value,
                          templateFileId: file.templateFileId,
                          uiIsDisplayed: file.uiIsDisplayed,
                          isContentVisibleForUser: file.isContentVisibleForUser
                        }
                       : file
              }))
          }}
          // onChange={(editorTab: EditorTab, newValue: string) => {
          //
          //   const currentFile = this.props.afterSolution.solutionFiles.find(p => p.id === editorTab.id)
          //
          //   if (!currentFile) {
          //     Logger.error("file not found (onChange)")
          //     errorDialog(getI18n(this.props.langId,'Error'),
          //       getI18n(this.props.langId,'Could not find editor file')
          //     )
          //     return
          //   }
          //
          //   this.props.setAfterSolutionFiles(
          //     this.props.afterSolution.solutionFiles.map<SolutionFileDoExerciseFullBase>(file => {
          //       return file.id === currentFile.id
          //         ? {
          //           id: file.id,
          //           displayIndex: file.displayIndex,
          //           fileNameWithExtension: file.fileNameWithExtension,
          //           isEditableByUser: file.isEditableByUser,
          //           content: newValue,
          //           templateFileId: file.templateFileId,
          //           uiIsDisplayed: file.uiIsDisplayed,
          //           uiEditSession: editorTab.editSession
          //         }
          //         : file
          //     })
          //   )
          // }}

          // onSomeEditorSessionChanged={(editorTabs: ReadonlyArray<EditorTab>) => {
          //   this.props.setAfterSolutionFiles(
          //     this.props.afterSolution.solutionFiles.map<SolutionFileDoExerciseFullBase>(file => {
          //       const changedTab = editorTabs.find(p => p.id === file.id)
          //       return changedTab
          //              ? {
          //           ...file,
          //           uiEditSession: changedTab.editSession
          //         }
          //              : file
          //     }))
          // }}

          hasFileChangedMap={this.props.hasFileChangedMap}
          onUpdateHasFileChangedMap={(changedFile: EditorTab, allTabs: ReadonlyArray<EditorTab>) => {
            //performance is tooooo bad...
            //this.props.setHasFileChangedMapDoEditor(updateHasFileChangedMap(changedFile, allTabs, this.props.hasFileChangedMap))
          }}

        />
      </div>)
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(afterEditorPanelView)
