/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {Icon, List} from "semantic-ui-react";
import TestNumber from '../testsPanel/testNumber'
import {RunnableCustomTestFrontendExclusive} from "../../../../types/customTests";
import {askDialog} from "../../../../helpers/dialogHelper";


import orderBy from 'lodash-es/orderBy'
import {AssessedSubmitTestFrontendExclusive} from "../../../../types/testResults";
import {RunnableExerciseTestFrontendExclusive} from "../../../../types/exerciseEditor";
import {getI18n} from "../../../../../i18n/i18nRoot";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string

  readonly customTests: ReadonlyArray<RunnableCustomTestFrontendExclusive>

  readonly canUserEditExercise: boolean

  /**
   * true: this panel displays after tests, false: not
   */
  readonly areAfterTests: boolean

  readonly canUserEditCustomTests: boolean

  readonly isSomeTestRunning: boolean

  readonly selectedConsoleViewTest: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null


  readonly setSelectedConsoleViewTest: (test: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => void

  readonly openUpdateCustomTestDialog: (test: RunnableCustomTestFrontendExclusive) => void

  readonly runCustomTest: (test: RunnableCustomTestFrontendExclusive) => Promise<boolean>

  readonly deleteCustomTestDoEditor: (test: RunnableCustomTestFrontendExclusive) => void

  /**
   * e.g. used to save changes in the editor
   */
  readonly beforeRunningTests: () => Promise<void>

}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class TestsPanel extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div className="fh">

        <List selection divided relaxed>
          {
            orderBy<RunnableCustomTestFrontendExclusive>(this.props.customTests,
                                                         (test) => test.displayIndex
            )
              .map((customTest, index) => {
                return (
                  //style hidden is needed because the test running animation (.test-running) rotates and box heights gets bigger... after 45 deg
                  <List.Item key={customTest.id} style={{overflow: 'hidden'}}
                             className={`test-line pad-left-right default-cursor fg-colored ` +
                             (this.props.selectedConsoleViewTest
                              && this.props.selectedConsoleViewTest.id === customTest.id
                              && this.props.areAfterTests === this.props.selectedConsoleViewTest.uiIsAfterTest
                              ? 'line-active'
                              : ''
                             )}

                  >
                    <List.Content className="v-centered-i" floated="right">

                    </List.Content>
                    <List.Content>
                      <div className="flexed">
                        <TestNumber number={(index + 1)} isValid={customTest.result
                                                                  ? customTest.result.passed
                                                                  : null}
                                    className="mar-right"
                                    isTestRunning={customTest.uiIsRunning}
                                    isEnabled={!this.props.isSomeTestRunning && this.props.canUserEditExercise}
                                    onClick={async () => {

                                      //save changes if code changed
                                      await this.props.beforeRunningTests()

                                      this.props.runCustomTest(customTest)

                                    }}
                        />

                        <div className="flexed v-centered flex-stretch">
                          <div>
                            {
                              customTest.displayName
                            }
                          </div>

                          {
                            customTest.result &&
                            <div className="clickable mar-left v-centered"
                                 onClick={() => {
                                   this.props.setSelectedConsoleViewTest(customTest)
                                 }}
                            >
                              <Icon name="info circle"/>
                              <span>
                                {
                                  getI18n(this.props.langId, 'Result')
                                }
                              </span>
                            </div>
                          }
                        </div>

                        {
                          this.props.canUserEditCustomTests &&
                          <div className="clickable v-centered mar-right"
                               onClick={() => {
                                 this.props.openUpdateCustomTestDialog(customTest)
                               }}
                          >
                            <Icon name="write"/>
                            <span>
                              {
                                getI18n(this.props.langId, 'Change')
                              }
                            </span>
                          </div>
                        }
                        {
                          this.props.canUserEditCustomTests === false &&
                          <div className="clickable v-centered mar-right"
                               onClick={() => {
                                 this.props.openUpdateCustomTestDialog(customTest)
                               }}
                          >
                            <Icon name="info circle"/>
                            <span>
                              {
                                getI18n(this.props.langId, 'Show content')
                              }
                            </span>
                          </div>
                        }
                        {
                          this.props.canUserEditCustomTests &&
                          <div className="clickable v-centered"
                               onClick={async () => {

                                 const shouldRemove = await askDialog(getI18n(this.props.langId, 'Delete custom test'),
                                                                      `${getI18n(this.props.langId,
                                                                                 'Are you sure you want to delete the custom test?'
                                                                      )} (${customTest.displayName})`, this.props.langId
                                 )

                                 if (shouldRemove) {
                                   // this.props.deleteCustomTestDoEditor(this.props.releaseCode, customTest,
                                   //   this.props.customTests)
                                   this.props.deleteCustomTestDoEditor(customTest)
                                 }
                               }}
                          >
                            <Icon name="remove"/>
                            <span>
                              {
                                getI18n(this.props.langId, 'Delete')
                              }
                            </span>
                          </div>
                        }

                      </div>
                    </List.Content>
                  </List.Item>
                )
              })
          }

        </List>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(TestsPanel)
