/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../../../state/reducers";
import FullSiteDialog from '../../../../../fullSiteDialog'
import ChangeCustomTestView, {changeCustomTestViewEditorId} from './changeCustomTestView'
import {
  cancelCustomTestDialog,
  saveCustomTestDialog
} from "../../../../../../state/actions/doExerciseSite/dialogs/changeCustomTestDialog/changeCustomTestActions";


import Logger from "../../../../../../helpers/logger";
import {notifySuccess} from "../../../../../../helpers/notificationHelper";
import {getI18n} from "../../../../../../../i18n/i18nRoot";
import {AceEditorsStorageHelper} from "../../../../../codeEditors/aceEditors/aceEditorsStorageHelper";
import {ErrorHelper} from '../../../../../../helpers/errorHelper'
import TestHints from '../../../../../helpers/tests/testServerHints'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string

  readonly canUserEditCustomTests: boolean
}

const mapStateToProps = (rootState: RootState , props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test

    ...props,
    isBusy: rootState.changeCustomTestDialogState.isBusy || rootState.testTypesState.isLoading,
    isCreating: rootState.changeCustomTestDialogState.isCreating,
    isValid: rootState.changeCustomTestDialogState.isValid,
    customTest: rootState.changeCustomTestDialogState.customTest,
    releaseCode: rootState.doExerciseSiteState.exercise.exerciseRelease.releaseCode,
    allRunnableCustomTests: rootState.doExerciseSiteState.exercise.customTests,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  cancelCustomTestDialog,
  saveCustomTestDialog,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class ChangeCustomTestDialog extends React.Component<Props, any> {
  render(): JSX.Element {

    const onSaveFunc = async () => {

      if (!this.props.canUserEditCustomTests) return

      try {
        await this.props.saveCustomTestDialog(this.props.isCreating, this.props.releaseCode, this.props.customTest,
                                              this.props.allRunnableCustomTests
        )
      } catch (err) {

        if (this.props.isCreating) {
          ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Error', 'Could not create custom test')

        } else {
          ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Error', 'Could not update custom test')
        }
        Logger.error(`could not ${this.props.isCreating
                                  ? 'create'
                                  : 'update'} custom test, err: ${err.message}`)
        return
      }

      AceEditorsStorageHelper.removeEditor(changeCustomTestViewEditorId)

      //notify because the user needs to know that he/she not need to save something e.g. solution or exercise
      //this was already done by the save button click
      if (this.props.isCreating) {
        notifySuccess(getI18n(this.props.langId, 'Created custom test'))
      } else {
        notifySuccess(getI18n(this.props.langId, 'Updated custom test'))
      }

    }

    return (<FullSiteDialog
        isBusy={this.props.isBusy}
        isValid={this.props.isValid}
        caption={<div className="flexed">
          <h2 className="mar-right">
            {this.props.isCreating
             ? getI18n(this.props.langId, 'Create custom test')
             : getI18n(this.props.langId, 'Change custom test')}
          </h2>

          <TestHints/>

        </div>}
        onSave={this.props.canUserEditCustomTests ? onSaveFunc : null}
        onCancel={() => {
          this.props.cancelCustomTestDialog()
          AceEditorsStorageHelper.removeEditor(changeCustomTestViewEditorId)
        }}
      >
        <ChangeCustomTestView
          canUserEditCustomTests={this.props.canUserEditCustomTests}
        />
      </FullSiteDialog>)
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ChangeCustomTestDialog)
