/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import {RouteComponentProps, withRouter} from "react-router";
import {isNotEmptyOrSpaces, isNotNullOrUndefined} from "../../../validation/validations";
import Spinner from "../../helpers/spinner";
import InterceptionOverlay from '../../helpers/InterceptOverlay'
import LockSolutionOverlay from './lockSolutionsOverlay/lockSolutionOverlay'
import PanelGrid from "../../panelSystem/panelGrid/panelGrid";
import Panel from "../../panelSystem/panel/panel";
import TabContentHost from "../../panelSystem/panel/tabContentHost";
import PreviewPanelView from '../exerciseEditorSite/previewPanel/previewPanelView'
import {
  createInitialAfterSolution,
  loadDoExerciseSite,
  resetAfterCustomTestResults,
  resetCustomTestResults,
  setBotLeftPanelViewSize,
  setBotRightPanelViewSize,
  setBottomLeftSelectedTabIndex,
  setBottomRightSelectedTabIndex,
  setDoExerciseDummy,
  setDoExerciseHDividerPosOffset,
  setDoExerciseVDividerPosOffset,
  setExercisePreviewFrontendDoExercise,
  setIsLeftTimeStringDisplayed,
  setIsOnGotFocusInterceptionOverlayDisplayedDoExerciseSite,
  setLastKnownEditorContent_DoExerciseSite,
  setSelectedConsoleViewTest,
  setSolutionVersion,
  setTopLeftPanelViewSize,
  setTopLeftSelectedTabIndex,
  setTopRightPanelViewSize,
  setTopRightSelectedTabIndex
} from "../../../state/actions/doExerciseSite/doExerciseSiteActions";

import TestsPanel from './testsPanel/testsPanelView'
import TestPanelHeaderBar from './testsPanel/testsPanelHeaderBar'

import CustomTestsPanel from './customTestsPanel/customTestsPanelView'
import CustomTestPanelHeaderBar from './customTestsPanel/customTestsPanelHeaderBar'

import AfterCustomTestsPanelHeaderBar from './afterCustomTestsPanel/afterCustomTestsPanelHeaderBar'

import ConsolePanelView from './consolePanel/consolePanelView'
import ConsolePanelHeaderBar from './consolePanel/consolePanelHeaderBar'

import EditorPanelView from './editorPanel/editorPanelView'
import EditorPanelHeaderBar from './editorPanel/editorPanelHeaderBar'

import ChangeCustomTestDialog from './customTestsPanel/dialog/customTestsDialog/changeCustomTestDialog'
import ViewTestDialog from './testsPanel/viewTestDialog/viewTestDialog'

import EditorSettingsDialog from '../../editorSettingsDialog/editorSettingsDialog'
import {getServerTime} from "../../../communicationLayer/timeLayer";
import {updateExercisePreviews} from "../../../helpers/editorHelper";
import moment, {Moment} from "moment";
import {ExercisePreviewsFrontendExclusive} from "../allAccessibleExercisesSite/openExercises/allOpenExercisesSite";
import {ReleaseStartType} from "../../../types/ReleaseStartType";
import {createAndRefreshAssetUrlsAndObjects, freeAssetUrlsAndObjects} from "../../../helpers/assetHelper";


import SubmitTestsPanelHeaderBar from './submitTestsPanel/submitTestsPanelHeaderBar'

import {
  runAfterCustomTest,
  runAfterNormalTest,
  runAfterSubmitTest,
  runCustomDoExerciseTest,
  runDoExerciseTest,
  saveDoExerciseSolution,
  updateDoExerciseLockSolutionsFlagAsync
} from "../../../state/actions/doExerciseSite/doExerciseCrudActions";
import {AssessedSubmitTestFrontendExclusive} from "../../../types/testResults";
import {RunnableExerciseTestFrontendExclusive} from "../../../types/exerciseEditor";
import {RunnableCustomTestFrontendExclusive} from "../../../types/customTests";
import {
  cancelViewTestDialog,
  openViewTestDialog
} from "../../../state/actions/doExerciseSite/dialogs/viewTestDialog/viewTestDialogActions";
import {RunTestCommandForBackend} from "../../../types/testsAndTestProtocol";
import {debugOriginUrl} from "../../../../debug";
import {ReleaseDurationType} from "../../../types/ReleaseDurationType";


import history from "../../../routerHistory";
import * as constants from '../../../constants'
import {refreshAuthTokenLifetimeIntervalWhenEditingInMs} from '../../../constants'
import AssessmentResultPanelHeaderBar from './assessmentResultPanel/assessmentResultPanelHeaderBar'
import {FrontendSettingsManager} from "../../../helpers/frontendSettingsManager";
import {errorDialog, infoDialog} from "../../../helpers/dialogHelper";
//editor for solving the exercise after the release finished or the time is up?? or just release finished?
import AfterEditorPanelView from './afterEditorPanel/afterEditorPanelView'
import AfterNormalTestsPanelHeaderBar from './afterNormalTestsPanel/afterNormalTestsPanelHeaderBar'
import AfterSubmitTestsPanelHeaderBar from './afterSubmitTestsPanel/afterSubmitTestsPanelHeaderBar'
import AfterEditorPanelHeaderBar from './afterEditorPanel/afterEditorPanelHeaderBar'
import Logger from "../../../helpers/logger";
import {
  setHasUnsavedChanges,
  setSelectedTabIdDoEditor
} from "../../../state/actions/doExerciseSite/editorActions/doEditorActions";
import {LoginTimeoutHelper} from "../../../helpers/loginTimeoutHelper";
import {
  openCreateCustomTestDialog,
  openUpdateCustomTestDialog
} from "../../../state/actions/doExerciseSite/dialogs/changeCustomTestDialog/changeCustomTestActions";
import {deleteCustomTestDoEditor} from "../../../state/actions/doExerciseSite/sub/customTestActions";
import {accessExerciseWithReleaseCode} from "../../../communicationLayer/exerciseReleasesLayer";
import {
  setIsSiteHeaderBarDisplayedAndEnabledAction,
  setIsSubHeaderBarCollapsed
} from "../../../state/actions/siteHeaderBarActions";
import {criticalLeftWorkingTimeInS} from "./headerBarContent";
import {Button, Icon, Input} from "semantic-ui-react";
import {getI18n} from "../../../../i18n/i18nRoot";
import {AceEditorsStorageHelper} from "../../codeEditors/aceEditors/aceEditorsStorageHelper";
import {saveDoExerciseAfterSolutionAsync} from '../../../state/actions/doExerciseSite/afterSolutionCrudActions'
import {HelpPopup} from '../../helpers/helpPopup'
import {ErrorHelper} from '../../../helpers/errorHelper'
import {setGlobalTaskDescriptionFontSizeInPx} from '../../../state/actions/globalUiActions'
import ZoomPreviewPanelActions from '../exerciseEditorSite/previewPanel/zoomPreviewPanelActions'
import {updateAuthTokenLifetime} from '../../../communicationLayer/loginLayer'
import MarkdownRenderer from '../../renderContentComponents/markdownRenderer'
import {QuestionType} from '../../../types/questions'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    doExercise: rootState.doExerciseSiteState.exercise,
    exerciseDescription: rootState.doExerciseSiteState.exercise.exerciseDescription,
    solution: rootState.doExerciseSiteState.solution,
    solutionPLangId: rootState.doExerciseSiteState.solution.pLangId,

    taskDescriptionFontSizeInPx: rootState.globalUiState.taskDescriptionFontSizeInPx,

    afterCustomTestsMerged: rootState.doExerciseSiteState.afterCustomTests.map<RunnableCustomTestFrontendExclusive>(
      p => {
        const customTest = rootState.doExerciseSiteState.exercise.customTests.find(test => test.id === p.id)
        return {
          ...customTest,
          result: p.result,
          uiIsRunning: p.uiIsRunning,
          uiIsAfterTest: true
        }
      }),

    pLangs: rootState.pLangsState.pLangs,

    releaseCode: rootState.doExerciseSiteState.exercise.exerciseRelease.releaseCode,
    //not needed here (inside editor panel) but keep it to know that we need to wait for loading
    //pLangs: rootState.pLangsState.pLangs,

    submitTestResults: rootState.doExerciseSiteState.submitTestResults,


    selectedConsoleViewTest: rootState.doExerciseSiteState.selectedConsoleViewTest,
    testTypes: rootState.testTypesState.testTypes,

    exercisePreview: rootState.doExerciseSiteState.exercisePreviewFrontendTemp,


    afterSolution: rootState.doExerciseSiteState.afterSolution,
    afterNormalTests: rootState.doExerciseSiteState.afterNormalTests,
    afterSubmitTests: rootState.doExerciseSiteState.afterSubmitTests,

    selectedPLangId: rootState.doExerciseSiteState.selectedPLangId,
    isCustomTestDialogDisplayed: rootState.doExerciseSiteState.isCustomTestDialogDisplayed,
    isViewTestDialogDisplayed: rootState.doExerciseSiteState.isViewTestDialogDisplayed,
    isCodeEditorSettingsDialogDisplayed: rootState.doExerciseSiteState.isCodeEditorSettingsDialogDisplayed,


    canUserCreateCustomTests: ((rootState.doExerciseSiteState.exercisePreviewFrontendTemp
      && rootState.doExerciseSiteState.exercisePreviewFrontendTemp.isStillOpen)
      || (rootState.doExerciseSiteState.exercise.exerciseRelease.isReleased
        && rootState.doExerciseSiteState.exercise.exerciseRelease.hasLimitedWorkingTime === false
        && rootState.doExerciseSiteState.exercise.exerciseRelease.releaseDurationType === ReleaseDurationType.Infinitely)) &&
      (rootState.doExerciseSiteState.exercisePreviewFrontendTemp
        && rootState.doExerciseSiteState.exercisePreviewFrontendTemp.isPermanentlyLocked === false),

    canUserEditExercise: ((rootState.doExerciseSiteState.exercisePreviewFrontendTemp
      && rootState.doExerciseSiteState.exercisePreviewFrontendTemp.isStillOpen)
      || (rootState.doExerciseSiteState.exercise.exerciseRelease.isReleased
        && rootState.doExerciseSiteState.exercise.exerciseRelease.hasLimitedWorkingTime === false
        && rootState.doExerciseSiteState.exercise.exerciseRelease.releaseDurationType === ReleaseDurationType.Infinitely)) &&
      (rootState.doExerciseSiteState.exercisePreviewFrontendTemp
        && rootState.doExerciseSiteState.exercisePreviewFrontendTemp.isPermanentlyLocked === false),

    // isSomeTestRunning: rootState.doExerciseSiteState.exercise.tests.some(p => p.uiIsRunning)
    // || rootState.doExerciseSiteState.exercise.customTests.some(p => p.uiIsRunning),

    /**
     * true: display the submit test results (not longer released and assessment finished)
     * false: not
     */
    displaySubmitTestResults: !rootState.doExerciseSiteState.exercise.exerciseRelease.isReleased
      && rootState.doExerciseSiteState.exercise.exerciseRelease.hasAutomaticAssessmentFinished,

    displayAssessmentResult: !rootState.doExerciseSiteState.exercise.exerciseRelease.isReleased
      && rootState.doExerciseSiteState.exercise.exerciseRelease.hasAutomaticAssessmentFinished &&
      rootState.doExerciseSiteState.assessmentResult
      && (isNotEmptyOrSpaces(
        rootState.doExerciseSiteState.assessmentResult.feedbackForStudent) || rootState.doExerciseSiteState.assessmentResult.manualPoints !== null),

    assessmentResult: rootState.doExerciseSiteState.assessmentResult,

    topRightSelectedTabIndex: rootState.doExerciseSiteState.topRightSelectedTabIndex,
    topLeftSelectedTabIndex: rootState.doExerciseSiteState.topLeftSelectedTabIndex,
    bottomRightSelectedTabIndex: rootState.doExerciseSiteState.bottomRightSelectedTabIndex,
    bottomLeftSelectedTabIndex: rootState.doExerciseSiteState.bottomLeftSelectedTabIndex,

    topRightPanelViewSize: rootState.doExerciseSiteState.topRightPanelViewSize,
    topLeftPanelViewSize: rootState.doExerciseSiteState.topLeftPanelViewSize,
    botRightPanelViewSize: rootState.doExerciseSiteState.botRightPanelViewSize,
    botLeftPanelViewSize: rootState.doExerciseSiteState.botLeftPanelViewSize,

    hDividerPosOffset: rootState.doExerciseSiteState.hDividerPosOffset,
    vDividerPosOffset: rootState.doExerciseSiteState.vDividerPosOffset,

    isSomeTestRunning: rootState.doExerciseSiteState.exercise.tests.some(p => p.uiIsRunning)
      || rootState.doExerciseSiteState.exercise.customTests.some(p => p.uiIsRunning)
      || rootState.doExerciseSiteState.afterNormalTests.some(p => p.uiIsRunning)
      || rootState.doExerciseSiteState.afterSubmitTests.some(p => p.uiIsRunning)
      || rootState.doExerciseSiteState.afterCustomTests.some(p => p.uiIsRunning)
    ,

    isNoDialogDisplayed: !rootState.doExerciseSiteState.isCustomTestDialogDisplayed
      && !rootState.doExerciseSiteState.isViewTestDialogDisplayed
      && !rootState.doExerciseSiteState.isCodeEditorSettingsDialogDisplayed,

    //only used to check if user can run normal tests
    canUserSolveExercise: ((rootState.doExerciseSiteState.exercisePreviewFrontendTemp
      && rootState.doExerciseSiteState.exercisePreviewFrontendTemp.isStillOpen
      && rootState.doExerciseSiteState.exercise.exerciseRelease.isReleased)
      || (rootState.doExerciseSiteState.exercise.exerciseRelease.isReleased
        && rootState.doExerciseSiteState.exercise.exerciseRelease.hasLimitedWorkingTime === false
        && rootState.doExerciseSiteState.exercise.exerciseRelease.releaseDurationType === ReleaseDurationType.Infinitely)) &&
      (rootState.doExerciseSiteState.exercisePreviewFrontendTemp
        && rootState.doExerciseSiteState.exercisePreviewFrontendTemp.isPermanentlyLocked === false),

    editorHasUnsavedChanges: rootState.doExerciseSiteState.doEditorState.hasUnsavedChanges,
    afterEditorHasUnsavedChanges: rootState.doExerciseSiteState.afterEditorState.hasUnsavedChanges,

    //release should be not active and assessment finished then the after tests can be run
    releaseFinishedAndAutomaticAssessmentHasFinished: !rootState.doExerciseSiteState.exercise.exerciseRelease.isReleased &&
      rootState.doExerciseSiteState.exercise.exerciseRelease.hasAutomaticAssessmentFinished,

    canDoAfterEditor: rootState.doExerciseSiteState.exercisePreviewFrontendTemp !== null &&
      rootState.doExerciseSiteState.exercisePreviewFrontendTemp.isStillOpen === false,

    dummy: rootState.doExerciseSiteState.dummy,


    //see loadDoExerciseSite used data
    isLoading: rootState.doExerciseSiteState.isLoading
      || rootState.testTypesState.isLoading
      || rootState.pLangsState.isLoading
      || rootState.systemSettingsSiteState.isLoading
      || rootState.globalSystemSettingsState.isLoading,

    isOnGotFocusInterceptionOverlayDisplayed: rootState.doExerciseSiteState.isOnGotFocusInterceptionOverlayDisplayed,

    isSiteHeaderBarDisplayedAndEnabled: rootState.siteHeaderBarState.isSiteHeaderBarDisplayedAndEnabled,
    //if this.props.exercisePreview.isStillOpen === false we ignore this flag because we then edit the after solution
    lockSolutionsFlagAndIsStillOpen: rootState.doExerciseSiteState.lockSolutionsFlag
      && rootState.doExerciseSiteState.exercisePreviewFrontendTemp !== null && rootState.doExerciseSiteState.exercisePreviewFrontendTemp.isStillOpen,
    lockSolutionsFlag: rootState.doExerciseSiteState.lockSolutionsFlag,


    selectedTabIdDoEditor: rootState.doExerciseSiteState.doEditorState.selectedTabIdDoEditor,
    selectedAfterSolutionTabIdDoEditor: rootState.doExerciseSiteState.afterEditorState.selectedTabIdDoEditor,

    lastKnownEditorContent: rootState.doExerciseSiteState.lastKnownEditorContent,

    langId: rootState.i18nState.langId
  }
}

// @ts-ignore
const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setTopRightSelectedTabIndex,
                                                                        setBottomRightSelectedTabIndex,
                                                                        setTopLeftSelectedTabIndex,
                                                                        setBottomLeftSelectedTabIndex,
                                                                        loadDoExerciseSite,
                                                                        setDoExerciseHDividerPosOffset,
                                                                        setDoExerciseVDividerPosOffset,

                                                                        setExercisePreviewFrontendDoExercise,

                                                                        setTopRightPanelViewSize,
                                                                        setTopLeftPanelViewSize,
                                                                        setBotRightPanelViewSize,
                                                                        setBotLeftPanelViewSize,

                                                                        saveDoExerciseSolution,
                                                                        setSelectedConsoleViewTest,

                                                                        runDoExerciseTest,
                                                                        openViewTestDialog,
                                                                        cancelViewTestDialog,
                                                                        setSolutionVersion,

                                                                        runAfterNormalTest,
                                                                        runAfterSubmitTest,
                                                                        setHasUnsavedChanges,
                                                                        setSelectedTabIdDoEditor,

                                                                        openCreateCustomTestDialog,
                                                                        runCustomDoExerciseTest,
                                                                        runAfterCustomTest,
                                                                        resetCustomTestResults,
                                                                        resetAfterCustomTestResults,

                                                                        openUpdateCustomTestDialog,
                                                                        deleteCustomTestDoEditor,
                                                                        setIsSiteHeaderBarDisplayedAndEnabledAction,
                                                                        setIsSubHeaderBarCollapsed,
                                                                        setIsLeftTimeStringDisplayed,

                                                                        setIsOnGotFocusInterceptionOverlayDisplayedDoExerciseSite,
                                                                        createInitialAfterSolution,
                                                                        saveDoExerciseAfterSolutionAsync,

                                                                        updateDoExerciseLockSolutionsFlagAsync,

                                                                        setGlobalTaskDescriptionFontSizeInPx,
                                                                        setLastKnownEditorContent_DoExerciseSite,
                                                                      }, dispatch)


enum DoExerciseMode {
  /**
   * hide the menu bar so that the user
   * cannot navigate (some other program should hide the url e.g. safe exam browser)
   *
   * should not count is also not displayed (because it's an exam ;))
   *
   * THIS IS ONLY LEGACY SUPPORT we now need to specify all exam options in the release manually
   */
  exam = 'exam'
}

interface MatchedProps {
  readonly releaseCode: string
  readonly mode: DoExerciseMode | null
}


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


// noinspection TsLint
let updateLeftTimeIntervals = -1
const onGotFocusBackInputId = 'ongotfocushelper'

const taskContentPrintId = 'task-content-printable'
const feedbackContentPrintId = 'feedback-content-printable'

class doExerciseSite extends React.Component<Props & RouteComponentProps<MatchedProps>, any> {

  lastPageVisibilityWasHidden: boolean | null = null
  hasFocus = false
  firstOnFocus = true


  refreshTokenTimeoutHandler = 0

  async componentWillMount(): Promise<void> {

    const releaseCode = this.props.match.params.releaseCode
    const mode = this.props.match.params.mode //THIS COULD BE 'exam'... because of legacy support we keep this...

    //it's safer to set to disabled and when we know that we should enable it
    //then set it back to enabled
    this.props.setIsSiteHeaderBarDisplayedAndEnabledAction(false)

    if ((mode && mode === DoExerciseMode.exam)) {
      Logger.warn('exam url option is not longer supported... we now use exam options from the release')
    }


    if (isNotNullOrUndefined(releaseCode) === false) {
      errorDialog(getI18n(this.props.langId, 'Error'),
                  getI18n(this.props.langId, 'Some url parameters were not found/set'),
                  this.props.langId
      )
      return
    }

    //after a reload (or login --> relogin) we can rset the last known content
    //so the next interval will trigger the auth  token update
    this.props.setLastKnownEditorContent_DoExerciseSite('')

    if (LoginTimeoutHelper.isReLogin && releaseCode === LoginTimeoutHelper.lastKnownDoEditorReleaseCode) {
      //do not reload or reset site state...
      //user re logged in he/she want's to continue
      //if we would reload then the code would be gone (even though the frontend settings would store a copy... but that's bad style)


      //but we need to re-create the assets because componentWillUnmount cleared everything

      createAndRefreshAssetUrlsAndObjects(this.props.exerciseDescription.assets)
      setDoExerciseDummy(false) //re render because we now have assets --> only right 1 render

      //we also need to renew the timer
      Logger.debug('doExerciseSite', 'setting up timers (refresh)')
      await this.setupTimeUsedUpIntervals()


      //if we re login ... display the header bar
      if (this.props.doExercise.exerciseRelease.hideSiteHeaderBar) {
        Logger.log('exam mode for do exercise site (release param)')
      } else {
        this.props.setIsSiteHeaderBarDisplayedAndEnabledAction(true)
      }

      return

    } else {


      //if the user (or via a link) the release code is entered before it was entered in the
      //enter release code dialog to it here...

      try {
        const test = await accessExerciseWithReleaseCode(releaseCode, true)
      } catch (err) {
        //already handled by setup.ts
        return
      }

      LoginTimeoutHelper.lastKnownDoEditorReleaseCode = releaseCode
    }


    //we need to freeze the manager because the multi tab editor would overwrite the solution when the
    //sessions were created

    FrontendSettingsManager.setIsFrozen(true)

    //reset ace storage
    AceEditorsStorageHelper.removeAll()

    try {
      await this.props.loadDoExerciseSite(releaseCode)
    } catch (err) {

      ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Error', 'Cannot load some exercise site parts')

      FrontendSettingsManager.setIsFrozen(false)
      return
    }


    if (this.props.doExercise.exerciseRelease.hideSiteHeaderBar) {
      Logger.log('exam mode for do exercise site (release param)')
    } else {
      this.props.setIsSiteHeaderBarDisplayedAndEnabledAction(true)
    }


    if (this.props.doExercise.exerciseRelease.shouldClearClipboard) {

      //we can't move this to the header because we need to await the exercise.release to be loaded

      //we cannot clear the clipboard in js...
      //copyToClipboard is not working even with settimeout
      this.lastPageVisibilityWasHidden = document.hidden

      document.addEventListener('visibilitychange', this.onPageVisibilityChanged.bind(this))
      // document.addEventListener('focus', this.onWindowGotFocus.bind(this), true)
      // document.addEventListener('blur', this.onWindowLostFocus.bind(this), true)
      window.onfocus = this.onWindowGotFocus.bind(this)
      window.onblur = this.onWindowLostFocus.bind(this)
    }


    Logger.debug('doExerciseSite', 'setting up timers')
    await this.setupTimeUsedUpIntervals()

    FrontendSettingsManager.setIsFrozen(false)

    // Logger.debug('doExerciseSite', 'checking for newer frontend solution version')
    // await this.checkForNewerFrontendSolutionVersion()

  }


  componentDidMount(): void {

    //could be moved to  componentWillMount but not needed
    this.refreshTokenTimeoutHandler = setInterval(this.hasEditorChangedAndUpdateAuthTokenLifetime.bind(this),
                                                  refreshAuthTokenLifetimeIntervalWhenEditingInMs
    )
  }

  componentWillUnmount(): void {

    clearInterval(updateLeftTimeIntervals)
    freeAssetUrlsAndObjects()
    setDoExerciseDummy(true) //enable re rendering

    clearInterval(this.refreshTokenTimeoutHandler)

    if (this.props.doExercise.exerciseRelease.shouldClearClipboard) {
      document.removeEventListener('visibilitychange', this.onPageVisibilityChanged)
      // document.removeEventListener('focus', this.onWindowGotFocus)
      // document.removeEventListener('blur', this.onWindowLostFocus)
      window.onfocus = null
      window.onblur = null

    }
  }

  /**
   * this checks if the normal/after editor current file content has changed since the last check
   * if yes we renew the auth token lifetime on the backend
   * else not
   */
  async hasEditorChangedAndUpdateAuthTokenLifetime(): Promise<void> {

    // noinspection TsLint
    let currentEditorContent = ''

    if (!this.props.canUserEditExercise) {

      if (!this.props.afterSolution) return //or error?

      //use the after editor content
      const file = this.props.afterSolution.solutionFiles.find(
        p => p.id === this.props.selectedAfterSolutionTabIdDoEditor)
      currentEditorContent = file
                             ? file.content
                             : ''
    } else {

      const file = this.props.solution.solutionFiles.find(p => p.id === this.props.selectedTabIdDoEditor)
      currentEditorContent = file
                             ? file.content
                             : ''
    }

    Logger.debug('[do exercise editor site]', `this.props.lastKnownEditorContent !== currentEditorContent: ${this.props.lastKnownEditorContent !== currentEditorContent}`)
    if (this.props.lastKnownEditorContent !== currentEditorContent) {
      //something changed... update auth token

      try {
        await updateAuthTokenLifetime()
        Logger.debug('[do exercise editor site]', 'updated auth lifetime because editor content changed')
      } catch (err) {
        Logger.warn('could not refresh auth token lifetime', err)
      }
    } else {
      Logger.debug('[do exercise editor site]',
                   'last known editor content and currentEditorContent were equal ... no auth token lifetime refresh'
      )
    }

    this.props.setLastKnownEditorContent_DoExerciseSite(currentEditorContent)
  }

  /**
   * called when the tab changed (or when minimized)
   * but NOT when the user changes windows focus
   *
   * the interception overlay is sometimes not working (especially if the do exercise sit loaded) and then we click outside...)
   */
  onPageVisibilityChanged(): void {

    if (this.lastPageVisibilityWasHidden && document.hidden === false) {
      //window got focus back
      this.props.setIsOnGotFocusInterceptionOverlayDisplayedDoExerciseSite(true)
    }

    // Logger.log("new state hidden: " + document.hidden)
    this.lastPageVisibilityWasHidden = document.hidden
  }

  onWindowLostFocus(e: FocusEvent): void {

    const el = e.target as HTMLElement

    if (el && el.id === onGotFocusBackInputId) {
      //ignore
      return
    }

    if (this.hasFocus === true) {
      this.hasFocus = false;
      this.props.setIsOnGotFocusInterceptionOverlayDisplayedDoExerciseSite(true)
    }

    // Logger.log('lost focus')
    this.hasFocus = false
  }

  onWindowGotFocus(e: FocusEvent): void {


    if (this.firstOnFocus) {
      this.props.setIsOnGotFocusInterceptionOverlayDisplayedDoExerciseSite(true)
      this.firstOnFocus = false
      return
    }

    const el = e.target as HTMLElement

    if (el && el.id === onGotFocusBackInputId) {
      //ignore
      return
    }
    this.hasFocus = true
    console.log("got focus")
    console.log(this.hasFocus)
  }

  // async checkForNewerFrontendSolutionVersion(): Promise<void> {
  //
  //   //check if we have a newer frontend only version but only if the use can edit...
  //
  //   if (!this.props.canUserSolveExercise) {
  //     Logger.debug('doExerciseSite',
  //                  ' cancel checking for newer frontend solution version because canUserSolveExercise is false/not set'
  //     )
  //     return
  //   }
  //
  //   //only if there is an old saved version
  //   const frontendSettings = FrontendSettingsManager.getFrontendSettings()
  //   if (frontendSettings.doEditorSolution) {
  //
  //     //this implies  FrontendSettings.lastKnownDoEditorReleaseId === frontendSettings.doEditorReleaseId
  //     //because ids are unique, sequential
  //     if (frontendSettings.doEditorSolution.id === this.props.solution.id) {
  //       //frontend version could be newer duo to browser crashes
  //
  //       //ignore file visibility
  //       const frontendVersion = JSON.stringify(frontendSettings.doEditorSolution,
  //                                              (key: string, value: any) => {
  //                                                if (key === 'createdAt') {
  //                                                  return ''//value.toString() //ignore because we only wont to compare ids & contents...
  //                                                }
  //                                                if (key === 'isDisplayed') {
  //                                                  return 'true'
  //                                                }
  //                                                if (key === 'displayIndex') {
  //                                                  return '0'
  //                                                }
  //                                                if (key === 'shouldNotCount') {
  //                                                  return 'false'
  //                                                }
  //                                                return value
  //                                              }
  //       )
  //
  //       //ignore file visibility & case date time moment object to string
  //       const backendVersion = JSON.stringify(this.props.solution,
  //                                             (key: string, value: any) => {
  //                                               if (key === 'createdAt') {
  //                                                 return ''//value.toString()  //ignore because we only wont to compare ids & contents...
  //                                               }
  //                                               if (key === 'isDisplayed') {
  //                                                 return 'true'
  //                                               }
  //                                               if (key === 'displayIndex') {
  //                                                 return '0'
  //                                               }
  //                                               if (key === 'shouldNotCount') {
  //                                                 return 'false'
  //                                               }
  //                                               return value
  //                                             }
  //       )
  //
  //
  //       //both have som props unset that don't matter
  //       // noinspection TsLint
  //       let cleanedFrontendVersion: SolutionVersionDoExerciseFullBase = JSON.parse(frontendVersion)
  //       // noinspection TsLint
  //       let cleanedBackendVersion: SolutionVersionDoExerciseFullBase = JSON.parse(backendVersion)
  //
  //
  //       cleanedFrontendVersion = {
  //         ...cleanedFrontendVersion,
  //         solutionFiles: orderBy(cleanedFrontendVersion.solutionFiles, (p: SolutionFileDoExerciseFullBase) => p.id)
  //       }
  //
  //       cleanedBackendVersion = {
  //         ...cleanedBackendVersion,
  //         solutionFiles: orderBy(cleanedBackendVersion.solutionFiles, (p: SolutionFileDoExerciseFullBase) => p.id)
  //       }
  //
  //
  //       //if (frontendVersion === backendVersion) {
  //       const areEqual = isEqual(cleanedFrontendVersion, cleanedBackendVersion)
  //
  //       if (areEqual) {
  //         //all ok same
  //         Logger.debug("doExerciseSite", "compared frontend code version and backend ... EQUAL!");
  //
  //       } else {
  //         //take the frontend version
  //
  //         //TODO remove but there is still a bug where this occurs even if the code is equal...
  //         Logger.debug("doExerciseSite", "not equal: cleanedFrontendVersion", cleanedFrontendVersion)
  //         Logger.debug("doExerciseSite", "not equal: cleanedBackendVersion", cleanedBackendVersion)
  //
  //         const shouldOverwrite = await askDialog(getI18n(this.props.langId, 'Recover files'),
  //                                                 getI18n(this.props.langId,
  //                                                         "If you press 'Yes' then the new files from your browser are taken. If you press 'No' then the files from server are taken. If you regret your decision you can reload the page and the question will be asked again."
  //                                                 )
  //         )
  //
  //         if (shouldOverwrite) {
  //
  //           Logger.debug("doExerciseSite",
  //                        "setting frontend solution version as the solution version (because not equal)"
  //           );
  //
  //           this.props.setSolutionVersion({
  //                                           ...this.props.solution,
  //                                           id: frontendSettings.doEditorSolution.id,
  //                                           // mainFileId: frontendSettings.doEditorSolution.mainFileId, //the user cannot change the main file
  //                                           // pLangId: frontendSettings.doEditorSolution.pLangId,
  //                                           createdAt: frontendSettings.doEditorSolution.createdAt.clone(),
  //                                           solutionFiles: frontendSettings.doEditorSolution.solutionFiles.map(p => {
  //                                             return {
  //                                               ...p,
  //                                             }
  //                                           })
  //                                         })
  //
  //
  //           const currPLang = this.props.pLangs.find(p => p.id === frontendSettings.doEditorSolution.pLangId)
  //
  //           if (!currPLang) {
  //             Logger.error('could not find plang')
  //           }
  //           //then update the ace storage because there are the current files stored
  //           AceEditorsStorageHelper.removeEditor(doEditorEditorPanelViewEditorId)
  //           const storedTabs = frontendSettings.doEditorSolution.solutionFiles.map<AceStoredTabLike>(p => {
  //             return {
  //               id: p.id,
  //               content: p.content,
  //               language: currPLang.internalName
  //             }
  //           })
  //           AceEditorsStorageHelper.addEditorWithTabs(doEditorEditorPanelViewEditorId, storedTabs)
  //
  //           //we need to pre select the first file...
  //           //see multiFileAceEditor > componentWillUpdate for explanation
  //           if (frontendSettings.doEditorSolution.solutionFiles.length > 0) {
  //
  //             const visibleFiles = frontendSettings.doEditorSolution.solutionFiles.filter(p => p.displayIndex >= 0)
  //             if (visibleFiles.length === 0) {
  //               //else we would still have the old file selected that we just overwritten with frontend files
  //               this.props.setSelectedTabIdDoEditor(-1)
  //             }
  //           }
  //
  //
  //           //also reset the selected tab index
  //
  //           //mark changes
  //           this.props.setHasUnsavedChanges(true)
  //         }
  //
  //       }
  //     } else {
  //
  //       //this can happen if
  //       //the user solves the solution on another pc then changes back --> take greater id (because unique, sequential)
  //       //the user starts solving another exercise (or just release)
  //
  //       if (frontendSettings.lastKnownDoEditorReleaseId !== frontendSettings.doEditorReleaseId
  //         || frontendSettings.lastKnownDoEditorReleaseId === -1 && frontendSettings.doEditorReleaseId === -1) {
  //         //user started to solve another exercise (release)... clear old solution
  //         //overwrite the old with the new one...
  //         FrontendSettingsManager.setDoEditorSolution({...this.props.solution})
  //
  //       } else {
  //         //user starts solving the same release but the frontend version id is higher --> take it
  //         if (frontendSettings.doEditorSolution.id > this.props.solution.id) {
  //
  //           const shouldOverwrite = await askDialog(getI18n(this.props.langId, 'Recover files'),
  //                                                   getI18n(this.props.langId,
  //                                                           "If you press 'Yes' then the new files from your browser are taken. If you press 'No' then the files from server are taken. If you regret your decision you can reload the page and the question will be asked again."
  //                                                   )
  //           )
  //
  //           if (shouldOverwrite) {
  //
  //             Logger.debug("doExerciseSite",
  //                          "setting frontend solution version as the solution version (because newer id)"
  //             );
  //             this.props.setSolutionVersion({
  //                                             ...frontendSettings.doEditorSolution,
  //                                             shouldNotCount: this.props.solution.shouldNotCount,
  //                                             note: this.props.solution.note,
  //                                             pLangId: this.props.solution.pLangId
  //                                           })
  //
  //             const currPLang = this.props.pLangs.find(p => p.id === frontendSettings.doEditorSolution.pLangId)
  //
  //             if (!currPLang) {
  //               Logger.error('could not find plang')
  //             }
  //             //then update the ace storage because there are the current files stored
  //             AceEditorsStorageHelper.removeEditor(doEditorEditorPanelViewEditorId)
  //             const storedTabs = frontendSettings.doEditorSolution.solutionFiles.map<AceStoredTabLike>(p => {
  //               return {
  //                 id: p.id,
  //                 content: p.content,
  //                 language: currPLang.internalName
  //               }
  //             })
  //             AceEditorsStorageHelper.addEditorWithTabs(doEditorEditorPanelViewEditorId, storedTabs)
  //
  //             //we need to pre select the first file...
  //             //see multiFileAceEditor > componentWillUpdate for explaination
  //             if (frontendSettings.doEditorSolution.solutionFiles.length > 0) {
  //
  //               const visibleFiles = frontendSettings.doEditorSolution.solutionFiles.filter(p => p.displayIndex >= 0)
  //               if (visibleFiles.length === 0) {
  //                 //else we would still have the old file selected that we just overwritten with frontend files
  //                 this.props.setSelectedTabIdDoEditor(-1)
  //               }
  //             }
  //
  //             //mark changes
  //             this.props.setHasUnsavedChanges(true)
  //           }
  //
  //         }
  //       }
  //     }
  //
  //     Logger.debug("doExerciseSite", "end checking frontend & backend version!");
  //
  //   }
  // }


  async setupTimeUsedUpIntervals(): Promise<void> {


    // noinspection TsLint
    let serverTime: Moment
    try {
      serverTime = await getServerTime();
    } catch (err) {
      //TODO better error
      Logger.error('doExerciseSite',
                   `cannot get time from server, thus cannot calculate left time ... redirect to ${constants.homeLinkPath}`
      )
      history.push(constants.homeLinkPath)
      return
    }

    const clientServerTimeDiff = moment().diff(serverTime, 'seconds')

    const wasAtSomePointEditable = this.checkTimeUsedUp(clientServerTimeDiff, null) //call it immediately because the first tick of the interval will be in 1 sec
    updateLeftTimeIntervals = setInterval(() => this.checkTimeUsedUp(clientServerTimeDiff, wasAtSomePointEditable),
                                          1000
    )

  }

  /**
   * returns true: the user can edit the exercise, false: not (time is up or the release auto ended time)
   * @param clientServerTimeDiff
   * @param wasAtSomePointEditable true: display a dialog that the time is up and the user can use the subsequent editor, false: was already not editable
   */
  checkTimeUsedUp(clientServerTimeDiff: number, wasAtSomePointEditable: boolean | null): boolean {

    const releaseCopy: ExercisePreviewsFrontendExclusive = {
      id: this.props.doExercise.exerciseRelease.id,
      isDisplayed: true,
      isReleased: this.props.doExercise.exerciseRelease.isReleased,
      displayName: '',
      isPermanentlyLocked: this.props.doExercise.isPermanentlyLocked,
      automaticEndAt: this.props.doExercise.exerciseRelease.automaticEndAt,
      hasLimitedWorkingTime: this.props.doExercise.exerciseRelease.hasLimitedWorkingTime,
      availableWorkingTimeInMinutes: this.props.doExercise.exerciseRelease.availableWorkingTimeInMinutes,
      firstAccessedAt: this.props.doExercise.firstAccessedAt,
      shortDescription: '',
      shouldNotCount: false,
      releaseDurationType: this.props.doExercise.exerciseRelease.releaseDurationType,
      tags: [],
      generatedCode: this.props.doExercise.exerciseRelease.releaseCode,
      isStillOpen: false,
      leftTimeString: '',
      releaseStartType: this.props.doExercise.exerciseRelease.releaseStartType,
      isVisibleToAllAfterRelease: false,
      releasedForPLangId: null,
      leftTimeInS: null
    }

    const rest = updateExercisePreviews(
      [this.props.exercisePreview
       ? this.props.exercisePreview
       : releaseCopy], clientServerTimeDiff, this.props.langId)

    const copy = rest[0]

    //Logger.debug('doExerciseSite', `setExercisePreviewFrontendDoExercise with ${JSON.stringify(copy)}`)

    this.props.setExercisePreviewFrontendDoExercise(copy)

    if (!copy.isStillOpen) {

      Logger.log('time is up or release already finished ... clearing check interval')
      clearInterval(updateLeftTimeIntervals)

      this.props.setIsLeftTimeStringDisplayed(true)

      if (this.props.afterSolution === null) {
        //also create the first after solution if needed
        this.props.createInitialAfterSolution()
      }

      //we are now sure that the user can edit the after solution (the conditional rendering will now be true)...
      //preselect the after editor because the user probably want to edit it
      this.props.setTopLeftSelectedTabIndex(1)

      if (wasAtSomePointEditable === true) {
        infoDialog('', getI18n(this.props.langId,
                               "Time is up! You can still do the exercise for your own in the subsequent editor (the subsequent solution will not be used for assessment through). To run tests use the life ring test-tabs."
                   ),
                   this.props.langId
        )
      }

      return false
    }

    //is released and no limited working time --> always allowed to solve
    if (copy.isStillOpen && copy.isReleased && !copy.hasLimitedWorkingTime
      && (copy.releaseStartType === ReleaseStartType.Manual || copy.releaseDurationType === ReleaseDurationType.Infinitely)) {
      clearInterval(updateLeftTimeIntervals)
      return true
    }

    if (copy.leftTimeInS !== null && copy.leftTimeInS <= criticalLeftWorkingTimeInS) {
      this.props.setIsSubHeaderBarCollapsed(false)
      this.props.setIsLeftTimeStringDisplayed(true)
    }

    return true
  }

  render(): JSX.Element {

    const isExerciseLocked = this.props.exercisePreview !== null && this.props.exercisePreview.isPermanentlyLocked

    return (
      <div className="fh">

        {
          this.props.isLoading &&
          <Spinner/>
        }

        {
          //first display got focus again interception overlay then this
          this.props.lockSolutionsFlagAndIsStillOpen && this.props.isOnGotFocusInterceptionOverlayDisplayed === false &&
          <LockSolutionOverlay>
            <p className="fg-colored" style={{fontSize: '16px'}}>
              <span className="mar-right-half">
                {
                  getI18n(this.props.langId,
                          'Your solution is currently locked for the submission process'
                  )
                }
              </span>
              <HelpPopup className="fg-colored" defaultText={getI18n(this.props.langId,
                                                                     "Note that locking the solution is optional, your solution will be assessed (maybe by a tutor) regardless if it's locked or not"
              )}/>
            </p>

            <div style={{marginTop: '0.5em'}} className="flexed">

              {
                !this.props.doExercise.exerciseRelease.hideExerciseLeaveActions &&
                <div className="mar-right">
                  <Button icon labelPosition='left' onClick={() => {
                    history.push(
                      constants.openExercisesLinkPath)
                  }}>
                    <Icon name="reply"/>
                    {
                      getI18n(this.props.langId, "Leave")
                    }
                  </Button>
                </div>
              }

              <div>
                <Button icon labelPosition='left' onClick={() => {
                  this.props.updateDoExerciseLockSolutionsFlagAsync(this.props.releaseCode, false)
                }}>
                  <Icon name="lock open"/>
                  {
                    getI18n(this.props.langId, "Unlock solution for editing")
                  }
                </Button>

              </div>
            </div>

          </LockSolutionOverlay>
        }
        {
          this.props.isOnGotFocusInterceptionOverlayDisplayed &&
          <InterceptionOverlay>

            {
              //copy only work if the input is visible, enabled...
              //because of react the user cannot change the value of the input...
              //so this should be ok (even though it's ugly)
            }


            <p className="fg-colored">
              {
                getI18n(this.props.langId,
                        'To resume click the button. This will set the clipboard content to a single whitespace.'
                )
              }
            </p>

            <Input id={onGotFocusBackInputId} value=' '/>
            <Button style={{color: 'rgba(0, 0, 0, 0.6)'}} onClick={() => {
              const input = document.getElementById(onGotFocusBackInputId) as HTMLInputElement
              input.select()
              document.execCommand("Copy");

              this.props.setIsOnGotFocusInterceptionOverlayDisplayedDoExerciseSite(false)
            }}>
              {
                getI18n(this.props.langId, 'Resume')
              }
            </Button>

          </InterceptionOverlay>
        }
        {
          this.props.isCustomTestDialogDisplayed &&
          <ChangeCustomTestDialog
            canUserEditCustomTests={this.props.canUserCreateCustomTests}
          />
        }
        {
          this.props.isViewTestDialogDisplayed &&
          <ViewTestDialog
            cancelViewTestDialog={() => {
              this.props.cancelViewTestDialog()
            }}
            getDownloadTestAssetLink={(testId: number, fileId: number, isSubmitTest: boolean) => {
              return `${debugOriginUrl}/api/assets/${this.props.releaseCode}/${testId}/${fileId}/${isSubmitTest}/false`
            }}
          />
        }
        {
          this.props.isCodeEditorSettingsDialogDisplayed &&
          <EditorSettingsDialog/>
        }

        {
          this.props.isNoDialogDisplayed &&
          <PanelGrid
            width="100%"
            height="100%"

            topRightPanelViewSize={this.props.topRightPanelViewSize}
            topLeftPanelViewSize={this.props.topLeftPanelViewSize}
            botRightPanelViewSize={this.props.botRightPanelViewSize}
            botLeftPanelViewSize={this.props.botLeftPanelViewSize}

            hDividerPosOffset={this.props.hDividerPosOffset}
            vDividerPosOffset={this.props.vDividerPosOffset}
            onHDividerPosOffsetChanged={(Offset) => {
              this.props.setDoExerciseHDividerPosOffset(Offset)
            }}
            onVDividerPosOffsetChanged={(Offset) => {
              this.props.setDoExerciseVDividerPosOffset(Offset)
            }}

            topLeftPanel={
              <Panel
                applyDefaultPadding={false}
                selectedTabIndex={this.props.topLeftSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setTopLeftSelectedTabIndex(index)
                }}

                onSizeChangedClicked={this.props.setTopLeftPanelViewSize}
                panelViewSize={this.props.topLeftPanelViewSize}
              >
                <TabContentHost
                  title={getI18n(this.props.langId, 'Editor')}
                  icon={'write'}
                  applyDefaultPadding={false}
                  tabHeaderBar={
                    <EditorPanelHeaderBar/>
                  }
                >
                  <EditorPanelView/>
                </TabContentHost>

                {
                  this.props.canDoAfterEditor && this.props.afterSolution !== null &&
                  <TabContentHost
                    title={getI18n(this.props.langId, 'Subsequent editor')}
                    icon={'life ring'}
                    applyDefaultPadding={false}
                    defaultHelpText={getI18n(this.props.langId,
                                             'The editor that is displayed after the release & the automatic assessment have finished. You can try to solve the exercise (again) with this editor BUT you cannot save!'
                    )}
                    tabHeaderBar={
                      <AfterEditorPanelHeaderBar/>
                    }
                  >
                    <AfterEditorPanelView/>
                  </TabContentHost>
                }

              </Panel>
            }

            topRightPanel={
              <Panel
                selectedTabIndex={this.props.topRightSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setTopRightSelectedTabIndex(index)
                }}
                onSizeChangedClicked={this.props.setTopRightPanelViewSize}
                panelViewSize={this.props.topRightPanelViewSize}
                additionalActionsRightIcons={
                  <ZoomPreviewPanelActions printId={this.props.topRightSelectedTabIndex === 0
                                                    ? taskContentPrintId
                                                    : feedbackContentPrintId}
                                           canPrint={!this.props.doExercise.exerciseRelease.hidePrintOptions}/>
                }
              >
                <TabContentHost
                  title={getI18n(this.props.langId, 'Task description')}
                  icon={'align left'}
                >
                  <PreviewPanelView printId={taskContentPrintId}
                                    descriptionContent={this.props.doExercise.exerciseDescription.content}
                                    assets={this.props.doExercise.exerciseDescription.assets}
                                    dummy={this.props.dummy}
                                    fontSizeInPx={this.props.taskDescriptionFontSizeInPx}
                  />
                </TabContentHost>

                {
                  this.props.displayAssessmentResult &&
                  <TabContentHost
                    title={getI18n(this.props.langId, 'Manual assessment')}
                    icon={'talk'}
                    defaultHelpText={getI18n(this.props.langId,
                                             'This is the result of the manual assessment. It is only visible if the tutor has left feedback or manual points for you.'
                    )}
                    tabHeaderBar={
                      <AssessmentResultPanelHeaderBar/>
                    }
                  >
                    <MarkdownRenderer
                      printId={feedbackContentPrintId}
                      content={this.props.assessmentResult.feedbackForStudent}
                      assets={[]}
                      contentType={QuestionType.markdown}
                      dummy={this.props.dummy}
                      fontSizeInPx={this.props.taskDescriptionFontSizeInPx}
                    />
                  </TabContentHost>
                }

              </Panel>
            }

            bottomLeftPanel={
              <Panel
                selectedTabIndex={this.props.bottomLeftSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setBottomLeftSelectedTabIndex(index)
                }}
                onSizeChangedClicked={this.props.setBotLeftPanelViewSize}
                panelViewSize={this.props.botLeftPanelViewSize}
              >
                <TabContentHost
                  title={getI18n(this.props.langId, 'Normal tests')}
                  icon={'bug'}
                  tabHeaderBar={
                    <TestPanelHeaderBar/>
                  }
                  defaultHelpText={getI18n(this.props.langId,
                                           'If the editor has unsaved changes and you run a test then the files will be auto saved before running the test'
                  )}
                >
                  <TestsPanel
                    displayWeightIndicatorsForResults={false}
                    areAfterTests={false}
                    tests={this.props.doExercise.tests}
                    selectedConsoleViewTest={this.props.selectedConsoleViewTest}
                    isUserAllowedToRunTests={this.props.canUserSolveExercise}
                    editorHasUnsavedChanges={this.props.editorHasUnsavedChanges}
                    isSomeTestRunning={this.props.isSomeTestRunning}
                    onSaveSolution={async () => {
                      //reset test results because this is only called if the editor has unsaved changes
                      await this.props.saveDoExerciseSolution(this.props.releaseCode, true, this.props.solution,
                                                              this.props.langId
                      )
                    }}
                    setSelectedConsoleViewTest={(test: RunnableExerciseTestFrontendExclusive
                      | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                      this.props.setSelectedConsoleViewTest(test)
                    }}
                    onRunTest={(test: RunnableExerciseTestFrontendExclusive
                      | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive) => {

                      const command: RunTestCommandForBackend = {
                        generatedCode: this.props.doExercise.exerciseRelease.releaseCode,
                        plangId: this.props.solutionPLangId,
                        testIds: [test.id],
                        isNormalTest: true,
                        isCustomTest: false,
                        isSubmitTest: false,
                      }

                      this.props.runDoExerciseTest(command)
                    }}
                    openViewTestDialog={(runnableTest: RunnableExerciseTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                      this.props.openViewTestDialog(runnableTest)
                    }}
                  />

                </TabContentHost>

                <TabContentHost
                  title={getI18n(this.props.langId, 'Custom tests')}
                  icon={'shield'}
                  tabHeaderBar={
                    <CustomTestPanelHeaderBar

                      isSomeTestRunning={this.props.isSomeTestRunning}
                      customTests={this.props.doExercise.customTests}
                      canUserCreateCustomTests={this.props.canUserCreateCustomTests}

                      openCreateCustomTestDialog={this.props.openCreateCustomTestDialog}

                      runCustomTests={async (tests: ReadonlyArray<RunnableCustomTestFrontendExclusive>) => {

                        //a template
                        // noinspection TsLint
                        let tempCommand: RunTestCommandForBackend = {
                          generatedCode: this.props.releaseCode,
                          plangId: this.props.solutionPLangId,
                          testIds: tests.map(p => p.id),
                          isNormalTest: false,
                          isCustomTest: true,
                          isSubmitTest: false,
                        }

                        const hasAnyTestFailed = await this.props.runCustomDoExerciseTest(tempCommand,
                                                                                          this.props.langId
                        )

                        return hasAnyTestFailed as any //wrong infered because magic redux binding
                      }}

                      resetCustomTestResults={this.props.resetCustomTestResults}

                      beforeRunningTests={async () => {
                        if (this.props.editorHasUnsavedChanges) {
                          //reset test results because this is only called if the editor has unsaved changes
                          await this.props.saveDoExerciseSolution(this.props.releaseCode, true, this.props.solution,
                                                                  this.props.langId
                          )
                        }
                      }}

                    />
                  }
                  defaultHelpText={getI18n(this.props.langId,
                                           'If the editor has unsaved changes and you run a test then the files will be auto saved before running the test'
                  )}
                >
                  <CustomTestsPanel
                    areAfterTests={false}
                    customTests={this.props.doExercise.customTests}
                    canUserEditExercise={this.props.canUserEditExercise}
                    canUserEditCustomTests={this.props.canUserEditExercise}
                    isSomeTestRunning={this.props.isSomeTestRunning}
                    selectedConsoleViewTest={this.props.selectedConsoleViewTest}
                    setSelectedConsoleViewTest={this.props.setSelectedConsoleViewTest}
                    openUpdateCustomTestDialog={this.props.openUpdateCustomTestDialog}

                    runCustomTest={async (test: RunnableCustomTestFrontendExclusive) => {

                      //a template
                      // noinspection TsLint
                      let tempCommand: RunTestCommandForBackend = {
                        generatedCode: this.props.releaseCode,
                        plangId: this.props.solutionPLangId,
                        testIds: [test.id],
                        isNormalTest: false,
                        isCustomTest: true,
                        isSubmitTest: false,
                      }

                      const hasAnyTestFailed = await this.props.runCustomDoExerciseTest(tempCommand, this.props.langId)

                      return hasAnyTestFailed as any //wrong infered because magic redux binding
                    }}

                    beforeRunningTests={async () => {
                      if (this.props.editorHasUnsavedChanges) {
                        //reset test results because this is only called if the editor has unsaved changes
                        await this.props.saveDoExerciseSolution(this.props.releaseCode, true, this.props.solution,
                                                                this.props.langId
                        )
                      }
                    }}
                    deleteCustomTestDoEditor={(test: RunnableCustomTestFrontendExclusive) => {
                      this.props.deleteCustomTestDoEditor(this.props.releaseCode, test,
                                                          this.props.doExercise.customTests
                      )
                    }}

                  />
                </TabContentHost>

                {
                  this.props.displaySubmitTestResults &&
                  <TabContentHost
                    title={getI18n(this.props.langId, 'Submit tests')}
                    icon={'flag checkered'}
                    tabHeaderBar={
                      <SubmitTestsPanelHeaderBar submitTestResults={this.props.submitTestResults}/>

                    }
                    defaultHelpText={getI18n(this.props.langId,
                                             'These are the submit tests. They are executed and visible after the release ended and the automatic assessment has run.'
                    )}
                  >

                    {
                      //submit test panel for submit tests ... don't allow the user to run tests
                    }
                    <TestsPanel
                      areAfterTests={false}
                      displayWeightIndicatorsForResults={true}
                      tests={this.props.submitTestResults}
                      selectedConsoleViewTest={this.props.selectedConsoleViewTest}
                      isUserAllowedToRunTests={false}
                      editorHasUnsavedChanges={null}
                      isSomeTestRunning={false}
                      onSaveSolution={null}
                      setSelectedConsoleViewTest={(test: RunnableExerciseTestFrontendExclusive
                        | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                        this.props.setSelectedConsoleViewTest(test)
                      }}
                      onRunTest={null}
                      openViewTestDialog={(runnableTest: RunnableExerciseTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                        this.props.openViewTestDialog(runnableTest)
                      }}
                    />

                  </TabContentHost>
                }

                {
                  //--- after finish normal tests, custom, submit tests
                }
                {
                  this.props.canDoAfterEditor && this.props.afterSolution !== null &&
                  <TabContentHost
                    title={getI18n(this.props.langId, 'Normal tests')}
                    icon={'life ring'}
                    tabHeaderBar={
                      <AfterNormalTestsPanelHeaderBar/>
                    }
                    defaultHelpText={getI18n(this.props.langId,
                                             'These tests are executed using the subsequent editor content. They are available when you cannot longer edit your original solution.'
                    )}
                  >
                    <TestsPanel
                      areAfterTests={true}
                      displayWeightIndicatorsForResults={false}
                      tests={this.props.afterNormalTests}
                      selectedConsoleViewTest={this.props.selectedConsoleViewTest}
                      isUserAllowedToRunTests={true}
                      editorHasUnsavedChanges={this.props.afterEditorHasUnsavedChanges}
                      isSomeTestRunning={this.props.isSomeTestRunning}
                      onSaveSolution={async () => {
                        //reset test results because this is only called if the editor has unsaved changes
                        await this.props.saveDoExerciseAfterSolutionAsync(this.props.releaseCode,
                                                                          this.props.solution.pLangId, true,
                                                                          this.props.afterSolution,
                                                                          this.props.langId
                        )
                      }}
                      setSelectedConsoleViewTest={(test: RunnableExerciseTestFrontendExclusive
                        | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                        this.props.setSelectedConsoleViewTest(test)
                      }}
                      onRunTest={(test: RunnableExerciseTestFrontendExclusive
                        | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive) => {

                        const command: RunTestCommandForBackend = {
                          generatedCode: this.props.doExercise.exerciseRelease.releaseCode,
                          isSubmitTest: false,
                          testIds: [test.id],
                          isNormalTest: true,
                          plangId: this.props.afterSolution.pLangId,
                          isCustomTest: false
                        }

                        this.props.runAfterNormalTest(command)

                      }}
                      openViewTestDialog={(runnableTest: RunnableExerciseTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                        this.props.openViewTestDialog(runnableTest)
                      }}
                    />

                  </TabContentHost>
                }


                {
                  this.props.releaseFinishedAndAutomaticAssessmentHasFinished && this.props.afterSolution !== null &&
                  <TabContentHost
                    title={getI18n(this.props.langId, 'Submit tests')}
                    icon={'life ring'}
                    tabHeaderBar={
                      <AfterSubmitTestsPanelHeaderBar submitTestResults={this.props.afterSubmitTests}/>
                    }
                    defaultHelpText={getI18n(this.props.langId,
                                             'These submit tests are executed using the subsequent editor content. They are available when the automatic assessment ran. Note that these submit tests are executed on the normal test server (not the submit test server), so there might be different settings e.g. timeouts. Ask a tutor to execute your submit tests on the submit test server'
                    )}
                  >
                    <TestsPanel
                      areAfterTests={true}
                      displayWeightIndicatorsForResults={true}
                      tests={this.props.afterSubmitTests}
                      selectedConsoleViewTest={this.props.selectedConsoleViewTest}
                      isUserAllowedToRunTests={this.props.releaseFinishedAndAutomaticAssessmentHasFinished}
                      editorHasUnsavedChanges={this.props.afterEditorHasUnsavedChanges}
                      isSomeTestRunning={this.props.isSomeTestRunning}
                      onSaveSolution={async () => {
                        //reset test results because this is only called if the editor has unsaved changes
                        await this.props.saveDoExerciseAfterSolutionAsync(this.props.releaseCode,
                                                                          this.props.solution.pLangId, true,
                                                                          this.props.afterSolution,
                                                                          this.props.langId
                        )
                      }}
                      setSelectedConsoleViewTest={(test: RunnableExerciseTestFrontendExclusive
                        | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                        this.props.setSelectedConsoleViewTest(test)
                      }}
                      onRunTest={(test: RunnableExerciseTestFrontendExclusive
                        | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive) => {

                        const command: RunTestCommandForBackend = {
                          generatedCode: this.props.doExercise.exerciseRelease.releaseCode,
                          isSubmitTest: true,
                          testIds: [test.id],
                          isNormalTest: false,
                          plangId: this.props.afterSolution.pLangId,
                          isCustomTest: false
                        }

                        this.props.runAfterSubmitTest(command)

                      }}
                      openViewTestDialog={(runnableTest: RunnableExerciseTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                        this.props.openViewTestDialog(runnableTest)
                      }}
                    />

                  </TabContentHost>
                }

                {
                  //currently we don't support changing (add/edit/delete) custom tests because
                  //we would need to keep them in sync (normal & after) ... backend supports this already

                  this.props.canDoAfterEditor && this.props.afterSolution !== null &&
                  <TabContentHost
                    title={getI18n(this.props.langId, 'Custom tests')}
                    icon={'life ring'}
                    tabHeaderBar={
                      <AfterCustomTestsPanelHeaderBar
                        isSomeTestRunning={this.props.isSomeTestRunning}
                        customTests={this.props.afterCustomTestsMerged}
                        canUserCreateCustomTests={this.props.canUserCreateCustomTests}

                        openCreateCustomTestDialog={this.props.openCreateCustomTestDialog}

                        runCustomTests={async (tests: ReadonlyArray<RunnableCustomTestFrontendExclusive>) => {

                          //a template
                          // noinspection TsLint
                          let tempCommand: RunTestCommandForBackend = {
                            generatedCode: this.props.releaseCode,
                            plangId: this.props.solutionPLangId,
                            testIds: tests.map(p => p.id),
                            isNormalTest: false,
                            isCustomTest: true,
                            isSubmitTest: false,
                          }

                          const hasAnyTestFailed = await this.props.runAfterCustomTest(tempCommand)

                          return hasAnyTestFailed as any //wrong infered because magic redux binding
                        }}

                        resetCustomTestResults={this.props.resetAfterCustomTestResults}

                        beforeRunningTests={async () => {
                          if (this.props.afterEditorHasUnsavedChanges) {
                            //reset test results because this is only called if the editor has unsaved changes
                            await this.props.saveDoExerciseAfterSolutionAsync(this.props.releaseCode,
                                                                              this.props.solution.pLangId, true,
                                                                              this.props.afterSolution,
                                                                              this.props.langId
                            )
                          }
                        }}
                      />
                    }
                    defaultHelpText={getI18n(this.props.langId,
                                             'These tests are executed using the subsequent editor content. They are available when you cannot longer edit your original solution.'
                    )}
                  >
                    <CustomTestsPanel
                      areAfterTests={true}
                      customTests={this.props.afterCustomTestsMerged}
                      canUserEditExercise={true}
                      canUserEditCustomTests={false}
                      isSomeTestRunning={this.props.isSomeTestRunning}
                      selectedConsoleViewTest={this.props.selectedConsoleViewTest}
                      setSelectedConsoleViewTest={this.props.setSelectedConsoleViewTest}
                      openUpdateCustomTestDialog={this.props.openUpdateCustomTestDialog}

                      runCustomTest={async (test: RunnableCustomTestFrontendExclusive) => {

                        //a template
                        // noinspection TsLint
                        let tempCommand: RunTestCommandForBackend = {
                          generatedCode: this.props.releaseCode,
                          plangId: this.props.solutionPLangId,
                          testIds: [test.id],
                          isNormalTest: false,
                          isCustomTest: true,
                          isSubmitTest: false,
                        }

                        const hasAnyTestFailed = await this.props.runAfterCustomTest(tempCommand)

                        return hasAnyTestFailed as any //wrong infered because magic redux binding
                      }}

                      beforeRunningTests={async () => {
                        if (this.props.afterEditorHasUnsavedChanges) {
                          //reset test results because this is only called if the editor has unsaved changes
                          await this.props.saveDoExerciseAfterSolutionAsync(this.props.releaseCode,
                                                                            this.props.solution.pLangId, true,
                                                                            this.props.afterSolution,
                                                                            this.props.langId
                          )
                        }
                      }}
                      deleteCustomTestDoEditor={(test: RunnableCustomTestFrontendExclusive) => {
                        this.props.deleteCustomTestDoEditor(this.props.releaseCode, test,
                                                            this.props.doExercise.customTests
                        )
                      }}
                    />

                  </TabContentHost>
                }

              </Panel>
            }

            bottomRightPanel={
              <Panel
                selectedTabIndex={this.props.bottomRightSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setBottomRightSelectedTabIndex(index)
                }}
                onSizeChangedClicked={this.props.setBotRightPanelViewSize}
                panelViewSize={this.props.botRightPanelViewSize}
              >
                <TabContentHost
                  title={getI18n(this.props.langId, 'Console output')}
                  icon={'terminal'}
                  tabHeaderBar={
                    <ConsolePanelHeaderBar test={this.props.selectedConsoleViewTest} testTypes={this.props.testTypes}
                    />
                  }
                >
                  <ConsolePanelView test={this.props.selectedConsoleViewTest} protocolItems={null}
                                    testTypes={this.props.testTypes}
                                    displayFilterConsoleItemsOptions={true}
                  />
                </TabContentHost>
              </Panel>
            }


          >
          </PanelGrid>
        }


      </div>
    )
  }
}

export default withRouter(connect(mapStateToProps, mapDispatchToProps)(doExerciseSite))
