/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {Icon} from "semantic-ui-react";
import {
  runDoExerciseTest,
  saveDoExerciseSolution
} from "../../../../state/actions/doExerciseSite/doExerciseCrudActions";
import {RunTestCommandForBackend} from "../../../../types/testsAndTestProtocol";
import {HelpPopup} from "../../../helpers/helpPopup";
import Logger from "../../../../helpers/logger";
import {isTestResultValid} from "../../../../helpers/testHelper";
import {ReleaseDurationType} from "../../../../types/ReleaseDurationType";
import orderBy from 'lodash-es/orderBy'


import {SimpleVDivider} from "../../../helpers/simpleVDivider";
import {resetNormalTestResults} from "../../../../state/actions/doExerciseSite/doExerciseSiteActions";
import {getI18n} from "../../../../../i18n/i18nRoot";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    tests: rootState.doExerciseSiteState.exercise.tests,
    releaseCode: rootState.doExerciseSiteState.exercise.exerciseRelease.releaseCode,
    solutionPLangId: rootState.doExerciseSiteState.solution.pLangId,
    systemSettings: rootState.globalSystemSettingsState.systemSettings,
    editorHasUnsavedChanges: rootState.doExerciseSiteState.doEditorState.hasUnsavedChanges,
    solution: rootState.doExerciseSiteState.solution,

    isSomeTestRunning: rootState.doExerciseSiteState.exercise.tests.some(p => p.uiIsRunning)
      || rootState.doExerciseSiteState.exercise.customTests.some(p => p.uiIsRunning),

    canUserEditExercise: ((rootState.doExerciseSiteState.exercisePreviewFrontendTemp
      && rootState.doExerciseSiteState.exercisePreviewFrontendTemp.isStillOpen)
      || (rootState.doExerciseSiteState.exercise.exerciseRelease.isReleased
        && rootState.doExerciseSiteState.exercise.exerciseRelease.hasLimitedWorkingTime === false
        && rootState.doExerciseSiteState.exercise.exerciseRelease.releaseDurationType === ReleaseDurationType.Infinitely)) &&
      (rootState.doExerciseSiteState.exercisePreviewFrontendTemp
        && rootState.doExerciseSiteState.exercisePreviewFrontendTemp.isPermanentlyLocked === false),

    maxNumberOfTestsWithOneRequest: rootState.globalSystemSettingsState.systemSettings
                                    && rootState.globalSystemSettingsState.systemSettings.maxNumberOfTestsWithOneRequest >= 0
                                    ? rootState.globalSystemSettingsState.systemSettings.maxNumberOfTestsWithOneRequest
                                    : 1,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        runDoExerciseTest,
                                                                        saveDoExerciseSolution,
                                                                        resetNormalTestResults,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class testsPanelHeader extends React.Component<Props, any> {
  render(): JSX.Element {

    return (
      <div className={!this.props.isSomeTestRunning
                      ? "flexed"
                      : 'flexed div-disabled'}>

        <div
          className={`mar-right-half ${(this.props.canUserEditExercise && this.props.tests.length > 0)
                                       ? 'clickable'
                                       : 'div-disabled' }`}
          onClick={async () => {
            //save changes if code changed
            //this also ensures that the old/now invalid after saving test results are cleared
            if (this.props.editorHasUnsavedChanges) {
              //reset test results because this is only called if the editor has unsaved changes
              await this.props.saveDoExerciseSolution(this.props.releaseCode, true, this.props.solution,
                                                      this.props.langId
              )
            }

            const maxNumberOfTestsWithOneRequest = this.props.maxNumberOfTestsWithOneRequest

            Logger.debug("test panel header bar",
                         `running all test with ${maxNumberOfTestsWithOneRequest} test in parallel`
            )


            const orderedTests = orderBy(this.props.tests, (test) => test.displayIndex)

            const validTests = orderedTests.filter(p => isTestResultValid(p))

            //a template
            // noinspection TsLint
            let tempCommand: RunTestCommandForBackend = {
              generatedCode: this.props.releaseCode,
              plangId: this.props.solutionPLangId,
              testIds: [],
              isNormalTest: true,
              isCustomTest: false,
              isSubmitTest: false,
            }

            // noinspection TsLint
            for (let i = 0; i < orderedTests.length; i++) {
              const test = orderedTests[i]

              //test was false or all tests have passed already
              if (isTestResultValid(test) === false || validTests.length === orderedTests.length) {
                //add
                tempCommand = {
                  ...tempCommand,
                  testIds: [...tempCommand.testIds, test.id]
                }
              } else {
                //test was already valid
              }

              //we reached the limit
              if (tempCommand.testIds.length === maxNumberOfTestsWithOneRequest) {
                const hasAnyTestFailed = await this.props.runDoExerciseTest(tempCommand)

                //clear test ids
                tempCommand = {
                  ...tempCommand,
                  testIds: []
                }

                //stop if at least 1 test failed
                if (hasAnyTestFailed) {
                  //return instead of break else we would run the last check below which would be always true
                  return
                }
              }
            }

            //we might have some last tests but not reached the max num of tests with one request...
            if (tempCommand.testIds.length > 0) {
              //we missed the last < maxNumberOfTestsWithOneRequest tests
              //these are still in the last tempCommand
              await this.props.runDoExerciseTest(tempCommand)
            }

          }}
        >
          <Icon name="forward"/>
          <span>
            {
              getI18n(this.props.langId, 'Run all tests')
            }
          </span>
        </div>

        <HelpPopup
          defaultText={
            `${getI18n(this.props.langId,
                       'Runs all not passed tests. If all tests have passed then all tests are re-run. When at least one test fails then the next x tests are not executed.'
            )} ${getI18n(this.props.langId,
                         'Max parallel tests'
            )}: ${this.props.maxNumberOfTestsWithOneRequest}`}
          wide
        />

        <SimpleVDivider/>

        <div className={this.props.canUserEditExercise
                        ? 'clickable mar-right-half'
                        : 'div-disabled mar-right-half'}
             onClick={() => {
               this.props.resetNormalTestResults()
             }}
        >
          <Icon name="remove"/>
          <span>
          {
            getI18n(this.props.langId, 'Clear test results')
          }
          </span>
        </div>
        <HelpPopup
          defaultText={getI18n(this.props.langId,
                               'This only discards the results in the frontend, the results on the server are still there'
          )}
          wide
        />

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(testsPanelHeader)
