/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../../state/reducers";
import {Form, Icon, List, Tab} from "semantic-ui-react";
import MaterialInput from '../../../../material/materialInput'
import SimpleCodeEditorWrapper from '../../../../codeEditors/simpleCodeEditorWrapper'
import ConsolePanelView from '../../consolePanel/consolePanelView'
import {TestProtocolItem} from "../../../../../types/testResults";
import {
  convertSizeToHumanReadableSize,
  convertTestProtocolFromString
} from "../../../../../helpers/convertersAndTransformers";
import {FilePreviewFromBackend, TestAssetFromBackendWithData} from "../../../../../types/exerciseEditor";


import {
  aceCompareFilesTestLanguage,
  aceRegexLanguage,
  aceTestProtocolLanguage,
  aceTextLanguage
} from "../../../../../helpers/editorHelper";
import {EditorTab} from "../../../../codeEditors/multiTabCodeEditorProps";
import {getI18n} from "../../../../../../i18n/i18nRoot";
import {KnownInternalTestTypes} from '../../../../../types/testTypes'
import CompareFilesTestSettingsView
  from '../../../exerciseEditorSite/exerciseTestsPanel/dialogs/compareFilesTestSettingsView'
import TestSettingsView from '../../../../helpers/tests/testSettingsView'
import {TestTypeLimits} from '../../../../../constants'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string

  readonly getDownloadTestAssetLink: (testId: number, fileId: number, isSubmitTest: boolean) => string
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    test: rootState.viewTestDialogState.runnableTest,
    testTypes: rootState.testTypesState.testTypes,
    getDownloadTestAssetLink: props.getDownloadTestAssetLink,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here

                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


export const viewTestViewEditorId = 'viewTestViewEditorId'

//TODO maybe merge this with src/components/exerciseEditorSite/editorTestsPanel/dialogs/changeTestView.tsx
//and or src/components/doExerciseSite/customTestsDialog/changeCustomTestView.tsx
//they display basically the same data...
class viewTestView extends React.Component<Props, any> {

  render(): JSX.Element {

    const testType = this.props.testTypes.find(p => p.id === this.props.test.testTypeId)

    //we fake a test result to reuse the console view
    const visualTest: ReadonlyArray<TestProtocolItem> | null = convertTestProtocolFromString(this.props.test.content,
                                                                                             testType
    )

    const tempEditorTab: EditorTab = {
      id: this.props.test.id,
      content: this.props.test.content,
      language: testType.internalName === KnownInternalTestTypes.regexTest
                ? aceRegexLanguage
                : testType.internalName === KnownInternalTestTypes.blackBoxTest
                  ? aceTestProtocolLanguage
                  : testType.internalName === KnownInternalTestTypes.compareFilesTest
                    ? aceCompareFilesTestLanguage
                    : aceTextLanguage,
      isReadonly: true,
      fileNameWithExtensions: this.props.test.displayName,
      displayIndex: this.props.test.displayIndex,
      isMainFile: true,
      isContentVisibleForUser: true,
    }

    const isCompareFilesTest = testType.internalName === KnownInternalTestTypes.compareFilesTest

    return (
      <div className="fh flexed-h">
        <div className="ui form divided-checkboxes fh" style={{height: 'auto' /*firefox fix*/}}>

          <Form.Group widths='equal'>
            <Form.Field>
              <label>
                {
                  getI18n(this.props.langId, 'Type')
                }
              </label>
              <MaterialInput
                //this is vad because this would switch from uncontrolled to controlled component but it's disabled...
                value={testType
                       ? testType.displayName
                       : getI18n(this.props.langId, 'Unknown test type')}
                disabled
              />
            </Form.Field>

            <Form.Field>
              <label>
                {
                  getI18n(this.props.langId, 'Points')
                }
              </label>
              <MaterialInput
                value={this.props.test.weight}
                disabled
              />
            </Form.Field>

            <Form.Field>
              <label>
                {
                  getI18n(this.props.langId, 'Name')
                }
              </label>
              <MaterialInput
                value={this.props.test.displayName}
                disabled
              />
            </Form.Field>
          </Form.Group>

        </div>

        <Tab menu={{
          secondary: true,
          pointing: true
        }} className="fh flexed-h" panes={
          [
            {
              menuItem: getI18n(this.props.langId, 'Test content visually'),
              render: () => {
                return (
                  <div className="mar-bot">
                    <ConsolePanelView test={null} protocolItems={visualTest} testTypes={this.props.testTypes}
                                      displayFilterConsoleItemsOptions={false}
                    />
                  </div>
                )
              }
            },
            {
              menuItem: getI18n(this.props.langId, 'Test content'),
              render: () => {
                return (

                  <div className="fh flexed-h flex-stretch-0">

                    {
                      isCompareFilesTest &&
                      <div>
                        <CompareFilesTestSettingsView files={this.props.test.files}
                                                      testContent={this.props.test.content}
                                                      setTestContent={null}
                                                      isReadonly={true}
                        />
                      </div>
                    }

                    <div className="flexed-h flex-stretch-0">

                      {
                        //do not embed monaco into form else some text are will get the min, max text area size...
                      }
                      <SimpleCodeEditorWrapper
                        id={viewTestViewEditorId}
                        editorTab={tempEditorTab}
                      />
                    </div>
                  </div>
                )
              }
            },
            {
              menuItem: getI18n(this.props.langId, 'Test settings & files'),
              render: () => {
                return (
                  <div className="ui form divided-checkboxes fh">

                    <TestSettingsView
                      testTypeLimits={this.props.test.isSubmitTest
                                      ? TestTypeLimits.IsSubmitTest
                                      : TestTypeLimits.IsNormalTest}
                      timeoutInMs={this.props.test.testSettings.timeoutInMs}
                      memoryLimitInKb={this.props.test.testSettings.memoryLimitInKb}
                      maxDiskSpaceInKb={this.props.test.testSettings.maxDiskSpaceInKb}
                      compileTimeoutInMs={this.props.test.testSettings.compileTimeoutInMs}
                      compilerOptions={this.props.test.testSettings.compilerOptions}

                      set_timeoutInMs={null}
                      set_memoryLimitInKb={null}
                      set_maxDiskSpaceInKb={null}
                      set_compileTimeoutInMs={null}
                      set_compilerOptions={null}
                    />

                    <h4 className="ui dividing header">
                      {
                        getI18n(this.props.langId, 'Files')
                      }
                    </h4>

                    <div>

                      <List>
                        {
                          this.props.test.files.map((file: FilePreviewFromBackend) => {
                            return (
                              <List.Item key={file.id}>
                                <List.Icon size='large' verticalAlign='middle'>
                                  <a target="_blank" download
                                     href={this.props.getDownloadTestAssetLink(this.props.test.id, file.id,
                                                                               this.props.test.isSubmitTest
                                     )}>
                                    <Icon name='download'/>
                                  </a>
                                </List.Icon>
                                <List.Content>
                                  <List.Header>

                                    <div className="flexed">
                                      <div className="mar-right-half">
                                        {
                                          file.originalName
                                        }
                                      </div>

                                      <div className="not-important-colored">
                                        (
                                        {convertSizeToHumanReadableSize(file.sizeInBytes)}
                                        )
                                      </div>
                                    </div>

                                  </List.Header>
                                  <List.Description>
                                    <div className="not-important-colored">
                                      {
                                        file.mimeType
                                      }
                                    </div>
                                  </List.Description>
                                </List.Content>
                              </List.Item>
                            )
                          })
                        }
                      </List>

                    </div>


                  </div>
                )
              }
            },
          ]

        }/>


      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(viewTestView)
