/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {Icon} from "semantic-ui-react";
import {InputOnChangeEvent} from "../../../../types/reactEvents";
import {FileWithData} from "../../../../types/exerciseEditor";
import {setExerciseEditorAssets} from "../../../../state/actions/exerciseEditorSite/subSets/editorExerciseDescriptionActions";
import {getNextFrontendId} from "../../../../helpers/frontendIdHelper";
import {assetPrefix} from "../../../../constants";
import {readFileAsAsset, readFileAsText} from "../../../../helpers/fileReader";
import Logger from "../../../../helpers/logger";


import {errorDialog} from "../../../../helpers/dialogHelper";
import {getI18n} from "../../../../../i18n/i18nRoot";
import {HelpPopup} from '../../../helpers/helpPopup'

//const css = require('./styles.styl');

export interface MyProps {

  /**
   * only allowed for old exercises, else we cannot add a connection from the asset ref to the exercise
   */
  readonly isEnabled: boolean

  readonly assets: ReadonlyArray<FileWithData>

  readonly setAssets: (assets: ReadonlyArray<FileWithData>) => void

  readonly openAddAssetsModal: () => void
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


class AssetsPanelHeaderBar extends React.Component<Props, any> {

  render(): JSX.Element {
    return (
      <div className="flexed">

        <div className={this.props.isEnabled
                        ? 'clickable'
                        : 'div-disabled'}
             onClick={() => {
               this.props.openAddAssetsModal()
             }}
        >
          <Icon name="write"/>
          <span>
            {
              getI18n(this.props.langId, 'Manage')
            }
          </span>
        </div>

        {
          this.props.isEnabled === false &&
          <HelpPopup className="mar-left-half" defaultText={getI18n(this.props.langId,
                                                                    "Managing assets is only allowed for saved exercises (not for new exercises)"
          )}/>
        }
        <HelpPopup className="mar-left-half warning-colored" defaultText={getI18n(this.props.langId,
                                                                                  "All changes to assets are applied immediately in the backend (saving is not needed for this)!"
        )}/>


      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(AssetsPanelHeaderBar)
