/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {List, Icon} from "semantic-ui-react";
import {ExerciseCodeTemplateFullBase} from "../../../../types/exerciseEditor";
import CodeTemplateDetailsView from './codeTemplateDetailsView'
import {
  setCodeTemplates,
  setUiCodeTemplateFilesDisplayedHash
} from "../../../../state/actions/exerciseEditorSite/subSets/editorExerciseCodeTemplatesActions";
import {openEditCodeTemplateDialog} from "../../../../state/actions/exerciseEditorSite/dialogs/changeCodeTemplateDialog/changeCodeTemplateDialogActions";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    codeTemplates: rootState.exerciseEditorState.editorExercise.codeTemplates,
    uiCodeTemplateFilesDisplayedHash: rootState.exerciseEditorState.uiCodeTemplateFilesDisplayedHash,
    numReleases: rootState.exerciseEditorState.numReleases,
    pLangs: rootState.pLangsState.pLangs
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  setUiCodeTemplateFilesDisplayedHash,
  openEditCodeTemplateDialog,
  setCodeTemplates,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class TemplatesPanelView extends React.Component<Props, any> {
  render(): JSX.Element {

    return (
      <div className="fh">

        <List divided relaxed>
          {
            this.props.codeTemplates.map(codeTemplate => {
              return (
                <List.Item key={codeTemplate.id}>
                  <List.Content>
                    <div className="flexed">

                      <div className={`mar-right ${this.props.numReleases === 0 ? 'clickable ' : 'div-disabled'}`}
                           onClick={() => {

                             if (this.props.numReleases > 0) return

                             this.props.setCodeTemplates(
                               this.props.codeTemplates.filter(p => p.id !== codeTemplate.id)
                             )

                           }}
                      >
                        <Icon name="remove"/>
                      </div>

                      <div className="mar-right">
                        {
                          this.props.pLangs.find(p => p.id === codeTemplate.pLangId) &&
                          this.props.pLangs.find(p => p.id === codeTemplate.pLangId).displayName
                        }
                      </div>

                      <div className="clickable mar-right"
                           onClick={() => {
                             const isDisplayed = this.props.uiCodeTemplateFilesDisplayedHash[codeTemplate.id]

                             this.props.setUiCodeTemplateFilesDisplayedHash({
                               ...this.props.uiCodeTemplateFilesDisplayedHash,
                               [codeTemplate.id]: !isDisplayed
                             })
                           }}
                      >
                        {
                          !this.props.uiCodeTemplateFilesDisplayedHash[codeTemplate.id] &&
                          <Icon name="chevron right"/>
                        }
                        {
                          this.props.uiCodeTemplateFilesDisplayedHash[codeTemplate.id] &&
                          <Icon name="chevron down"/>
                        }

                      </div>

                      <div className="clickable"
                           onClick={() => {
                             this.props.openEditCodeTemplateDialog(codeTemplate)
                           }}
                      >
                        <Icon name="write"/>
                      </div>
                    </div>

                    {
                      this.props.uiCodeTemplateFilesDisplayedHash[codeTemplate.id] &&
                      <div>
                        <CodeTemplateDetailsView codeTemplate={codeTemplate} allCodeTemplates={this.props.codeTemplates} />
                      </div>
                    }

                  </List.Content>
                </List.Item>
              )
            })
          }
        </List>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(TemplatesPanelView)
