/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {List, Icon} from "semantic-ui-react";
import {openEditEditorTestDialog} from "../../../../state/actions/exerciseEditorSite/dialogs/changeTestDialog/changeTestDialogActions";
import {setEditorTests} from "../../../../state/actions/exerciseEditorSite/subSets/editorExerciseTestsActions";


import {ExerciseTestForBackend} from "../../../../types/exerciseEditor";
import orderBy from 'lodash-es/orderBy'
import {getI18n} from "../../../../../i18n/i18nRoot";


//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string

  /**
   * true: manages submit tests,
   * false: the component manages normal tests
   */
  readonly isForSubmitTests: boolean
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    tests: rootState.exerciseEditorState.editorExercise.tests.filter(
      p => p.isSubmitTest === props.isForSubmitTests),
    oppositeTests: rootState.exerciseEditorState.editorExercise.tests.filter(
      p => p.isSubmitTest !== props.isForSubmitTests), //needed setting display index
    isForSubmitTests: props.isForSubmitTests,
    allTests: rootState.exerciseEditorState.editorExercise.tests,
    testTypes: rootState.testTypesState.testTypes,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  openEditEditorTestDialog,
  setEditorTests
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class ChangeTestsPanelView extends React.Component<Props, any> {


  /**
   * moves the test up (to the front)
   * @param {ExerciseTestForBackend} oldTest
   */
  moveTestUp(oldTest: ExerciseTestForBackend): void {
    //(>= 0, 0 is first, 1 second, ...)
    //because the backend could have done some wrong stuff... apply new indices to all tests...

    // noinspection TsLint
    let count = 0;

    const oldIndex = this.props.tests.indexOf(oldTest)

    if (oldIndex === 0) {
      //code below would throw because of oldIndex - 1
      //we might need to correct wrong indices here...
      const copyList = this.props.tests.map((test, i) => {
        return {
          ...test,
          displayIndex: i
        }
      })

      //this also sorts by display index
      this.props.setEditorTests(copyList.concat(this.props.oppositeTests))
      return;
    }

    const copyList = this.props.tests.map((test, i) => {

      //e.g. we want to swap 2, 3 (we have 0, 1, 2, 3, 4)
      //where 3 is the old index

      if (i < oldIndex - 1) {
        count++;
        return test //e.g. 0,1
      }
      else if (i === oldIndex - 1) { //test before the oldTest, count is 2
        return {
          ...test,
          displayIndex: count + 1 //will be 3, count is 2 after this
        }
      }
      else if (i === oldIndex) { //count is 2
        return {
          ...test,
          displayIndex: count++ //will be 3, count is 3 after this
        }
      }
      else { //count is 4
        count++
        return {
          ...test,
          displayIndex: count
        }
      }
    })

    //this also sorts by display index
    this.props.setEditorTests(copyList.concat(this.props.oppositeTests))
  }

  /**
   * moves the test up (to the front), it's then the first test
   * @param {ExerciseTestForBackend} oldTest
   */
  moveTestUpAbsolute(oldTest: ExerciseTestForBackend): void {
    //(>= 0, 0 is first, 1 second, ...)
    //because the backend could have done some wrong stuff... apply new indices to all tests...

    // noinspection TsLint
    let count = 1;
    const copyList = this.props.tests.map((test, i) => {

      return test.id === oldTest.id
        ? {
          ...test,
          displayIndex: 0
        }
        : {
          ...test,
          displayIndex: count++
        }
    })

    //sort else this function does not work!
    //this also sorts by display index
    this.props.setEditorTests(copyList.concat(this.props.oppositeTests))
  }

  /**
   * moves the test down (to the back)
   * @param {ExerciseTestForBackend} oldTest
   */
  moveTestDown(oldTest: ExerciseTestForBackend): void {
    //(>= 0, 0 is first, 1 second, ...)
    //because the backend could have done some wrong stuff... apply new indices to all tests...

    // noinspection TsLint
    let count = 0;
    const oldIndex = this.props.tests.indexOf(oldTest)

    if (oldIndex === this.props.tests.length - 1) {
      //code below would throw because of oldIndex - 1
      //we might need to correct wrong indices here...
      const copyList = this.props.tests.map((test, i) => {
        return {
          ...test,
          displayIndex: i
        }
      })
      //sort else this oldIndex would be wrong
      //this also sorts by display index
      this.props.setEditorTests(copyList.concat(this.props.oppositeTests))
      return;
    }

    const copyList = this.props.tests.map((test, i) => {

      //e.g. we want to swap 2, 3 (we have 0, 1, 2, 3, 4)
      //where 2 is the old index
      if (i <= oldIndex - 1) {
        count++;
        return test //e.g. 0,1
      }
      else if (i === oldIndex) { //count is 2
        return {
          ...test,
          displayIndex: ++count //will be 3, count is 3 after this
        }
      }
      else if (i === oldIndex + 1) { //test after the oldTest, count is 3
        return {
          ...test,
          displayIndex: count - 1 //will be 2, count is 3 after this
        }
      }
      else { //count is 4
        count++
        return {
          ...test,
          displayIndex: count
        }
      }
    })

    //sort else this oldIndex would be wrong
    //this also sorts by display index
    this.props.setEditorTests(copyList.concat(this.props.oppositeTests))
  }

  /**
   * moves the test down (to the back), it's then the last test
   * @param {ExerciseTestForBackend} oldTest
   */
  moveTestDownAbsolute(oldTest: ExerciseTestForBackend): void {
    //(>= 0, 0 is first, 1 second, ...)
    //because the backend could have done some wrong stuff... apply new indices to all tests...

    // noinspection TsLint
    let count = 0;
    const copyList = this.props.tests.map((test, i) => {

      return test.id === oldTest.id
        ? {
          ...test,
          displayIndex: this.props.tests.length - 1
        }
        : {
          ...test,
          displayIndex: count++
        }
    })

    //sort else this function does not work!
    //this also sorts by display index
    this.props.setEditorTests(copyList.concat(this.props.oppositeTests))
  }

  render(): JSX.Element {

    const list: ReadonlyArray<ExerciseTestForBackend> = orderBy(this.props.tests, p => p.displayIndex)

    return (
      <div>

        <List divided relaxed>
          {
            list.map(test => {
              return (
                <List.Item key={test.id} className="flexed-i-no-wrap has-hover-visible-child">
                  <List.Icon name='remove' size='large' verticalAlign='middle' className="clickable v-centered-i"
                             onClick={() => {
                               this.props.setEditorTests(this.props.allTests.filter(p => p.id !== test.id))
                             }}
                  />
                  <List.Content className="flex-stretch" style={{flex: '1 1'}}>
                    <List.Header>
                      {
                        test.displayName
                      }
                    </List.Header>
                    <List.Description style={{color: 'rgba(0,0,0,0.5)'}}>
                      {
                        this.props.testTypes.find(p => p.id === test.testTypeId) &&
                        this.props.testTypes.find(p => p.id === test.testTypeId).displayName
                      }
                    </List.Description>
                  </List.Content>

                  <div className="v-centered mar-right flexed">
                    <div className="flexed mar-right hover-child">
                      <Icon name="angle down" className="clickable" onClick={() => {
                        this.moveTestDown(test)
                      }}/>
                      <Icon name="angle double down" className="clickable" onClick={() => {
                        this.moveTestDownAbsolute(test)
                      }}/>
                      <Icon name="angle up" className="clickable" onClick={() => {
                        this.moveTestUp(test)
                      }}/>
                      <Icon name="angle double up" className="clickable" onClick={() => {
                        this.moveTestUpAbsolute(test)
                      }}/>
                    </div>

                    <div className="clickable" onClick={() => {
                      this.props.openEditEditorTestDialog(test)
                    }}
                    >
                      <Icon name="write"/>
                      <span>
                      {
                        getI18n(this.props.langId,'Change')
                      }
                      </span>
                    </div>
                  </div>
                </List.Item>
              )
            })
          }
        </List>


      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ChangeTestsPanelView)
