/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import {Icon} from "semantic-ui-react";
import {SimpleVDivider} from "../../helpers/simpleVDivider";
import CommonHeaderBarContentWrapper from '../../commonHeaderBarContentWrapper'
import {toJSONAndReplaceIds} from "../../../helpers/jsonTransformer";
import {InputOnChangeEvent} from "../../../types/reactEvents";
import {readFileAsText} from "../../../helpers/fileReader";
import Logger from "../../../helpers/logger";
import {
  EditorExerciseForBackend, EditorExerciseFromBackend, ExerciseFromBackendWithData,
  ExerciseTestFromBackend,
  TestAssetFromBackendWithData
} from "../../../types/exerciseEditor";
import {setEditorExercise} from "../../../state/actions/exerciseEditorSite/exerciseEditorSiteActions";
import {
  createEditorExerciseAsync, lockEditorExercisePermanentlyAsync, updateEditorExerciseAsync
} from "../../../state/actions/exerciseEditorSite/exerciseEditorCrudActions";
import history from '../../../routerHistory'
import * as constants from "../../../constants";
import {getExerciseEditorEditToLink} from "../../../constants";
import {addCloseWindowEvent, clearCloseWindowEvent} from "../../../helpers/eventsHelper";
import {Prompt} from "react-router";
import {freeAssetUrlsAndObjects} from "../../../helpers/assetHelper";
import {getNextFrontendId} from "../../../helpers/frontendIdHelper";
import {notifyError, notifyInfo, notifySuccess} from "../../../helpers/notificationHelper";
import * as mousetrap from "mousetrap";


import {getEditorExercise, getEditorExerciseWithData} from "../../../communicationLayer/exerciseEditorLayer";
import {askDialog, errorDialog} from "../../../helpers/dialogHelper";
import {mergeEditorExerciseTestAssetContents} from "../../../helpers/convertersAndTransformers";
import {getI18n, getRawI18n} from "../../../../i18n/i18nRoot";
import {ErrorHelper} from '../../../helpers/errorHelper'
import {HelpPopup} from '../../helpers/helpPopup'

const fileSaver = require('file-saver');

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    editorExercise: rootState.exerciseEditorState.editorExercise,
    oldGroupId: rootState.exerciseEditorState.oldGroupId,
    groups: rootState.exerciseEditorState.groups,
    savedNewExercise: rootState.exerciseEditorState.savedNewExercise,

    isLoading: rootState.exerciseEditorState.isLoading
      || rootState.testTypesState.isLoading
      || rootState.pLangsState.isLoading
      || rootState.manageTagsState.isLoading
      || rootState.manageGroupExercisesState.isLoading,

    isHeaderBarDisabled: rootState.exerciseEditorState.isLoading || rootState.exerciseEditorState.isChangeEditorSettingsDialogDisplayed || rootState.exerciseEditorState.isChangeCodeTemplateDialogDisplayed || rootState.exerciseEditorState.isChangeSelectedTagsDialogDisplayed || rootState.exerciseEditorState.isChangeTestDialogDisplayed,
    langId: rootState.i18nState.langId,

    isNoDialogDisplayed: !rootState.exerciseEditorState.isChangeTestDialogDisplayed
      && !rootState.exerciseEditorState.isChangeSelectedTagsDialogDisplayed
      && !rootState.exerciseEditorState.isChangeCodeTemplateDialogDisplayed
      && !rootState.exerciseEditorState.isChangeEditorSettingsDialogDisplayed
    ,

    currentUserId: rootState.userDataSettingsSate.userData !== null
                   ? rootState.userDataSettingsSate.userData.id
                   : null,

    comesFromOwnExercisesSite: rootState.exerciseEditorState.comesFromOwnExercisesSite,
    isLoggingOut: rootState.siteHeaderBarState.isLoggingOut,
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setEditorExercise,
                                                                        createEditorExerciseAsync,
                                                                        updateEditorExerciseAsync,
                                                                        lockEditorExercisePermanentlyAsync,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


const importJsonFileExerciseId = 'exerciseEditor-file-import'

class HeaderBar extends React.Component<Props, any> {

  htmlSaveButton: HTMLDivElement | undefined

  componentDidMount(): void {
    addCloseWindowEvent()
    mousetrap.bindGlobal(["ctrl+s", "meta+s"], this.simulateSaveButtonPress)
  }

  componentWillUnmount(): void {
    this.clearHandlers()
  }

  clearHandlers(): void {
    clearCloseWindowEvent()
    freeAssetUrlsAndObjects()
    mousetrap.unbind(["ctrl+s", "meta+s"])
  }

  simulateSaveButtonPress = async (e: ExtendedKeyboardEvent | null) => {

    if (this.props.editorExercise.exerciseProperties.isPermanentlyLocked) {
      Logger.log('exercise is locked, cannot save')
      return
    }

    if (!this.htmlSaveButton) {
      Logger.log('htmlSaveButton was undefined')
      return
    }
    const editorTextArea = document.querySelector(':focus') as HTMLTextAreaElement

    if (!editorTextArea) {
      //then the editor is probably up to date...
      Logger.log('we assume the editor has no focus and thus is up to date')
    }

    if (editorTextArea)
      editorTextArea.blur() //this should blur the editor before saving... (and update the store files)

    await this.onSaveKeyPress(e)

    if (editorTextArea)
      editorTextArea.focus()
  }

  onSaveKeyPress = async (e: ExtendedKeyboardEvent | null) => {

    if (e) {
      e.preventDefault()
    }

    if (this.props.isNoDialogDisplayed === false) {
      return
    }

    if (this.props.editorExercise.id <= 0) {
      //creating (backend uses ids > 0

      // noinspection TsLint
      let created = false

      //this catches and returns always true/false
      created = (await this.props.createEditorExerciseAsync(this.props.editorExercise)) as any as boolean

      if (!created) {
        notifyError(getI18n(this.props.langId, 'Could not save exercise'))
        return
      }

      //because we only get the new id back (intentionally)
      //navigate to the load exercise editor site

      //actually we don't need to get the state because we await in the method which re populates the props
      //but to be sure
      if (this.props.savedNewExercise) { //state.getState().exerciseEditorState.savedNewExercise //works but wrong typing
        const newExerciseId = this.props.editorExercise.id //state.getState().exerciseEditorState.editorExercise.id
        this.clearHandlers()

        history.push(getExerciseEditorEditToLink(newExerciseId, this.props.comesFromOwnExercisesSite))
      }


    } else {
      //updating

      const success = await this.props.updateEditorExerciseAsync(this.props.editorExercise)

      if (!success) {
        notifyError(getI18n(this.props.langId, 'Could not update exercise'))
        return
      }

      notifySuccess(getI18n(this.props.langId, 'Updated exercise'))

    }
  }


  render(): JSX.Element {

    const oldGroupRole = this.props.groups.find(p => p.id === this.props.oldGroupId)

    // tslint:disable-next-line
    let isAllowedToLock = false


    if (this.props.editorExercise.creatorId !== null && this.props.currentUserId === this.props.editorExercise.creatorId) {
      isAllowedToLock = true
    }

    if (!oldGroupRole) {
      Logger.warn('could not get user role for current (old) user group')
    } else {

      if (oldGroupRole.groupRole.canLockExercisesPermanently) {
        isAllowedToLock = true
      }
    }

    const leftArea = (<div>

      <Prompt
        when={this.props.savedNewExercise === false && !this.props.isLoggingOut}
        message={getI18n(this.props.langId,
                         'If you have unsaved changes then they will be discarded. Do you want to leave the page anyway?'
        )}/>

      {
        !this.props.editorExercise.exerciseProperties.isPermanentlyLocked &&
        <div className="inline-flex">
          <div className={`clickable ${this.props.editorExercise.id <= 0
                                       ? 'ok-colored'
                                       : 'orange-colored'}`}
               ref={(r) => this.htmlSaveButton = r}
               onClick={async () => {
                 this.onSaveKeyPress(null)
               }}
          >
            <Icon name={this.props.editorExercise.id <= 0
                        ? 'asterisk'
                        : 'save'}/>
            <span>
            {this.props.editorExercise.id <= 0 && getI18n(this.props.langId, 'Create exercise')}
              {this.props.editorExercise.id > 0 && getI18n(this.props.langId, 'Change exercise')}
          </span>
          </div>


          <SimpleVDivider/>
        </div>
      }

      <div className="clickable" onClick={() => {

        //we already have a prompt before leaving the page
        //or use replace?
        history.push(
          this.props.comesFromOwnExercisesSite
          ? constants.ownExercisesLinkPath
          : constants.groupExercisesLinkPath
        )

      }}>
        <Icon name="reply"/>
        <span>
          {getI18n(this.props.langId, 'Leave')}
        </span>
      </div>

    </div>)

    const rightArea = (<div>
      <div className="clickable"
           onClick={async () => {


             //when we create a new exercise (not copying) then the test asset contents are not loaded...
             // noinspection TsLint
             let exerciseToExport: ExerciseFromBackendWithData | EditorExerciseFromBackend = this.props.editorExercise

             if (this.props.editorExercise.id > 0) {

               notifyInfo(getI18n(this.props.langId, 'Now loading test assets (before exporting)...'))

               try {
                 //load the full exercise but only take the test asset contents
                 const exerciseWithTestContents = await getEditorExerciseWithData(this.props.editorExercise.id)


                 exerciseToExport = mergeEditorExerciseTestAssetContents({
                                                                           ...this.props.editorExercise,
                                                                           numReleases: 0, // set to 0 because when we import we create a new exercise... and we can only create templates if we have no releases
                                                                           tests: this.props.editorExercise.tests,
                                                                         },
                                                                         exerciseWithTestContents.tests
                 )


               } catch (err) {
                 ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Server error', 'Could not load test assets')

                 return
               }
             } else {
               //when we create a new exercise the test assets are still stored at frontend (in memory)
             }

             const value = toJSONAndReplaceIds(exerciseToExport)

             // noinspection TsLint
             let fileName = `${this.props.editorExercise.exerciseProperties.displayName.trim()}.json`

             if (!fileName) {
               fileName = "exercise.json"
             }

             const blob = new Blob([value], {type: "application/json;charset=utf-8"});
             fileSaver.saveAs(blob, fileName)
           }}
      >
        <Icon name="upload"/>
        <span>
          {getI18n(this.props.langId, 'Export exercise')}
        </span>
      </div>

      {(this.props.editorExercise.id <= 0 && false) && <SimpleVDivider/>}

      {
        //only allow import for new exercises (for old ones there is the copy func)
        //------import DISABLED ---
        // because now we store asset files (exercise description, test assets) in the file system and we need to
        //create them one by one again but they may already exists because this is probably an exported exercise...
        //... maybe we can implement this in a future version again...
        (this.props.editorExercise.id <= 0 && false) && <div className="clickable"
                                                  onClick={() => {

                                                    const inputEl = document.getElementById(importJsonFileExerciseId)
                                                    inputEl.click()
                                                  }}
        >
          <Icon name="download"/>
          <span>
            {getI18n(this.props.langId, 'Import exercise')}
          </span>
          <input id={importJsonFileExerciseId} type="file" className="collapsed" accept="application/json"
                 onChange={async (e: InputOnChangeEvent) => {
                   const files: FileList = e.currentTarget.files

                   // noinspection TsLint
                   let text: string = null
                   try {
                     text = await readFileAsText(files[0])
                   } catch (err) {
                     Logger.error(`exercise editor: ${err.message}`)
                   }

                   if (!text) {
                     Logger.warn(`exercise editor: something went wrong reading the file`)
                     return
                   }

                   //TODO must this is somehow insecure....
                   //check if the parsed has every needed props
                   // noinspection TsLint
                   let parsed = JSON.parse(text) as EditorExerciseForBackend

                   parsed = {
                     ...parsed,
                     id: getNextFrontendId()
                   }

                   this.props.setEditorExercise(parsed)
                 }}
          />
        </div>}


      {
        this.props.editorExercise.id > 0 && this.props.oldGroupId > 0 &&
        <div className="flexed">

          <SimpleVDivider/>

          {
            this.props.editorExercise.exerciseProperties.isPermanentlyLocked === false &&
            <div className={!isAllowedToLock
                            ? 'div-disabled'
                            : 'clickable'} onClick={async () => {

              if (this.props.editorExercise.exerciseProperties.isPermanentlyLocked) return

              const userGroup = this.props.groups.find(p => p.id === this.props.oldGroupId)

              if (!userGroup) {
                Logger.warn(`could not find old user group with id ${this.props.oldGroupId}`)
                return
              }

              const shouldLock = await askDialog(getI18n(this.props.langId, "Lock exercise permanently"),
                                                 `${getI18n(this.props.langId,
                                                            "Are you sure you want to lock the current exercise permanently? You cannot longer update or delete the exercise or any dependent entities. This means pending releases (automatic) will not be released, releases for this exercise cannot be created, updated or deleted. Pending assessments for solutions to this exercise will not run and tutors will not be able to update the assessments. Any changes you made to this exercise (in this view) before clicking this button will not be applied because the exercise then then locked! The exercise will be visible in the group (if not already) and you will not be longer the owner. The exercise will be locked in the group: "
                                                 )} '${userGroup.displayName}'. `, this.props.langId
              )

              if (!shouldLock) {
                return
              }

              const shouldLock2 = await askDialog(getI18n(this.props.langId, "Lock exercise permanently"),
                                                  getI18n(this.props.langId,
                                                          "Are you really sure you want to lock this exercise? This cannot be reverted!"
                                                  ), this.props.langId
              )

              if (!shouldLock2) {
                return
              }

              this.props.lockEditorExercisePermanentlyAsync(this.props.editorExercise)

            }}>
              <Icon className={this.props.editorExercise.exerciseProperties.isPermanentlyLocked
                               ? 'warning-colored'
                               : ''} name="lock"/>

              <span>
              {
                getI18n(this.props.langId, "Lock exercise permanently")
              }
            </span>

            </div>
          }

          {
            this.props.editorExercise.exerciseProperties.isPermanentlyLocked &&
            <div>
              <Icon name="lock"/>
              <span>
                {
                  getI18n(this.props.langId, "Exercise is lock")
                }
              </span>
            </div>
          }

          <HelpPopup className="mar-left-half" defaultText={getI18n(this.props.langId,
                                                                    "You cannot longer update or delete the exercise or any dependent entities. This means pending releases (automatic) will not be released, releases for this exercise cannot be created,updated or deleted. Pending assessments for solutions to this exercise will not run and tutors will not be able to update the assessments."
          )} wide/>

        </div>
      }

    </div>)


    return (<CommonHeaderBarContentWrapper leftArea={leftArea} rightArea={rightArea}
                                           disabled={this.props.isLoading || this.props.isHeaderBarDisabled}/>)
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(HeaderBar)
