import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from '../../../../state/reducers'
import {Icon} from 'semantic-ui-react'
import {browser_isSafari, defaultTaskDescriptionFontSizeInPx, tempPrintDivId} from '../../../../constants'
import {setGlobalTaskDescriptionFontSizeInPx} from '../../../../state/actions/globalUiActions'
import Logger from '../../../../helpers/logger'
import {SimpleVDivider} from '../../../helpers/simpleVDivider'
import {PrintHelper} from '../../../../helpers/printHelper'

export interface MyProps {
  //readonly test: string
  readonly canPrint?: boolean

  /**
   * the content to print
   */
  readonly printId: string
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props,
    taskDescriptionFontSizeInPx: rootState.globalUiState.taskDescriptionFontSizeInPx,
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setGlobalTaskDescriptionFontSizeInPx,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


class ZoomPreviewPanelActions extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div className="flexed-no-wrap">

        <Icon name="zoom out" className="clickable" onClick={() => {
          this.props.setGlobalTaskDescriptionFontSizeInPx(this.props.taskDescriptionFontSizeInPx - 1)

        }}/>
        <Icon name="undo" className="clickable" onClick={() => {
          this.props.setGlobalTaskDescriptionFontSizeInPx(defaultTaskDescriptionFontSizeInPx)

        }}/>
        <Icon name="zoom in" className="clickable" onClick={() => {

          this.props.setGlobalTaskDescriptionFontSizeInPx(this.props.taskDescriptionFontSizeInPx + 1)
        }}/>

        {
          this.props.canPrint &&
            <SimpleVDivider/>
        }

        {
          this.props.canPrint &&
          <Icon name="print" className="clickable" onClick={() => {

            const el = document.getElementById(`${this.props.printId}`)

            if (!el) {
              Logger.warn('found none or too many printable contents')
              return
            }

            PrintHelper.printContent(el.innerHTML)

          }}/>
        }
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ZoomPreviewPanelActions)
