/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {Divider} from "semantic-ui-react";
import SaveButton from "../../../buttons/saveButton";
import CancelButton from "../../../buttons/cancelButton";
import ChangeGroupRoleView from './changeGroupRoleView'
import {cancelEditGroupRole} from "../../../../state/actions/groupRolesSite/groupRolesSiteActions";
import Spinner from "../../../helpers/spinner";
import {
  createGroupRolesAsync, updateGroupRolesAsync
} from "../../../../state/actions/groupRolesSite/dialog/groupRoleDialogCrudActions";
import {getI18n} from "../../../../../i18n/i18nRoot";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    dialogState: rootState.groupRolesChangeDialogState,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  createGroupRolesAsync,
  updateGroupRolesAsync,
  cancelEditGroupRole

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class changeGroupRoleDialog extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div>

        {
          this.props.dialogState.isBusy &&
          <Spinner/>
        }

        <div className="view-padding">
          <div>
            <div className="view-caption">
              <h2>
                {
                  this.props.dialogState.isCreating &&
                  getI18n(this.props.langId,'Create group role')
                }
                {
                  !this.props.dialogState.isCreating &&
                  getI18n(this.props.langId,'Change group role')
                }
              </h2>
            </div>
          </div>

          <div className="view-content">
            <ChangeGroupRoleView/>

            <Divider/>

            <div className="flexed">
              <SaveButton className="mar-right" disabled={!this.props.dialogState.isValid}
                          onClick={() => {

                            if (this.props.dialogState.isCreating) {
                              this.props.createGroupRolesAsync(this.props.dialogState.groupRole)
                            } else {
                              this.props.updateGroupRolesAsync(this.props.dialogState.groupRole)
                            }
                            //setIsChangeDialogDisplayed and reset gets called in reducer
                          }}
              />
              <CancelButton onClick={() => {
                this.props.cancelEditGroupRole()
              }}/>
            </div>

          </div>
        </div>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(changeGroupRoleDialog)
