/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import {Card, Icon, Checkbox, Divider} from "semantic-ui-react";
import {openEditGroupRoleDialog} from "../../../state/actions/groupRolesSite/groupRolesSiteActions";
import {deleteGroupRoleAsync} from "../../../state/actions/groupRolesSite/groupRolesCrudActions";
import {askDialog} from "../../../helpers/dialogHelper";


import {getI18n} from "../../../../i18n/i18nRoot";
import {HelpPopup} from '../../helpers/helpPopup'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    groupRoles: rootState.groupRolesState.groupRoles,
    userSystemRole: rootState.userDataSettingsSate.userData
                    && rootState.userDataSettingsSate.userData.systemRole
                    ? rootState.userDataSettingsSate.userData.systemRole
                    : null,
    defaultGroupRoleId: rootState.globalSystemSettingsState.systemSettings.defaultGroupRoleId,
    defaultGroupCreatorGroupRoleId: rootState.globalSystemSettingsState.systemSettings.defaultGroupCreatorGroupRoleId,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        editGroupRole: openEditGroupRoleDialog,
                                                                        deleteGroupRoleAsync
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class GroupRolesCardsView extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div>
        <div className="material-table--caption-line-padding">
          <Card.Group>
            {
              this.props.groupRoles.map((groupRole, index) => {
                return (<Card key={groupRole.id}>
                    <Card.Content>
                      <Card.Header>
                        <span>{groupRole.displayName}</span>
                      </Card.Header>

                      <Card.Description>
                        <Checkbox label={getI18n(this.props.langId, 'Can create exercises')}
                                  disabled
                                  checked={groupRole.canCreateExercises}/>
                        <Checkbox label={getI18n(this.props.langId, 'Can change exercises')}
                                  disabled
                                  checked={groupRole.canChangeExercises}/>
                        <Checkbox label={getI18n(this.props.langId, 'Can delete exercises')}
                                  disabled
                                  checked={groupRole.canDeleteExercises}/>
                        <Checkbox label={getI18n(this.props.langId, 'Can lock exercises permanently')}
                                  disabled
                                  checked={groupRole.canLockExercisesPermanently}/>

                        <Divider/>

                        <Checkbox label={getI18n(this.props.langId, 'Can manage exercise releases')}
                                  disabled checked={groupRole.canManageExerciseReleases}/>
                        <Checkbox label={getI18n(this.props.langId, 'Can see exercises from other members')} disabled
                                  checked={groupRole.canSeeExercisesFromOthersInGroup}/>

                        <Checkbox label={getI18n(this.props.langId, 'Can assess exercises from group')}
                                  disabled
                                  checked={groupRole.canAssessExercises}/>


                        <Divider/>

                        <Checkbox label={getI18n(this.props.langId, 'Can add users to group')}
                                  disabled
                                  checked={groupRole.canAddUserToGroup}/>
                        <Checkbox label={getI18n(this.props.langId, 'Can remove members from group')}
                                  disabled
                                  checked={groupRole.canRemoveMemberFromGroup}/>
                        <Checkbox label={getI18n(this.props.langId, 'Can change other members group role')}
                                  disabled
                                  checked={groupRole.canChangeOtherMembersRole}/>

                        <Checkbox label={getI18n(this.props.langId, 'Can see other members')}
                                  disabled
                                  checked={groupRole.canSeeOtherMembers}/>
                        <Checkbox label={getI18n(this.props.langId, 'Can change group data')}
                                  disabled
                                  checked={groupRole.canChangeGroupData}/>

                      </Card.Description>

                    </Card.Content>

                    <Card.Content extra>
                      <div className="two-flexed">
                        {
                          (this.props.userSystemRole && this.props.userSystemRole.canChangeRoles) &&
                          <div className="clickable"
                               onClick={() => {
                                 this.props.editGroupRole(groupRole)
                               }}
                          >
                            <Icon name="write"/>
                            <span>
                            {
                              getI18n(this.props.langId, 'Change group role')
                            }
                          </span>
                          </div>
                        }

                        {
                          (groupRole.id === this.props.defaultGroupRoleId || groupRole.id === this.props.defaultGroupCreatorGroupRoleId) &&
                          <div>
                            <HelpPopup defaultText={getI18n(this.props.langId,
                                                            "This is a default group role and cannot be deleted"
                            )}/>
                          </div>
                        }

                        {
                          (this.props.userSystemRole && this.props.userSystemRole.canDeleteRoles) && groupRole.id !== this.props.defaultGroupRoleId && groupRole.id !== this.props.defaultGroupCreatorGroupRoleId &&
                          <div className="clickable"
                               onClick={async () => {

                                 const shouldRemove = await askDialog(getI18n(this.props.langId, 'Delete group role'),
                                                                      `${getI18n(this.props.langId,
                                                                                 'Are you sure you want to delete the group role?'
                                                                      )} (${groupRole.displayName})`, this.props.langId
                                 )

                                 if (shouldRemove) {
                                   this.props.deleteGroupRoleAsync(groupRole)
                                 }

                               }}
                          >
                            <Icon name="remove"/>
                            <span>
                            {
                              getI18n(this.props.langId, 'Delete group role')
                            }
                          </span>
                          </div>
                        }
                      </div>
                    </Card.Content>
                  </Card>
                )
              })
            }
          </Card.Group>
        </div>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(GroupRolesCardsView)
