/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import SinglePanelSiteWrapper from '../../singlePanelSiteWrapper'
import GroupRolesCardsView from './groupRolesCardsView'
import MaterialSearchInput from '../../material/materialSearchInput'
import Spinner from "../../helpers/spinner";
import ChangeGroupRoleDialog from './dialogs/changeGroupRoleDialog'
import {getGroupRolesAsync} from "../../../state/actions/groupRolesSite/groupRolesCrudActions";
import {loadGroupRolesSite} from "../../../state/actions/groupRolesSite/groupRolesSiteActions";


import {getI18n} from "../../../../i18n/i18nRoot";
import {errorDialog} from '../../../helpers/dialogHelper'
import Logger from '../../../helpers/logger'
import {ErrorHelper} from '../../../helpers/errorHelper'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    isChangeDialogDisplayed: rootState.groupRolesState.isChangeDialogDisplayed,
    isLoading: rootState.groupRolesState.isLoading,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  loadGroupRolesSite
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class GroupRolesSite extends React.Component<Props, any> {

  async componentWillMount(): Promise<void> {
    try {
      await this.props.loadGroupRolesSite()
    } catch(err) {
      ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Error', 'Could not load site data')
    }
  }

  render(): JSX.Element {
    return (
      <SinglePanelSiteWrapper>

        {
          this.props.isLoading &&
          <Spinner/>
        }

        {
          !this.props.isChangeDialogDisplayed &&
          <div className="view-padding">
            <div className="view-head-line">
              <div className="view-caption">
                <h1>
                  {
                    getI18n(this.props.langId,'Group roles')
                  }
                </h1>
              </div>
              <div className="view-options">
                {/*<MaterialSearchInput/>*/}
              </div>
            </div>

            <div className="view-content">
              <GroupRolesCardsView/>
            </div>
          </div>
        }

        {
          !this.props.isLoading && this.props.isChangeDialogDisplayed &&
          <ChangeGroupRoleDialog/>
        }

      </SinglePanelSiteWrapper>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(GroupRolesSite)
