/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import {Icon, Table} from "semantic-ui-react";
import * as constants from '../../../constants'
import {setSelectedGroupId} from "../../../state/actions/singleGroupSite/singleGroupSiteActions";
import {
  openEditGroupDialog,
  quitGroup,
  setActivePaginationPage,
  setPaginationPageSize,
  setSortByKey
} from "../../../state/actions/groupsSite/groupsSiteActions";
import {Link} from "react-router-dom";
import {UserGroupBase, UserGroupWithExerciseCountFromBackend} from "../../../types/group";
import {isCurrentSortingColumnPaginated, rotateSorting} from "../../../helpers/tablesHelper";
import {copyToClipboard} from "../../../helpers/clipboard";
import {askDialog} from "../../../helpers/dialogHelper";
import {deleteUserGroupAsync} from "../../../state/actions/groupsSite/groupsCrudActions";
import {HelpPopup} from "../../helpers/helpPopup";
import {getI18n} from "../../../../i18n/i18nRoot";
import PaginationTableFooter from '../../helpers/paginationTableFooter'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string

}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    groups: rootState.groupsState.groups,
    sortByKey: rootState.groupsState.pagination.orderByProp,
    sortDirection: rootState.groupsState.pagination.isOrderByDesc,
    pagination: rootState.groupsState.pagination,

    systemRole: rootState.userDataSettingsSate.userData
                && rootState.userDataSettingsSate.userData.systemRole
                ? rootState.userDataSettingsSate.userData.systemRole
                : null,

    defaultUserGroupId: rootState.globalSystemSettingsState.systemSettings.defaultUserGroupId,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setSelectedGroupId,
                                                                        openEditGroupDialog,
                                                                        quitGroup,
                                                                        setSortByKey,
                                                                        deleteUserGroupAsync,
                                                                        setPaginationPageSize,
                                                                        setActivePaginationPage
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


class GroupsTableView extends React.Component<Props, any> {

  render(): JSX.Element {


    return (
      <div>
        <Table sortable selectable padded striped singleLine className="material-table">

          <Table.Header>
            <Table.Row>
              <Table.HeaderCell>
                <span>
                  {
                    getI18n(this.props.langId, 'Open')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<UserGroupWithExerciseCountFromBackend>(this.props.sortByKey, 'displayName',
                                                                 this.props.sortDirection
                  )
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('displayName',
                                                rotateSorting<UserGroupWithExerciseCountFromBackend>(this.props.sortDirection,
                                                                             this.props.sortByKey,
                                                                             'displayName'
                                                )
                        )
                      }}
                >

                  {
                    getI18n(this.props.langId, 'Name')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<UserGroupWithExerciseCountFromBackend>(this.props.sortByKey, 'email',
                                                                 this.props.sortDirection
                  )
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('email',
                                                rotateSorting<UserGroupWithExerciseCountFromBackend>(this.props.sortDirection,
                                                                             this.props.sortByKey,
                                                                             'email'
                                                )
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Email')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span>
                    {
                      getI18n(this.props.langId, 'Change')
                    }
                  </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span>
                  {
                    getI18n(this.props.langId, 'Leave')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span className="mar-right-half">
                    {
                      getI18n(this.props.langId, 'Delete')
                    }
                  </span>

              </Table.HeaderCell>
            </Table.Row>
          </Table.Header>

          <Table.Body>
            {
              this.props.groups.map((group, index) => {
                return (
                  <Table.Row key={group.id}>
                    <Table.Cell>
                      {
                        (group.groupRole && group.groupRole.canSeeOtherMembers) &&
                        <div className="clickable">
                          <Link to={constants.getSingleGroupLinkPath(group.id)}>
                            <Icon name="share"/>
                          </Link>
                        </div>
                      }

                    </Table.Cell>

                    <Table.Cell>
                          <span>
                            {group.displayName}
                          </span>
                    </Table.Cell>

                    <Table.Cell>
                      {
                        group.email &&
                        <Icon name="copy" className="clickable"
                              onClick={() => {
                                copyToClipboard(group.email)
                              }}
                        />
                      }
                      <span>
                          {group.email}
                        </span>
                    </Table.Cell>
                    <Table.Cell>

                      {
                        group.groupRole && group.groupRole.canChangeGroupData &&
                        <div className="clickable"
                             onClick={() => {
                               this.props.openEditGroupDialog(group)
                             }}
                        >
                          <Icon name="write"/>
                        </div>
                      }
                    </Table.Cell>

                    <Table.Cell>

                      {
                        <div className="clickable" onClick={async () => {

                          const shouldRemove = await askDialog(getI18n(this.props.langId, 'Leave group'),
                                                               `${getI18n(this.props.langId,
                                                                          'Are you sure you want to leave the group?'
                                                               )} (${group.displayName})`, this.props.langId
                          )

                          if (shouldRemove) {
                            this.props.quitGroup(group)
                          }


                        }}>
                          <Icon name="remove" className="clickable"/>
                        </div>
                      }

                    </Table.Cell>

                    <Table.Cell>

                      {
                        group.id === this.props.defaultUserGroupId &&
                          <HelpPopup className="warning-colored" defaultText={getI18n(this.props.langId, "This is the default group and cannot be deleted")} />
                      }
                      {
                        group.id !== this.props.defaultUserGroupId && group.numExercisesInGroup > 0 &&
                        <HelpPopup defaultText={getI18n(this.props.langId, "Only groups without exercises can be deleted")} />
                      }
                      {
                        this.props.systemRole && this.props.systemRole.canDeleteGroups && group.id !== this.props.defaultUserGroupId && group.numExercisesInGroup === 0 &&
                        <div className="clickable"
                             onClick={async () => {

                               const shouldRemove = await askDialog(getI18n(this.props.langId, 'Delete group'),
                                                                    `${getI18n(this.props.langId,
                                                                               'Are you sure you want to delete the group? This will also delete all exercises (and connected releases) managed by this group.'
                                                                    )} (${group.displayName})`, this.props.langId
                               )

                               if (shouldRemove) {
                                 this.props.deleteUserGroupAsync(group.id)
                               }


                             }}>
                          <Icon name="trash" className="clickable"/>
                        </div>
                      }


                    </Table.Cell>

                  </Table.Row>
                )
              })
            }
          </Table.Body>
          <PaginationTableFooter
            pagination={this.props.pagination}
            setPaginationPageSize={this.props.setPaginationPageSize}
            setActivePaginationPage={this.props.setActivePaginationPage}
          />
        </Table>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(GroupsTableView)
