/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import SinglePanelSiteWrapper from '../../singlePanelSiteWrapper'
import MaterialSearchInput from '../../material/materialSearchInput'
import GroupsTableView from './groupsListView'
import Spinner from "../../helpers/spinner";
import ChangeGroupDataDialog from './dialogs/changeGroupDataDialog'
import {loadGroupsSite, setSearchText} from "../../../state/actions/groupsSite/groupsSiteActions";


import {getI18n} from "../../../../i18n/i18nRoot";
import {InputOnChangeEvent} from '../../../types/reactEvents'
import {ErrorHelper} from '../../../helpers/errorHelper'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    isLoading: rootState.groupsState.isLoading,
    isChangeDialogDisplayed: rootState.groupsState.isChangeDialogDisplayed,
    langId: rootState.i18nState.langId,
    pagination: rootState.groupsState.pagination
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        loadGroupsSite,
                                                                        setSearchText
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class GroupsSite extends React.Component<Props, any> {


  async componentWillMount(): Promise<void> {
    try {
        await this.props.loadGroupsSite()
    } catch(err) {
      ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Error', 'Could not load site data')
    }
  }

  render(): JSX.Element {
    return (
      <div>
        <SinglePanelSiteWrapper>

          {
            this.props.isLoading &&
            <Spinner/>
          }

          {
            !this.props.isChangeDialogDisplayed &&
            <div className="view-padding">
              <div className="view-head-line">
                <div className="view-caption">
                  <h1>
                    {
                      getI18n(this.props.langId, 'Groups')
                    }
                  </h1>
                </div>
                <div className="view-options">
                  <MaterialSearchInput
                    value={this.props.pagination.searchText}
                    onChange={(e: InputOnChangeEvent) => {
                      this.props.setSearchText(e.currentTarget.value)
                    }}
                  />
                </div>
              </div>

              <div className="view-content">
                  <GroupsTableView/>
              </div>
            </div>
          }


          {
            !this.props.isLoading && this.props.isChangeDialogDisplayed &&
            <ChangeGroupDataDialog/>
          }


        </SinglePanelSiteWrapper>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(GroupsSite)
