/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import ChangeUserDataView from './changeUserDataView'
import {Divider, Icon} from "semantic-ui-react";
import SaveButton from "../../../buttons/saveButton";
import CancelButton from "../../../buttons/cancelButton";
import {setIsChangeUserDataDialogDisplayed} from "../../../../state/actions/manageActivatedUsersSite/manageActivatedUsersSiteActions";
import {resetChangeActivatedUserDataDialog} from "../../../../state/actions/manageActivatedUsersSite/dialogs/changeActivatedUserDataDialogActions";
import {updateActivatedUserDataAsync} from "../../../../state/actions/manageActivatedUsersSite/dialogs/changeActivatedUserDataCrudActions";
import Spinner from "../../../helpers/spinner";


import {getI18n} from "../../../../../i18n/i18nRoot";
import Logger from '../../../../helpers/logger'
import {errorDialog} from '../../../../helpers/dialogHelper'
import {getPersonalDataDownloadLinkPart} from '../../../../constants'
import {HelpPopup} from '../../../helpers/helpPopup'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    isBusy: rootState.changeActivatedUsersDataDialogState.isBusy,
    isValid: rootState.changeActivatedUsersDataDialogState.isValid,
    userData: rootState.changeActivatedUsersDataDialogState.userData,
    systemRoles: rootState.manageActivatedUsersState.systemRoles,
    currentUserId: rootState.userDataSettingsSate.userData.id,
    currentUserSystemRole: rootState.userDataSettingsSate.userData
                           && rootState.userDataSettingsSate.userData.systemRole
                           ? rootState.userDataSettingsSate.userData.systemRole
                           : null,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setIsChangeUserDataDialogDisplayed,
                                                                        resetChangeActivatedUserDataDialog,
                                                                        updateActivatedUserDataAsync
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class ChangeUserDataDialog extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div>

        {
          this.props.isBusy &&
          <Spinner/>
        }

        <div className="view-padding">
          <div className="view-head-line">

            <div className="view-caption">
              <h2>
                {
                  getI18n(this.props.langId, 'Change user data')
                }
              </h2>
            </div>

            <div className="view-options">

              {
                this.props.currentUserSystemRole && this.props.currentUserSystemRole.canDownloadPersonalDataFromOthers &&
                <div className="flexed view-head-line-centered mar-left">
                  <a href={`${getPersonalDataDownloadLinkPart(this.props.userData.id)}`} download target="_blank">
                    <div className="clickable">
                      <Icon name="download"/>
                      <span>
                      {
                        getI18n(this.props.langId, 'Download personal data')
                      }
                    </span>
                    </div>
                  </a>
                </div>
              }

            </div>
          </div>

          <div className="view-content">
            <ChangeUserDataView>
              <div className="flexed">
                <SaveButton className="mar-right" disabled={!this.props.isValid}
                            onClick={() => {
                              this.props.updateActivatedUserDataAsync(this.props.userData,
                                                                      this.props.userData.systemRole
                              )
                            }}
                />
                <CancelButton onClick={() => {
                  this.props.setIsChangeUserDataDialogDisplayed(false)
                  this.props.resetChangeActivatedUserDataDialog()
                }}/>
              </div>
            </ChangeUserDataView>
          </div>
        </div>


      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ChangeUserDataDialog)
