/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import SinglePanelSiteWrapper from '../../singlePanelSiteWrapper'
import ActivatedUsersListView from './activatedUsersListView'
import {RouteComponentProps} from 'react-router'
import MaterialSearchInput from '../../material/materialSearchInput'
import Spinner from "../../helpers/spinner";
import ChangeUserDataDialog from './dialogs/changeUserDataDialog'
import {
  loadManageActivatedUsersSite,
  setSearchText
} from "../../../state/actions/manageActivatedUsersSite/manageActivatedUsersSiteActions";


import {getI18n} from "../../../../i18n/i18nRoot";
import {InputOnChangeEvent, OnKeyPressEvent} from '../../../types/reactEvents'
import {errorDialog} from '../../../helpers/dialogHelper'
import Logger from '../../../helpers/logger'
import {ErrorHelper} from '../../../helpers/errorHelper'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    isLoading: rootState.manageActivatedUsersState.isLoading,
    isChangeUserDataDialogDisplayed: rootState.manageActivatedUsersState.isChangeUserDataDialogDisplayed,
    langId: rootState.i18nState.langId,
    pagination: rootState.manageActivatedUsersState.pagination
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        loadManageActivatedUsersSite,
                                                                        setSearchText,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class ManageUsersSite extends React.Component<Props & RouteComponentProps<any>, any> {

  async componentWillMount(): Promise<void> {
    try {
      await this.props.loadManageActivatedUsersSite()
    } catch(err) {
      ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Error', 'Could not load site data')
    }
  }

  render(): JSX.Element {

    return (
      <SinglePanelSiteWrapper>

        {
          this.props.isLoading &&
          <Spinner/>
        }

        {
          !this.props.isChangeUserDataDialogDisplayed &&
          <div className="view-padding">
            <div className="view-head-line">
              <div className="view-caption">
                <h1>
                  {
                    getI18n(this.props.langId, 'Manage activated users')
                  }
                </h1>
              </div>
              <div className="view-options">

                <MaterialSearchInput
                  value={this.props.pagination.searchText}
                  onChange={(e: InputOnChangeEvent) => {
                    this.props.setSearchText(e.currentTarget.value)
                  }}
                />
              </div>
            </div>

            <div className="view-content">
              <ActivatedUsersListView/>
            </div>
          </div>
        }

        {
          !this.props.isLoading && this.props.isChangeUserDataDialogDisplayed &&
          <ChangeUserDataDialog/>
        }

      </SinglePanelSiteWrapper>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ManageUsersSite)
