/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import {CustomProjectPreviewFullBase} from "../../../types/customProjects";
import {Icon, Table} from "semantic-ui-react";


import {isCurrentSortingColumnPaginated, rotateSorting} from "../../../helpers/tablesHelper";
import {Link} from "react-router-dom";
import {displayDateAndTime} from "../../../helpers/convertersAndTransformers";
import {
  deleteOwnCustomProjectAsync, setActivePaginationPage, setPaginationPageSize,
  setSortByKey
} from "../../../state/actions/manageCustomProjectsSite/manageCustomProjectsSiteActions";
import {askDialog} from "../../../helpers/dialogHelper";
import {getCustomProjectToLinkPath} from "../../../constants";
import {getI18n} from "../../../../i18n/i18nRoot";
import PaginationTableFooter from '../../helpers/paginationTableFooter'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    customProjectPreviews: rootState.manageCustomProjectsSiteState.customProjectPreviews,

    pLangs: rootState.pLangsState.pLangs,

    sortByKey: rootState.manageCustomProjectsSiteState.pagination.orderByProp,
    sortDirection: rootState.manageCustomProjectsSiteState.pagination.isOrderByDesc,
    pagination: rootState.manageCustomProjectsSiteState.pagination,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setSortByKey,
                                                                        deleteOwnCustomProjectAsync,
                                                                        setActivePaginationPage,
                                                                        setPaginationPageSize
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class listView extends React.Component<Props, any> {
  render(): JSX.Element {

    const list: ReadonlyArray<CustomProjectPreviewFullBase> = this.props.customProjectPreviews

    return (
      <div>
        <Table selectable sortable padded striped singleLine className="material-table">

          <Table.Header>
            <Table.Row>

              <Table.HeaderCell>
                {
                  getI18n(this.props.langId, 'Open')
                }
              </Table.HeaderCell>

              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<CustomProjectPreviewFullBase>(this.props.sortByKey,
                                                                                'displayName',
                                                                                this.props.sortDirection
                  )
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('displayName',
                                                rotateSorting<CustomProjectPreviewFullBase>(this.props.sortDirection,
                                                                                            this.props.sortByKey,
                                                                                            'displayName'
                                                )
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Name')
                  }
                </span>
              </Table.HeaderCell>

              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<CustomProjectPreviewFullBase>(this.props.sortByKey,
                                                                                'lastEditorPLangId',
                                                                                this.props.sortDirection
                  )
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('lastEditorPLangId',
                                                rotateSorting<CustomProjectPreviewFullBase>(this.props.sortDirection,
                                                                                            this.props.sortByKey,
                                                                                            'lastEditorPLangId'
                                                )
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Programming language')
                  }
                </span>
              </Table.HeaderCell>

              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<CustomProjectPreviewFullBase>(this.props.sortByKey,
                                                                                'lastUpdatedAt',
                                                                                this.props.sortDirection
                  )
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('lastUpdatedAt',
                                                rotateSorting<CustomProjectPreviewFullBase>(this.props.sortDirection,
                                                                                            this.props.sortByKey,
                                                                                            'lastUpdatedAt'
                                                )
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Last updated at')
                  }
                </span>
              </Table.HeaderCell>

              <Table.HeaderCell>
                {
                  getI18n(this.props.langId, 'Delete')
                }
              </Table.HeaderCell>
            </Table.Row>
          </Table.Header>

          <Table.Body>
            {
              list.map((preview, index) => {
                return (
                  <Table.Row key={preview.id}>

                    <Table.Cell>
                      <Link to={getCustomProjectToLinkPath(preview.id)} className="clickable">
                        <Icon name="share"/>
                      </Link>
                    </Table.Cell>

                    <Table.Cell>
                      <span>
                        {
                          preview.displayName
                        }
                      </span>
                    </Table.Cell>

                    <Table.Cell>
                      <span>
                        {
                          this.props.pLangs.find(p => p.id === preview.lastEditorPLangId) &&
                          this.props.pLangs.find(p => p.id === preview.lastEditorPLangId).displayName
                        }
                      </span>
                    </Table.Cell>

                    <Table.Cell>
                      <span>
                        {
                          displayDateAndTime(preview.lastUpdatedAt, -1)
                        }
                      </span>
                    </Table.Cell>

                    <Table.Cell>

                      <div className="clickable"
                           onClick={async () => {

                             const shouldRemove = await askDialog(getI18n(this.props.langId, 'Delete custom project'),
                                                                  `${getI18n(this.props.langId,
                                                                             'Are you sure you want to delete your project?'
                                                                  )} (${preview.displayName})`, this.props.langId
                             )

                             if (shouldRemove) {
                               this.props.deleteOwnCustomProjectAsync(preview)
                             }

                           }}
                      >
                        <Icon name="trash"></Icon>
                      </div>

                    </Table.Cell>

                  </Table.Row>
                )
              })
            }
          </Table.Body>
          <PaginationTableFooter
            pagination={this.props.pagination}
            setActivePaginationPage={this.props.setActivePaginationPage}
            setPaginationPageSize={this.props.setPaginationPageSize}

          />

        </Table>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(listView)
