/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import {Icon} from "semantic-ui-react";
import {SimpleVDivider} from "../../helpers/simpleVDivider";
import CommonHeaderBarContentWrapper from '../../commonHeaderBarContentWrapper'
import {
  activateMultipleNewUserAsync,
  deleteMultipleNewUserAsync
} from "../../../state/actions/manageNewUsersSite/manageNewUserSiteCrudActions";


import {getI18n} from "../../../../i18n/i18nRoot";
import {askDialog} from '../../../helpers/dialogHelper'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    isLoading: rootState.manageNewUsersState.isLoading,
    newUsers: rootState.manageNewUsersState.newUsers,
    systemRole: rootState.userDataSettingsSate.userData
    && rootState.userDataSettingsSate.userData.systemRole
      ? rootState.userDataSettingsSate.userData.systemRole
      : null,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  activateMultipleNewUserAsync,
  deleteMultipleNewUserAsync
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class headerBar extends React.Component<Props, any> {


  render(): JSX.Element {

    const numSelected = this.props.newUsers.filter(p => p.uiIsSelected).length
    const isSomeUserSelected = numSelected > 0

    const leftArea = (
      <div>
        {
          (this.props.systemRole && this.props.systemRole.canManageNewUsers) &&
          <div className={isSomeUserSelected ? 'clickable' : 'div-disabled'}
               onClick={async () => {

                 const shouldActivate = await askDialog(getI18n(this.props.langId, "Activated selected user(s)"),
                   `${getI18n(this.props.langId, "Are you sure you want to activate the selected user(s)?")} (${numSelected})`, this.props.langId)

                 if (!shouldActivate) return

                   this.props.activateMultipleNewUserAsync(
                   this.props.newUsers
                     .filter(p => p.uiIsSelected)
                     .map(p => {
                       return {
                         userId: p.id,
                         systemRoleId: p.systemRoleId,
                       }
                     }))
               }}>
            <Icon name="checkmark"/>
            <span>
            {
              getI18n(this.props.langId,'Activate selected users')
            }
           </span>
          </div>
        }

        {
          (this.props.systemRole && this.props.systemRole.canManageNewUsers) &&
          <SimpleVDivider/>
        }

        {
          (this.props.systemRole && this.props.systemRole.canManageNewUsers) &&
          <div className={isSomeUserSelected ? 'clickable' : 'div-disabled'}
               onClick={async() => {

                 const shouldDelete = await askDialog(getI18n(this.props.langId, "Delete not activated user(s)"), getI18n(this.props.langId, "Are you sure you want to delete the not activated user(s)?"), this.props.langId)

                 if (!shouldDelete) return

                 this.props.deleteMultipleNewUserAsync(
                   this.props.newUsers.filter(p => p.uiIsSelected)
                 )
               }}
          >
            <Icon name="remove"/>
            <span>
            {
              getI18n(this.props.langId,'Delete selected users')
            }
           </span>
          </div>
        }

      </div>
    )


    return (
      <CommonHeaderBarContentWrapper leftArea={leftArea} disabled={this.props.isLoading}/>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(headerBar)
