/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import {NewUserFrontendExclusive} from "../../../types/users";
import {Checkbox, Table, Icon, Popup} from "semantic-ui-react";
import {CheckboxOnChangeEventData} from "../../../types/reactEvents";
import {copyToClipboard} from "../../../helpers/clipboard";
import {leftIconPopupOffset} from "../../../constants";
import {displayDateAndTime} from "../../../helpers/convertersAndTransformers";
import SelectOptionList, {SelectableOption} from "../../helpers/selectOptionList";
import {
  activateNewUserAsync,
  deleteNewUserAsync
} from "../../../state/actions/manageNewUsersSite/manageNewUserSiteCrudActions";
import {
  setAllNewUsersUiIsSelected, setNewUserSystemRole, setNewUserUiIsChangeSystemRolePopupDisplayed,
  setNewUserUiIsSelected
} from "../../../state/actions/manageNewUsersSite/manageNewUsersActions";
import {askDialog} from "../../../helpers/dialogHelper";


import {MyPopup} from "../../helpers/myPopup";
import {getI18n} from "../../../../i18n/i18nRoot";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly newUser: NewUserFrontendExclusive
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    newUser: props.newUser,
    systemRoles: rootState.manageNewUsersState.systemRoles,
    systemRole: rootState.userDataSettingsSate.userData
    && rootState.userDataSettingsSate.userData.systemRole
      ? rootState.userDataSettingsSate.userData.systemRole
      : null,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  setNewUserSystemRole,
  setNewUserUiIsChangeSystemRolePopupDisplayed,
  setNewUserUiIsSelected,
  setAllNewUsersUiIsSelected,
  activateNewUserAsync,
  deleteNewUserAsync,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class listViewItem extends React.Component<Props, any> {

  render(): JSX.Element {

    const newUser = this.props.newUser

    return (
      <Table.Row key={newUser.id}>
        <Table.Cell>
          <Checkbox checked={newUser.uiIsSelected}
                    onChange={(event: any, data: CheckboxOnChangeEventData) => {
                      return this.props.setNewUserUiIsSelected(newUser.id, data.checked)
                    }}/>
        </Table.Cell>

        <Table.Cell>
                      <span>
                        {newUser.lastName}
                      </span>
        </Table.Cell>

        <Table.Cell>
                      <span>
                        {newUser.firstName}
                      </span>
        </Table.Cell>
        <Table.Cell>
                      <span>
                        {newUser.token}
                      </span>
        </Table.Cell>
        <Table.Cell>
          <div>
            <Icon name="copy" className="clickable"
                  onClick={() => {
                    copyToClipboard(newUser.email)
                  }}
            />
            <span>
                          {newUser.email}
                        </span>
          </div>
        </Table.Cell>
        <Table.Cell>
          {
            (this.props.systemRole && this.props.systemRole.canManageNewUsers) &&
            <MyPopup
              trigger={
                <div className="clickable">
                  <Icon name="caret down" className="mar-right-half"/>
                  <span>{
                    newUser.systemRole ? newUser.systemRole.displayName : getI18n(this.props.langId,'None')
                  }</span>
                </div>
              }
              position="bottom left"
              on="click"
              onClose={() => this.props.setNewUserUiIsChangeSystemRolePopupDisplayed(newUser.id, false)}
              onOpen={() => this.props.setNewUserUiIsChangeSystemRolePopupDisplayed(newUser.id, true)}
              open={newUser.uiIsChangeSystemRolePopupDisplayed}
              content={
                <div>
                  <SelectOptionList
                    options={this.props.systemRoles}
                    defaultOption={{id: -1, displayName: getI18n(this.props.langId,'None')}}
                    optionClicked={(option: SelectableOption) => {

                      this.props.setNewUserSystemRole(newUser,
                        this.props.systemRoles.find(p => p.id === option.id))

                      this.props.setNewUserUiIsChangeSystemRolePopupDisplayed(newUser.id, false)
                    }}
                  ></SelectOptionList>
                </div>
              }
            />
          }

        </Table.Cell>
        <Table.Cell>
          <span>
            {
              displayDateAndTime(newUser.createdAt, -1)
            }
          </span>
        </Table.Cell>
        <Table.Cell>
          {
            (this.props.systemRole && this.props.systemRole.canManageNewUsers) &&
            <div className="clickable"
                 onClick={() =>
                   this.props.activateNewUserAsync({
                     userId: newUser.id,
                     systemRoleId: newUser.systemRoleId
                   })
                 }>
              <Icon name="checkmark"></Icon>
            </div>
          }


        </Table.Cell>
        <Table.Cell>
          {
            (this.props.systemRole && this.props.systemRole.canManageNewUsers) &&
            <div className="clickable"
                 onClick={async () => {

                   const shouldRemove = await askDialog(getI18n(this.props.langId,'Delete new user'),
                     `${getI18n(this.props.langId,'Are you sure you want to delete the new user?')} (${newUser.lastName}, ${newUser.firstName})`, this.props.langId)

                   if (shouldRemove) {
                     this.props.deleteNewUserAsync(newUser)
                   }

                 }}
            >
              <Icon name="remove"></Icon>
            </div>
          }
        </Table.Cell>
      </Table.Row>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(listViewItem)