/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";

import SinglePanelSiteWrapper from '../../singlePanelSiteWrapper'

import {RouteComponentProps} from 'react-router'

import MaterialSearchInput from '../../material/materialSearchInput'
import NewUsersListView from './newUsersListView'
import Spinner from "../../helpers/spinner";
import {loadManageNewUsersSite, setSearchText} from "../../../state/actions/manageNewUsersSite/manageNewUsersActions";


import {getI18n} from "../../../../i18n/i18nRoot";
import {InputOnChangeEvent} from '../../../types/reactEvents'
import {errorDialog} from '../../../helpers/dialogHelper'
import Logger from '../../../helpers/logger'
import {ErrorHelper} from '../../../helpers/errorHelper'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    isLoading: rootState.manageNewUsersState.isLoading,
    pagination: rootState.manageNewUsersState.pagination,
    langId: rootState.i18nState.langId
  }
}


const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        loadManageNewUsersSite,
                                                                        setSearchText
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class ManageUsersSite extends React.Component<Props & RouteComponentProps<any>, any> {

  async componentWillMount(): Promise<void> {
    try {
      await this.props.loadManageNewUsersSite()
    } catch(err) {
      ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Error', 'Could not load site data')
    }
  }


  render(): JSX.Element {
    return (
      <SinglePanelSiteWrapper>

        {
          this.props.isLoading &&
          <Spinner/>
        }

        <div className="view-padding">
          <div className="view-head-line">
            <div className="view-caption">
              <h1>
                {
                  getI18n(this.props.langId, 'Manage new users')
                }
              </h1>
            </div>
            <div className="view-options">
              <MaterialSearchInput
                value={this.props.pagination.searchText}
                onChange={(e: InputOnChangeEvent) => {
                  this.props.setSearchText(e.currentTarget.value)
                }}
              />
            </div>
          </div>

          <div className="view-content">
            <NewUsersListView/>
          </div>
        </div>

      </SinglePanelSiteWrapper>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ManageUsersSite)
