/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import {Checkbox, Table} from "semantic-ui-react";
import {
  setActivePaginationPage,
  setAllNewUsersUiIsSelected,
  setNewUserSystemRole,
  setNewUserUiIsChangeSystemRolePopupDisplayed,
  setNewUserUiIsSelected, setPaginationPageSize,
  setSortByKey
} from "../../../state/actions/manageNewUsersSite/manageNewUsersActions";
import {CheckboxOnChangeEventData} from "../../../types/reactEvents";
import {
  activateNewUserAsync,
  deleteNewUserAsync
} from "../../../state/actions/manageNewUsersSite/manageNewUserSiteCrudActions";
import {NewUserFrontendExclusive} from "../../../types/users";
import {isCurrentSortingColumnPaginated, rotateSorting} from "../../../helpers/tablesHelper";
import ListViewItem from './listViewItem'
import {getI18n} from "../../../../i18n/i18nRoot";
import PaginationTableFooter from '../../helpers/paginationTableFooter'
import {MyPopup} from '../../helpers/myPopup'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    systemRoles: rootState.manageNewUsersState.systemRoles,
    newUsers: rootState.manageNewUsersState.newUsers,
    sortByKey: rootState.manageNewUsersState.pagination.orderByProp,
    sortDirection: rootState.manageNewUsersState.pagination.isOrderByDesc,
    langId: rootState.i18nState.langId,
    pagination: rootState.manageNewUsersState.pagination,
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setNewUserSystemRole,
                                                                        setNewUserUiIsChangeSystemRolePopupDisplayed,
                                                                        setNewUserUiIsSelected,
                                                                        setAllNewUsersUiIsSelected,
                                                                        activateNewUserAsync,
                                                                        deleteNewUserAsync,
                                                                        setSortByKey,
                                                                        setActivePaginationPage,
                                                                        setPaginationPageSize,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class NewUsersListView extends React.Component<Props, any> {
  render(): JSX.Element {

    const list: ReadonlyArray<NewUserFrontendExclusive> = this.props.newUsers

    return (
      <div>
        <Table sortable selectable padded striped singleLine className="material-table">
          <Table.Header>
            <Table.Row>
              <Table.HeaderCell>
                <MyPopup
                  trigger={
                    <Checkbox
                      checked={
                        this.props.newUsers.every(p => p.uiIsSelected) && this.props.newUsers.length > 0
                      }
                      onChange={(e: any, data: CheckboxOnChangeEventData) => {
                        this.props.setAllNewUsersUiIsSelected(data.checked)
                      }}
                    />
                  }
                  on="hover"
                  content={getI18n(this.props.langId, "Will only select all items on the current page")}
                />

              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<NewUserFrontendExclusive>(this.props.sortByKey, 'lastName',
                                                                            this.props.sortDirection
                  )
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('lastName',
                                                rotateSorting(this.props.sortDirection, this.props.sortByKey,
                                                              'lastName'
                                                )
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Last name')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<NewUserFrontendExclusive>(this.props.sortByKey, 'firstName',
                                                                            this.props.sortDirection
                  )
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('firstName',
                                                rotateSorting(this.props.sortDirection, this.props.sortByKey,
                                                              'firstName'
                                                )
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'First name')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<NewUserFrontendExclusive>(this.props.sortByKey, 'token',
                                                                            this.props.sortDirection
                  )
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('token',
                                                rotateSorting(this.props.sortDirection, this.props.sortByKey, 'token')
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Token')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<NewUserFrontendExclusive>(this.props.sortByKey, 'email',
                                                                            this.props.sortDirection
                  )
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('email',
                                                rotateSorting(this.props.sortDirection, this.props.sortByKey, 'email')
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Email')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span>
                  {
                    getI18n(this.props.langId, 'System role')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<NewUserFrontendExclusive>(this.props.sortByKey, 'createdAt',
                                                                            this.props.sortDirection
                  )
                }
              >
                <span  className="clickable"
                       onClick={() => {
                         this.props.setSortByKey('createdAt',
                                                 rotateSorting(this.props.sortDirection, this.props.sortByKey, 'createdAt')
                         )
                       }}
                >
                  {
                    getI18n(this.props.langId, 'Created at')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span>
                  {
                    getI18n(this.props.langId, 'Activate')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span>
                  {
                    getI18n(this.props.langId, 'Delete')
                  }
                </span>
              </Table.HeaderCell>
            </Table.Row>
          </Table.Header>

          <Table.Body>
            {
              list.map((newUser, index) => {
                return (
                  <ListViewItem key={newUser.id} newUser={newUser}/>
                )
              })
            }

          </Table.Body>
          <PaginationTableFooter
            pagination={this.props.pagination}
            setActivePaginationPage={this.props.setActivePaginationPage}
            setPaginationPageSize={this.props.setPaginationPageSize}
          />
        </Table>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(NewUsersListView)