/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import SinglePanelSiteWrapper from '../../singlePanelSiteWrapper'
import MaterialSearchInput from '../../material/materialSearchInput'
import TableView from '../manageOwnOrGroupExerciseComponents/listView'
import Spinner from "../../helpers/spinner";
import {
  deleteOwnExerciseAsync, getOwnExercisesAsync,
  loadOwnExercisesSite,
  setActivePaginationPage, setIsLoading,
  setPaginationPageSize,
  setSearchText,
  setSortByKey
} from "../../../state/actions/manageOwnExercisesSite/ownExercisesSiteActions";
import {EditableExercisePreviewFromBackend} from "../../../types/exercisePreview";


import TagFilterPanel from '../../tagComponents/tagsFilterPanel'
import {getI18n} from "../../../../i18n/i18nRoot";
import {InputOnChangeEvent} from '../../../types/reactEvents'
import {ErrorHelper} from '../../../helpers/errorHelper'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    tags: rootState.manageTagsState.tags,
    exercisePreviews: rootState.manageOwnExercisesState.ownExercises,
    isLoading: rootState.manageOwnExercisesState.isLoading
      || rootState.manageTagsState.isLoading,
    sortByKey: rootState.manageOwnExercisesState.pagination.orderByProp,
    sortDirection: rootState.manageOwnExercisesState.pagination.isOrderByDesc,
    pagination: rootState.manageOwnExercisesState.pagination,

    selectedFilterTagsIds: rootState.tagsFilterState.selectedFilterTagsIds,
    selectedNegativeFilterTagsIds: rootState.tagsFilterState.selectedNegativeFilterTagsIds,

    isTagsFilterDisplayed: rootState.manageOwnExercisesState.isTagsFilterDisplayed,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        loadOwnExercisesSite,
                                                                        setSortByKey,
                                                                        deleteOwnExerciseAsync,
                                                                        setSearchText,
                                                                        setActivePaginationPage,
                                                                        setPaginationPageSize,
                                                                        getOwnExercisesAsync,
                                                                        setIsLoading,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class OwnExercisesSite extends React.Component<Props, any> {

  async componentWillMount(): Promise<void> {
    try {
        await this.props.loadOwnExercisesSite()
    } catch(err) {
      ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Error', 'Could not load site data')
    }
  }

  render(): JSX.Element {


    const previewsList = this.props.exercisePreviews
    // sortAndFilterExercisePreviews(this.props.exercisePreviews, this.props.sortDirection,
    //                                                  this.props.sortByKey, this.props.selectedFilterTagsIds,
    //                                                  this.props.selectedNegativeFilterTagsIds
    // )

    return (

      <div className="multiple-single-panels">

        {
          this.props.isTagsFilterDisplayed &&
          <TagFilterPanel
            isLoading={this.props.isLoading}
            allTags={this.props.tags}
            onTagsUpdated={this.props.getOwnExercisesAsync}
          />
        }

        <SinglePanelSiteWrapper>

          {
            this.props.isLoading &&
            <Spinner/>
          }

          <div className="view-padding">
            <div className="view-head-line">
              <div className="view-caption">
                <h1>
                  {
                    getI18n(this.props.langId, 'Own exercises')
                  }
                </h1>
              </div>
              <div className="view-options">
                <MaterialSearchInput
                  value={this.props.pagination.searchText}
                  onChange={(e: InputOnChangeEvent) => {
                    this.props.setSearchText(e.currentTarget.value)
                  }}
                />
              </div>
            </div>

            <div className="view-content">
                <TableView
                  isLoading={false}
                  groupId={null}
                  exercisePreviews={previewsList} comesFromOwnExercisesSite={true}
                  sortByKey={this.props.sortByKey}
                  sortDirection={this.props.sortDirection}
                  setSortByKey={this.props.setSortByKey}
                  pagination={this.props.pagination}
                  setActivePaginationPage={this.props.setActivePaginationPage}
                  setPaginationPageSize={this.props.setPaginationPageSize}
                  onDeleteExercise={(exercisePreview: EditableExercisePreviewFromBackend) => {
                    this.props.deleteOwnExerciseAsync(exercisePreview.id)
                  }}
                  currentUserGroupRole={null}
                  setIsLoading={this.props.setIsLoading}
                />
            </div>
          </div>

        </SinglePanelSiteWrapper>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(OwnExercisesSite)
