/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import ListView from './listView'
import MaterialSearchInput from '../../../material/materialSearchInput'
import {RouteComponentProps, withRouter} from "react-router";
import {parseBoolean, parseId} from "../../../../helpers/convertersAndTransformers";
import {
  loadExerciseReleasesSite,
  resetExerciseReleasesSite,
  setComesFromOwnExercisesSite,
  setSearchText,
  setSelectedExerciseId
} from "../../../../state/actions/ownOrGroupExerciseActions/releasesSiteActions/exerciseReleasesSiteActions";
import SinglePanelSiteWrapper from '../../../singlePanelSiteWrapper'
import Spinner from "../../../helpers/spinner";
import {Link} from "react-router-dom";
import * as constants from '../../../../constants'
import ZoomedExerciseReleaseDialog from './dialogs/zoomedExerciseReleaseDialog/zoomedExerciseReleaseDialog'
import ChangeExerciseReleaseDialog from './dialogs/changeExerciseReleaseDialog/changeExerciseReleaseDialog'
import {BackArrow} from "../../../helpers/backArrow";


import {getI18n} from "../../../../../i18n/i18nRoot";
import {InputOnChangeEvent} from '../../../../types/reactEvents'
import Logger from '../../../../helpers/logger'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    exercises: rootState.releaseSiteState.exercise,
    selectedExerciseId: rootState.releaseSiteState.selectedExerciseId,
    isLoading: rootState.releaseSiteState.isLoading
      || rootState.pLangsState.isLoading,
    comesFromOwnExercisesSite: rootState.releaseSiteState.comesFromOwnExercisesSite,
    isZoomedExerciseReleaseDisplayed: rootState.releaseSiteState.isZoomedExerciseReleaseDisplayed,
    isChangeDialogDisplayed: rootState.releaseSiteState.isChangeDialogDisplayed,
    langId: rootState.i18nState.langId,
    pagination: rootState.releaseSiteState.pagination
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setSelectedExerciseId,
                                                                        loadExerciseReleasesSite,
                                                                        setComesFromOwnExercisesSite,
                                                                        resetExerciseReleasesSite,
                                                                        setSearchText,
                                                                      }, dispatch)

interface MatchedProps {
  readonly exerciseId: string
  readonly comesFromOwnExercisesSite: string
}

const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


//TODO enforce group role ... currently we don't know the group role ... maybe ask server exercise.userGroupId --> member --> role
class ReleasesView extends React.Component<Props & RouteComponentProps<MatchedProps>, any> {

  componentWillMount(): void {
    const exerciseId = parseId(this.props.match.params.exerciseId)
    const comesFromOwnExercisesSite = parseBoolean(this.props.match.params.comesFromOwnExercisesSite)

    //don't do this in the load action because we need to set som props before
    this.props.resetExerciseReleasesSite(false)

    this.props.setComesFromOwnExercisesSite(comesFromOwnExercisesSite)


    if (exerciseId === null || comesFromOwnExercisesSite === null) {
      Logger.error('could not parse exercise id or comesFromOwnExercisesSite')
      return
    }

    this.props.setSelectedExerciseId(exerciseId)
    //exercises.length === 0 this is possible if a user opens this page directly in the browser
    this.props.loadExerciseReleasesSite(exerciseId, comesFromOwnExercisesSite)

  }

  render(): JSX.Element {
    const selectedExercise = this.props.exercises

    return (
      <SinglePanelSiteWrapper>

        {
          this.props.isLoading &&
          <Spinner/>
        }

        {
          !this.props.isZoomedExerciseReleaseDisplayed &&
          !this.props.isChangeDialogDisplayed &&
          <div className="view-padding">
            <div className="view-head-line">
              <div className="view-caption">
                <div className="mar-right back-arrow">
                  <Link
                    to={this.props.comesFromOwnExercisesSite
                        ? constants.ownExercisesLinkPath
                        : constants.groupExercisesLinkPath}>
                    <BackArrow/>
                  </Link>
                </div>
                <h2 className="clear-margin">
                  {
                    selectedExercise &&
                    `${getI18n(this.props.langId, 'Releases for')} ${selectedExercise.displayName}`
                  }
                </h2>
              </div>
              <div className="view-options">
                <MaterialSearchInput
                  value={this.props.pagination.searchText}
                  onChange={(e: InputOnChangeEvent) => {
                    this.props.setSearchText(e.currentTarget.value)
                  }}
                />
              </div>
            </div>

            <div className="view-content">
              <ListView/>
            </div>
          </div>
        }

        {
          this.props.isZoomedExerciseReleaseDisplayed &&
          <ZoomedExerciseReleaseDialog exercise={selectedExercise}/>
        }

        {
          this.props.isChangeDialogDisplayed &&
          <ChangeExerciseReleaseDialog/>
        }

      </SinglePanelSiteWrapper>
    )
  }
}

export default withRouter(connect(mapStateToProps, mapDispatchToProps)(ReleasesView))
