/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import {Icon, Form, Dropdown, DropdownItemProps} from "semantic-ui-react";
import MaterialSelectWrapper from '../../material/materialSelectWrapper'
import {SyntheticEvent} from "react";
import {setLangId, setTheme} from "../../../state/actions/settingsSite/userSettingsActions";


import {availableEditorThemeNames, InternalCodeEditorThemes} from "../../../helpers/themes";
import {getI18n} from "../../../../i18n/i18nRoot";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    isLoading: rootState.userDataSettingsSate.isLoadingMiscSettings || rootState.langsState.isLoading,
    langs: rootState.langsState.langs,
    userSettingsCopy: rootState.userDataSettingsSate.userSettingsCopy,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  setTheme,
  setLangId
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;



class MiscSettingsView extends React.Component<Props, any> {


  componentWillMount(): void {
    //pre select first lang else we get -1 or something...

    if (this.props.langs.length > 0) {
      this.props.setLangId(this.props.langs[0].id)
    }

    if (!this.props.userSettingsCopy.theme) {
      this.props.setTheme(availableEditorThemeNames.dark)
    }

  }

  render(): JSX.Element {

    return (
      <div>
        <Form.Group widths='equal'>
          <Form.Field>
            <label>
              {
                getI18n(this.props.langId,'Language')
              }
            </label>
            <MaterialSelectWrapper
              value={this.props.userSettingsCopy.langId}
            >
              <select value={this.props.userSettingsCopy.langId}
                      disabled //TODO remove when we have real option
                      onChange={(e: SyntheticEvent<HTMLSelectElement>) => {
                        const id = parseInt(e.currentTarget.value, 10)
                        this.props.setLangId(id)
                      }}
              >
                {
                  this.props.langs.map(lang => {
                    return (
                      <option key={lang.id} value={lang.id}>{lang.displayName}</option>
                    )
                  })
                }
              </select>
            </MaterialSelectWrapper>
          </Form.Field>

          <Form.Field>
            <label>
              {
                getI18n(this.props.langId,'Design')
              }
            </label>

            <MaterialSelectWrapper
              value={this.props.userSettingsCopy.theme ? this.props.userSettingsCopy.theme : ''}
            >
              <select value={this.props.userSettingsCopy.theme ? this.props.userSettingsCopy.theme : ''}
                      disabled //TODO remove when we have real theme option
                      onChange={(e: SyntheticEvent<HTMLSelectElement>) => {
                        const themeKey = e.currentTarget.value
                        this.props.setTheme(themeKey)
                      }}
              >
                <optgroup label={getI18n(this.props.langId,'Light')}>

                  <option key={InternalCodeEditorThemes.light} value={InternalCodeEditorThemes.light}>
                    {
                      getI18n(this.props.langId,'Default light')
                    }
                  </option>

                </optgroup>

                <optgroup label={getI18n(this.props.langId,'Dark')}>

                  <option key={InternalCodeEditorThemes.dark} value={InternalCodeEditorThemes.dark}>
                    {
                      getI18n(this.props.langId,'Default dark')
                    }
                  </option>

                </optgroup>

              </select>
            </MaterialSelectWrapper>

          </Form.Field>

        </Form.Group>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(MiscSettingsView)
