/**
 * Created by janis dähne.
 */

import * as React from "react";
import {bindActionCreators, Dispatch} from "redux";
import {connect} from "react-redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import SinglePanelSiteWrapper from '../../singlePanelSiteWrapper'
import {Form, Icon} from "semantic-ui-react";
import MaterialInput from '../../material/materialInput'
import {SimpleVDivider} from "../../helpers/simpleVDivider";
import Spinner from "../../helpers/spinner";
import {ChangeState} from "../../../state/reducers/settingsSite/settingsSiteReducers";
import MiscSettingsChangeView from './miscSettingsChangeView'
import {setUserSettingsChangeStateAction} from "../../../state/actions/settingsSite/userSettingsSiteActions";
import {resetUserSettings} from "../../../state/actions/settingsSite/userSettingsActions";
import {updateUserSettingsAsync} from "../../../state/actions/settingsSite/userSettingsSiteCrudActions";
import {LangFullBase} from "../../../types/languages";


import {getEditorInternalThemeName} from "../../../helpers/themes";
import {getI18n} from "../../../../i18n/i18nRoot";
import {debugOriginUrl} from '../../../../debug'
import {HelpPopup} from '../../helpers/helpPopup'
import {getPersonalDataDownloadLinkPart} from '../../../constants'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    isLoading: rootState.userDataSettingsSate.isLoadingMiscSettings || rootState.langsState.isLoading,
    changeState: rootState.userDataSettingsSate.userSettingsChangeState,
    userSettings: rootState.userDataSettingsSate.userData.settings
                  ? rootState.userDataSettingsSate.userData.settings
                  : null,
    langs: rootState.langsState.langs,
    userSettingsCopy: rootState.userDataSettingsSate.userSettingsCopy,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  setUserSettingsChangeStateAction,
  resetUserSettings,
  updateUserSettingsAsync
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


class MiscSettingsView extends React.Component<Props, any> {
  render(): JSX.Element {

    // noinspection TsLint
    let themeName = getI18n(this.props.langId,'Not selected')

    if (this.props.userSettings) {
      // noinspection TsLint
      themeName = getEditorInternalThemeName(this.props.userSettings.theme)
    }


    // noinspection TsLint
    let lang: LangFullBase = null

    if (this.props.userSettings) {
      lang = this.props.langs.find(p => p.id === this.props.userSettings.langId)
    }

    const langName = lang
                     ? lang.displayName
                     : getI18n(this.props.langId,'Automatic')

    return (<SinglePanelSiteWrapper>

        {this.props.isLoading && <Spinner/>}

        <div className="view-padding">

          <div className="view-head-line">

            <div className="view-caption">
              <h2>
                {getI18n(this.props.langId,'Miscellaneous')}
              </h2>
            </div>

            <div className="view-options">
              {this.props.changeState === ChangeState.notActive && <div className="flexed view-head-line-centered">
                <div className="clickable"
                     onClick={() => this.props.setUserSettingsChangeStateAction(ChangeState.active)}
                >
                  <Icon name="write"/>
                  <span>
                      {getI18n(this.props.langId,'Change')}
                    </span>
                </div>

                {/*
                  not fully implemented...
                  <SimpleVDivider/>

                  <div className="clickable"
                       onClick={() => {
                         this.props.resetUserSettings()
                       }}
                  >
                    <Icon name="undo"/>
                    <span>Zurücksetzen</span>
                  </div>
                  */}
              </div>}

              {this.props.changeState === ChangeState.active && <div className="flexed view-head-line-centered">
                <div className="clickable"
                     onClick={() => {
                       this.props.updateUserSettingsAsync({
                         id: this.props.userSettingsCopy.id,
                         langId: this.props.userSettingsCopy.langId,
                         theme: this.props.userSettingsCopy.theme
                       })

                       this.props.setUserSettingsChangeStateAction(ChangeState.notActive)
                     }}
                >
                  <Icon name="save"/>
                  <span>
                      {getI18n(this.props.langId,'Save')}
                    </span>
                </div>

                <SimpleVDivider/>

                <div className="clickable"
                     onClick={() => {
                       this.props.resetUserSettings()
                       this.props.setUserSettingsChangeStateAction(ChangeState.notActive)
                     }}
                >
                  <Icon name="remove"/>
                  <span>
                      {getI18n(this.props.langId,'Discard')}
                    </span>
                </div>
              </div>}
            </div>

            <div className="flexed view-head-line-centered mar-left">
              <a href={`${getPersonalDataDownloadLinkPart(null)}`} download target="_blank">
                <div className="clickable">
                  <Icon name="download"/>
                  <span>
                      {
                        getI18n(this.props.langId, 'Download personal data')
                      }
                    </span>
                </div>
              </a>
              <HelpPopup className="mar-left-half" defaultText={getI18n(this.props.langId, "This will download all your personal data that we store into a zip file. This can take some time!")} />
            </div>

          </div>

          <div className="view-content">
            <div className="ui form">

              {this.props.changeState === ChangeState.notActive && <Form.Group widths='equal'>
                <Form.Field>
                  <label>
                    {getI18n(this.props.langId,'Language')}
                  </label>
                  <MaterialInput
                    value={langName}
                    disabled
                  />
                </Form.Field>

                <Form.Field>
                  <label>
                    {getI18n(this.props.langId,'Design')}
                  </label>
                  <MaterialInput
                    // value={themeName}
                    //TODO remove when we have real theme option
                    value={getI18n(this.props.langId,'Default light')}
                    disabled
                  />
                </Form.Field>

              </Form.Group>}

              {this.props.changeState === ChangeState.active && <MiscSettingsChangeView></MiscSettingsChangeView>}

            </div>
          </div>

        </div>

      </SinglePanelSiteWrapper>)
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(MiscSettingsView)
