/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../../state/reducers";
import Spinner from "../../../../helpers/spinner";
import AddUserView from './addUserView'
import {setIsAddUsersDialogDisplayed} from "../../../../../state/actions/singleGroupSite/singleGroupSiteActions";
import {BackArrow} from "../../../../helpers/backArrow";
import {
  cancelAddUsersDialog,
  loadAddUsersDialog,
  resetAddUsersDialog,
  setSearchText
} from "../../../../../state/actions/singleGroupSite/dialogs/addUsers/addUsersDialogActions";
import MaterialSearchInput from '../../../../material/materialSearchInput'


import {getI18n} from "../../../../../../i18n/i18nRoot";
import {InputOnChangeEvent} from '../../../../../types/reactEvents'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    isBusy: rootState.addUsersDialogState.isBusy,
    isValid: rootState.addUsersDialogState.isValid,
    currentUserId: rootState.userDataSettingsSate.userData.id,
    members: rootState.singleGroupState.members,
    langId: rootState.i18nState.langId,
    pagination: rootState.addUsersDialogState.pagination
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setIsAddUsersDialogDisplayed,
                                                                        loadAddUsersDialog,
                                                                        cancelAddUsersDialog,
                                                                        setSearchText,
                                                                        resetAddUsersDialog
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class addUserDialog extends React.Component<Props, any> {

  componentWillMount(): void {
    this.props.loadAddUsersDialog()
  }

  render(): JSX.Element {
    return (
      <div>
        {
          this.props.isBusy &&
          <Spinner/>
        }

        <div className="view-padding">
          <div className="view-head-line">
            <div className="view-caption">
              <div className="mar-right back-arrow">
                <div className="clickable"
                     onClick={() => {
                       this.props.cancelAddUsersDialog()
                     }}
                >
                  <BackArrow/>
                </div>
              </div>

              <h2 className="clear-margin">

                {
                  getI18n(this.props.langId, 'Add users to group')
                }
              </h2>
            </div>
            <div className="view-options">
              <MaterialSearchInput
                value={this.props.pagination.searchText}
                onChange={(e: InputOnChangeEvent) => {
                  this.props.setSearchText(e.currentTarget.value)
                }}
              />
            </div>
          </div>

          <div className="view-content">
            <AddUserView/>
          </div>
        </div>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(addUserDialog)
