/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../../state/reducers/index";
import {Table, Checkbox, Icon, Button} from "semantic-ui-react"
import {CheckboxOnChangeEventData} from "../../../../../types/reactEvents";
import {
  addMembersAndExitAsync,
  invertAllSelections, setActivePaginationPage,
  setAllUsersSelected, setPaginationPageSize,
  setSortByKey,
  setUserSelected
} from "../../../../../state/actions/singleGroupSite/dialogs/addUsers/addUsersDialogActions";
import {AddableUserFrontendExclusive, UserPreviewFromBackend} from "../../../../../types/users";
import {isCurrentSortingColumnPaginated, rotateSorting} from "../../../../../helpers/tablesHelper";
import ListViewItem from './listViewItem'
import {getI18n} from "../../../../../../i18n/i18nRoot";
import {MyPopup} from '../../../../helpers/myPopup'
import PaginationTableFooter from '../../../../helpers/paginationTableFooter'
import {askDialog} from '../../../../../helpers/dialogHelper'
import {HelpPopup} from '../../../../helpers/helpPopup'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    //...rootState.users.activatedUsers,
    members: rootState.singleGroupState.members,
    selectedGroupId: rootState.singleGroupState.selectedGroupId,
    groupRoles: rootState.singleGroupState.groupRoles,
    group: rootState.singleGroupState.group,
    users: rootState.addUsersDialogState.users,
    sortByKey: rootState.addUsersDialogState.pagination.orderByProp,
    sortDirection: rootState.addUsersDialogState.pagination.isOrderByDesc,
    pagination: rootState.addUsersDialogState.pagination,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setUserSelected,
                                                                        setAllUsersSelected,
                                                                        invertAllSelections,
                                                                        setSortByKey,
                                                                        setPaginationPageSize,
                                                                        setActivePaginationPage,
                                                                        addMembersAndExitAsync,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class AddUserView extends React.Component<Props, any> {

  handleUserSelection = (selectedUser: AddableUserFrontendExclusive, isSelected: boolean) => {
    this.props.setUserSelected(selectedUser, isSelected)
  }

  render(): JSX.Element {

    const list: ReadonlyArray<AddableUserFrontendExclusive> = this.props.users

    const selectedUsersIds = list.filter(p => p.uiIsSelected).map(p => p.id)

    return (
      <div>

        <div className="two-flexed-stretch-left">

          <div className="flexed">

            <div className={selectedUsersIds.length > 0
                            ? 'clickable mar-right-half'
                            : 'div-disabled mar-right-half'}
                 onClick={async () => {

                   // noinspection TsLint
                   let shouldAdd = false

                   // if (selectedUsersIds.length > 1) {

                   shouldAdd = await askDialog(getI18n(this.props.langId, 'Add users to group'),
                                               `${getI18n(this.props.langId,
                                                          'Are you sure you want to add the selected user(s) to the group?'
                                               )} (${selectedUsersIds.length} users)`, this.props.langId
                   )
                   // }

                   if (shouldAdd) {
                     this.props.addMembersAndExitAsync(
                       this.props.selectedGroupId,
                       this.props.groupRoles,
                       {
                         groupId: this.props.group.id,
                         userIds: selectedUsersIds
                       }
                     )
                   }

                 }}
            >
              <Icon name="plus"/>
              <span>
                {
                  getI18n(this.props.langId, 'Add selected users')
                }
              </span>
            </div>
            <HelpPopup defaultText={getI18n(this.props.langId, "If you change the page in the table then the selected state is lost. Thus make sure to add the users before changing the table page.")}/>

          </div>
        </div>

        <Table sortable selectable padded striped singleLine className="material-table">
          <Table.Header>
            <Table.Row>
              <Table.HeaderCell>

                <div className="flexed">

                  <MyPopup
                    trigger={
                      <Checkbox
                        className="mar-right"
                        checked={this.props.users.every(
                          p => p.uiIsSelected || p.isAlreadyMember) && this.props.users.length > 0}
                        onChange={(e, data: CheckboxOnChangeEventData) => {
                          this.props.setAllUsersSelected(data.checked)
                        }}
                      />
                    }
                    on="hover"
                    content={getI18n(this.props.langId, "Will only select all items on the current page")}
                  />

                  <div>
                    <MyPopup
                      trigger={
                        <div className="clickable"
                          onClick={() => this.props.invertAllSelections()}
                        >
                          <Icon name="retweet"/>
                        </div>
                      }
                      content={getI18n(this.props.langId, 'Invert selection')}
                    />
                  </div>
                </div>

              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<UserPreviewFromBackend>(this.props.sortByKey, 'lastName',
                                                                          this.props.sortDirection
                  )
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('lastName',
                                                rotateSorting<UserPreviewFromBackend>(this.props.sortDirection,
                                                                                      this.props.sortByKey,
                                                                                      'lastName'
                                                )
                        )
                      }}
                >

                  {
                    getI18n(this.props.langId, 'Last name')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell sorted={
                isCurrentSortingColumnPaginated<UserPreviewFromBackend>(this.props.sortByKey, 'firstName',
                                                                        this.props.sortDirection
                )
              }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('firstName',
                                                rotateSorting<UserPreviewFromBackend>(this.props.sortDirection,
                                                                                      this.props.sortByKey,
                                                                                      'firstName'
                                                )
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'First name')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell sorted={
                isCurrentSortingColumnPaginated<UserPreviewFromBackend>(this.props.sortByKey, 'token',
                                                                        this.props.sortDirection
                )
              }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('token',
                                                rotateSorting<UserPreviewFromBackend>(this.props.sortDirection,
                                                                                      this.props.sortByKey,
                                                                                      'token'
                                                )
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Token')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<UserPreviewFromBackend>(this.props.sortByKey, 'isActivated',
                                                                          this.props.sortDirection
                  )
                }
              >
                <Icon name="user outline"/>
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('isActivated',
                                                rotateSorting<UserPreviewFromBackend>(this.props.sortDirection,
                                                                                      this.props.sortByKey,
                                                                                      'isActivated'
                                                )
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Is activated')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell sorted={
                isCurrentSortingColumnPaginated<UserPreviewFromBackend>(this.props.sortByKey, 'email',
                                                                        this.props.sortDirection
                )
              }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('email',
                                                rotateSorting<UserPreviewFromBackend>(this.props.sortDirection,
                                                                                      this.props.sortByKey,
                                                                                      'email'
                                                )
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Email')
                  }
                </span>
              </Table.HeaderCell>
            </Table.Row>
          </Table.Header>

          <Table.Body>
            {
              list.map((user, index) => {
                //console.log("render")
                //don't use arrow function to handle this because this would
                //return always a new function (??) and thus all items are re rendered...
                return (
                  <ListViewItem key={user.id} user={user}
                                setUserSelected={this.handleUserSelection}
                  />
                )
              })
            }
          </Table.Body>
          <PaginationTableFooter
            pagination={this.props.pagination}
            setPaginationPageSize={this.props.setPaginationPageSize}
            setActivePaginationPage={this.props.setActivePaginationPage}

          />
        </Table>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(AddUserView)