/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../../state/reducers";
import {AddableUserFrontendExclusive} from "../../../../../types/users";
import {Checkbox, Icon, Table} from "semantic-ui-react";
import {CheckboxOnChangeEventData} from "../../../../../types/reactEvents";
import {MyPopup} from '../../../../helpers/myPopup'
import {HelpPopup} from '../../../../helpers/helpPopup'
import {getI18n} from '../../../../../../i18n/i18nRoot'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly user: AddableUserFrontendExclusive

  readonly setUserSelected: (user: AddableUserFrontendExclusive, isSelected: boolean) => void
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    user: props.user,
    setUserSelected: props.setUserSelected,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class listViewItem extends React.Component<Props, any> {

  render(): JSX.Element {

    const user = this.props.user

    return (
      <Table.Row>
        <Table.Cell>
          <div>
            {
              user.isAlreadyMember &&
                <HelpPopup
                defaultText={getI18n(this.props.langId, "User is already a member")}
                />
            }
            {
              !user.isAlreadyMember &&
              <Checkbox checked={user.uiIsSelected}
                        onChange={(e, data: CheckboxOnChangeEventData) => {
                          this.props.setUserSelected(user, data.checked)
                        }}
              />
            }

          </div>
        </Table.Cell>

        <Table.Cell>
          <span>{user.lastName}</span>
        </Table.Cell>

        <Table.Cell>
          <span>{user.firstName}</span>
        </Table.Cell>

        <Table.Cell>
          <span>{user.token}</span>
        </Table.Cell>

        <Table.Cell>
          {
            //TODO icon or checkbox??
            //checkbox is more precises than different icons??
            //<Icon name={member.isActivated ? 'user' : 'user outline'} />
          }
          {
            <Checkbox checked={user.isActivated} disabled/>
          }
        </Table.Cell>

        <Table.Cell>
          <Icon name="copy" className="clickable"/>
          <span>{user.email}</span>
        </Table.Cell>
      </Table.Row>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(listViewItem)