/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {Checkbox, Form} from "semantic-ui-react";
import MaterialInput from '../../../material/materialInput'
import {
  setCanChangeOtherUsersSystemRole,
  setCanChangeRoles,
  setCanChangeSystemSettings,
  setCanChangeUserData,
  setCanCreateGroups,
  setCanCreateRoles,
  setCanDeleteActivatedUsers,
  setCanDeleteGroups,
  setCanDeleteRoles, setCanDownloadPersonalDataFromOthers,
  setCanManageNewUsers,
  setCanManageTags,
  setCanViewDashboard,
  setDisplayName,
  setEmail
} from "../../../../state/actions/systemRolesSite/dialog/systemRoleSetActions";
import {
  validationRules
} from "../../../../state/reducers/systemRolesSite/dialog/systemRoleReducer";
import {
  validationMessageKeys,
} from "../../../../state/reducers/systemRolesSite/dialog/systemRoleReducerValidation";
import {getI18n} from "../../../../../i18n/i18nRoot";


//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    systemRole: rootState.systemRolesChangeDialogState.systemRole,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setDisplayName,
                                                                        setEmail,
                                                                        setCanCreateGroups,
                                                                        setCanDeleteGroups,
                                                                        setCanCreateRoles,
                                                                        setCanChangeRoles,
                                                                        setCanDeleteRoles,
                                                                        setCanManageNewUsers,
                                                                        setCanChangeOtherUsersSystemRole,
                                                                        setCanDeleteActivatedUsers,
                                                                        setCanChangeUserData,
                                                                        setCanChangeSystemSettings,
                                                                        setCanManageTags,
                                                                        setCanViewDashboard,
                                                                        setCanDownloadPersonalDataFromOthers,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class ChangeSystemRoleView extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div>
        <div className="ui form divided-checkboxes">
          <Form.Group widths='equal'>

            <Form.Field required>
              <label>
                {
                  getI18n(this.props.langId, 'Name')
                }
              </label>
              <MaterialInput value={this.props.systemRole.displayName}
                             onChange={(e) => this.props.setDisplayName(e.currentTarget.value)}
                             validations={validationRules.displayName}
                             validationMessageKey={validationMessageKeys.displayName}
              />
            </Form.Field>
            <Form.Field>
              <label>
                {
                  getI18n(this.props.langId, 'Email')
                }
              </label>
              <MaterialInput value={this.props.systemRole.email}
                             onChange={(e) => this.props.setEmail(e.currentTarget.value)}
                             validations={validationRules.email}
                             validationMessageKey={validationMessageKeys.email}
              />
            </Form.Field>
          </Form.Group>


          <Form.Group widths='equal'>
            <Form.Field>
              <h4 className="ui dividing header">
                {
                  getI18n(this.props.langId, 'Groups')
                }
              </h4>

              <Form.Field>
                <Checkbox label={getI18n(this.props.langId, 'Can create groups')}
                          checked={this.props.systemRole.canCreateGroups}
                          onChange={(e, data) => {
                            this.props.setCanCreateGroups(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={getI18n(this.props.langId, 'Can delete groups')}
                          checked={this.props.systemRole.canDeleteGroups}
                          onChange={(e, data) => {
                            this.props.setCanDeleteGroups(data.checked)
                          }}
                />
              </Form.Field>
            </Form.Field>
            <Form.Field>
              <h4 className="ui dividing header">
                {
                  getI18n(this.props.langId, 'System roles')
                }
              </h4>

              <Form.Field>
                <Checkbox label={getI18n(this.props.langId, 'Can create system & group roles')}
                          checked={this.props.systemRole.canCreateRoles}
                          onChange={(e, data) => {
                            this.props.setCanCreateRoles(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={getI18n(this.props.langId, 'Can change system & group  roles')}
                          checked={this.props.systemRole.canChangeRoles}
                          onChange={(e, data) => {
                            this.props.setCanChangeRoles(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={getI18n(this.props.langId, 'Can delete system & group  roles')}
                          checked={this.props.systemRole.canDeleteRoles}
                          onChange={(e, data) => {
                            this.props.setCanDeleteRoles(data.checked)
                          }}
                />
              </Form.Field>
            </Form.Field>
          </Form.Group>


          <h4 className="ui dividing header">
            {
              getI18n(this.props.langId, 'Miscellaneous')
            }
          </h4>
          <Form.Group widths='equal'>


            <Form.Field>
              <Form.Field>
                <Checkbox label={getI18n(this.props.langId, 'Can manage new users')}
                          checked={this.props.systemRole.canManageNewUsers}
                          onChange={(e, data) => {
                            this.props.setCanManageNewUsers(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={getI18n(this.props.langId, 'Can change other users system role')}
                          checked={this.props.systemRole.canChangeOtherUsersSystemRole}
                          onChange={(e, data) => {
                            this.props.setCanChangeOtherUsersSystemRole(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={getI18n(this.props.langId, 'Can delete & view activated users')}
                          checked={this.props.systemRole.canDeleteActivatedUsers}
                          onChange={(e, data) => {
                            this.props.setCanDeleteActivatedUsers(data.checked)
                          }}
                />
              </Form.Field>
            </Form.Field>
            <Form.Field>
              <Form.Field>
                <Checkbox label={getI18n(this.props.langId, 'Can change other user data & view activated users')}
                          checked={this.props.systemRole.canChangeUserData}
                          onChange={(e, data) => {
                            this.props.setCanChangeUserData(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={getI18n(this.props.langId, 'Can change system settings')}
                          checked={this.props.systemRole.canChangeSystemSettings}
                          onChange={(e, data) => {
                            this.props.setCanChangeSystemSettings(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={getI18n(this.props.langId, 'Can manage tags')}
                          checked={this.props.systemRole.canManageTags}
                          onChange={(e, data) => {
                            this.props.setCanManageTags(data.checked)
                          }}
                />
              </Form.Field>

              <Form.Field>
                <Checkbox label={getI18n(this.props.langId, 'Can view dashboard')}
                          checked={this.props.systemRole.canViewDashboard}
                          onChange={(e, data) => {
                            this.props.setCanViewDashboard(data.checked)
                          }}
                />
              </Form.Field>

              <Form.Field>
                <Checkbox label={getI18n(this.props.langId, 'Can download personal data from others')}
                          checked={this.props.systemRole.canDownloadPersonalDataFromOthers}
                          onChange={(e, data) => {
                            this.props.setCanDownloadPersonalDataFromOthers(data.checked)
                          }}
                />
              </Form.Field>

            </Form.Field>
          </Form.Group>
        </div>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ChangeSystemRoleView)
