/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import {Card, Icon, Checkbox, Divider} from "semantic-ui-react";
import {openEditSystemRoleDialog} from "../../../state/actions/systemRolesSite/systemRolesSiteActions";
import {deleteSystemRolesAsync} from "../../../state/actions/systemRolesSite/systemRolesCrudActions";
import {copyToClipboard} from "../../../helpers/clipboard";
import {askDialog} from "../../../helpers/dialogHelper";


import {getI18n} from "../../../../i18n/i18nRoot";
import Logger from '../../../helpers/logger'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    systemRoles: rootState.systemRolesState.systemRoles,
    lockedSystemRoleIds: rootState.systemRolesState.lockedSystemRoleIds,

    userSystemRole: rootState.userDataSettingsSate.userData
                    && rootState.userDataSettingsSate.userData.systemRole
                    ? rootState.userDataSettingsSate.userData.systemRole
                    : null,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        openEditSystemRoleDialog,
                                                                        deleteSystemRolesAsync
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class SystemRolesCardsView extends React.Component<Props, any> {
  render(): JSX.Element {

    const couldNotGetLockedSystemRoles = this.props.lockedSystemRoleIds.length === 0

    if (couldNotGetLockedSystemRoles) {
      Logger.warn('could not get locked system roles, disabling change/delete for all system roles')
    }

    return (
      <div>
        <Card.Group>
          {
            this.props.systemRoles.map((systemRole, index) => {

              const isLocked = couldNotGetLockedSystemRoles || this.props.lockedSystemRoleIds.some(
                p => p === systemRole.id)

              return (<Card key={systemRole.id}>
                  <Card.Content>
                    <Card.Header>
                      <span>{systemRole.displayName}</span>
                    </Card.Header>

                    <Card.Meta>
                      <Icon name="copy" style={{
                        visibility: (systemRole.email
                                     ? null
                                     : 'hidden')
                      }}
                            className="clickable mar-right-half"
                            onClick={() => {
                              copyToClipboard(systemRole.email)
                            }}
                      />
                      <span>{systemRole.email}</span>
                    </Card.Meta>

                    <Card.Description>
                      <Checkbox label={getI18n(this.props.langId, 'Can create groups')}
                                disabled checked={systemRole.canCreateGroups}/>
                      <Checkbox label={getI18n(this.props.langId, 'Can delete groups')}
                                disabled checked={systemRole.canDeleteGroups}/>

                      <Divider/>

                      <Checkbox label={getI18n(this.props.langId, 'Can create system & group roles')} disabled
                                checked={systemRole.canCreateRoles}/>
                      <Checkbox label={getI18n(this.props.langId, 'Can change system & group  roles')} disabled
                                checked={systemRole.canChangeRoles}/>
                      <Checkbox label={getI18n(this.props.langId, 'Can delete system & group  roles')} disabled
                                checked={systemRole.canDeleteRoles}/>

                      <Divider/>

                      <Checkbox label={getI18n(this.props.langId, 'Can manage new users')} disabled
                                checked={systemRole.canManageNewUsers}/>
                      <Checkbox label={getI18n(this.props.langId, 'Can change other users system role')} disabled
                                checked={systemRole.canChangeOtherUsersSystemRole}/>
                      <Checkbox label={getI18n(this.props.langId, 'Can delete & view activated users')} disabled
                                checked={systemRole.canDeleteActivatedUsers}/>
                      <Checkbox label={getI18n(this.props.langId, 'Can change other user data & view activated users')}
                                disabled
                                checked={systemRole.canChangeUserData}/>
                      <Checkbox label={getI18n(this.props.langId, 'Can change system settings')} disabled
                                checked={systemRole.canChangeSystemSettings}/>
                      <Checkbox label={getI18n(this.props.langId, 'Can manage tags')} disabled
                                checked={systemRole.canManageTags}/>

                      <Checkbox label={getI18n(this.props.langId, 'Can view dashboard')} disabled
                                checked={systemRole.canViewDashboard}/>

                      <Checkbox label={getI18n(this.props.langId, 'Can download personal data from others')} disabled
                                checked={systemRole.canDownloadPersonalDataFromOthers}/>

                    </Card.Description>

                  </Card.Content>

                  {
                    isLocked === false &&
                    <Card.Content extra>
                      <div className="two-flexed">
                        {
                          (this.props.userSystemRole && this.props.userSystemRole.canChangeRoles) &&
                          <div className="clickable"
                               onClick={() => {


                                 this.props.openEditSystemRoleDialog(systemRole)
                               }}
                          >
                            <Icon name="write"/>
                            <span>
                          {
                            getI18n(this.props.langId, 'Change system role')
                          }
                        </span>
                          </div>
                        }
                        {
                          (this.props.userSystemRole && this.props.userSystemRole.canDeleteRoles) &&
                          <div className="clickable"
                               onClick={async () => {

                                 const shouldRemove = await askDialog(getI18n(this.props.langId, 'Delete system role'),
                                                                      `${getI18n(this.props.langId,
                                                                                 'Are you sure you want to delete the system role?'
                                                                      )} (${systemRole.displayName})`, this.props.langId
                                 )

                                 if (shouldRemove) {
                                   this.props.deleteSystemRolesAsync(systemRole)
                                 }

                               }}
                          >
                            <Icon name="remove"/>
                            <span>
                          {
                            getI18n(this.props.langId, 'Delete system role')
                          }
                        </span>
                          </div>
                        }
                      </div>
                    </Card.Content>
                  }
                </Card>
              )
            })
          }
        </Card.Group>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(SystemRolesCardsView)
