/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import SinglePanelSiteWrapper from '../../singlePanelSiteWrapper'
import {Input, Icon} from "semantic-ui-react";
import SystemRolesCardsView from './systemRolesCardsView'
import MaterialSearchInput from '../../material/materialSearchInput'
//const css = require('./styles.styl');
import ChangeSystemRoleDialog from './dialogs/changeSystemRoleDialog'
import {Dimmer, Loader} from "semantic-ui-react";
import Spinner from "../../helpers/spinner";
import {getSystemRolesAsync} from "../../../state/actions/systemRolesSite/systemRolesCrudActions";
import {loadSystemRolesSite} from "../../../state/actions/systemRolesSite/systemRolesSiteActions";


import {getI18n} from "../../../../i18n/i18nRoot";
import {errorDialog} from '../../../helpers/dialogHelper'
import Logger from '../../../helpers/logger'
import {ErrorHelper} from '../../../helpers/errorHelper'

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    isChangeDialogDisplayed: rootState.systemRolesState.isChangeDialogDisplayed,
    isLoading: rootState.systemRolesState.isLoading,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here,
  loadSystemRolesSite
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class SystemRolesSite extends React.Component<Props, any> {

  async componentWillMount(): Promise<void> {
    //this is called when we navigate to this site (dialog does not trigger this currently)

    //WHY not use the constructor instead?
    //--> https://github.com/reactjs/react-redux/issues/129
    //seems to throw warnings in some cases... this is the recommended way

    try {
      await this.props.loadSystemRolesSite()
    } catch(err) {
      ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Error', 'Could not load site data')
    }

  }

  render(): JSX.Element {
    return (
      <SinglePanelSiteWrapper>

        {
          this.props.isLoading &&
          <Spinner/>
        }

        {
          !this.props.isChangeDialogDisplayed &&
          <div className="view-padding">
            <div className="view-head-line">
              <div className="view-caption">
                <h1>
                  {
                    getI18n(this.props.langId,'System roles')
                  }
                </h1>
              </div>
              <div className="view-options">
                {/*<MaterialSearchInput/>*/}
              </div>
            </div>

            <div className="view-content">
              <SystemRolesCardsView/>
            </div>
          </div>
        }
        {
          !this.props.isLoading && this.props.isChangeDialogDisplayed &&
          <ChangeSystemRoleDialog/>
        }


      </SinglePanelSiteWrapper>

    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(SystemRolesSite)
