import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from '../../../state/reducers'
import SinglePanelSiteWrapper from '../../singlePanelSiteWrapper'
import {getI18n} from '../../../../i18n/i18nRoot'
import {Form, Icon, Table} from 'semantic-ui-react'
import Spinner from '../../helpers/spinner'
import MaterialInput from '../../material/materialInput'
import {askDialog} from '../../../helpers/dialogHelper'
import {createLangAsync, deleteLangAsync, updateLangAsync} from '../../../state/actions/langsView/langCrudActions'
import {setEditingLangId, setLangDisplayName, setLangShortcut} from '../../../state/actions/langsView/langsActions'
import {HelpPopup} from '../../helpers/helpPopup'
import {refreshServerMessageIntervalInS} from '../../../constants'

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {

    isLoading: rootState.langsState.isLoading,
    allLangs: rootState.langsState.langs,
    langCopy: rootState.langsState.langCopy,
    editingLangId: rootState.langsState.editingLangId,

    langId: rootState.i18nState.langId,
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setEditingLangId,
                                                                        setLangDisplayName,
                                                                        setLangShortcut,

                                                                        createLangAsync,
                                                                        updateLangAsync,
                                                                        deleteLangAsync,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

/**
 * copied from the pLangs settings
 */
class PlangSettingsView extends React.Component<Props, any> {


  render(): JSX.Element {

    // tslint:disable-next-line
    let langsList = this.props.allLangs

    if (this.props.editingLangId !== null && this.props.editingLangId <= 0) {
      langsList = langsList.concat([this.props.langCopy])
    }

    return (
      <SinglePanelSiteWrapper>

        {
          this.props.isLoading &&
          <Spinner/>
        }
        <div className="view-padding">
          <div className="view-head-line">
            <div className="view-caption">
              <h1 className="clear-margin">
                {
                  getI18n(this.props.langId, "Languages")
                }
              </h1>
              <HelpPopup className="mar-left-half" style={{marginTop: '0.8em'}} wide="very"
                         defaultText={getI18n(this.props.langId,
                                              'Note that adding a language here will not automatically translate all text. To add a new language you need to change the sources and add a file in the i18n directory. You can see i18n/en.ts for an example. After that you need to add the language to i18n/i18nRoot.ts in the KnownLangs enum where the value must be the same as the language shortcut in this table.'
                         )}/>
            </div>
            <div className="view-options">

            </div>
          </div>

          <div className="view-content">

            <div className="ui form">

              <Form.Field>

                <Table selectable padded striped singleLine className="material-table">
                  <Table.Header>
                    <Table.Row>
                      <Table.HeaderCell>
                        <span>
                          {
                            getI18n(this.props.langId, 'Edit')
                          }
                        </span>
                      </Table.HeaderCell>
                      <Table.HeaderCell>
                        <span>
                          {
                            getI18n(this.props.langId, 'Name')
                          }
                        </span>
                        <HelpPopup wide className="mar-left-half"
                                   defaultText={getI18n(this.props.langId, "Just for display purposes")}/>
                      </Table.HeaderCell>
                      <Table.HeaderCell>
                        <span>
                          {
                            getI18n(this.props.langId, 'Shortcut')
                          }
                        </span>
                        <HelpPopup wide className="mar-left-half" defaultText={getI18n(this.props.langId,
                                                                                       "The shortcut code is used to identify the language internally. There must be a lagnuage file (i18n) for that shortcut to work."
                        )}/>
                      </Table.HeaderCell>
                      <Table.HeaderCell>
                        <span>
                          {
                            getI18n(this.props.langId, 'Delete')
                          }
                        </span>
                        <HelpPopup wide className="mar-left-half"
                                   defaultText={getI18n(this.props.langId, "At least one language needs to be there")}/>
                      </Table.HeaderCell>
                    </Table.Row>
                  </Table.Header>

                  <Table.Body>
                    {
                      langsList.map((lang, index) => {

                        const isEditingThisPlang = lang.id === this.props.editingLangId

                        //when we edit an old plang...
                        if (isEditingThisPlang) {
                          lang = this.props.langCopy
                        }
                        //a new plang is also stored in this.props.pLangCopy but is added at the end of the list

                        return (
                          <Table.Row key={lang.id}>
                            <Table.Cell>
                              <div className="flexed div-disabled">

                                {
                                  this.props.editingLangId === null &&
                                  <div className="clickable" onClick={() => {
                                    this.props.setEditingLangId(lang.id)
                                  }}>
                                    <Icon name="write"/>
                                  </div>
                                }

                                {
                                  isEditingThisPlang &&
                                  <div className="clickable" onClick={async () => {

                                    //createPLangAsync or updatePLangsAsync should clear the edit mode

                                    if (lang.id <= 0) {
                                      //create
                                      await this.props.createLangAsync(this.props.langCopy)
                                    } else {
                                      //update
                                      await this.props.updateLangAsync(lang)
                                    }
                                  }}>
                                    <Icon name="save"/>
                                  </div>
                                }
                                {
                                  isEditingThisPlang &&
                                  <div className="clickable mar-left" onClick={() => {
                                    this.props.setEditingLangId(null)
                                  }}>
                                    <Icon name="remove"/>
                                  </div>
                                }


                              </div>

                            </Table.Cell>
                            <Table.Cell>
                              {
                                isEditingThisPlang === false &&
                                <span>{lang.displayName}</span>
                              }

                              {
                                isEditingThisPlang &&
                                <MaterialInput value={lang.displayName} onChange={(e) => {
                                  this.props.setLangDisplayName(e.currentTarget.value)
                                }}/>

                              }
                            </Table.Cell>

                            <Table.Cell>
                              {
                                isEditingThisPlang === false &&
                                <span>{lang.langShortcut}</span>
                              }

                              {
                                isEditingThisPlang &&
                                <MaterialInput value={lang.langShortcut} onChange={(e) => {
                                  this.props.setLangShortcut(e.currentTarget.value)
                                }}/>

                              }
                            </Table.Cell>

                            <Table.Cell>
                              <div className={langsList.length > 1 && false
                                              ? 'clickable'
                                              : 'div-disabled'} onClick={async () => {

                                if (langsList.length === 1) return

                                const shouldDelete = await askDialog(getI18n(this.props.langId, "Delete"),
                                                                     `${getI18n(this.props.langId,
                                                                                "Are you sure you want to delete this language?"
                                                                     )} (${lang.displayName})`,
                                                                     this.props.langId
                                )

                                if (!shouldDelete) return

                                await this.props.deleteLangAsync(lang)

                              }}>
                                <Icon name="trash"/>
                              </div>
                            </Table.Cell>

                          </Table.Row>
                        )
                      })
                    }

                    {
                      //not supported
                      // this.props.editingLangId === null &&
                      // <Table.Row>
                      //   <Table.Cell colSpan={10}>
                      //
                      //     <div>
                      //       <div className="clickable" onClick={() => {
                      //         this.props.setEditingLangId(-1)
                      //       }}>
                      //         <Icon name="plus"/>
                      //       </div>
                      //     </div>
                      //
                      //   </Table.Cell>
                      //
                      // </Table.Row>
                    }
                  </Table.Body>

                </Table>


              </Form.Field>


            </div>

          </div>
        </div>

      </SinglePanelSiteWrapper>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(PlangSettingsView)
