import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from '../../../state/reducers'
import Spinner from '../../helpers/spinner'
import SinglePanelSiteWrapper from '../../singlePanelSiteWrapper'
import {getI18n, LangObj} from '../../../../i18n/i18nRoot'
import {HelpPopup} from '../../helpers/helpPopup'
import {testServerPublicSettingsNotAvailableValue} from '../../../constants'
import {Form, Icon} from 'semantic-ui-react'
import {MyPopup} from '../../helpers/myPopup'
import MaterialInput from '../../material/materialInput'
import {TestServerPublicSettings} from '../../../types/systemSettings'

export interface MyProps {
  readonly title: keyof LangObj
  readonly testServerPublicSettings: TestServerPublicSettings

  readonly testServerConfigUiUrl: string
}

const mapStateToProps = (rootState: RootState , props: MyProps) => {
  return {
    //test0: rootState...
    ...props,
    isLoading: rootState.systemSettingsSiteState.isLoading,

    langId: rootState.i18nState.langId,
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here

                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


/**
 * this is only a readonly view of the test server settings (because there is no api to change these, we would have to handle account management...)
 */
class TestServerSettingsView extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <SinglePanelSiteWrapper>

        {
          this.props.isLoading &&
          <Spinner/>
        }


        <div className="view-padding">
          <div className="view-head-line">
            <div className="view-caption">
              <h1 className="clear-margin">
                {
                  getI18n(this.props.langId, this.props.title)
                }
              </h1>

              {
                this.props.testServerPublicSettings.hardTimeoutInMs === testServerPublicSettingsNotAvailableValue &&
                  <HelpPopup icon="warning sign" className="warning-colored mar-left-half v-centered" defaultText={getI18n(this.props.langId, "The test-server settings are not available")} />
              }

              {
                this.props.testServerConfigUiUrl &&
                <MyPopup
                  trigger={
                    <a className="mar-left-half" href={this.props.testServerConfigUiUrl} target="_blank">
                      <Icon name="external alternate" />
                    </a>
                  }
                  content={getI18n(this.props.langId, "Click to open the test server configuration page")}
                />
              }
            </div>
            <div className="view-options">

            </div>
          </div>

          <div className="view-content">

            <div className="ui form">


              <Form.Group widths="equal">
                {
                  //hard timeout for all test server tests
                }
                <Form.Field>

                  <div className="form-field-label-with-icon">
                    <label>
                      {
                        getI18n(this.props.langId, "Timeout in ms")
                      }
                    </label>
                    <HelpPopup wide
                               className="mar-left-half"
                               defaultText={getI18n(this.props.langId,
                                                    "The max time the user program is allowed to run in ms"
                               )}
                    />

                  </div>
                  <MaterialInput type="number" value={this.props.testServerPublicSettings.hardTimeoutInMs} disabled
                  />
                </Form.Field>

                {
                  //hard timeout for all test server tests
                }
                <Form.Field>

                  <div className="form-field-label-with-icon">
                    <label>
                      {
                        getI18n(this.props.langId, "Compile timeout in ms")
                      }
                    </label>
                    <HelpPopup wide
                               className="mar-left-half"
                               defaultText={getI18n(this.props.langId,
                                                    "The max time the user program is allowed to compile in ms"
                               )}
                    />

                  </div>
                  <MaterialInput type="number" value={this.props.testServerPublicSettings.hardCompileTimeoutInMs} disabled
                  />
                </Form.Field>


                <Form.Field>

                  <div className="form-field-label-with-icon">
                    <label>
                      {
                        getI18n(this.props.langId, "Max. RAM in kb")
                      }
                    </label>

                  </div>
                  <MaterialInput type="number" value={this.props.testServerPublicSettings.hardMemoryLimitInKb} disabled
                  />
                </Form.Field>

                <Form.Field>

                  <div className="form-field-label-with-icon">
                    <label>
                      {
                        getI18n(this.props.langId, "Max disk space in kb")
                      }
                    </label>

                  </div>
                  <MaterialInput type="number" value={this.props.testServerPublicSettings.hardDiskSpaceLimitInKb} disabled
                  />
                </Form.Field>

              </Form.Group>

              <Form.Group widths="equal">

                <Form.Field>

                  <div className="form-field-label-with-icon">
                    <label>
                      {
                        getI18n(this.props.langId, "Max. tests with one request")
                      }
                    </label>
                    <HelpPopup wide
                               className="mar-left-half"
                               defaultText={getI18n(this.props.langId,
                                                    "If one runs multiple tests this is the number of tests that are actually run with one requests. If there are tests left one must issue a new requests to run all tests"
                               )}
                    />

                  </div>
                  <MaterialInput type="number" value={this.props.testServerPublicSettings.maxNumberOfTestsWithOneRequest} disabled
                  />
                </Form.Field>

              </Form.Group>

            </div>
          </div>
        </div>

      </SinglePanelSiteWrapper>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(TestServerSettingsView)
