/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {Icon} from "semantic-ui-react";
import {HelpPopup} from "../../../helpers/helpPopup";
import {SimpleVDivider} from "../../../helpers/simpleVDivider";
import {
  resetDoExerciseSolutionToTemplate,
  setIsDoCodeEditorSettingsDialogDisplayed
} from "../../../../state/actions/doExerciseSite/doExerciseSiteActions";
import {openEditEditorSettingsDialog} from "../../../../state/actions/editorSettingsDialog/editorSettingsDialogActions";
import {setTutorViewIsCodeEditorSettingsDialogDisplayed} from "../../../../state/actions/tutorViewSite/tutorViewSiteActions";


import {getI18n} from "../../../../../i18n/i18nRoot";
import {MyPopup} from '../../../helpers/myPopup'
import {displayDateAndTime} from '../../../../helpers/convertersAndTransformers'
import Logger from '../../../../helpers/logger'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    codeEditorSettings: rootState.userDataSettingsSate.userData.settings ?
      rootState.userDataSettingsSate.userData.settings.codeEditorSettings
      : null,
    exerciseId: rootState.tutorViewSiteState.exercise.id,
    userSolutionPLangId: rootState.tutorViewSiteState.userSolution.pLangId,
    codeTemplates: rootState.tutorViewSiteState.allCodeTemplates,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  setTutorViewIsCodeEditorSettingsDialogDisplayed,
  openEditEditorSettingsDialog,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class editorPanelHeaderBar extends React.Component<Props, any> {


  render(): JSX.Element {

    if (!this.props.codeTemplates) return null

    const selectedCodeTemplate = this.props.codeTemplates.find(p => p.pLangId === this.props.userSolutionPLangId)

    if (!selectedCodeTemplate) {
      Logger.warn(`could not find code template for plangId ${this.props.userSolutionPLangId} (exercise Id ${this.props.exerciseId})`)
      return null
    }

    return (
      <div className="flexed">

        <div className="clickable"
             onClick={() => {
               this.props.openEditEditorSettingsDialog(this.props.codeEditorSettings,
                 this.props.setTutorViewIsCodeEditorSettingsDialogDisplayed)
             }}
        >
          <Icon name="settings"/>
          <span>
            {
              getI18n(this.props.langId,'Editor settings')
            }
          </span>
        </div>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(editorPanelHeaderBar)
