/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import MultiTabCodeEditorWrapper from "../../../codeEditors/multiTabCodeEditorWrapper";
import Logger from "../../../../helpers/logger";
import {errorDialog} from "../../../../helpers/dialogHelper";
import {EditorTab} from "../../../codeEditors/multiTabCodeEditorProps";
import {getI18n} from "../../../../../i18n/i18nRoot";
import {tutorViewCodeTemplateEditorPanelViewEditorId} from '../../../../constants'
import {
  closeTabChangeCodeTemplateEditorTutorView,
  openTabChangeCodeTemplateEditorTutorView,
  setIsFileExplorerDisplayedCodeTemplateEditorTutorView,
  setSelectedTabIdCodeTemplateEditorTutorView
} from '../../../../state/actions/tutorViewSite/editorActions/codeTemplateEditorTutorActions'
import {ExerciseCodeTemplateFileFullBase} from '../../../../types/exerciseEditor'
import {getTabModeFromExtension} from '../../../../helpers/editorHelper'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test

    pLangs: rootState.pLangsState.pLangs,
    codeTemplates: rootState.tutorViewSiteState.allCodeTemplates,
    userSolutionPLangId: rootState.tutorViewSiteState.userSolution.pLangId,
    exerciseId: rootState.tutorViewSiteState.exercise.id,

    selectedTabIdDoEditor: rootState.tutorViewSiteState.codeTemplateEditorState.selectedTabIdDoEditor,
    isFileExplorerDisplayedDoEditor: rootState.tutorViewSiteState.codeTemplateEditorState.isFileExplorerDisplayedDoEditor,
    newFileNameDoEditor: rootState.tutorViewSiteState.codeTemplateEditorState.newFileNameDoEditor,
    renamingFileIdDoEditor: rootState.tutorViewSiteState.codeTemplateEditorState.renamingFileIdDoEditor,
    isRenamingFileDoEditor: rootState.tutorViewSiteState.codeTemplateEditorState.isRenamingFileDoEditor,
    renamingFileNameDoEditor: rootState.tutorViewSiteState.codeTemplateEditorState.renamingFileNameDoEditor,
    hasFileChangedMap: rootState.tutorViewSiteState.codeTemplateEditorState.hasFileChangedMap,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setIsFileExplorerDisplayedCodeTemplateEditorTutorView,
                                                                        setSelectedTabIdCodeTemplateEditorTutorView,

                                                                        openTabChangeCodeTemplateEditorTutorView,
                                                                        closeTabChangeCodeTemplateEditorTutorView,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class editorPanelView extends React.Component<Props, any> {


  componentWillMount(): void {

  }

  render(): JSX.Element {

    if (!this.props.codeTemplates) return null

    const selectedCodeTemplate = this.props.codeTemplates.find(p => p.pLangId === this.props.userSolutionPLangId)

    if (!selectedCodeTemplate) {
      Logger.warn(`could not find code template for plangId ${this.props.userSolutionPLangId} (exercise Id ${this.props.exerciseId})`)
      return null
    }

    const pLang = this.props.pLangs.find(p => p.id === this.props.userSolutionPLangId)

    if (!pLang) {
      //TODO only error if really not found
      //on first render we have no plang but thats no and error!!
      return null
    }

    const editorTabs = selectedCodeTemplate.templateFiles.map<EditorTab>(p => {
      return {
        id: p.id,
        fileNameWithExtensions: p.fileNameWithExtension,
        content: p.content,
        isMainFile: p.id === selectedCodeTemplate.mainFileId,
        isReadonly: true, //all template files are readonly
        displayIndex: p.displayIndex,
        language: getTabModeFromExtension(p.fileNameWithExtension, this.props.pLangs),
        isContentVisibleForUser: true,
      }
    })
      .sort((a, b) => a.fileNameWithExtensions.localeCompare(b.fileNameWithExtensions))

    return (<div className="fh hidden-overflow-i">
      <MultiTabCodeEditorWrapper
        id={tutorViewCodeTemplateEditorPanelViewEditorId}
        height="100%"
        debounceTimeInMs={undefined}
        selectedTabId={this.props.selectedTabIdDoEditor}
        canPrint={true}
        onSelectedTabIdChanged={(tabId: number) => {
          this.props.setSelectedTabIdCodeTemplateEditorTutorView(tabId)
        }}

        isFileExplorerDisplayed={this.props.isFileExplorerDisplayedDoEditor}
        onChangeIsFileExplorerDisplayed={(isDisplayed: boolean) => {
          this.props.setIsFileExplorerDisplayedCodeTemplateEditorTutorView(isDisplayed)
        }}

        newFileName={this.props.newFileNameDoEditor}
        onNewFileNameChanged={null}

        renamingFileId={this.props.renamingFileIdDoEditor}
        onRenamingFileIdChanged={null}

        isRenamingFile={this.props.isRenamingFileDoEditor}
        onIsRenamingFileChanged={null}

        renamingFileName={this.props.renamingFileNameDoEditor}
        onRenamingFileNameChanged={null}

        hasUnsavedChanges={false}
        onHasUnsavedChangesChanged={null}

        userCanCreateNewFiles={false}

        onCreateFileClicked={null}

        onOpenTabClicked={(editorTab: EditorTab) => {
          const file = selectedCodeTemplate.templateFiles.find(p => p.id === editorTab.id)

          if (!file) {

            errorDialog(getI18n(this.props.langId, 'Error'), getI18n(this.props.langId, 'Could not find editor file'),this.props.langId)
            Logger.error("file not found (onOpenTabClicked)")

            return
          }

          const mappedFiles = selectedCodeTemplate.templateFiles.map<ExerciseCodeTemplateFileFullBase>(p => {
            return p.id === file.id
                   ? {
                ...file,
                uiIsDisplayed: true,
              }
                   : p
          })

          this.props.openTabChangeCodeTemplateEditorTutorView(file, mappedFiles, this.props.selectedTabIdDoEditor, selectedCodeTemplate.id)
        }}

        onCloseTabClicked={(editorTab: EditorTab) => {

          const file = selectedCodeTemplate.templateFiles.find(p => p.id === editorTab.id)

          if (!file) {

            errorDialog(getI18n(this.props.langId, 'Error'), getI18n(this.props.langId, 'Could not find editor file'),this.props.langId)
            Logger.error("file not found (onCloseTabClicked)")

            return
          }

          const mappedFiles = selectedCodeTemplate.templateFiles.map<ExerciseCodeTemplateFileFullBase>(p => {
            return p.id === file.id
                   ? {
                ...file,
                uiIsDisplayed: false,
              }
                   : p
          })

          this.props.closeTabChangeCodeTemplateEditorTutorView(file, mappedFiles, this.props.selectedTabIdDoEditor, selectedCodeTemplate.id)
        }}

        onFileRename={null}

        onRemoveFile={null}

        editorTabs={editorTabs}
        onBlur={null}

        hasFileChangedMap={this.props.hasFileChangedMap}
        onUpdateHasFileChangedMap={(changedFile: EditorTab, allTabs: ReadonlyArray<EditorTab>) => {
        }}

      />
    </div>)
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(editorPanelView)
