/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {Icon} from "semantic-ui-react";
import {HelpPopup} from "../../../helpers/helpPopup";
import {SimpleVDivider} from "../../../helpers/simpleVDivider";
import {
  resetDoExerciseSolutionToTemplate,
  setIsDoCodeEditorSettingsDialogDisplayed
} from "../../../../state/actions/doExerciseSite/doExerciseSiteActions";
import {openEditEditorSettingsDialog} from "../../../../state/actions/editorSettingsDialog/editorSettingsDialogActions";
import {
  setFeedbackForStudentPanelSelectedSubTabIndex,
  setTutorViewIsCodeEditorSettingsDialogDisplayed
} from "../../../../state/actions/tutorViewSite/tutorViewSiteActions";


import {getI18n} from "../../../../../i18n/i18nRoot";
import {MyPopup} from '../../../helpers/myPopup'
import {displayDateAndTime} from '../../../../helpers/convertersAndTransformers'
import Logger from '../../../../helpers/logger'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    releaseFinishedAndAutomaticAssessmentHasFinished: !rootState.tutorViewSiteState.exercise.exerciseRelease.isReleased &&
      rootState.tutorViewSiteState.exercise.exerciseRelease.hasAutomaticAssessmentFinished,
    feedbackForStudentPanelSelectedSubTabIndex: rootState.tutorViewSiteState.feedbackForStudentPanelSelectedSubTabIndex,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setFeedbackForStudentPanelSelectedSubTabIndex,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class feedbackPanelHeaderBar extends React.Component<Props, any> {

  render(): JSX.Element {

    return (
      <div className="flexed">

        {
          this.props.releaseFinishedAndAutomaticAssessmentHasFinished &&
          <div className="flexed">
            <div className={`clickable${this.props.feedbackForStudentPanelSelectedSubTabIndex === 0
                                        ? ' toggled-active--color'
                                        : ''}`}
                 onClick={() => {
                   this.props.setFeedbackForStudentPanelSelectedSubTabIndex(0)
                 }}
            >
              <Icon name="write"/>
              <span>
                {
                  getI18n(this.props.langId, 'Editor')
                }
              </span>
            </div>

            {
              //TODO when semantic ui react has font awesome >= 5.7.x use markdown icon...
            }
            <HelpPopup className="mar-left-half" defaultText={getI18n(this.props.langId, "Editor supports Markdown (Icon)")}/>
          </div>
        }

        {
          this.props.releaseFinishedAndAutomaticAssessmentHasFinished &&
          <SimpleVDivider/>
        }

        <div
          className={`clickable${(this.props.feedbackForStudentPanelSelectedSubTabIndex === 1 || this.props.releaseFinishedAndAutomaticAssessmentHasFinished === false)
                                 ? ' toggled-active--color'
                                 : ''}`}
          onClick={() => {
            this.props.setFeedbackForStudentPanelSelectedSubTabIndex(1)
          }}
        >
          <Icon name="eye"/>
          <span>
            {
              getI18n(this.props.langId, 'Preview')
            }
          </span>
        </div>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(feedbackPanelHeaderBar)
