/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {Icon} from "semantic-ui-react";
import {RunTutorTestCommandForBackend} from "../../../../types/testsAndTestProtocol";
import {HelpPopup} from "../../../helpers/helpPopup";
import Logger from "../../../../helpers/logger";
import {isTestResultValid} from "../../../../helpers/testHelper";
import {runNormalTestAsTutor} from "../../../../state/actions/tutorViewSite/tutorViewSiteCrudActions";


import {SimpleVDivider} from "../../../helpers/simpleVDivider";
import {resetTutorNormalTestResults} from "../../../../state/actions/tutorViewSite/tutorViewSiteActions";
import {getI18n} from "../../../../../i18n/i18nRoot";
import orderBy from 'lodash-es/orderBy'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    tests: rootState.tutorViewSiteState.normalTestsForTutor,
    releaseCode: rootState.tutorViewSiteState.exercise.exerciseRelease.releaseCode,
    solutionPLangId: rootState.tutorViewSiteState.tutorEditorSolution.pLangId,
    systemSettings: rootState.globalSystemSettingsState.systemSettings,
    solution: rootState.tutorViewSiteState.tutorEditorSolution,

    isSomeTestRunning: rootState.tutorViewSiteState.exercise.tests.some(p => p.uiIsRunning)
    || rootState.tutorViewSiteState.exercise.customTests.some(p => p.uiIsRunning)
    || rootState.tutorViewSiteState.normalTestsForTutor.some(p => p.uiIsRunning)
    || rootState.tutorViewSiteState.submitTestsForTutor.some(p => p.uiIsRunning),

    maxNumberOfTestsWithOneRequest: rootState.globalSystemSettingsState.systemSettings
    && rootState.globalSystemSettingsState.systemSettings.maxNumberOfTestsWithOneRequest >= 0
                                    ? rootState.globalSystemSettingsState.systemSettings.maxNumberOfTestsWithOneRequest : 1,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  runNormalTestAsTutor,
  resetTutorNormalTestResults
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class testsPanelHeader extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div className="flexed">

        <div className={`mar-right-half ${(!this.props.isSomeTestRunning  && this.props.tests.length > 0) ? 'clickable' : 'div-disabled' }`}
             onClick={async () => {

               const maxNumberOfTestsWithOneRequest = this.props.maxNumberOfTestsWithOneRequest

               Logger.debug("normal test panel header bar (tutor view)",
                 `running all test with ${maxNumberOfTestsWithOneRequest} test in parallel`)

               const orderedTests = orderBy(this.props.tests, (test) => test.displayIndex)

               const validTests = orderedTests.filter(p => isTestResultValid(p))

               //a template
               // noinspection TsLint
               let tempCommand: RunTutorTestCommandForBackend = {
                 releaseCode: this.props.releaseCode,
                 isSubmitTest: false,
                 testIds: [],
                 isNormalTest: true,
                 plangId: this.props.solutionPLangId,
                 mainFileId: this.props.solution.mainFileId,
                 solutionFiles: this.props.solution.solutionFiles
               }

               // noinspection TsLint
               for (let i = 0; i < orderedTests.length; i++) {
                 const test = orderedTests[i]

                 if (isTestResultValid(test) === false || validTests.length === orderedTests.length) {
                   //add
                   tempCommand = {
                     ...tempCommand,
                     testIds: [...tempCommand.testIds, test.id]
                   }
                 } else {
                   //test was already valid
                 }

                 //we reached the limit
                 if (tempCommand.testIds.length === maxNumberOfTestsWithOneRequest) {
                   const hasAnyTestFailed = await this.props.runNormalTestAsTutor(tempCommand)

                   //clear test ids
                   tempCommand = {
                     ...tempCommand,
                     testIds: []
                   }

                   //stop if at least 1 test failed
                   if (hasAnyTestFailed) {
                     //return instead of break else we would run the last check below which would be always true
                     return
                   }
                 }
               }

               //we might have some last tests but not reached the max num of tests with one request...
               if (tempCommand.testIds.length > 0) {
                 //we missed the last < maxNumberOfTestsWithOneRequest tests
                 //these are still in the last tempCommand
                 await this.props.runNormalTestAsTutor(tempCommand)
               }

             }}
        >
          <Icon name="forward"/>
          <span>
            {
              getI18n(this.props.langId,'Run all tests')
            }
          </span>
        </div>

        <HelpPopup
          defaultText={`${getI18n(this.props.langId,'Runs all not passed tests. If all tests have passed then all tests are re-run. When at least one test fails then the next x tests are not executed.')} ${getI18n(this.props.langId,
            'Max parallel tests')}: ${this.props.maxNumberOfTestsWithOneRequest}`}
          wide
        />

        <SimpleVDivider/>

        <div className="clickable mar-right-half"
             onClick={() => {
               this.props.resetTutorNormalTestResults()
             }}
        >
          <Icon name="remove"/>
          <span>
          {
            getI18n(this.props.langId, 'Clear test results')
          }
          </span>
        </div>
        <HelpPopup
          defaultText={getI18n(this.props.langId,'This only discards the results in the frontend, the results on the server are still there')}
          wide
        />


      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(testsPanelHeader)
