/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import MultiTabCodeEditorWrapper from "../../../codeEditors/multiTabCodeEditorWrapper";
import {
  closeTabChangeUserSolutionEditorTutorView,
  openTabChangeUserSolutionEditorTutorView,
  setIsFileExplorerDisplayedUserSolutionEditorTutorView,
  setSelectedTabIdUserSolutionEditorTutorView
} from "../../../../state/actions/tutorViewSite/editorActions/userSolutionEditorActions";
import {SolutionFileDoExerciseFullBase} from "../../../../types/exerciseSolution";
import Logger from "../../../../helpers/logger";
import {errorDialog} from "../../../../helpers/dialogHelper";
import {EditorTab} from "../../../codeEditors/multiTabCodeEditorProps";
import {getI18n} from "../../../../../i18n/i18nRoot";
import {multiFileEditorDebounceInMs, tutorViewUserSolutionEditorPanelViewEditorId} from '../../../../constants'
import {getTabModeFromExtension} from '../../../../helpers/editorHelper'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test

    pLangs: rootState.pLangsState.pLangs,
    solution: rootState.tutorViewSiteState.userSolution,

    selectedTabIdDoEditor: rootState.tutorViewSiteState.userSolutionEditorState.selectedTabIdDoEditor,
    isFileExplorerDisplayedDoEditor: rootState.tutorViewSiteState.userSolutionEditorState.isFileExplorerDisplayedDoEditor,
    newFileNameDoEditor: rootState.tutorViewSiteState.userSolutionEditorState.newFileNameDoEditor,
    renamingFileIdDoEditor: rootState.tutorViewSiteState.userSolutionEditorState.renamingFileIdDoEditor,
    isRenamingFileDoEditor: rootState.tutorViewSiteState.userSolutionEditorState.isRenamingFileDoEditor,
    renamingFileNameDoEditor: rootState.tutorViewSiteState.userSolutionEditorState.renamingFileNameDoEditor,
    hasUnsavedChanges: rootState.tutorViewSiteState.userSolutionEditorState.hasUnsavedChanges,
    hasFileChangedMap: rootState.tutorViewSiteState.userSolutionEditorState.hasFileChangedMap,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  setIsFileExplorerDisplayedUserSolutionEditorTutorView,
  setSelectedTabIdUserSolutionEditorTutorView,

  openTabChangeUserSolutionEditorTutorView,
  closeTabChangeUserSolutionEditorTutorView,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class editorPanelView extends React.Component<Props, any> {


  componentWillMount(): void {

  }

  render(): JSX.Element {

    const pLang = this.props.pLangs.find(p => p.id === this.props.solution.pLangId)

    if (!pLang) {
      //TODO only error if really not found
      //on first render we have no plang but thats no and error!!
      return null
    }

    const editorTabs = this.props.solution.solutionFiles.map<EditorTab>(p => {
      return {
        id: p.id,
        fileNameWithExtensions: p.fileNameWithExtension,
        content: p.content,
        isMainFile: p.id === this.props.solution.mainFileId,
        isReadonly: true,
        displayIndex: p.displayIndex,
        language: getTabModeFromExtension(p.fileNameWithExtension, this.props.pLangs),
        isContentVisibleForUser: p.isContentVisibleForUser
      }
    })
      .sort((a, b) => a.fileNameWithExtensions.localeCompare(b.fileNameWithExtensions))

    return (<div className="fh hidden-overflow-i">
        <MultiTabCodeEditorWrapper
          id={tutorViewUserSolutionEditorPanelViewEditorId}
          height="100%"
          debounceTimeInMs={undefined}
          selectedTabId={this.props.selectedTabIdDoEditor}
          canPrint={true}
          onSelectedTabIdChanged={(tabId: number) => {
            this.props.setSelectedTabIdUserSolutionEditorTutorView(tabId)
          }}

          isFileExplorerDisplayed={this.props.isFileExplorerDisplayedDoEditor}
          onChangeIsFileExplorerDisplayed={(isDisplayed: boolean) => {
            this.props.setIsFileExplorerDisplayedUserSolutionEditorTutorView(isDisplayed)
          }}

          newFileName={this.props.newFileNameDoEditor}
          onNewFileNameChanged={null}

          renamingFileId={this.props.renamingFileIdDoEditor}
          onRenamingFileIdChanged={null}

          isRenamingFile={this.props.isRenamingFileDoEditor}
          onIsRenamingFileChanged={null}

          renamingFileName={this.props.renamingFileNameDoEditor}
          onRenamingFileNameChanged={null}

          hasUnsavedChanges={false}
          onHasUnsavedChangesChanged={null}

          userCanCreateNewFiles={false}

          onCreateFileClicked={null}

          onOpenTabClicked={(editorTab: EditorTab) => {
            const file = this.props.solution.solutionFiles.find(p => p.id === editorTab.id)

            if (!file) {

              errorDialog(getI18n(this.props.langId, 'Error'), getI18n(this.props.langId, 'Could not find editor file'),this.props.langId)
              Logger.error("file not found (onOpenTabClicked)")

              return
            }

            const mappedFiles = this.props.solution.solutionFiles.map<SolutionFileDoExerciseFullBase>(p => {
              return p.id === file.id
                     ? {
                  ...file,
                  uiIsDisplayed: true,
                }
                     : p
            })

            this.props.openTabChangeUserSolutionEditorTutorView(file, mappedFiles, this.props.selectedTabIdDoEditor)
          }}

          onCloseTabClicked={(editorTab: EditorTab) => {

            const file = this.props.solution.solutionFiles.find(p => p.id === editorTab.id)

            if (!file) {

              errorDialog(getI18n(this.props.langId, 'Error'), getI18n(this.props.langId, 'Could not find editor file'),this.props.langId)
              Logger.error("file not found (onCloseTabClicked)")

              return
            }

            const mappedFiles = this.props.solution.solutionFiles.map<SolutionFileDoExerciseFullBase>(p => {
              return p.id === file.id
                     ? {
                  ...file,
                  uiIsDisplayed: false,
                }
                     : p
            })

            this.props.closeTabChangeUserSolutionEditorTutorView(file, mappedFiles, this.props.selectedTabIdDoEditor)
          }}

          onFileRename={null}

          onRemoveFile={null}

          editorTabs={editorTabs}
          onBlur={null}
          // onChange={(editorTab: EditorTab, newValue: string) => {
          // }}
          // onSomeEditorSessionChanged={() => {
          //
          // }}

          hasFileChangedMap={this.props.hasFileChangedMap}
          onUpdateHasFileChangedMap={(changedFile: EditorTab, allTabs: ReadonlyArray<EditorTab>) => {
            //performance is tooooo bad...
            //this.props.setHasFileChangedMapDoEditor(updateHasFileChangedMap(changedFile, allTabs, this.props.hasFileChangedMap))
          }}

        />
      </div>)
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(editorPanelView)
