/**
 * Created by janis dähne.
 */
import {AwaitActions, MultiActions} from "../state/actions/types";
import {FileIdAssociation, SolutionFileDoExerciseFullBase} from "../types/exerciseSolution";
import {ExercisePreviewsFrontendExclusive} from "../components/sites/allAccessibleExercisesSite/openExercises/allOpenExercisesSite";
import moment, {Moment} from "moment";
import {ReleaseDurationType} from "../types/ReleaseDurationType";
import {padLeft} from "./stringHelper";
import {errorDialog} from "./dialogHelper";
import Logger from "./logger";
import {ReleaseStartType} from "../types/ReleaseStartType";
import {EditorTab} from "../components/codeEditors/multiTabCodeEditorProps";
import {ExerciseCodeTemplateFileFullBase} from "../types/exerciseEditor";
import {getI18n, KnownLangs} from "../../i18n/i18nRoot";
import {EditorWrapperTab} from "../components/codeEditors/aceEditors/aceEditorsStorageHelper";
import {defaultEditorMode} from '../constants'
import {PLangFullBase} from '../types/pLangs'


/**
 * the possible editor highlight languages, found in
 * node_modules/brace/mode/*.js
 * you can use in terminal ts or find ./ -print to get the list
 * @see possibleMarkdownHighlightPLanguages for markdown language code fence
 *
 * for real names you can check https://ace.c9.io/build/kitchen-sink.html and inspect the mode select in html... here are all options (14.2.2019)
 */
 /* <select id="-mode">
 * <option value="ace/mode/abap">ABAP</option>
 * <option value="ace/mode/abc">ABC</option>
 * <option value="ace/mode/actionscript">ActionScript</option>
 * <option value="ace/mode/ada">ADA</option>
 * <option value="ace/mode/apache_conf">Apache Conf</option>
 * <option value="ace/mode/asciidoc">AsciiDoc</option>
 * <option value="ace/mode/asl">ASL</option>
 * <option value="ace/mode/assembly_x86">Assembly x86</option>
 * <option value="ace/mode/autohotkey">AutoHotKey</option>
 * <option value="ace/mode/apex">Apex</option>
 * <option value="ace/mode/batchfile">BatchFile</option>
 * <option value="ace/mode/bro">Bro</option>
 * <option value="ace/mode/c_cpp">C and C++</option>
 * <option value="ace/mode/c9search">C9Search</option>
 * <option value="ace/mode/cirru">Cirru</option>
 * <option value="ace/mode/clojure">Clojure</option>
 * <option value="ace/mode/cobol">Cobol</option>
 * <option value="ace/mode/coffee">CoffeeScript</option>
 * <option value="ace/mode/coldfusion">ColdFusion</option>
 * <option value="ace/mode/csharp">C#</option>
 * <option value="ace/mode/csound_document">Csound Document</option>
 * <option value="ace/mode/csound_orchestra">Csound</option>
 * <option value="ace/mode/csound_score">Csound Score</option>
 * <option value="ace/mode/css">CSS</option>
 * <option value="ace/mode/curly">Curly</option>
 * <option value="ace/mode/d">D</option>
 * <option value="ace/mode/dart">Dart</option>
 * <option value="ace/mode/diff">Diff</option>
 * <option value="ace/mode/dockerfile">Dockerfile</option>
 * <option value="ace/mode/dot">Dot</option>
 * <option value="ace/mode/drools">Drools</option>
 * <option value="ace/mode/edifact">Edifact</option>
 * <option value="ace/mode/eiffel">Eiffel</option>
 * <option value="ace/mode/ejs">EJS</option>
 * <option value="ace/mode/elixir">Elixir</option>
 * <option value="ace/mode/elm">Elm</option>
 * <option value="ace/mode/erlang">Erlang</option>
 * <option value="ace/mode/forth">Forth</option>
 * <option value="ace/mode/fortran">Fortran</option>
 * <option value="ace/mode/fsharp">FSharp</option>
 * <option value="ace/mode/fsl">FSL</option>
 * <option value="ace/mode/ftl">FreeMarker</option>
 * <option value="ace/mode/gcode">Gcode</option>
 * <option value="ace/mode/gherkin">Gherkin</option>
 * <option value="ace/mode/gitignore">Gitignore</option>
 * <option value="ace/mode/glsl">Glsl</option>
 * <option value="ace/mode/gobstones">Gobstones</option>
 * <option value="ace/mode/golang">Go</option>
 * <option value="ace/mode/graphqlschema">GraphQLSchema</option>
 * <option value="ace/mode/groovy">Groovy</option>
 * <option value="ace/mode/haml">HAML</option>
 * <option value="ace/mode/handlebars">Handlebars</option>
 * <option value="ace/mode/haskell">Haskell</option>
 * <option value="ace/mode/haskell_cabal">Haskell Cabal</option>
 * <option value="ace/mode/haxe">haXe</option>
 * <option value="ace/mode/hjson">Hjson</option>
 * <option value="ace/mode/html">HTML</option>
 * <option value="ace/mode/html_elixir">HTML (Elixir)</option>
 * <option value="ace/mode/html_ruby">HTML (Ruby)</option>
 * <option value="ace/mode/ini">INI</option>
 * <option value="ace/mode/io">Io</option>
 * <option value="ace/mode/jack">Jack</option>
 * <option value="ace/mode/jade">Jade</option>
 * <option value="ace/mode/java">Java</option>
 * <option value="ace/mode/javascript">JavaScript</option>
 * <option value="ace/mode/json">JSON</option>
 * <option value="ace/mode/jsoniq">JSONiq</option>
 * <option value="ace/mode/jsp">JSP</option>
 * <option value="ace/mode/jssm">JSSM</option>
 * <option value="ace/mode/jsx">JSX</option>
 * <option value="ace/mode/julia">Julia</option>
 * <option value="ace/mode/kotlin">Kotlin</option>
 * <option value="ace/mode/latex">LaTeX</option>
 * <option value="ace/mode/less">LESS</option>
 * <option value="ace/mode/liquid">Liquid</option>
 * <option value="ace/mode/lisp">Lisp</option>
 * <option value="ace/mode/livescript">LiveScript</option>
 * <option value="ace/mode/logiql">LogiQL</option>
 * <option value="ace/mode/lsl">LSL</option>
 * <option value="ace/mode/lua">Lua</option>
 * <option value="ace/mode/luapage">LuaPage</option>
 * <option value="ace/mode/lucene">Lucene</option>
 * <option value="ace/mode/makefile">Makefile</option>
 * <option value="ace/mode/markdown">Markdown</option>
 * <option value="ace/mode/mask">Mask</option>
 * <option value="ace/mode/matlab">MATLAB</option>
 * <option value="ace/mode/maze">Maze</option>
 * <option value="ace/mode/mel">MEL</option>
 * <option value="ace/mode/mixal">MIXAL</option>
 * <option value="ace/mode/mushcode">MUSHCode</option>
 * <option value="ace/mode/mysql">MySQL</option>
 * <option value="ace/mode/nix">Nix</option>
 * <option value="ace/mode/nsis">NSIS</option>
 * <option value="ace/mode/objectivec">Objective-C</option>
 * <option value="ace/mode/ocaml">OCaml</option>
 * <option value="ace/mode/pascal">Pascal</option>
 * <option value="ace/mode/perl">Perl</option>
 * <option value="ace/mode/perl6">Perl 6</option>
 * <option value="ace/mode/pgsql">pgSQL</option>
 * <option value="ace/mode/php_laravel_blade">PHP (Blade Template)</option>
 * <option value="ace/mode/php">PHP</option>
 * <option value="ace/mode/puppet">Puppet</option>
 * <option value="ace/mode/pig">Pig</option>
 * <option value="ace/mode/powershell">Powershell</option>
 * <option value="ace/mode/praat">Praat</option>
 * <option value="ace/mode/prolog">Prolog</option>
 * <option value="ace/mode/properties">Properties</option>
 * <option value="ace/mode/protobuf">Protobuf</option>
 * <option value="ace/mode/python">Python</option>
 * <option value="ace/mode/r">R</option>
 * <option value="ace/mode/razor">Razor</option>
 * <option value="ace/mode/rdoc">RDoc</option>
 * <option value="ace/mode/red">Red</option>
 * <option value="ace/mode/rhtml">RHTML</option>
 * <option value="ace/mode/rst">RST</option>
 * <option value="ace/mode/ruby">Ruby</option>
 * <option value="ace/mode/rust">Rust</option>
 * <option value="ace/mode/sass">SASS</option>
 * <option value="ace/mode/scad">SCAD</option>
 * <option value="ace/mode/scala">Scala</option>
 * <option value="ace/mode/scheme">Scheme</option>
 * <option value="ace/mode/scss">SCSS</option>
 * <option value="ace/mode/sh">SH</option>
 * <option value="ace/mode/sjs">SJS</option>
 * <option value="ace/mode/slim">Slim</option>
 * <option value="ace/mode/smarty">Smarty</option>
 * <option value="ace/mode/snippets">snippets</option>
 * <option value="ace/mode/soy_template">Soy Template</option>
 * <option value="ace/mode/space">Space</option>
 * <option value="ace/mode/sql">SQL</option>
 * <option value="ace/mode/sqlserver">SQLServer</option>
 * <option value="ace/mode/stylus">Stylus</option>
 * <option value="ace/mode/svg">SVG</option>
 * <option value="ace/mode/swift">Swift</option>
 * <option value="ace/mode/tcl">Tcl</option>
 * <option value="ace/mode/terraform">Terraform</option>
 * <option value="ace/mode/tex">Tex</option>
 * <option value="ace/mode/text">Text</option>
 * <option value="ace/mode/textile">Textile</option>
 * <option value="ace/mode/toml">Toml</option>
 * <option value="ace/mode/tsx">TSX</option>
 * <option value="ace/mode/twig">Twig</option>
 * <option value="ace/mode/typescript">Typescript</option>
 * <option value="ace/mode/vala">Vala</option>
 * <option value="ace/mode/vbscript">VBScript</option>
 * <option value="ace/mode/velocity">Velocity</option>
 * <option value="ace/mode/verilog">Verilog</option>
 * <option value="ace/mode/vhdl">VHDL</option>
 * <option value="ace/mode/visualforce">Visualforce</option>
 * <option value="ace/mode/wollok">Wollok</option>
 * <option value="ace/mode/xml">XML</option>
 * <option value="ace/mode/xquery">XQuery</option>
 * <option value="ace/mode/yaml">YAML</option>
 * <option value="ace/mode/django">Django</option>
 * </select>
 *
 */
export const possibleEditorHighlightPLanguages: ReadonlyArray<string> = [
  'io',
  'csound_document',
  'html',
  'rhtml',
  'html_ruby',
  'sh',
  'cirru',
  'jack',
  'haml',
  'vhdl',
  'prolog',
  'jsx',
  'powershell',
  'graphqlschema',
  'c_cpp',
  'liquid',
  'perl',
  'typescript',
  'verilog',
  'smarty',
  'gobstones',
  'red',
  'logiql',
  'dart',
  'd',
  'sass',
  'matlab',
  'ruby',
  'pgsql',
  'curly',
  'livescript',
  'xml',
  'mips_assembler',
  'drools',
  'plain_text',
  'gherkin',
  'mipsassembler',
  'mask',
  'sparql',
  'coldfusion',
  'java',
  'objectivec',
  'glsl',
  'lucene',
  'applescript',
  'ada',
  'lua',
  'django',
  'lisp',
  'xquery',
  'properties',
  'c9search',
  'soy_template',
  'tsx',
  'cobol',
  'tex',
  'html_elixir',
  'haskell_cabal',
  'r',
  'ejs',
  'mushcode',
  'nix',
  'jade',
  'handlebars',
  'csound_score',
  'erlang',
  'sqlserver',
  'ocaml',
  'abap',
  'fortran',
  'elm',
  'jssm',
  'hjson',
  'jsp',
  'haskell',
  'rdoc',
  'latex',
  'julia',
  'jsoniq',
  'javascript',
  'textile',
  'nsis',
  'lean',
  'protobuf',
  'dot',
  'space',
  'sjs',
  'less',
  'coffee',
  'praat',
  'asciidoc',
  'razor',
  'php',
  'ini',
  'sql',
  'ftl',
  'mysql',
  'toml',
  'scala',
  'rust',
  'groovy',
  'abc',
  'live_script',
  'apache_conf',
  'scad',
  'batchfile',
  'gcode',
  'csound_orchestra',
  'mavens_mate_log',
  'scheme',
  'rst',
  'autohotkey',
  'haxe',
  'clojure',
  'swig',
  'velocity',
  'wollok',
  'json',
  'csharp',
  'tcl',
  'eiffel',
  'stylus',
  'maze',
  'svg',
  'elixir',
  'twig',
  'lsl',
  'vbscript',
  'forth',
  'css',
  'luapage',
  'yaml',
  'gitignore',
  'pascal',
  'turtle',
  'vala',
  'kotlin',
  'golang',
  'diff',
  'snippets',
  'makefile',
  'python',
  'assembly_x86',
  'dockerfile',
  'text',
  'actionscript',
  'swift',
  'pig',
  'markdown',
  'scss',
  'mel',
  'bro',
].sort()


/**
 * returns the editor mode string and also requires the the editor mode async
 * seems as ace can require the langs by itself
 * @see SimpleAceEditor we set ace.config.set('modePath', '/static/ace-modes')
 * @param fileNameWithExtension
 * @param allPLangs
 */
export function getTabModeFromExtension(fileNameWithExtension: string, allPLangs: ReadonlyArray<PLangFullBase>): string {


  const index = fileNameWithExtension.lastIndexOf(".")

  if (index === -1) {
    return defaultEditorMode
  }

  const extensionWithDot = fileNameWithExtension.substr(index).trim()

  // tslint:disable-next-line
  let pLang: PLangFullBase | null = null

  // tslint:disable-next-line
  for(let i = 0; i < allPLangs.length;i++) {
    const _pLang = allPLangs[i]

    if (!_pLang.fileExtensionsWithDot) continue

    const fileExtensions = _pLang.fileExtensionsWithDot.split(',').map(p => p.trim())

    if (fileExtensions.indexOf(extensionWithDot) === -1) continue

    pLang = _pLang
    break
  }

  if (!pLang) return defaultEditorMode

  // if (knownPLangFileExtensionsMap.hasOwnProperty(extensionWithDot) === false) {
  //   return defaultEditorMode
  // }
  // return knownPLangFileExtensionsMap[extensionWithDot]

  if (possibleEditorHighlightPLanguages.indexOf(pLang.editorHighlightModeName) === -1) {
    Logger.warn('plang editorHighlightModeName was set but not found in possibleEditorHighlightPLanguages, using default')
    return defaultEditorMode
  }



  return pLang.editorHighlightModeName
}


interface TabLike {
  readonly id: number
  readonly displayIndex: number
}

export const aceTextLanguage = 'text'
export const aceRegexLanguage = 'regex_test'
export const aceTestProtocolLanguage = 'blackbox_test'
export const aceCompareFilesTestLanguage = 'blackbox_compareFiles_test'

/**
 * this only sets the display indices DOES NOT CHANGE THE VISIBILITY
 * @param {TabLike} targetFile
 * @param {boolean} isDisplayed
 * @param {ReadonlyArray<TabLike>} templateFiles
 * @param {number} selectedFileId
 * @param {(newFiles: ReadonlyArray<TabLike>) => any} setFilesAction
 * @param {(newTabId: number) => any} setSelectedEditorTabIdAction
 * @returns {MultiActions}
 */
export function changeTabVisibility(targetFile: TabLike,
                                    isDisplayed: boolean,
                                    templateFiles: ReadonlyArray<TabLike>,
                                    selectedFileId: number,
                                    setFilesAction: (newFiles: ReadonlyArray<TabLike>) => void,
                                    setSelectedEditorTabIdAction: (newTabId: number) => void,
): MultiActions {
  return dispatch => {

    //this would do nothing and it's a bug when we execute this because this would hide the file
    if (isDisplayed === true && targetFile.id === selectedFileId) return

    const newTempFiles = getNewTabDisplayIndices(targetFile, isDisplayed, templateFiles)

    dispatch(setFilesAction(newTempFiles))

    //set new selected tab if necessary
    const nextActiveTabId = getNewActiveTabId(targetFile, isDisplayed, selectedFileId, newTempFiles)

    //if null we closed a not active tab
    if (nextActiveTabId === null) return

    dispatch(setSelectedEditorTabIdAction(nextActiveTabId))

  }
}

/**
 * returns temp files with the new display indices
 * -x is not visible
 * +x is visible
 * @param targetFile
 * @param isDisplayed
 * @param templateFiles
 */
export function getNewTabDisplayIndices(targetFile: TabLike, isDisplayed: boolean,
                                        templateFiles: ReadonlyArray<TabLike>,
): ReadonlyArray<TabLike> {

  // noinspection TsLint
  let currDisplayIndex = 1
  // noinspection TsLint
  let currNotDisplayedIndex = -1

  const newTempFiles = templateFiles.map(file => {
    return file.id === targetFile.id
           ? {
        ...file,
        displayIndex: isDisplayed
                      ? currDisplayIndex++
                      : currNotDisplayedIndex--
      }
           : {
        ...file,
        displayIndex: (file.displayIndex >= 0)
                      ? currDisplayIndex++
                      : currNotDisplayedIndex--
      }
  })
  return newTempFiles
}


/**
 * returns the next active tab id
 * if we close a tab we try to set the file left of the current tab
 * @param targetFile
 * @param isDisplayed true: open a tab, false: close a tab
 * @param selectedFileId
 * @param newTempFiles
 */
export function getNewActiveTabId(targetFile: TabLike,
                                  isDisplayed: boolean,
                                  selectedFileId: number,
                                  newTempFiles: ReadonlyArray<TabLike>,
): number | null {


  if (targetFile.id === selectedFileId) {
    //now selected the next file... or nothing //close the active file //else we would not be here (see changeTabVisibility)

    const visibleFiles = newTempFiles.filter(p => p.displayIndex >= 0)

    if (visibleFiles.length === 0) {
      //no files is left open...
      return -1
    }
    else if (visibleFiles.length === 1) {

      if (visibleFiles[0].id === targetFile.id) { //we have only one file and this is the only one visible...
        return -1
      }

      return visibleFiles[0].id
    }

    const visibleFilesWithTargetFile = newTempFiles.filter(p => p.displayIndex >= 0 || p.id === targetFile.id)

    //we have at least 2 in visibleFilesWithTargetFile
    const targetFileIndex = visibleFilesWithTargetFile.findIndex(p => p.id === targetFile.id)

    if (targetFileIndex === 0) {
      return visibleFilesWithTargetFile[1].id
    }

    //return the file left of the file we close
    return visibleFilesWithTargetFile[targetFileIndex - 1].id

  } else if (targetFile.id !== selectedFileId && isDisplayed === true) {

    //we clicked a file that is not the current active file ... make it the active file
    return targetFile.id
  }

  return null
}

/**
 * checks if the options differs or not
 * @param {T} newSettings
 * @param {T} oldSettings
 * @returns {boolean}
 */
export function hasEditorSettingsChanged<T>(newSettings: T, oldSettings: T): boolean {

  if (!newSettings) return true
  if (!oldSettings) return true

  const oldKeys = Object.keys(oldSettings)
  const newKeys = Object.keys(newSettings)

  if (oldKeys.length !== newKeys.length) return true


  // noinspection TsLint
  for (const key in newSettings) {

    const oldKey = oldKeys.find(p => p === key)
    if (!oldKey) return false

    //check vals
    const newVal = newSettings[key]
    const oldVal = oldSettings[oldKey]

    if (newVal !== oldVal) return true

  }

  return false
}

export interface HasFileChangedMap {
  // noinspection TsLint
  /**
   * id: the file id,
   * bool: has changed?
   *
   * not readonly for performance
   */
  [id: string]: boolean
}

/**
 * updates (or creates) the hasFileChangedMap to display the has changed states in the editor
 * @param {EditorTab} changedFile
 * @param {ReadonlyArray<EditorTab>} allTabs
 * @param {HasFileChangedMap} hasFileChangedMap
 * @returns {HasFileChangedMap}
 */
export function updateHasFileChangedMap(
  changedFile: EditorTab, allTabs: ReadonlyArray<EditorTab>, hasFileChangedMap: HasFileChangedMap): HasFileChangedMap {

  // noinspection TsLint
  let copy: HasFileChangedMap = {}
  const currentFileKey = changedFile.id.toString()

  for (const tab of allTabs) {

    const key = tab.id.toString()
    // noinspection TsLint
    let hasChanged = hasFileChangedMap[key]

    if (hasChanged === undefined) {
      hasChanged = false
    }

    copy[key] = currentFileKey === key
                ? true
                : hasChanged
  }

  return copy

}


/**
 * DO NOT USE THIS !! only use this if really necessary
 *
 * this sets the frontend file ids to the backend file ids
 *
 * we cannot return a copy of the array else the editor undo won't work...
 * if we had only 1 version of the file in the backend then the id would stay
 * the same and this method would be not longer needed...
 *
 * @param {ReadonlyArray<SolutionFileDoExerciseFullBase>} frontendFiles
 * @param {ReadonlyArray<SolutionFileDoExerciseFullBase>} filesFromBackend
 * @param fileIdAssociationMap the map which stores which frontend id belongs to which backend saved file id
 * @param langId
 * @returns {ReadonlyArray<SolutionFileDoExerciseFullBase>}
 */
export function updateFilesReference(frontendFiles: ReadonlyArray<SolutionFileDoExerciseFullBase>,
                                     filesFromBackend: ReadonlyArray<SolutionFileDoExerciseFullBase>,
                                     fileIdAssociationMap: ReadonlyArray<FileIdAssociation>,
                                     langId: KnownLangs
): ReadonlyArray<SolutionFileDoExerciseFullBase> {


  //take the ids from the files from backend to the frontend files

  // noinspection TsLint
  for (let frontendFile of frontendFiles) {

    const fileIdTuple = fileIdAssociationMap.find(p => p.frontendFileId === frontendFile.id)

    if (fileIdTuple === undefined) {

      //we save the frontend version so if we create/delete a frontend file the backend should this too...
      //so normally this cannot happen

      Logger.debug('[editorHelper]', 'Could not find solution file in func updateFileReferencesAceStorage, frontend tab id: ' + frontendFile.id)

      errorDialog(getI18n(langId, 'Error'),
                  getI18n(langId, 'Could not find solution file'), langId
      )
      continue
    }

    const backendFile: SolutionFileDoExerciseFullBase = filesFromBackend.find(p => p.id === fileIdTuple.backendFileId)

    if (!backendFile) {

      Logger.debug('[editorHelper]', 'Could not find solution file in func updateFileReferencesAceStorage, backend tab id: ' + fileIdTuple.backendFileId)

      //TODO what on error?
      errorDialog(getI18n(langId, 'Error'),
                  getI18n(langId, 'Could not find solution file'), langId
      )
      continue
    }

    // @ts-ignore
    frontendFile.id = backendFile.id
    // @ts-ignore
    frontendFile.content = backendFile.content
    // @ts-ignore
    frontendFile.fileNameWithExtension = backendFile.fileNameWithExtension
    // @ts-ignore
    frontendFile.templateFileId = backendFile.templateFileId
    // @ts-ignore
    frontendFile.isEditableByUser = backendFile.isEditableByUser
  }

  return frontendFiles
}

/**
 * updates the stored tabs ids with the ones from backend
 * @param storedTabs
 * @param filesFromBackend
 * @param fileIdAssociationMap
 * @param langId
 */
export function updateFileReferencesAceStorage(storedTabs: ReadonlyArray<EditorWrapperTab>,
                                               filesFromBackend: ReadonlyArray<SolutionFileDoExerciseFullBase>,
                                               fileIdAssociationMap: ReadonlyArray<FileIdAssociation>,
                                               langId: KnownLangs
): void {
  //take the ids from the files from backend to the frontend files

  // noinspection TsLint
  for (let frontendFile of storedTabs) {

    const fileIdTuple = fileIdAssociationMap.find(p => p.frontendFileId === frontendFile.tab.id)

    if (fileIdTuple === undefined) {

      //we save the frontend version so if we create/delete a frontend file the backend should this too...
      //so normally this cannot happen

      Logger.debug('[editorHelper]', 'Could not find solution file in func updateFileReferencesAceStorage, frontend tab id: ' + frontendFile.tab.id)

      errorDialog(getI18n(langId, 'Error'),
                  getI18n(langId, 'Could not find solution file'), langId
      )
      continue
    }

    const backendFile: SolutionFileDoExerciseFullBase = filesFromBackend.find(p => p.id === fileIdTuple.backendFileId)

    if (!backendFile) {

      Logger.debug('[editorHelper]', 'Could not find solution file in func updateFileReferencesAceStorage, backend tab id: ' + fileIdTuple.backendFileId)

      //TODO what on error?
      errorDialog(getI18n(langId, 'Error'),
                  getI18n(langId, 'Could not find solution file'), langId
      )
      continue
    }

    // @ts-ignore
    frontendFile.tab.id = backendFile.id
  }

}

interface SomeTab {
  readonly id: number,
  readonly  content: string
}


//recalculates time is up stuff
export function updateExercisePreviews(previews: ReadonlyArray<ExercisePreviewsFrontendExclusive>,
                                       clientServerTimeDiffInS: number,
                                       langId: KnownLangs
): ReadonlyArray<ExercisePreviewsFrontendExclusive> {


  const serverNow: Moment = moment().add(clientServerTimeDiffInS, 'seconds')

  const copies = previews.map<ExercisePreviewsFrontendExclusive>((preview: ExercisePreviewsFrontendExclusive) => {
    // noinspection TsLint
    const availableWorkingTimeInSeconds: number = preview.availableWorkingTimeInMinutes * 60


    if (preview.firstAccessedAt && !preview.firstAccessedAt.isValid()) {
      Logger.error(
        `firstAccessedAt was invalid: ${preview.firstAccessedAt.toString()}(editorHelper, updateExercisePreviews)`)
    }

    const isAllowedToSolve = _userIsAllowedToSolveExercisePreview(preview, serverNow, langId)
    // noinspection TsLint
    let leftTimeString = ''
    // noinspection TsLint
    let leftTimeInS: number | null = null

    //set initial

    if (!preview.isReleased) {
      //we can solve this at any time
      const copy: ExercisePreviewsFrontendExclusive = {
        ...preview,
        isStillOpen: false,
        leftTimeInS
      }
      return copy
    }

    //we only need to display the left time if...
    // automaticEndAt (automatic end option is set)
    // or hasLimitedWorkingTime AND firstAccessedAt is set

    //we have an automatic end time that limits the left time
    //check for automatic start because we can set automaticEndAt && ReleaseDurationType.Limited and the change to manually...
    //in this case automaticEndAt && ReleaseDurationType.Limited should be ignored
    if (
      (preview.automaticEndAt !== null && preview.releaseDurationType === ReleaseDurationType.Limited && preview.releaseStartType === ReleaseStartType.Automatic)
      || (preview.hasLimitedWorkingTime && preview.firstAccessedAt)
    ) {

      // noinspection TsLint
      let diffBeforeReleaseEnd = Number.POSITIVE_INFINITY
      // noinspection TsLint
      let diffToWorkingTimeDeadlineInSeconds = Number.POSITIVE_INFINITY

      if (preview.firstAccessedAt && preview.hasLimitedWorkingTime) {
        // noinspection TsLint
        let diffInSeconds = serverNow.diff(preview.firstAccessedAt, 'seconds')
        diffToWorkingTimeDeadlineInSeconds = Math.floor(availableWorkingTimeInSeconds - diffInSeconds)
      }

      if (preview.automaticEndAt && preview.releaseDurationType === ReleaseDurationType.Limited
        && preview.releaseStartType === ReleaseStartType.Automatic
      ) {
        diffBeforeReleaseEnd = preview.automaticEndAt.diff(serverNow, 'seconds')
      }


      // noinspection TsLint
      let minDeadlineInS = Math.min(diffToWorkingTimeDeadlineInSeconds, diffBeforeReleaseEnd)

      if (minDeadlineInS === Number.POSITIVE_INFINITY) {
        Logger.warn("minDeadline was invalid (Number.POSITIVE_INFINITY) (editorHelper, updateExercisePreviews)")
        Logger.warn(`firstAccessedAt: ${preview.firstAccessedAt} (editorHelper, updateExercisePreviews)`)
        Logger.warn(`hasLimitedWorkingTime: ${preview.hasLimitedWorkingTime} (editorHelper, updateExercisePreviews)`)

        errorDialog(getI18n(langId, 'Error'), getI18n(langId, 'Something went wrong calculating left working time'), langId)

        const copy: ExercisePreviewsFrontendExclusive = {
          ...preview,
          leftTimeString: getI18n(langId, 'Error'),
          leftTimeInS: null,
          isStillOpen: false,
        }
        return copy

      } else if (isNaN(minDeadlineInS)) {

        Logger.warn("minDeadline was NaN (editorHelper, updateExercisePreviews)")
        Logger.warn(`firstAccessedAt: ${preview.firstAccessedAt} (editorHelper, updateExercisePreviews)`)
        Logger.warn(`hasLimitedWorkingTime: ${preview.hasLimitedWorkingTime} (editorHelper, updateExercisePreviews)`)
        Logger.warn(
          `because of the last error we give the user the full working time... if that was a mistake the server should throw an error`)

        errorDialog(getI18n(langId, 'Error'),
                    getI18n(langId,
                            'Something went wrong calculating left working time'
                    ), langId
        )

        // firstAccessedAt is invalid --> DateHelper.getDiffInSeconds(firstAccessedAt, now) is NaN
        leftTimeString = preview.availableWorkingTimeInMinutes + ' min'
        // noinspection TsLint
        leftTimeInS = preview.availableWorkingTimeInMinutes * 60

        const copy: ExercisePreviewsFrontendExclusive = {
          ...preview,
          leftTimeString,
          leftTimeInS,
          isStillOpen: true
        }
        return copy

      }


      const availableWorkingTimeInMinutesInS = preview.availableWorkingTimeInMinutes * 60
      //if the automatic end says 40 min left but the limited working time says 30 (and we haven't started yet... use the 30)
      if (!preview.firstAccessedAt && preview.hasLimitedWorkingTime && minDeadlineInS > availableWorkingTimeInMinutesInS) {
        minDeadlineInS = availableWorkingTimeInMinutesInS
      }

      leftTimeString = getLeftTimeString(minDeadlineInS, langId)
      leftTimeInS = minDeadlineInS

      if (minDeadlineInS <= 0) {
        //after the time is up we do not need to check for this any longer

        const copy: ExercisePreviewsFrontendExclusive = {
          ...preview,
          leftTimeString: getI18n(langId, 'Time is up'),
          leftTimeInS: 0,
          isStillOpen: false,
        }
        return copy

        //clearInterval(clearHandle)
      }

      const copy: ExercisePreviewsFrontendExclusive = {
        ...preview,
        leftTimeString,
        leftTimeInS,
        isStillOpen: minDeadlineInS > 0,
      }
      return copy

    }


    //user has full working time left
    leftTimeString = getLeftTimeString(preview.availableWorkingTimeInMinutes * 60, langId)
    // noinspection TsLint
    leftTimeInS = preview.availableWorkingTimeInMinutes * 60

    const copy: ExercisePreviewsFrontendExclusive = {
      ...preview,
      leftTimeString,
      leftTimeInS,
      isStillOpen: isAllowedToSolve,
    }
    return copy


  })

  return copies
}


const dayInSeconds = 86400
const hourInSeconds = 3600
const minuteInSeconds = 60

/**
 * converts the given seconds into a time string like:
 * 60s = 1 m
 * ... = 1 day 3 h 5 m
 * minimum is 1 min
 * @param leftTimeInS must not have fraction digits
 * @param langId
 */
export function getLeftTimeString(leftTimeInS: number, langId: KnownLangs): string {

  const days = Math.floor(leftTimeInS / dayInSeconds)

  leftTimeInS = days > 0
                    ? Math.floor(leftTimeInS % dayInSeconds) //why floor? if we use ints this is not needed?
                    : leftTimeInS

  const hours = Math.floor(leftTimeInS / hourInSeconds)

  leftTimeInS = hours > 0
                      ? Math.floor(leftTimeInS % hourInSeconds)
                      : leftTimeInS

  const minutes = Math.floor(leftTimeInS / minuteInSeconds)

  leftTimeInS = minutes > 0
                      ? Math.floor(leftTimeInS % minuteInSeconds)
                      : leftTimeInS

  // noinspection TsLint
  let result = `${days > 0 ? days.toString() + ' ' + (days === 1 ? getI18n(langId, 'day') : getI18n(langId, 'days')) + ' ' : ''}${days > 0 || hours > 0  ? padLeft(hours, 2) + ':' : ''}${padLeft(minutes, 2)}:${padLeft(leftTimeInS, 2)} `

  if (days === 0) {
      if (hours > 0) {
        result += getI18n(langId, 'h')
      } else {

        //if minutes is 0 then we display it anyways...
        //so m is always correct
        result += getI18n(langId, 'm')

        //this is not needed because 00:01 is always displayed...
        // if (minutes === 0) {
        //   result += getI18n(langId, 's')
        // } else {
        //   result += getI18n(langId, 'm')
        // }
      }
  }

  return result
}

function _userIsAllowedToSolveExercisePreview(preview: ExercisePreviewsFrontendExclusive, serverNow: Moment, langId: KnownLangs): boolean {

  if (!preview.isReleased) return false

  if (preview.isReleased && !preview.hasLimitedWorkingTime && preview.releaseStartType === ReleaseStartType.Manual) return true


  if (preview.firstAccessedAt){

    if (preview.hasLimitedWorkingTime) {
        //need to check
    }
    else if (preview.releaseStartType === ReleaseStartType.Automatic && preview.releaseDurationType === ReleaseDurationType.Limited) {
      //need to check automatic end
    }
    else {
      return true
    }
  }

  else {
    //preview.firstAccessedAt === null / not accessed yet

    //we can only check automatic end here

    //also check for automatic in case we switched back to manual the ReleaseDurationType.Limited could still be set
    if (preview.releaseStartType === ReleaseStartType.Automatic && preview.releaseDurationType === ReleaseDurationType.Limited) {
      //need to check automatic end
    }
    else {
      return true
    }

  }

  //TODO DUPLICATE see above
  if (preview.firstAccessedAt || preview.automaticEndAt || preview.hasLimitedWorkingTime) {

    // noinspection TsLint
    const availableWorkingTimeInSeconds: number = preview.availableWorkingTimeInMinutes * 60
    // noinspection TsLint
    let diffBeforeReleaseEnd = Number.POSITIVE_INFINITY
    // noinspection TsLint
    let diffToWorkingTimeDeadlineInSeconds = Number.POSITIVE_INFINITY

    if (preview.firstAccessedAt && preview.hasLimitedWorkingTime) {
      const diffInSeconds = serverNow.diff(preview.firstAccessedAt, 'seconds')
      diffToWorkingTimeDeadlineInSeconds = Math.floor(availableWorkingTimeInSeconds - diffInSeconds)
    }

    if (preview.automaticEndAt && preview.releaseStartType === ReleaseStartType.Automatic
      && preview.releaseDurationType === ReleaseDurationType.Limited) { //ReleaseDurationType.Limited --> preview.hasLimitedWorkingTime
      diffBeforeReleaseEnd = preview.automaticEndAt.diff(serverNow, 'seconds')
    }


    const minDeadline = Math.min(diffToWorkingTimeDeadlineInSeconds, diffBeforeReleaseEnd)


    if (minDeadline === Number.POSITIVE_INFINITY) {

      Logger.warn(
        "minDeadline was invalid (Number.POSITIVE_INFINITY) (editorHelper, _userIsAllowedToSolveExercisePreview)")
      Logger.warn(
        `firstAccessedAt: ${preview.firstAccessedAt.toString()} (editorHelper, _userIsAllowedToSolveExercisePreview)`)
      Logger.warn(
        `hasLimitedWorkingTime: ${preview.hasLimitedWorkingTime} (editorHelper, _userIsAllowedToSolveExercisePreview)`)

      errorDialog(getI18n(langId, 'Error'),
                  getI18n(langId,
                          'Something went wrong calculating left working time'
                  ), langId
      )

    } else if (isNaN(minDeadline)) {

      // firstAccessedAt is invalid --> DateHelper.getDiffInSeconds(firstAccessedAt, now) is NaN
      //assume user has the full working time

      Logger.warn(
        "minDeadline was NaN (Number.POSITIVE_INFINITY) (editorHelper, _userIsAllowedToSolveExercisePreview)")
      Logger.warn(
        `firstAccessedAt: ${preview.firstAccessedAt.toString()} (editorHelper, _userIsAllowedToSolveExercisePreview)`)
      Logger.warn(
        `hasLimitedWorkingTime: ${preview.hasLimitedWorkingTime} (editorHelper, _userIsAllowedToSolveExercisePreview)`)
      Logger.warn(
        `because of the last error we give the user the full working time... if that was a mistake the server should throw an error`)

      return true
    } else {

      if (minDeadline <= 0) {
        return false
      }
      //else still active/ open
    }
  }

  return true
}


/**
 * this is used so that the editor's componentWillUpdate method will be called with the right ForceEditorUpdateSessionDummy values
 * so that we get always a valid editor state (with undo, selection...)
 *
 *
 * if you wan't to call these funcs on your own see type MultiTabEditorState for usage
 *
 * @param {ReadonlyArray<SolutionFileDoExerciseFullBase>} solutionFiles
 * @param {(files: ReadonlyArray<SolutionFileDoExerciseFullBase>) => void} setSolutionFilesAction
 * @returns {AwaitActions}
 */
export function setSolutionFiles_AndRefreshEditor(solutionFiles: ReadonlyArray<ExerciseCodeTemplateFileFullBase>,
                                                  setSolutionFilesAction: (files: ReadonlyArray<ExerciseCodeTemplateFileFullBase>) => void,
): AwaitActions {
  return async (dispatch) => {


    dispatch(setSolutionFilesAction(solutionFiles))

    //if we don't wait then the editor won't re render and the change of MultiTabEditorState would not be noticed by
    //the editor...
    //TODO somehow make this better!!!

    // noinspection TsLint
    // await refreshEditorDelay()

  }
}



