import {
  PaginatedData,
  PaginationPostData,
  PaginationPostDataWithTagsFilter,
  PaginationStateData
} from '../types/pagination'
import {SortDirection} from '../types/sortDirection'
import {TagsConnectors} from '../state/reducers/tagsFilterReducer'


export class PaginationHelper {
  private constructor() {}


  /**
   * used to construct the pagination post data
   * @param state
   */
  public static getPaginationPostData<T>(state: PaginationStateData<T>): PaginationPostData {
    const paginationPostData: PaginationPostData = {
      isOrderByDesc: state.isOrderByDesc,
      page:state.currentPage,
      orderByProp: state.orderByProp as string,
      pageSize: state.pageSize,
      searchText: state.searchText
    }
    return paginationPostData
  }

  public static getPaginationPostDataWithTagsFilter<T>(state: PaginationStateData<T>, selectedFilterTagIds: ReadonlyArray<number>, selectedNegativeFilterTagsIds: ReadonlyArray<number>, logicalConnector: TagsConnectors): PaginationPostDataWithTagsFilter {
    const paginationPostData: PaginationPostDataWithTagsFilter = {
      isOrderByDesc: state.isOrderByDesc,
      page:state.currentPage,
      orderByProp: state.orderByProp as string,
      pageSize: state.pageSize,
      searchText: state.searchText,
      selectedFilterTagIds,
      selectedNegativeFilterTagsIds,
      logicalConnector
    }
    return paginationPostData
  }

  /**
   * usage:
   *
   * return {
   *   ...state,
   *   pagination: PaginationHelper.getPaginationStateSpreadObj(action.payload)
   * }
   * @param paginationData
   */
  public static getPaginationStateSpreadObj<T, TSorting>(paginationData: PaginatedData<T>): PaginationStateData<TSorting> {

    const stateData: PaginationStateData<TSorting> = {
      totalPages: paginationData.totalPages,
      pageSize: paginationData.pageSize,
      orderByProp: paginationData.orderByProp as any,
      isOrderByDesc: paginationData.isOrderByDesc,
      currentPage: paginationData.currentPage,
      searchText: paginationData.searchText,
      totalItems: paginationData.totalItems
    }

    return stateData
  }


  /**
   * map frontend to backend
   * @param sortDirection
   */
  public static getSortDirection(sortDirection: SortDirection | number): boolean | null {
    if (sortDirection === SortDirection.ascending) return false
    if (sortDirection === SortDirection.descending) return true

    return null
  }

}