/**
 * Created by janis dähne.
 */


/**
 * these are the names stored in the backend db
 * they are then translated into frontend (e.g. ace) themes
 */
export enum InternalCodeEditorThemes {
  light = 'light',
  dark = 'dark'
}

interface AvailableEditorThemesCollection {
  readonly light: string,
  readonly dark: string
}


//--- these are for the monaco editor...
// export const availableEditorThemeNames: AvailableEditorThemesCollection = {
//   light: {
//     'Standard Hell': 'vs'
//   },
//   dark: {
//     'Standard Dunkel': 'vs-dark'
//   },
// }

//--- these are for ace
//KEEP THE KEYS IN SYNC WITH InternalCodeEditorThemes
//if https://github.com/Microsoft/TypeScript/issues/2491 or https://github.com/Microsoft/TypeScript/issues/5683
//gets resolved then we can sync this with the editor
//see https://github.com/Microsoft/TypeScript/issues/13042 for current state
export const availableEditorThemeNames: AvailableEditorThemesCollection = {
  light: 'sqlserver',
  dark: 'monokai'
}

/**
 * tries to return the specific editor theme for the internal common theme name
 * of nothing is found then a default theme will be returned
 * @param {string} internalTheme
 * @returns {string}
 */
export function getEditorInternalThemeName(internalTheme: string): string {

  if (internalTheme === InternalCodeEditorThemes.light) {
    return availableEditorThemeNames[InternalCodeEditorThemes.light]
  }
  //else
  return availableEditorThemeNames[InternalCodeEditorThemes.dark]

}

/**
 * only the internal common theme name use getEditorInternalThemeName to get the real (editor specific) theme name
 * @type {InternalCodeEditorThemes}
 */
export const defaultEditorTheme: InternalCodeEditorThemes = InternalCodeEditorThemes.dark