/**
 * Created by janis dähne.
 */
import {
  ResetAction,
  SET_isTagsFilterDisplayedAction,
  SET_sortByKeyAction,
  SetActivePaginationPageAction,
  SetPaginationPageSizeAction,
  SetSearchTextAction
} from "../../../reducers/allAccessibleExercises/closedExercisesSite/closedExercisesSiteReducer";
import {
  getClosedExercisePreviewsViaCode,
  getClosedExercisePreviewsViaVisibility
} from "../../../../communicationLayer/exercisesOverviewLayer";
import {ActionType} from "../../../reducers/allAccessibleExercises/closedExercisesSite/closedExercisesSiteActionTypes";
import {ExercisePreviewFromBackend} from "../../../../types/exercisePreview";
import {AwaitActions, MultiActions} from "../../types";
import {loadManageTagsSite} from "../../manageTagsSite/manageTagsActions";
import {resetTagsFilter} from "../../tagsFilterActions";
import {GET_closedExercisesViaCodeAction} from '../../../reducers/allAccessibleExercises/closedExercisesSite/getClosedExercisesViaCodeCrudReducer'
import {delay} from '../../../../../debug'
import {GET_closedExercisesViaVisibilityAction} from '../../../reducers/allAccessibleExercises/closedExercisesSite/getClosedExercisesViaVisibilityCrudReducer'
import {PaginationPostDataWithTagsFilter} from '../../../../types/pagination'
import {PaginationHelper} from '../../../../helpers/paginationHelper'
import debounce from 'lodash-es/debounce'
import {searchInputDebounceInMs} from '../../../../constants'

export function loadClosedExerciseViaCodeAsync(): AwaitActions {
  return async (dispatch) => {

    dispatch(resetClosedExercisesSite())

    dispatch(resetTagsFilter())
    await dispatch(loadManageTagsSite())

    await dispatch(getClosedExercisesViaCode())
  }
}

export function loadClosedExerciseViaVisibilityAsync(): AwaitActions {
  return async (dispatch) => {

    dispatch(resetClosedExercisesSite())

    dispatch(resetTagsFilter())
    await dispatch(loadManageTagsSite())


    await dispatch(getClosedExercisesViaVisibility())
  }
}

export function _getClosedExercisesViaCode(paginationPostData: PaginationPostDataWithTagsFilter): GET_closedExercisesViaCodeAction {
  return {
    type: ActionType.GET_closedExercisesViaCode,
    payload: delay(getClosedExercisePreviewsViaCode(paginationPostData))
  }
}

export function getClosedExercisesViaCode(): MultiActions {
  return (dispatch, getState) => {

    const state = getState().closedExercisesState
    const tagsState = getState().tagsFilterState

    const paginationPostData = PaginationHelper.getPaginationPostDataWithTagsFilter(state.paginationViaCode,
                                                                                    tagsState.selectedFilterTagsIds,
                                                                                    tagsState.selectedNegativeFilterTagsIds,
                                                                                    tagsState.logicalConnector
    )
    dispatch(_getClosedExercisesViaCode(paginationPostData))
  }
}

export function _getClosedExercisesViaVisibility(paginationPostData: PaginationPostDataWithTagsFilter): GET_closedExercisesViaVisibilityAction {
  return {
    type: ActionType.GET_closedExercisesViaVisibility,
    payload: delay(getClosedExercisePreviewsViaVisibility(paginationPostData))
  }
}

export function getClosedExercisesViaVisibility(): MultiActions {
  return (dispatch, getState) => {

    const state = getState().closedExercisesState
    const tagsState = getState().tagsFilterState

    const paginationPostData = PaginationHelper.getPaginationPostDataWithTagsFilter(
      state.paginationViaVisibility,
      tagsState.selectedFilterTagsIds,
      tagsState.selectedNegativeFilterTagsIds,
      tagsState.logicalConnector
    )
    dispatch(_getClosedExercisesViaVisibility(paginationPostData))
  }
}


export function setIsTagsFilterDisplayed(isTagsFilterDisplayed: boolean): SET_isTagsFilterDisplayedAction {
  return {
    type: ActionType.SET_isTagsFilterDisplayed,
    isTagsFilterDisplayed,
  }
}


export function resetClosedExercisesSite(): ResetAction {
  return {
    type: ActionType.RESET
  }
}


//--- pagination

export function _setSortByKey(sortByKey: keyof ExercisePreviewFromBackend,
                              sortDirection: boolean | null,
                              viaCode: boolean
): SET_sortByKeyAction {
  return {
    type: ActionType.SET_sortByKey,
    sortByKey,
    sortDirection,
    viaCode
  }
}

export function setSortByKey(sortByKey: keyof ExercisePreviewFromBackend,
                             sortDirection: boolean | null,
                             viaCode: boolean
): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setSortByKey(sortByKey, sortDirection, viaCode))
    dispatch(viaCode
             ? getClosedExercisesViaCode()
             : getClosedExercisesViaVisibility())

  }
}

export function _setActivePaginationPage(activePage: number, viaCode: boolean): SetActivePaginationPageAction {
  return {
    type: ActionType.setPaginationActivePage,
    activePage,
    viaCode
  }
}

export function setActivePaginationPage(activePage: number, viaCode: boolean): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setActivePaginationPage(activePage, viaCode))
    dispatch(viaCode
             ? getClosedExercisesViaCode()
             : getClosedExercisesViaVisibility())
  }
}

export function _setPaginationPageSize(pageSize: number, viaCode: boolean): SetPaginationPageSizeAction {
  return {
    type: ActionType.setPaginationPageSize,
    pageSize,
    viaCode
  }
}

export function setPaginationPageSize(pageSize: number, viaCode: boolean): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setPaginationPageSize(pageSize, viaCode))
    dispatch(viaCode
             ? getClosedExercisesViaCode()
             : getClosedExercisesViaVisibility())
  }
}

export function _setSearchText(searchText: string, viaCode: boolean): SetSearchTextAction {
  return {
    type: ActionType.setSearchText,
    searchText,
    viaCode
  }
}

// noinspection TsLint
const debouncedRefresh = debounce((dispatch, getState, viaCode: boolean) => {

  dispatch(viaCode
           ? getClosedExercisesViaCode()
           : getClosedExercisesViaVisibility())

}, searchInputDebounceInMs)

export function setSearchText(searchText: string, viaCode: boolean): MultiActions {
  return (dispatch, getState) => {
    dispatch(_setSearchText(searchText, viaCode))

    debouncedRefresh(dispatch, getState, viaCode)
  }
}