/**
 * Created by janis dähne.
 */
import {GET_assessmentStatisticsAction} from "../../reducers/assessmentStatisticsSite/getAssessmentStatistics";
import {AwaitActions, MultiActions} from "../types";
import {ActionType} from "../../reducers/assessmentStatisticsSite/assessmentStatisticsSiteActionTypes";
import {delay} from "../../../../debug";
import {getFullAssessmentStatistics} from "../../../communicationLayer/assessmentStatisticsLayer";
import {
  ResetAction, SET_displayManualPointsLineAction,
  SET_displayNormalTestLineAction,
  SET_displaySubmitTestLineAction,
  SET_sortByKeyAction,
  SET_targetUserIdAction,
  SetActivePaginationPageAction,
  SetPaginationPageSizeAction,
  SetSearchTextAction
} from "../../reducers/assessmentStatisticsSite/assessmentStatisticsSiteReducer";
import {PaginationPostData} from '../../../types/pagination'
import {PaginationHelper} from '../../../helpers/paginationHelper'
import debounce from 'lodash-es/debounce'
import {searchInputDebounceInMs} from '../../../constants'
import {SingleAssessmentStatistic} from '../../../types/assessmentStatistics'
import {getPLangsAsync} from '../plangSettingsView/pLangsCrudActions'


export function setDisplaySubmitTestLine(displaySubmitTestLine: boolean): SET_displaySubmitTestLineAction {
  return {
    type: ActionType.SET_displaySubmitTestLine,
    displaySubmitTestLine,
  }
}

export function setDisplayNormalTestLine(displayNormalTestLine: boolean): SET_displayNormalTestLineAction {
  return {
    type: ActionType.SET_displayNormalTestLine,
    displayNormalTestLine,
  }
}

export function setDisplayManualPointsLine(displayManualPointsLine: boolean): SET_displayManualPointsLineAction {
  return {
    type: ActionType.SET_displayManualPointsLine,
    displayManualPointsLine,
  }
}


export function _getAssessmentStatisticsAsync(targetUserId: number, paginationPostData: PaginationPostData): GET_assessmentStatisticsAction {
  return {
    type: ActionType.GET_assessmentStatistics,
    payload: delay(getFullAssessmentStatistics(targetUserId, paginationPostData))
  }
}

export function getAssessmentStatisticsAsync(): MultiActions {
  return (dispatch, getState) => {

    const targetUserId = getState().assessmentStatisticsSiteState.targetUserId
    const paginationPostData = PaginationHelper.getPaginationPostData(getState().assessmentStatisticsSiteState.pagination)
    dispatch(_getAssessmentStatisticsAsync(targetUserId, paginationPostData))
  }
}
export function setTargetUserId(targetUserId: number): SET_targetUserIdAction {
  return {
    type: ActionType.SET_targetUserId,
    targetUserId
  }
}


export function loadDoExerciseSite(targetUserId: number): AwaitActions {

  return async (dispatch, getState) => {

    dispatch(resetAssessmentStatisticsSite())

    dispatch(setTargetUserId(targetUserId))

    await dispatch(getPLangsAsync())

    await dispatch(getAssessmentStatisticsAsync())

  }
}



export function resetAssessmentStatisticsSite(): ResetAction {
  return {
    type: ActionType.RESET,
  }
}


//--- pagination


export function _setSortByKey(sortByKey: keyof SingleAssessmentStatistic,
                              sortDirection: boolean | null,
): SET_sortByKeyAction {
  return {
    type: ActionType.SET_sortByKey,
    sortByKey,
    sortDirection
  }
}

export function setSortByKey(sortByKey: keyof SingleAssessmentStatistic,
                             sortDirection: boolean | null,
): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setSortByKey(sortByKey, sortDirection))
    dispatch(getAssessmentStatisticsAsync())

  }
}

export function _setActivePaginationPage(activePage: number): SetActivePaginationPageAction {
  return {
    type: ActionType.setPaginationActivePage,
    activePage
  }
}

export function setActivePaginationPage(activePage: number): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setActivePaginationPage(activePage))
    dispatch(getAssessmentStatisticsAsync())
  }
}

export function _setPaginationPageSize(pageSize: number): SetPaginationPageSizeAction {
  return {
    type: ActionType.setPaginationPageSize,
    pageSize
  }
}

export function setPaginationPageSize(pageSize: number): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setPaginationPageSize(pageSize))

    dispatch(getAssessmentStatisticsAsync())
  }
}

export function _setSearchText(searchText: string,): SetSearchTextAction {
  return {
    type: ActionType.setSearchText,
    searchText
  }
}

// noinspection TsLint
const debouncedRefresh = debounce((dispatch, getState) => {

  dispatch(getAssessmentStatisticsAsync())

}, searchInputDebounceInMs)

export function setSearchText(searchText: string): MultiActions {
  return (dispatch, getState) => {
    dispatch(_setSearchText(searchText))

    debouncedRefresh(dispatch, getState)
  }
}
