import {ActionType} from '../../reducers/doExerciseSite/doExerciseSiteActionTypes'
import {delay} from '../../../../debug'
import {GET_doExercise_afterSolutionAction} from '../../reducers/doExerciseSite/crud/afterSolution/getAfterSolutionReducer'
import {
  createOrUpdateAfterSolution,
  getDoExerciseAfterSolution
} from '../../../communicationLayer/doExerciseAfterSolutionLayer'
import {getI18n, KnownLangs} from '../../../../i18n/i18nRoot'
import {AwaitActionsWithResult, MultiActions} from '../types'
import {errorDialog} from '../../../helpers/dialogHelper'
import globalState from '../../state'
import {UPDATE_doExerciseSolutionAction} from '../../reducers/doExerciseSite/crud/updateDoEditorSolutionReducer'
import {AfterSolutionDoExerciseFullBase} from '../../../types/exerciseAfterSolution'
import {setAfterEditor_hasFileChangedMap, setAfterEditor_hasUnsavedChanges} from './editorActions/afterEditorActions'
import {UPDATE_doExercise_afterSolutionAction} from '../../reducers/doExerciseSite/crud/afterSolution/updateAfterSolution'
import {resetDoExerciseAfterTestResults, setAfterSolutionFirstDummy} from './doExerciseSiteActions'
import {getNextFrontendId} from '../../../helpers/frontendIdHelper'
import {RunnableAfterCustomTestFrontendExclusive} from '../../../types/customTests'
import {setAfterCustomTests} from './sub/afterCustomTestActions'
import {ErrorHelper} from '../../../helpers/errorHelper'


export function getDoExerciseAfterSolutionAsync(releaseCode: string, pLangId: number): GET_doExercise_afterSolutionAction {
  return {
    type: ActionType.GET_doExercise_afterSolution,
    payload: delay(getDoExerciseAfterSolution(releaseCode, pLangId))
  }
}


export function _createFirstDummyAfterSolution(releaseCode: string, pLangId: number): MultiActions {
  return (dispatch, getState) => {

    const currSolution = getState().doExerciseSiteState.solution
    const firstAfterSolution: AfterSolutionDoExerciseFullBase = {
      id: getNextFrontendId(),
      releaseId: currSolution.releaseId,
      userId: currSolution.userId,
      pLangId: currSolution.pLangId,
      mainFileId: currSolution.mainFileId,
      solutionFiles: [...currSolution.solutionFiles],
      lastUpdatedAt: currSolution.lastUpdatedAt
    }

    dispatch(setAfterSolutionFirstDummy(firstAfterSolution))
    //when a user clicks on execute test then we save the dummy solution
    dispatch(setAfterEditor_hasUnsavedChanges(true))

    //when we create a dummy solution then we might got here because time was used up...
    //because of this we might not have all after custom tests...
    //we set the after custom tests when we load the page (e..g there were none at page load)
    //then the user create one and then the time used up
    //we never change the after custom tests so we are missing the created custom test...
    //because we have no after solution there can't be some saved test results so just copy the custom tests

    const newAfterCustomTests = getState().doExerciseSiteState.exercise.customTests.map<RunnableAfterCustomTestFrontendExclusive>(p => {
      return {
        ...p,
        uiIsRunning: false,
        result: null,
        uiIsAfterTest: true
      }
    })

    dispatch(setAfterCustomTests(newAfterCustomTests))

  }
}

/**
 * this also reset has unsaved changes
 * @param {string} releaseCode
 * @param plangId
 * @param shouldClearAllTestResults
 * @param afterSolution
 * @param langId
 * @returns {UPDATE_doExerciseSolutionAction}
 */
export function saveDoExerciseAfterSolutionAsync(releaseCode: string, plangId: number, shouldClearAllTestResults: boolean, afterSolution: AfterSolutionDoExerciseFullBase, langId: KnownLangs): AwaitActionsWithResult<boolean> {
  return async (dispatch) => {

    try {
      await dispatch(_saveDoExerciseAfterSolution(releaseCode,plangId, shouldClearAllTestResults, afterSolution, langId))
    } catch(err) {

      ErrorHelper.makeServerErrorDialog(err, globalState.getState().i18nState.langId, 'Error', 'Could not save files')
      return false
    }

    if (shouldClearAllTestResults) {
      dispatch(resetDoExerciseAfterTestResults())
    }

    dispatch(setAfterEditor_hasUnsavedChanges(false))
    dispatch(setAfterEditor_hasFileChangedMap({}))


    return true
  }
}

export function _saveDoExerciseAfterSolution(releaseCode: string,plangId: number, shouldClearAllTestResults: boolean,  afterSolution: AfterSolutionDoExerciseFullBase, langId: KnownLangs): UPDATE_doExercise_afterSolutionAction {
  return {
    type: ActionType.UPDATE_doExercise_afterSolution,
    payload: delay(createOrUpdateAfterSolution(releaseCode, plangId, shouldClearAllTestResults, afterSolution)),
    meta: {
      langId
    }
  }
}
