/**
 * Created by janis dähne.
 */
import {
  ResetAction, SET_editorSettingsAction, SET_setIsDialogDisplayedActionCallbackAction,
  SetIsDialogDisplayedActionCallback
} from "../../reducers/editorSettingsDialog/editorSettingsDialogReducer";
import {ActionType} from "../../reducers/editorSettingsDialog/editorSettingsDialogActionTypes";
import {AwaitActions, MultiActions} from "../types";
import {CodeEditorSettingsFullBase, UserSettingsFullBase} from "../../../types/userData";
import {updateUserSettingsAsync} from "../settingsSite/userSettingsSiteCrudActions";





export function setEditorSettings(editorSettings: CodeEditorSettingsFullBase): SET_editorSettingsAction {
  return {
    type: ActionType.SET_editorSettings,
    editorSettings
  }
}

//this is used that the dialog knows how to close itself
export function setSetIsDialogDisplayedActionCallback(setIsDialogDisplayedActionCallback: SetIsDialogDisplayedActionCallback): SET_setIsDialogDisplayedActionCallbackAction {
  return {
    type: ActionType.SET_setIsDialogDisplayedActionCallback,
    setIsDialogDisplayedActionCallback
  }
}


export function openEditEditorSettingsDialog(editorSettings: CodeEditorSettingsFullBase | null,
                                             setIsDialogDisplayedAction: SetIsDialogDisplayedActionCallback): MultiActions {
  return dispatch => {

    dispatch(resetChangeEditorSettingsDialog())

    if (editorSettings !== null) {
      dispatch(setEditorSettings(editorSettings))
    }

    dispatch(setSetIsDialogDisplayedActionCallback(setIsDialogDisplayedAction))

    dispatch(setIsDialogDisplayedAction(true))
  }
}

export function applyChangedEditorSettingsDialogAsync(editorSettings: CodeEditorSettingsFullBase,
                                                      userSettings: UserSettingsFullBase | null,
                                                      setIsDialogDisplayedAction: SetIsDialogDisplayedActionCallback): AwaitActions {
  return async (dispatch) => {

    const newUserSettings: UserSettingsFullBase = {
      ...userSettings,
      codeEditorSettings: editorSettings
    }

    //use the user settings site action because we use the settings from there
    await dispatch(updateUserSettingsAsync(newUserSettings))

    dispatch(resetChangeEditorSettingsDialog())
    dispatch(setIsDialogDisplayedAction(false))
  }
}

export function cancelEditorSettingsDialog(setIsDialogDisplayedAction: SetIsDialogDisplayedActionCallback): MultiActions {
  return dispatch => {

    dispatch(resetChangeEditorSettingsDialog())
    dispatch(setIsDialogDisplayedAction(false))
  }
}


export function resetChangeEditorSettingsDialog(): ResetAction {
  return {
    type: ActionType.RESET
  }
}