/**
 * Created by janis dähne.
 */
import {ActionType} from "../../../../reducers/exerciseEditorSite/dialogs/changeCodeTemplateDialog/changeCodeTemplateDialogActionTypes";
import {ExerciseCodeTemplateFileFullBase, ExerciseCodeTemplateFullBase} from "../../../../../types/exerciseEditor";
import {
  ResetAction,
  SET_changeCodeTemplateNewFileNameAction,
  SET_hasFileChangedMapAction,
  SET_hasUnsavedChangesAction,
  SET_isCreatingAction,
  SET_isMonacoFileExplorerDisplayedAction,
  SET_isRenamingCodeTemplateFileAction,
  SET_renamingCodeTemplateFileNameAction,
  SET_renamingFileIdCodeTemplateAction
} from "../../../../reducers/exerciseEditorSite/dialogs/changeCodeTemplateDialog/changeCodeTemplateDialogReducer";
import {MultiActions} from "../../../types";
import {
  setChangCodeTemplate,
  setSelectedEditorTabIdChangeCodeTemplate,
  setTemplateFiles
} from "./changeCodeTemplateActions";
import {setIsChangeCodeTemplateDialogDisplayed} from "../../exerciseEditorSiteActions";
import {setCodeTemplates} from "../../subSets/editorExerciseCodeTemplatesActions";
import {changeTabVisibility, HasFileChangedMap} from "../../../../../helpers/editorHelper";


export function setHasFileChangedMapExerciseEditor(hasFileChangedMap: HasFileChangedMap): SET_hasFileChangedMapAction {
  return {
    type: ActionType.SET_hasFileChangedMap,
    hasFileChangedMap
  }
}

export function setHasUnsavedChanges(hasUnsavedChanges: boolean): SET_hasUnsavedChangesAction {
  return {
    type: ActionType.SET_hasUnsavedChanges,
    hasUnsavedChanges
  }
}

export function setIsCreatingCodeTemplate(isCreating: boolean): SET_isCreatingAction {
  return {
    type: ActionType.SET_isCreating,
    isCreating
  }
}

export function setIsMonacoFileExplorerDisplayed(isMonacoFileExplorerDisplayed: boolean): SET_isMonacoFileExplorerDisplayedAction {
  return {
    type: ActionType.SET_isMonacoFileExplorerDisplayed,
    isMonacoFileExplorerDisplayed
  }
}

export function setChangeCodeTemplateNewFileName(changeCodeTemplateNewFileName: string): SET_changeCodeTemplateNewFileNameAction {
  return {
    type: ActionType.SET_changeCodeTemplateNewFileName,
    changeCodeTemplateNewFileName
  }
}
export function setRenamingFileIdCodeTemplate(renamingFileIdCodeTemplate: number): SET_renamingFileIdCodeTemplateAction {
  return {
    type: ActionType.SET_renamingFileIdCodeTemplate,
    renamingFileIdCodeTemplate
  }
}



export function setIsRenamingCodeTemplateFile(isRenamingCodeTemplateFile: boolean): SET_isRenamingCodeTemplateFileAction {
  return {
    type: ActionType.SET_isRenamingCodeTemplateFile,
    isRenamingCodeTemplateFile
  }
}

export function setRenamingCodeTemplateFileName(renamingCodeTemplateFileName: string): SET_renamingCodeTemplateFileNameAction {
  return {
    type: ActionType.SET_renamingCodeTemplateFileName,
    renamingCodeTemplateFileName
  }
}



export function openTabChangeCodeTemplate(file: ExerciseCodeTemplateFileFullBase,
                                          templateFiles: ReadonlyArray<ExerciseCodeTemplateFileFullBase>,
                                          selectedFileId: number): MultiActions {
  return changeTabVisibilityChangeCodeTemplate(file, true, templateFiles, selectedFileId)
}

export function closeTabChangeCodeTemplate(file: ExerciseCodeTemplateFileFullBase,
                                           templateFiles: ReadonlyArray<ExerciseCodeTemplateFileFullBase>,
                                           selectedFileId: number): MultiActions {
  return changeTabVisibilityChangeCodeTemplate(file, false, templateFiles, selectedFileId)
}


export function changeTabVisibilityChangeCodeTemplate(targetFile: ExerciseCodeTemplateFileFullBase,
                                                      isDisplayed: boolean,
                                                      templateFiles: ReadonlyArray<ExerciseCodeTemplateFileFullBase>,
                                                      selectedFileId: number): MultiActions {

  return changeTabVisibility(targetFile, isDisplayed, templateFiles, selectedFileId, setTemplateFiles, setSelectedEditorTabIdChangeCodeTemplate)
  //TODO remove
  // return dispatch => {
  //
  //   //this would do nothing and it's a bug when we execute this because this would hide the file
  //   if (isDisplayed === true && targetFile.id === selectedFileId) return
  //
  //   // noinspection TsLint
  //   let currDisplayIndex = 1
  //   // noinspection TsLint
  //   let currNotDisplayedIndex = -1
  //
  //   const newTempFiles = templateFiles.map(file => {
  //     return file.id === targetFile.id
  //       ? {
  //         ...file,
  //         displayIndex: isDisplayed ? currDisplayIndex++ : currNotDisplayedIndex--
  //       }
  //       : {
  //         ...file,
  //         displayIndex: (file.displayIndex >= 0) ? currDisplayIndex++ : currNotDisplayedIndex--
  //       }
  //   })
  //
  //   dispatch(setTemplateFiles(newTempFiles))
  //
  //   //set new selected tab if necessary
  //
  //   if (targetFile.id === selectedFileId) {
  //     //now selected the next file... or nothing
  //
  //     const visibleFiles = newTempFiles.filter(p => p.displayIndex >= 0)
  //
  //     if (visibleFiles.length === 0) {
  //       //no files is left open...
  //       dispatch(setSelectedEditorTabIdChangeCodeTemplate(-1))
  //     } else {
  //
  //       const visibleFile = getMaxEntity(visibleFiles, p => p.displayIndex)
  //
  //       dispatch(setSelectedEditorTabIdChangeCodeTemplate(visibleFile.id))
  //     }
  //   } else if (targetFile.id !== selectedFileId && isDisplayed === true) {
  //
  //     //we clicked a file that is not the current active file ... make it the active file
  //     dispatch(setSelectedEditorTabIdChangeCodeTemplate(targetFile.id))
  //
  //   }
  // }
}


export function openCreateCodeTemplateDialog(): MultiActions {
  return dispatch => {

    dispatch(resetChangeCodeTemplateDialog())
    dispatch(setIsCreatingCodeTemplate(true))

    dispatch(setIsChangeCodeTemplateDialogDisplayed(true))
  }
}

export function openEditCodeTemplateDialog(codeTemplate: ExerciseCodeTemplateFullBase): MultiActions {
  return dispatch => {

    dispatch(resetChangeCodeTemplateDialog())
    dispatch(setIsCreatingCodeTemplate(false))
    dispatch(setChangCodeTemplate(codeTemplate))

    //select first file if we have one

    if (codeTemplate.templateFiles.length > 0 && codeTemplate.templateFiles[0].displayIndex >= 0) {
      dispatch(setSelectedEditorTabIdChangeCodeTemplate(codeTemplate.templateFiles[0].id))
    }

    dispatch(setIsChangeCodeTemplateDialogDisplayed(true))
  }
}

export function applyChangCodeTemplateDialog(codeTemplate: ExerciseCodeTemplateFullBase, isCreating: boolean,
                                             allCodeTemplates: ReadonlyArray<ExerciseCodeTemplateFullBase>): MultiActions {
  return (dispatch) => {

    if (isCreating) {

      dispatch(setCodeTemplates(
        [...allCodeTemplates, {...codeTemplate}]
      ))

    } else {

      dispatch(setCodeTemplates(
        allCodeTemplates.map(p => {
          return p.id === codeTemplate.id ?
            {
              ...codeTemplate
            }
            : p
        })
      ))
    }


    dispatch(resetChangeCodeTemplateDialog())
    dispatch(setIsChangeCodeTemplateDialogDisplayed(false))
  }
}

export function cancelEditorTestDialog(): MultiActions {
  return dispatch => {

    dispatch(resetChangeCodeTemplateDialog())
    dispatch(setIsChangeCodeTemplateDialogDisplayed(false))
  }
}

export function resetChangeCodeTemplateDialog(): ResetAction {
  return {
    type: ActionType.RESET,
  }
}