/**
 * Created by janis dähne.
 */
import {GET_editorExerciseAction} from "../../reducers/exerciseEditorSite/getEditorExerciseReducer";
import {ActionType} from "../../reducers/exerciseEditorSite/editorExerciseSiteActionTypes";
import {delay} from "../../../../debug";
import {
  attachExitingFileToExerciseDescription, attachExitingFileToExerciseTest,
  createEditorExercise,
  detachFileFromExerciseDescription, detachFileFromExerciseTest,
  getEditorExercise,
  getPossibleExerciseDescriptionFiles, getPossibleExerciseTestFiles, getSingleExerciseDescriptionAssetFile,
  lockEditorExercisePermanently,
  updateEditorExercise
} from "../../../communicationLayer/exerciseEditorLayer";
import {CREATE_editorExerciseAction} from "../../reducers/exerciseEditorSite/createEditorExerciseReducer";
import {EditorExerciseForBackend} from "../../../types/exerciseEditor";
import {UPDATE_editorExerciseAction} from "../../reducers/exerciseEditorSite/updateEditorExerciseReducer";
import {AwaitActions, AwaitActionsWithResult, MultiActions} from "../types";
import {createAndRefreshAssetUrlsAndObjects} from "../../../helpers/assetHelper";
import Logger from "../../../helpers/logger";
import {LOCK_exercisePermanentlyAction} from '../../reducers/exerciseEditorSite/crud/lockExercisePermanentlyReducer'
import {ATTACH_markdownAssetAction} from '../../reducers/exerciseEditorSite/crud/assets/attachMarkdownAssetPreviewReducer'
import {GET_markdownAssetPreviewsAction} from '../../reducers/exerciseEditorSite/crud/assets/getMarkdownAssetPreviewsReducer'
import {DETACH_markdownAssetPreviewAction} from '../../reducers/exerciseEditorSite/crud/assets/detachMarkdownAssetPreviewReducer'
import {LOAD_markdownAssetAction} from '../../reducers/exerciseEditorSite/crud/assets/loadSingleMarkdownAssetReducer'
import {setExerciseEditorAssets} from './subSets/editorExerciseDescriptionActions'
import {GET_testAssetPreviewsAction} from '../../reducers/exerciseEditorSite/crud/assets/getTestAssetPreviewsReducer'
import {ATTACH_testAssetAction} from '../../reducers/exerciseEditorSite/crud/assets/attachTestAssetPreviewReducer'
import {DETACH_testAssetAction} from '../../reducers/exerciseEditorSite/crud/assets/detachTestAssetPreviewReducer'


export function getEditorExerciseAsync(exerciseId: number): AwaitActions {
  return async(dispatch, getState) => {

    try {
      await dispatch(_getEditorExerciseAsync(exerciseId))
    } catch(err) {
      return
    }

    //create asset urls
    createAndRefreshAssetUrlsAndObjects(getState().exerciseEditorState.editorExercise.exerciseDescription.assets)
  }
}


function _getEditorExerciseAsync(exerciseId: number): GET_editorExerciseAction {
  return {
    type: ActionType.GET_editorExercise,
    payload: delay(getEditorExercise(exerciseId))
  }
}

export function createEditorExerciseAsync(exercise: EditorExerciseForBackend): AwaitActionsWithResult<boolean> {
  return async(dispatch, getState) => {

    try {
      await dispatch(_createEditorExerciseAsync(exercise))
    } catch(err) {
      Logger.error(`could not create new exercise, err: ${err.message}`)
      return false
    }

    return true
  }
}

export function _createEditorExerciseAsync(exercise: EditorExerciseForBackend): CREATE_editorExerciseAction {
  return {
    type: ActionType.CREATE_editorExercise,
    payload: delay(createEditorExercise(exercise))
  }
}

export function updateEditorExerciseAsync(exercise: EditorExerciseForBackend): AwaitActionsWithResult<boolean> {
  return async(dispatch, getState) => {

    try {
      await dispatch(_updateEditorExerciseAsync(exercise))
    } catch(err) {
      Logger.error(`could not update exercise, err: ${err.message}`)
      return false
    }

    return true
  }
}

export function _updateEditorExerciseAsync(exercise: EditorExerciseForBackend): UPDATE_editorExerciseAction {
  return {
    type: ActionType.UPDATE_editorExercise,
    payload: delay(updateEditorExercise(exercise))
  }
}

export function lockEditorExercisePermanentlyAsync(exercise: EditorExerciseForBackend): LOCK_exercisePermanentlyAction {
  return {
    type: ActionType.LOCK_exercisePermanently,
    payload: delay(lockEditorExercisePermanently(exercise))
  }
}


//--- exercise description / markdown asset files

export function getMarkdownAssetPreviewsAsync(exerciseId: number): GET_markdownAssetPreviewsAction {
  return {
    type: ActionType.GET_markdownAssetPreviews,
    payload: delay(getPossibleExerciseDescriptionFiles(exerciseId))
  }
}

export function attachExitingFileToExerciseDescriptionAsync(exerciseId: number, fileReferenceId: number, onProgress?: (percentage: number) => void): AwaitActions {
  return async (dispatch, getState) => {

    await dispatch(_attachExitingFileToExerciseDescriptionAsync(exerciseId, fileReferenceId))

    //after we connected the file to the exercise description...
    //we need to load it so it can be used!

    Logger.debug("exerciseEditorCrudActions", `after attaching file id: ${fileReferenceId} to exercise id: ${exerciseId}, we load the file content...`)

    await dispatch(_getSingleExerciseDescriptionAssetFileAsync(exerciseId, fileReferenceId, onProgress))
  }
}

export function _attachExitingFileToExerciseDescriptionAsync(exerciseId: number, fileReferenceId: number): ATTACH_markdownAssetAction {
  return {
    type: ActionType.ATTACH_markdownAsset,
    payload: delay(attachExitingFileToExerciseDescription(exerciseId, fileReferenceId))
  }
}


export function detachFileFromExerciseDescriptionAsync(exerciseId: number, fileReferenceId: number): DETACH_markdownAssetPreviewAction {
  return {
    type: ActionType.DETACH_markdownAsset,
    payload: delay(detachFileFromExerciseDescription(exerciseId, fileReferenceId)),
    meta: {
      assetPreviewId: fileReferenceId
    }
  }
}

export function _getSingleExerciseDescriptionAssetFileAsync(exerciseId: number, fileReferenceId: number, onProgress?: (percentage: number) => void): LOAD_markdownAssetAction {
  return {
    type: ActionType.LOAD_markdownAsset,
    payload: delay(getSingleExerciseDescriptionAssetFile(exerciseId, fileReferenceId, onProgress))
  }
}

//--- exercise test asset files

export function getTestAssetPreviewsAsync(exerciseId: number): GET_testAssetPreviewsAction {
  return {
    type: ActionType.GET_testAssetPreviews,
    payload: delay(getPossibleExerciseTestFiles(exerciseId))
  }
}

//this is also handled in the edit test dialog reducer to add the returned asset file also to the copied test
export function attachExitingFileToExerciseTestAsync(exerciseId: number, testId: number, fileReferenceId: number): ATTACH_testAssetAction {
  return {
    type: ActionType.ATTACH_testAsset,
    payload: delay(attachExitingFileToExerciseTest(exerciseId, testId, fileReferenceId)),
    meta: {
      testId
    }
  }
}

export function detachFileFromExerciseTestAsync(exerciseId: number, testId: number, fileReferenceId: number): DETACH_testAssetAction {
  return {
    type: ActionType.DETACH_testAsset,
    payload: delay(detachFileFromExerciseTest(exerciseId, testId, fileReferenceId)),
    meta: {
      testId,
      assetPreviewId: fileReferenceId
    }
  }
}

