/**
 * Created by janis dähne.
 */

import {ActionType} from "../../../reducers/exerciseEditorSite/editorExerciseSiteActionTypes";
import {ExerciseTestForBackend} from "../../../../types/exerciseEditor";
import {SET_testAction} from "../../../reducers/exerciseEditorSite/subSets/setEditorExerciseTestsReducer";
import {MultiActions} from "../../types";
import orderBy from 'lodash-es/orderBy'

//---ReadonlyArray<ExerciseTestForBackend>

/**
 * sets the tests for the exercise editor
 *
 * corrects sorts the test by display index
 *
 * displayIndices with < 0 are set to the last index
 * @param {ReadonlyArray<ExerciseTestForBackend>} tests
 * @returns {MultiActions}
 */
export function setEditorTests(tests: ReadonlyArray<ExerciseTestForBackend>): MultiActions {
  return dispatch => {

    //if we find any test with < 0 index this is probably a new test ... set the display index here

    const normalTests = tests.filter(p => !p.isSubmitTest)
    const submitTests = tests.filter(p => p.isSubmitTest === true)

    //e.g. -1, 0, 1, 2, 3

    const copy1 = renewIndices(normalTests)
    const copy2 = renewIndices(submitTests)

    const copy = copy1.concat(copy2)

    //sort again in case we changed any index
    dispatch(_setEditorTests(orderBy(copy, p => p.displayIndex)))
  }
}

function renewIndices(tests: ReadonlyArray<ExerciseTestForBackend>): ReadonlyArray<ExerciseTestForBackend> {

  // noinspection TsLint
  let count = 0

  const sortedTests = orderBy(tests, p => p.displayIndex)

  // tslint:disable-next-line
  let countWrong = 0

  const copy = sortedTests.map(test => {

    return test.displayIndex < 0
           ? {
        ...test,
        displayIndex: (tests.length - 1) + countWrong++
      }
           : {
        ...test,
        displayIndex: count++
      }
  })
  return copy
}

export function _setEditorTests(tests: ReadonlyArray<ExerciseTestForBackend>): SET_testAction {
  return {
    type: ActionType.SET_tests,
    tests
  }
}