/**
 * Created by janis dähne.
 */

import {SystemRoleFullBase} from "../../../types/userData";
import {
  ResetAction, SET_groupRolesAction, SET_isChangeDialogDisplayedAction,
} from "../../reducers/groupRolesSite/groupRolesSiteReducer";
import {GroupRoleWithPermissionFullBase} from "../../../types/groupRoles";
import {delay} from "../../../../debug";
import {getGroupRoles} from "../../../communicationLayer/groupRolesLayer";
import {ActionType} from "../../reducers/groupRolesSite/groupRolesSiteActionTypes";
import {ThunkAction} from "redux-thunk";
import {setIsCreating, resetChangeGroupRoleDialog} from "./dialog/changeGroupRoleDialogActions";
import {setTargetGroupRole} from "./dialog/groupRoleSetAction";
import {AwaitActions, MultiActions} from "../types";
import {getGroupRolesAsync} from "./groupRolesCrudActions";
import {getGlobalSystemSettingsAsync} from '../globalSystemSettings/globalSystemSettingsCrudActions'


export function setGroupRoles(groupRoles: ReadonlyArray<GroupRoleWithPermissionFullBase>): SET_groupRolesAction {
  return {
    type: ActionType.SET_groupRoles,
    groupRoles: groupRoles
  }
}


export function openCreateGroupRoleDialog(): MultiActions {
  return dispatch => {

    dispatch(resetChangeGroupRoleDialog())
    dispatch(setIsCreating(true))
    dispatch(setIsChangeDialogDisplayed(true))

  }
}

export function openEditGroupRoleDialog(groupRole: GroupRoleWithPermissionFullBase): MultiActions {
  return dispatch => {

    dispatch(resetChangeGroupRoleDialog())
    dispatch(setIsCreating(false))
    dispatch(setTargetGroupRole(groupRole))
    dispatch(setIsChangeDialogDisplayed(true))

  }
}

export function cancelEditGroupRole(): MultiActions {
  return dispatch => {

    dispatch(setIsChangeDialogDisplayed(false))
    dispatch(resetChangeGroupRoleDialog())

  }
}


export function setIsChangeDialogDisplayed(isDisplayed: boolean): SET_isChangeDialogDisplayedAction {
  return {
    type: ActionType.SET_isChangeDialogDisplayed,
    isDisplayed
  }
}

export function loadGroupRolesSite(): AwaitActions {
  return async(dispatch) => {

    dispatch(resetGroupRolesSite())

    await dispatch(getGroupRolesAsync())
    await dispatch(getGlobalSystemSettingsAsync()) //to get the default group roles

  }
}

export function resetGroupRolesSite(): ResetAction {
  return {
    type: ActionType.RESET
  }
}
