/**
 * Created by janis dähne.
 */
import {GET_groupsAction} from "../../reducers/groupsSite/getGroupsReducer";
import {delay} from "../../../../debug";
import {ActionType} from '../../reducers/groupsSite/groupsSiteActionTypes'
import {deleteUserGroup, getGroupsFromMe, removeMember} from "../../../communicationLayer/groupsLayer";
import {QUIT_groupAction} from "../../reducers/groupsSite/quitGroupReducer";
import {AwaitActions, MultiActions} from "../types";
import {UserGroupBase, UserGroupForBackend} from "../../../types/group";
import {DELETE_userGroupAction} from "../../reducers/groupsSite/deleteGroupReducer";
import {loadGroupsSite} from "./groupsSiteActions";
import {PaginationPostData} from '../../../types/pagination'
import {PaginationHelper} from '../../../helpers/paginationHelper'


function _quitGroupAsync(group: UserGroupForBackend): QUIT_groupAction {
  return {
    type: ActionType.QUIT_group,
    payload: delay(removeMember({
      groupId: group.id,
      useCurrentUser: true
    }))
  }
}

export function quitGroupAsync(group: UserGroupForBackend): AwaitActions {
  return async (dispatch) => {
    await dispatch(_quitGroupAsync(group))
  }
}

export function _deleteUserGroupAsync(userGroupId: number): DELETE_userGroupAction {
  return {
    type: ActionType.DELETE_userGroup,
    payload: delay(deleteUserGroup(userGroupId)),
  }
}

/**
 * deletes the given user group and reloads the page
 * @param {number} userGroupId
 * @returns {AwaitActions}
 */
export function deleteUserGroupAsync(userGroupId: number): AwaitActions {
  return async(dispatch, getState) => {

    await dispatch(_deleteUserGroupAsync(userGroupId))

    //then reload
    dispatch(loadGroupsSite())
  }
}



export function _getGroupsAsync(paginationPostData: PaginationPostData): GET_groupsAction {
  return {
    type: ActionType.GET_groups,
    payload: delay(getGroupsFromMe(paginationPostData))
}
}

/**
 *
 * @param paginationPostData pass null to get all groups
 */
export function getGroupsAsync(paginationPostData: PaginationPostData | null): MultiActions {
  return (dispatch, getState) => {

    //some other sites may need this data (all groups) so enable to pass custom pagination data
    if (paginationPostData) {
      dispatch(_getGroupsAsync(paginationPostData))
      return
    }

    const paginationPostDataFresh = PaginationHelper.getPaginationPostData(getState().groupsState.pagination)
    dispatch(_getGroupsAsync(paginationPostDataFresh))
  }
}
