/**
 * Created by janis dähne.
 */
import {ActionType} from "../../reducers/manageCustomProjectsSite/manageCustomProjectsSiteActionTypes";
import {delay} from "../../../../debug";
import {
  deleteOwnCustomProject, getOwnCustomProjectOverviews
} from "../../../communicationLayer/customProjectsOverviewLayer";
import {AwaitActions, MultiActions} from "../types";
import {
  ResetAction, SET_sortByKeyAction, SetActivePaginationPageAction, SetPaginationPageSizeAction, SetSearchTextAction
} from "../../reducers/manageCustomProjectsSite/manageCustomProjectsSiteReducer";
import {SortDirection} from "../../../types/sortDirection";
import {CustomProjectPreviewFullBase} from "../../../types/customProjects";
import {DELETE_customProjectAction} from "../../reducers/manageCustomProjectsSite/deleteCustomProjectReducer";
import {errorDialog} from "../../../helpers/dialogHelper";
import Logger from "../../../helpers/logger";
import {GET_customProjectOverviewsAction} from "../../reducers/manageCustomProjectsSite/getCustomProjectOverviewsReducer";
import {getI18n} from "../../../../i18n/i18nRoot";
import globalState from '../../state'
import {PaginationPostData} from '../../../types/pagination'
import {PaginationHelper} from '../../../helpers/paginationHelper'
import debounce from 'lodash-es/debounce'
import {searchInputDebounceInMs} from '../../../constants'
import {getSystemSettingsAsync} from '../systemSettingsSite/systemSettingsCrudActions'
import {
  getGlobalSystemSettingsAsync,
  getTestServerPublicSettingsAsync
} from '../globalSystemSettings/globalSystemSettingsCrudActions'
import {ErrorHelper} from '../../../helpers/errorHelper'
import {getPLangsAsync} from '../plangSettingsView/pLangsCrudActions'



export function _getOwnCustomProjectsAsync(paginationPostData: PaginationPostData): GET_customProjectOverviewsAction {
  return {
    type: ActionType.GET_customProjectOverviews,
    payload: delay(getOwnCustomProjectOverviews(paginationPostData))
  }
}

export function getOwnCustomProjectsAsync(): MultiActions {
  return (dispatch, getState) => {

    const paginationPostData = PaginationHelper.getPaginationPostData(getState().manageCustomProjectsSiteState.pagination)
    dispatch(_getOwnCustomProjectsAsync(paginationPostData))
  }
}

export function deleteOwnCustomProjectAsync(customProjectPreview: CustomProjectPreviewFullBase): AwaitActions {
  return async (dispatch, getState) => {

    try {
      await dispatch(_deleteOwnCustomProjectAsync(customProjectPreview))
    } catch (err) {

      ErrorHelper.makeServerErrorDialog(err, globalState.getState().i18nState.langId, 'Error', 'Could not delete custom project')

      Logger.error(`could not delete custom project with id: ${customProjectPreview.id}, err: ${err.message}`)
    }

    //refresh always
    dispatch(getOwnCustomProjectsAsync())

  }
}

export function _deleteOwnCustomProjectAsync(customProjectPreview: CustomProjectPreviewFullBase): DELETE_customProjectAction {
  return {
    type: ActionType.DELETE_customProject,
    payload: delay(deleteOwnCustomProject(customProjectPreview))
  }
}



export function loadManageCustomProjectSite(): AwaitActions {
  return async (dispatch) => {

    dispatch(resetManageCustomProjectsSite())

    await dispatch(getGlobalSystemSettingsAsync())

    await dispatch(getTestServerPublicSettingsAsync())

    await dispatch(getPLangsAsync())

    await dispatch(getOwnCustomProjectsAsync())

  }
}

export function resetManageCustomProjectsSite(): ResetAction {
  return {
    type: ActionType.RESET
  }
}





//--- pagination


export function _setSortByKey(sortByKey: keyof CustomProjectPreviewFullBase,
                              sortDirection: boolean | null
): SET_sortByKeyAction {
  return {
    type: ActionType.SET_sortByKey,
    sortByKey,
    sortDirection
  }
}

export function setSortByKey(sortByKey: keyof CustomProjectPreviewFullBase,
                             sortDirection: boolean | null
): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setSortByKey(sortByKey, sortDirection))
    dispatch(getOwnCustomProjectsAsync())

  }
}

export function _setActivePaginationPage(activePage: number): SetActivePaginationPageAction {
  return {
    type: ActionType.setPaginationActivePage,
    activePage
  }
}

export function setActivePaginationPage(activePage: number): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setActivePaginationPage(activePage))
    dispatch(getOwnCustomProjectsAsync())
  }
}

export function _setPaginationPageSize(pageSize: number): SetPaginationPageSizeAction {
  return {
    type: ActionType.setPaginationPageSize,
    pageSize
  }
}

export function setPaginationPageSize(pageSize: number): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setPaginationPageSize(pageSize))

    dispatch(getOwnCustomProjectsAsync())
  }
}

export function _setSearchText(searchText: string): SetSearchTextAction {
  return {
    type: ActionType.setSearchText,
    searchText
  }
}

// noinspection TsLint
const debouncedRefresh = debounce((dispatch, getState) => {

  dispatch(getOwnCustomProjectsAsync())

}, searchInputDebounceInMs)

export function setSearchText(searchText: string): MultiActions {
  return (dispatch, getState) => {
    dispatch(_setSearchText(searchText))

    debouncedRefresh(dispatch, getState)
  }
}
