import {GET_groupExercisesAction} from '../../reducers/manageGroupExercisesSite/getGroupExercisesReducer'
import {ActionType} from '../../reducers/manageGroupExercisesSite/groupExercisesSiteActionTypes'
import {delay} from '../../../../debug'
import {
  deleteExercise,
  getAccessibleExercisePreviewsByGroup,
  getGroupsWithAccessibleExercises
} from '../../../communicationLayer/exercisesOverviewLayer'
import {AwaitActions, MultiActions} from '../types'
import {DELETE_groupExerciseAction} from '../../reducers/manageGroupExercisesSite/deleteGroupExerciseReducer'
import {loadManageGroupExercisesSite} from './groupExercisesSiteActions'
import {GET_groupsForExercisesAction} from '../../reducers/manageGroupExercisesSite/getGroupsForExercisesCrudReducer'
import {PaginationHelper} from '../../../helpers/paginationHelper'
import {PaginationPostData, PaginationPostDataWithTagsFilter} from '../../../types/pagination'
import Logger from '../../../helpers/logger'

export function _getGroupExercisesAsync(groupId: number,
                                        paginationPostData: PaginationPostDataWithTagsFilter
): GET_groupExercisesAction {
  return {
    type: ActionType.GET_groupExercises,
    payload: delay(getAccessibleExercisePreviewsByGroup(groupId, paginationPostData)),
    meta: {
      groupId
    }
  }
}

export function getGroupExercisesAsync(groupId: number): MultiActions {
  return async (dispatch, getState) => {


    const groupTuple = getState().manageGroupExercisesState.groupExercises.find(p => p.userGroup.id === groupId)

    if (!groupTuple) {
      Logger.error('could not find group tuple to get exercises')
    }

    const tagsState = getState().tagsFilterState
    const postData = PaginationHelper.getPaginationPostDataWithTagsFilter(groupTuple.pagination,
                                                                          tagsState.selectedFilterTagsIds,
                                                                          tagsState.selectedNegativeFilterTagsIds,
                                                                          tagsState.logicalConnector
    )
    await dispatch(_getGroupExercisesAsync(groupId, postData))

  }
}

export function deleteGroupExercisesAsync(exerciseId: number, groupId: number): AwaitActions {
  return async (dispatch, getState) => {

    await dispatch(_deleteGroupExercisesAsync(exerciseId, groupId))

    await dispatch(getGroupExercisesAsync(groupId))

  }
}

export function _deleteGroupExercisesAsync(exerciseId: number, groupId: number): DELETE_groupExerciseAction {
  return {
    type: ActionType.DELETE_groupExercise,
    payload: delay(deleteExercise(exerciseId)),
    meta: {
      groupId
    }
  }
}


export function getGroupsForExercisesAsync(): GET_groupsForExercisesAction {
  return {
    type: ActionType.GET_groupsForExercises,
    payload: delay(getGroupsWithAccessibleExercises())
  }
}