/**
 * Created by janis dähne.
 */
import {RERUN_automaticAssessmentAction} from "../../../reducers/ownOrGroupExerciseReducers/submissionsSite/rerunAutomaticAssessmentReducer";
import {ActionType} from "../../../reducers/ownOrGroupExerciseReducers/submissionsSite/submissionsSiteActionTypes";
import {delay} from "../../../../../debug";
import {
  getFromRelease, importCsvAssessments,
  rerunAutomaticAssessment,
  rerunSingleAutomaticAssessment
} from "../../../../communicationLayer/submissionsLayer";
import {RERUN_singleAutomaticAssessmentAction} from "../../../reducers/ownOrGroupExerciseReducers/submissionsSite/rerunSingleAutomaticAssessmentReducer";
import {GET_submissionsAction} from "../../../reducers/ownOrGroupExerciseReducers/submissionsSite/getSubmissionsReducer";
import {
  getSingleGroupExercisePreview,
  getSingleOwnExercisePreview
} from '../../../../communicationLayer/exercisesOverviewLayer'
import {GET_singleGroupExerciseAction} from '../../../reducers/ownOrGroupExerciseReducers/submissionsSite/getSingleGroupExerciseCrudReducer'
import {GET_singleOwnExerciseAction} from '../../../reducers/ownOrGroupExerciseReducers/submissionsSite/getSingleOwnExerciseCrudReducer'
import {getSingleReleaseForExercise} from '../../../../communicationLayer/exerciseReleasesLayer'
import {GET_singleReleaseAction} from '../../../reducers/ownOrGroupExerciseReducers/submissionsSite/getSingleReleaseCrudReducer'
import {AwaitActions, MultiActions} from '../../types'
import {PaginationHelper} from '../../../../helpers/paginationHelper'
import {PaginationPostData} from '../../../../types/pagination'
import Logger from '../../../../helpers/logger'
import {UPLOAD_csvAssessmentFileAction} from '../../../reducers/ownOrGroupExerciseReducers/submissionsSite/uploadCsvAssessmentFileReducer'


export function _rerunAutomaticAssessmentActionAsync(releaseCode: string): RERUN_automaticAssessmentAction {
  return {
    type: ActionType.RERUN_automaticAssessment,
    payload: delay(rerunAutomaticAssessment(releaseCode))
  }
}


export function _rerunSingleAutomaticAssessmentActionAsync(releaseCode: string, userId: number, pLangId: number): RERUN_singleAutomaticAssessmentAction {
  return {
    type: ActionType.RERUN_singleAutomaticAssessment,
    payload: delay(rerunSingleAutomaticAssessment(releaseCode, userId, pLangId))
  }
}

export function _getSubmissions(releaseId: number, paginationPostData: PaginationPostData): GET_submissionsAction {
  return {
    type: ActionType.GET_submissions,
    payload: delay(getFromRelease(releaseId, paginationPostData))
  }
}


/**
 * this requires
 * @submissionsSiteState.release.id is loaded
 */
export function getSubmissions(): MultiActions {
  return async (dispatch, getState) => {

    if (!getState().submissionsSiteState.release) {
      Logger.log('could not get submissions because no release found... might be ok')
      return
    }

    const releaseId = getState().submissionsSiteState.release.id
    const paginationPostData = PaginationHelper.getPaginationPostData(getState().submissionsSiteState.pagination)
    await dispatch(_getSubmissions(releaseId, paginationPostData))
  }
}


export function getSingleOwnExerciseAsync(exerciseId: number): GET_singleOwnExerciseAction {
  return {
    type: ActionType.GET_singleOwnExercise,
    payload: delay(getSingleOwnExercisePreview(exerciseId)),
  }
}

export function getSingleGroupExerciseAsync(exerciseId: number): GET_singleGroupExerciseAction {
  return {
    type: ActionType.GET_singleGroupExercise,
    payload: delay(getSingleGroupExercisePreview(exerciseId)),
  }
}

export function getSingleReleaseAsync(exerciseId: number, releaseId: number): GET_singleReleaseAction {
  return {
    type: ActionType.GET_singleRelease,
    payload: delay(getSingleReleaseForExercise(exerciseId, releaseId)),
  }
}

export function uploadCsvAssessmentFile(releaseCode: string, content: ReadonlyArray<number>): UPLOAD_csvAssessmentFileAction {
  return {
    type: ActionType.UPLOAD_csvAssessmentFile,
    payload: delay(importCsvAssessments(releaseCode, content)),
  }
}
