/**
 * Created by janis dähne.
 */

import {ActionType} from "../../../reducers/ownOrGroupExerciseReducers/submissionsSite/submissionsSiteActionTypes";
import {
  ResetAction,
  SET_automaticRefreshViewIntervalCounterAction,
  SET_isAutomaticRefreshViewEnabledAction, SET_isExportCsvDialogDisplayedAction,
  SET_runningAssessmentAction,
  SET_selectedReleaseIdAction, SET_showLastEditingIpAddressAction,
  SET_sortByKeyAction,
  SetActivePaginationPageAction,
  SetPaginationPageSizeAction,
  SetSearchTextAction
} from "../../../reducers/ownOrGroupExerciseReducers/submissionsSite/submissionsSiteReducer";
import {AwaitActions, MultiActions} from "../../types";
import {
  _rerunAutomaticAssessmentActionAsync,
  _rerunSingleAutomaticAssessmentActionAsync,
  getSingleGroupExerciseAsync,
  getSingleOwnExerciseAsync,
  getSingleReleaseAsync,
  getSubmissions
} from "./submissionsCrudActions";
import {AssessmentFullBase} from "../../../../types/submissions";
import Logger from "../../../../helpers/logger";
import {errorDialog} from "../../../../helpers/dialogHelper";
import {getI18n} from "../../../../../i18n/i18nRoot";
import globalState from '../../../state'
import debounce from 'lodash-es/debounce'
import {searchInputDebounceInMs} from '../../../../constants'
import {getPLangsAsync} from '../../plangSettingsView/pLangsCrudActions'


export function setSelectedReleaseIdSubmissionsSite(exerciseReleaseId: number): SET_selectedReleaseIdAction {
  return {
    type: ActionType.SET_selectedReleaseId,
    id: exerciseReleaseId
  }
}

export function rerunSingleSubmissionAsync(releaseCode: string, runningAssessment: AssessmentFullBase,
                                           reloadSubmissions: boolean
): AwaitActions {
  return async (dispatch) => {

    await dispatch(
      _rerunSingleAutomaticAssessmentActionAsync(releaseCode, runningAssessment.userId, runningAssessment.pLangId))


    if (reloadSubmissions) {
      //then reload page
      dispatch(getSubmissions())
    }

  }
}

/**
 * this sets the assessment as running and also clears the result (if an assessment is given)
 * @param {AssessmentFullBase} runningAssessment
 * @returns {SET_runningAssessmentAction}
 */
export function setRunningAssessment(runningAssessment: AssessmentFullBase | null): SET_runningAssessmentAction {
  return {
    type: ActionType.SET_runningAssessment,
    runningAssessment
  }
}


export function setShowLastEditingIpAddress(showLastEditingIpAddress: boolean): SET_showLastEditingIpAddressAction {
  return {
    type: ActionType.SET_showLastEditingIpAddress,
    showLastEditingIpAddress
  }
}


/**
 *
 * this is not really that good because we don't get feedback from the server...
 * instead running every single assessment sequentially and report state is ALSO bad because the user
 * browser could crash or the user just closes the browser... so we need to start this on the server
 * @param {string} releaseCode
 * @returns {AwaitActions}
 */
export function rerunAllSubmissionsAsync(releaseCode: string): MultiActions {
  return (dispatch) => {

    dispatch(_rerunAutomaticAssessmentActionAsync(releaseCode))

    //don't await because this takes too long
    //then we cant reload... make a message that the user should manually refresh the page
  }
}

/**
 *
 * @param {number} exerciseId
 * @param {number} releaseId
 * @param comesFromOwnExercisesSite
 * and when the user starts with this site then the releases site is not loaded yet,
 * false: not
 * @param isRefreshingView on refresh we don't need to load all data
 * @returns {AwaitActions}
 */
export function loadSubmissionsSite(exerciseId: number,
                                    comesFromOwnExercisesSite: boolean,
                                    releaseId: number,
                                    isRefreshingView: boolean
): AwaitActions {
  return async (dispatch, getState) => {


    if (!isRefreshingView) {
      if (comesFromOwnExercisesSite) {
        await dispatch(getSingleOwnExerciseAsync(exerciseId))
      } else {
        await dispatch(getSingleGroupExerciseAsync(exerciseId))
      }
    }


    // if (!isRefreshingView) {
    //we always need to get the release too because else we won't get any new submission state
      await dispatch(getSingleReleaseAsync(exerciseId, releaseId))
    // }

    //workaround until we changed the api to use the release id instead of the code
    const release = getState().submissionsSiteState.release

    if (!release) {
      Logger.error("no release found (submissionsSiteActions)")

      errorDialog(getI18n(globalState.getState().i18nState.langId, 'Error'),
                  getI18n(globalState.getState().i18nState.langId, 'Could not find release'),globalState.getState().i18nState.langId
      )
      return
    }

    if (!isRefreshingView) {
      await dispatch(getPLangsAsync())
    }

    //the release should be loaded here (in the reducer state)
    await dispatch(getSubmissions())

  }
}


export function setAutomaticRefreshViewIntervalCounter(automaticRefreshViewIntervalCounter: number): SET_automaticRefreshViewIntervalCounterAction {
  return {
    type: ActionType.SET_automaticRefreshViewIntervalCounter,
    automaticRefreshViewIntervalCounter,
  }
}

export function setIsAutomaticRefreshViewEnabled(isAutomaticRefreshViewEnabled: boolean): SET_isAutomaticRefreshViewEnabledAction {
  return {
    type: ActionType.SET_isAutomaticRefreshViewEnabled,
    isAutomaticRefreshViewEnabled,
  }
}

export function resetSubmissionsSite(): ResetAction {
  return {
    type: ActionType.RESET,
  }
}

export function setIsExportCsvDialogDisplayed(isExportCsvDialogDisplayed: boolean): SET_isExportCsvDialogDisplayedAction {
  return {
    type: ActionType.SET_isExportCsvDialogDisplayed,
    isExportCsvDialogDisplayed,
  }
}


//--- pagination


export function _setSortByKey(sortByKey: keyof AssessmentFullBase,
                              sortDirection: boolean | null
): SET_sortByKeyAction {
  return {
    type: ActionType.SET_sortByKey,
    sortByKey,
    sortDirection
  }
}

export function setSortByKey(sortByKey: keyof AssessmentFullBase,
                             sortDirection: boolean | null
): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setSortByKey(sortByKey, sortDirection))
    dispatch(getSubmissions())

  }
}

export function _setActivePaginationPage(activePage: number): SetActivePaginationPageAction {
  return {
    type: ActionType.setPaginationActivePage,
    activePage
  }
}

export function setActivePaginationPage(activePage: number): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setActivePaginationPage(activePage))
    dispatch(getSubmissions())
  }
}

export function _setPaginationPageSize(pageSize: number): SetPaginationPageSizeAction {
  return {
    type: ActionType.setPaginationPageSize,
    pageSize
  }
}

export function setPaginationPageSize(pageSize: number): AwaitActions {
  return async (dispatch, getState) => {

    dispatch(_setPaginationPageSize(pageSize))

    await dispatch(getSubmissions())
  }
}

export function _setSearchText(searchText: string): SetSearchTextAction {
  return {
    type: ActionType.setSearchText,
    searchText
  }
}

// noinspection TsLint
const debouncedRefresh = debounce((dispatch, getState) => {

  dispatch(getSubmissions())

}, searchInputDebounceInMs)

export function setSearchText(searchText: string): MultiActions {
  return (dispatch, getState) => {
    dispatch(_setSearchText(searchText))

    debouncedRefresh(dispatch, getState)
  }
}
