/**
 * Created by janis dähne.
 */
import {ActionType} from "../../../../reducers/singleGroupSite/dialogs/addUsers/addsUserDialogActionTypes";
import {
  INVERT_allSelectionsAction,
  ResetAction,
  SET_all_users_uiIsSelectedAction,
  SET_sortByKeyAction,
  SET_user_uiIsSelectedAction, SetActivePaginationPageAction, SetPaginationPageSizeAction, SetSearchTextAction
} from "../../../../reducers/singleGroupSite/dialogs/addUsers/addUsersDialogReducer";
import {AddableUserFrontendExclusive, UserPreviewFromBackend} from "../../../../../types/users";
import {AwaitActions, MultiActions} from "../../../types";
import {setIsAddUsersDialogDisplayed} from "../../singleGroupSiteActions";
import {addMembersAsync, getUsersAsync} from "./addUsersDialogCrudActions";
import {AddUsersToGroupTuplesForBackend, SelectableGroupUserFrontend} from "../../../../../types/group";
import {GroupRoleWithPermissionFullBase} from "../../../../../types/groupRoles";
import {PaginationHelper} from '../../../../../helpers/paginationHelper'
import debounce from 'lodash-es/debounce'
import {searchInputDebounceInMs} from '../../../../../constants'
import {getMembersAsync} from '../../singleGroupSiteCrudActions'


export function setUserSelected(user: AddableUserFrontendExclusive, isSelected: boolean): SET_user_uiIsSelectedAction {
  return {
    type: ActionType.SET_user_uiIsSelected,
    user,
    isSelected
  }
}

export function setAllUsersSelected(isSelected: boolean): SET_all_users_uiIsSelectedAction {
  return {
    type: ActionType.SET_all_users_uiIsSelected,
    isSelected
  }
}


export function invertAllSelections(): INVERT_allSelectionsAction {
  return {
    type: ActionType.INVERT_allSelections,
  }
}

export function addMembersAndExitAsync(
  groupId: number,
  groupRoles: ReadonlyArray<GroupRoleWithPermissionFullBase>,
  addUsersTuple: AddUsersToGroupTuplesForBackend): AwaitActions {

  return async (dispatch, getState) => {

    await dispatch(addMembersAsync(addUsersTuple))

    const postData = PaginationHelper.getPaginationPostData(getState().singleGroupState.pagination)

    dispatch(getMembersAsync(groupId, groupRoles, postData))
    dispatch(setIsAddUsersDialogDisplayed(false))
    dispatch(resetAddUsersDialog())

  }
}

export function cancelAddUsersDialog(): MultiActions {
  return async (dispatch) => {

    dispatch(setIsAddUsersDialogDisplayed(false))
    dispatch(resetAddUsersDialog())

  }
}


export function resetAddUsersDialog(): ResetAction {
  return {
    type: ActionType.RESET,
  }
}

export function loadAddUsersDialog(): MultiActions {
  return async (dispatch, getState) => {

    dispatch(resetAddUsersDialog()) //this also applies the frontend settings page size

    const postData = PaginationHelper.getPaginationPostData(getState().addUsersDialogState.pagination)
    dispatch(getUsersAsync(getState().singleGroupState.members, postData))

  }
}

//--- pagination


export function _setSortByKey(sortByKey: keyof UserPreviewFromBackend,
                              sortDirection: boolean | null
): SET_sortByKeyAction {
  return {
    type: ActionType.SET_sortByKey,
    sortByKey,
    sortDirection
  }
}

export function setSortByKey(sortByKey: keyof UserPreviewFromBackend,
                             sortDirection: boolean | null
): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setSortByKey(sortByKey, sortDirection))

    const members = getState().singleGroupState.members
    const paginationPostData = PaginationHelper.getPaginationPostData( getState().addUsersDialogState.pagination)
    dispatch(getUsersAsync(members, paginationPostData))

  }
}

export function _setActivePaginationPage(activePage: number): SetActivePaginationPageAction {
  return {
    type: ActionType.setPaginationActivePage,
    activePage
  }
}

export function setActivePaginationPage(activePage: number): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setActivePaginationPage(activePage))

    const members = getState().singleGroupState.members
    const paginationPostData = PaginationHelper.getPaginationPostData( getState().addUsersDialogState.pagination)
    dispatch(getUsersAsync(members, paginationPostData))
  }
}

export function _setPaginationPageSize(pageSize: number): SetPaginationPageSizeAction {
  return {
    type: ActionType.setPaginationPageSize,
    pageSize
  }
}

export function setPaginationPageSize(pageSize: number): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setPaginationPageSize(pageSize))

    const members = getState().singleGroupState.members
    const paginationPostData = PaginationHelper.getPaginationPostData( getState().addUsersDialogState.pagination)
    dispatch(getUsersAsync(members, paginationPostData))
  }
}

export function _setSearchText(searchText: string): SetSearchTextAction {
  return {
    type: ActionType.setSearchText,
    searchText
  }
}

// noinspection TsLint
const debouncedRefresh = debounce((dispatch, getState) => {

  const members = getState().singleGroupState.members
  const paginationPostData = PaginationHelper.getPaginationPostData( getState().addUsersDialogState.pagination)
  dispatch(getUsersAsync(members, paginationPostData))

}, searchInputDebounceInMs)

export function setSearchText(searchText: string): MultiActions {
  return (dispatch, getState) => {
    dispatch(_setSearchText(searchText))

    debouncedRefresh(dispatch, getState)
  }
}