/**
 * Created by janis dähne.
 */
import {
  ResetAction,
  SET_all_member_uiIsSelectedAction,
  SET_isAddUsersDialogDisplayedAction,
  SET_member_uiIsSelectedAction,
  SET_selectedGroupIdAction,
  SET_sortByKeyAction,
  SET_user_uiIsChangeGroupRolePopupDisplayedAction,
  SetActivePaginationPageAction,
  SetPaginationPageSizeAction,
  SetSearchTextAction
} from "../../reducers/singleGroupSite/singleGroupSiteReducer";
import {ActionType} from "../../reducers/singleGroupSite/singleGroupSiteActionTypes";
import {AwaitActions, MultiActions} from "../types";
import {
  getMembersAsync,
  getSingleGroupAsync,
  removeMemberAsync,
  setMemberGroupRoleAsync
} from "./singleGroupSiteCrudActions";
import {getGroupRolesAsync, getGroupRolesForGroupAsync} from "../groupRolesSite/groupRolesCrudActions";
import {SelectableGroupUserFrontend} from "../../../types/group";
import {searchInputDebounceInMs} from '../../../constants'
import debounce from 'lodash-es/debounce'
import {PaginationHelper} from '../../../helpers/paginationHelper'
import {getGlobalSystemSettingsAsync} from '../globalSystemSettings/globalSystemSettingsCrudActions'


export function setSelectedGroupId(groupId: number): SET_selectedGroupIdAction {
  return {
    type: ActionType.SET_selectedGroupId,
    groupId
  }
}


export function setIsAddUsersDialogDisplayed(isDisplayed: boolean): SET_isAddUsersDialogDisplayedAction {
  return {
    type: ActionType.SET_isAddUsersDialogDisplayed,
    isDisplayed
  }
}

export function setUiIsChangeGroupRolePopupDisplayedAction(isDisplayed: boolean,
                                                           userId: number
): SET_user_uiIsChangeGroupRolePopupDisplayedAction {
  return {
    type: ActionType.SET_user_uiIsChangeGroupRolePopupDisplayed,
    isDisplayed,
    userId
  }
}


export function setMemberGroupRoleAndRefreshAsync(userId: number, groupId: number, groupRoleId: number): AwaitActions {
  return async (dispatch, getState) => {

    dispatch(setUiIsChangeGroupRolePopupDisplayedAction(false, userId))
    await dispatch(setMemberGroupRoleAsync(userId, groupId, groupRoleId, true))

    const groupRoles = getState().singleGroupState.groupRoles
    const paginationPostData = PaginationHelper.getPaginationPostData(getState().singleGroupState.pagination)
    dispatch(getMembersAsync(groupId, groupRoles, paginationPostData))

  }
}

export function removeMultipleMemberAndRefreshAsync(userIds: ReadonlyArray<number>, groupId: number): AwaitActions {
  return async (dispatch, getState) => {

    for (const userId of userIds) {
      await dispatch(removeMemberAsync(userId, groupId))
    }

    const groupRoles = getState().singleGroupState.groupRoles
    const paginationPostData = PaginationHelper.getPaginationPostData(getState().singleGroupState.pagination)
    dispatch(getMembersAsync(groupId, groupRoles, paginationPostData))
  }
}

export function removeMemberAndRefreshAsync(userId: number, groupId: number): AwaitActions {
  return async (dispatch, getState) => {

    await dispatch(removeMemberAsync(userId, groupId))

    const groupRoles = getState().singleGroupState.groupRoles
    const paginationPostData = PaginationHelper.getPaginationPostData(getState().singleGroupState.pagination)
    dispatch(getMembersAsync(groupId, groupRoles, paginationPostData))
  }
}


export function setMemberUiIsSelected(memberId: number, isSelected: boolean): SET_member_uiIsSelectedAction {
  return {
    type: ActionType.SET_member_uiIsSelected,
    memberId,
    isSelected
  }
}

export function setAllMemberUiIsSelected(isSelected: boolean): SET_all_member_uiIsSelectedAction {
  return {
    type: ActionType.SET_all_member_uiIsSelected,
    isSelected
  }
}

/**
 * loads the group roles, then the group members
 * @param {number} groupId
 * false: the user came from the groups site so they are loaded...
 * @returns {AwaitActions}
 */
export function loadSingleGroupSite(groupId: number): AwaitActions {
  return async (dispatch, getState) => {

    dispatch(resetSingleGroupSite())

    await dispatch(getSingleGroupAsync(groupId))

    await dispatch(getGlobalSystemSettingsAsync()) //to get the default group

    if (getState().singleGroupState.group.groupRole !== null &&
      getState().singleGroupState.group.groupRole.canSeeOtherMembers) {
      await dispatch(getGroupRolesForGroupAsync(groupId))
    }

    dispatch(setSelectedGroupId(groupId))

    const groupRoles = getState().singleGroupState.groupRoles
    const paginationPostData = PaginationHelper.getPaginationPostData(getState().singleGroupState.pagination)
    dispatch(getMembersAsync(groupId, groupRoles, paginationPostData))

  }
}

export function resetSingleGroupSite(): ResetAction {
  return {
    type: ActionType.RESET
  }
}


//--- pagination


export function _setSortByKey(sortByKey: keyof SelectableGroupUserFrontend,
                              sortDirection: boolean | null
): SET_sortByKeyAction {
  return {
    type: ActionType.SET_sortByKey,
    sortByKey,
    sortDirection
  }
}

export function setSortByKey(sortByKey: keyof SelectableGroupUserFrontend,
                             sortDirection: boolean | null
): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setSortByKey(sortByKey, sortDirection))

    const groupId = getState().singleGroupState.selectedGroupId
    const groupRoles = getState().singleGroupState.groupRoles
    const paginationPostData = PaginationHelper.getPaginationPostData(getState().singleGroupState.pagination)
    dispatch(getMembersAsync(groupId, groupRoles, paginationPostData))

  }
}

export function _setActivePaginationPage(activePage: number): SetActivePaginationPageAction {
  return {
    type: ActionType.setPaginationActivePage,
    activePage
  }
}

export function setActivePaginationPage(activePage: number): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setActivePaginationPage(activePage))

    const groupId = getState().singleGroupState.selectedGroupId
    const groupRoles = getState().singleGroupState.groupRoles
    const paginationPostData = PaginationHelper.getPaginationPostData(getState().singleGroupState.pagination)
    dispatch(getMembersAsync(groupId, groupRoles, paginationPostData))
  }
}

export function _setPaginationPageSize(pageSize: number): SetPaginationPageSizeAction {
  return {
    type: ActionType.setPaginationPageSize,
    pageSize
  }
}

export function setPaginationPageSize(pageSize: number): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setPaginationPageSize(pageSize))

    const groupId = getState().singleGroupState.selectedGroupId
    const groupRoles = getState().singleGroupState.groupRoles
    const paginationPostData = PaginationHelper.getPaginationPostData(getState().singleGroupState.pagination)
    dispatch(getMembersAsync(groupId, groupRoles, paginationPostData))
  }
}

export function _setSearchText(searchText: string): SetSearchTextAction {
  return {
    type: ActionType.setSearchText,
    searchText
  }
}

// noinspection TsLint
const debouncedRefresh = debounce((dispatch, getState) => {

  const groupId = getState().singleGroupState.selectedGroupId
  const groupRoles = getState().singleGroupState.groupRoles
  const paginationPostData = PaginationHelper.getPaginationPostData(getState().singleGroupState.pagination)
  dispatch(getMembersAsync(groupId, groupRoles, paginationPostData))

}, searchInputDebounceInMs)

export function setSearchText(searchText: string): MultiActions {
  return (dispatch, getState) => {
    dispatch(_setSearchText(searchText))

    debouncedRefresh(dispatch, getState)
  }
}
