/**
 * Created by janis dähne.
 */
import {ActionType} from "../../reducers/singleGroupSite/singleGroupSiteActionTypes";
import {delay} from "../../../../debug";
import {GroupRoleWithPermissionFullBase} from "../../../types/groupRoles";
import {
  changeMemberRole,
  getMembers,
  getSingleGroupFromMe,
  removeMember
} from "../../../communicationLayer/groupsLayer";
import {SET_memberGroupRoleAction} from "../../reducers/singleGroupSite/setMemberGroupRoleReducer";
import {REMOVE_memberAction} from "../../reducers/singleGroupSite/removeMemberReducer";
import {GET_membersAction} from '../../reducers/singleGroupSite/getMembersReducer'
import {GET_singleGroupAction} from '../../reducers/singleGroupSite/getSingleGroupReducer'
import {getGroupRoles} from '../../../communicationLayer/groupRolesLayer'
import {PaginationPostData} from '../../../types/pagination'


//normally we should do 1 request and get members + roles + group data ...
export function getMembersAsync(groupId: number, groupRoles: ReadonlyArray<GroupRoleWithPermissionFullBase>, paginationPostData: PaginationPostData): GET_membersAction {
  return {
    type: ActionType.GET_members,
    payload: delay(getMembers(groupId, paginationPostData)),
    meta: {
      groupRoles
    }
  }
}


export function removeMemberAsync(userId: number, groupId: number): REMOVE_memberAction {
  return {
    type: ActionType.REMOVE_member,
    payload: delay(removeMember({
      userId,
      groupId,
      useCurrentUser: false //in the singe group view the user cannot remove him/herself
    })),
  }
}

export function setMemberGroupRoleAsync(userId: number, groupId: number, groupRoleId: number, shouldHideLoading: boolean): SET_memberGroupRoleAction {
  return {
    type: ActionType.SET_memberGroupRole,
    payload: delay(changeMemberRole({
      userId,
      groupId,
      groupRoleId
    })),
    meta: {
      shouldHideLoading: shouldHideLoading
    }
  }
}


export function getSingleGroupAsync(groupId: number): GET_singleGroupAction {
  return {
    type: ActionType.GET_singleGroup,
    payload: delay(getSingleGroupFromMe(groupId))
  }
}