import {ActionBase, ActionType} from './closedExercisesSiteActionTypes'
import {initial, State} from './closedExercisesSiteReducer'
import {PaginatedData} from '../../../../types/pagination'
import {ExercisePreviewFromBackend} from '../../../../types/exercisePreview'
import {PaginationHelper} from '../../../../helpers/paginationHelper'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../index'
import {notExhaustive} from '../../_notExhausiveHelper'

export interface GET_closedExercisesViaVisibilityAction
  extends ActionBase, ReduxPromiseAction<PaginatedData<ExercisePreviewFromBackend>, GET_closedExercisesViaVisibility_PENDINGAction, GET_closedExercisesViaVisibility_FULFILLEDAction, GET_closedExercisesViaVisibility_REJECTEDAction> {
  readonly type: ActionType.GET_closedExercisesViaVisibility
  readonly payload: Promise<PaginatedData<ExercisePreviewFromBackend>>
}

//automatically dispatched by redux promise middleware
export interface GET_closedExercisesViaVisibility_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_closedExercisesViaVisibility_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_closedExercisesViaVisibility_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<PaginatedData<ExercisePreviewFromBackend>> {
  readonly type: ActionType.GET_closedExercisesViaVisibility_FULFILLED
  readonly payload: PaginatedData<ExercisePreviewFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_closedExercisesViaVisibility_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_closedExercisesViaVisibility_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_closedExercisesViaVisibility_PENDINGAction
  | GET_closedExercisesViaVisibility_FULFILLEDAction
  | GET_closedExercisesViaVisibility_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_closedExercisesViaVisibility_PENDING:
      return {
        ...state,
        isLoadingViaVisibility: true
      }
    case ActionType.GET_closedExercisesViaVisibility_FULFILLED:
      return {
        ...state,
        isLoadingViaVisibility: false,
        closedExercisePreviewsViaVisibility: action.payload.items,
        paginationViaVisibility: PaginationHelper.getPaginationStateSpreadObj(action.payload),
      }
    case ActionType.GET_closedExercisesViaVisibility_REJECTED:
      return {
        ...state,
        isLoadingViaVisibility: false,
        closedExercisePreviewsViaVisibility: [],

      }

    default:
      notExhaustive(action)
      return state
  }
}

