/**
 * Created by janis dähne.
 */
import {ActionType, initial, State, ActionBase} from "./enterReleaseCodeDialogReducer";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../../index";
import {notExhaustive} from "../../../_notExhausiveHelper";
import Logger from "../../../../../helpers/logger";
import {ExercisePreviewFromBackend} from "../../../../../types/exercisePreview";

export interface ACCESS_exerciseWithCodeAction
  extends ActionBase, ReduxPromiseAction<ExercisePreviewFromBackend, ACCESS_exerciseWithCode_PENDINGAction, ACCESS_exerciseWithCode_FULFILLEDAction, ACCESS_exerciseWithCode_REJECTEDAction> {
  readonly type: ActionType.ACCESS_exerciseWithCode
  readonly payload: Promise<ExercisePreviewFromBackend>
}

//automatically dispatched by redux promise middleware
export interface ACCESS_exerciseWithCode_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.ACCESS_exerciseWithCode_PENDING
}

//automatically dispatched by redux promise middleware
export interface ACCESS_exerciseWithCode_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ExercisePreviewFromBackend> {
  readonly type: ActionType.ACCESS_exerciseWithCode_FULFILLED
  readonly payload: ExercisePreviewFromBackend
}

//automatically dispatched by redux promise middleware
export interface ACCESS_exerciseWithCode_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.ACCESS_exerciseWithCode_REJECTED
  readonly payload: any
}


export type AllActions =
  ACCESS_exerciseWithCode_PENDINGAction
  | ACCESS_exerciseWithCode_FULFILLEDAction
  | ACCESS_exerciseWithCode_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.ACCESS_exerciseWithCode_PENDING:
      return {
        ...state,
        isBusy: true
      }
    case ActionType.ACCESS_exerciseWithCode_FULFILLED:
      return {
        ...state,
        isBusy: false,
        accessedExerciseRelease: action.payload
      }
    case ActionType.ACCESS_exerciseWithCode_REJECTED:
      return {
        ...state,
        isBusy: false,
        accessedExerciseRelease: null
      }

    default:
      notExhaustive(action)
      return state
  }
}

