import {PaginatedData} from '../../../../types/pagination'
import {ExercisePreviewFromBackend} from '../../../../types/exercisePreview'
import {PaginationHelper} from '../../../../helpers/paginationHelper'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../index'
import {notExhaustive} from '../../_notExhausiveHelper'
import {ActionBase, ActionType} from './openExercisesSiteActionTypes'
import {initial, State} from './openExercisesSiteReducer'
import {convertOpenExercisePreviewToFrontendOpenExercisePreview} from '../../../../helpers/convertersAndTransformers'


export interface GET_openExercisesViaCodeAction
  extends ActionBase, ReduxPromiseAction<PaginatedData<ExercisePreviewFromBackend>, GET_openExercisesViaCode_PENDINGAction, GET_openExercisesViaCode_FULFILLEDAction, GET_openExercisesViaCode_REJECTEDAction> {
  readonly type: ActionType.GET_openExercisesViaCode
  readonly payload: Promise<PaginatedData<ExercisePreviewFromBackend>>
}

//automatically dispatched by redux promise middleware
export interface GET_openExercisesViaCode_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_openExercisesViaCode_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_openExercisesViaCode_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<PaginatedData<ExercisePreviewFromBackend>> {
  readonly type: ActionType.GET_openExercisesViaCode_FULFILLED
  readonly payload: PaginatedData<ExercisePreviewFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_openExercisesViaCode_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_openExercisesViaCode_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_openExercisesViaCode_PENDINGAction
  | GET_openExercisesViaCode_FULFILLEDAction
  | GET_openExercisesViaCode_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_openExercisesViaCode_PENDING:
      return {
        ...state,
        isLoadingViaCode: true
      }
    case ActionType.GET_openExercisesViaCode_FULFILLED:
      return {
        ...state,
        isLoadingViaCode: false,
        openExercisePreviewsViaCode: convertOpenExercisePreviewToFrontendOpenExercisePreview(action.payload.items),
        paginationViaCode: PaginationHelper.getPaginationStateSpreadObj(action.payload),
      }
    case ActionType.GET_openExercisesViaCode_REJECTED:
      return {
        ...state,
        isLoadingViaCode: false,
        openExercisePreviewsViaCode: []
      }

    default:
      notExhaustive(action)
      return state
  }
}

