/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {ActionBase, ActionType} from "./assessmentStatisticsSiteActionTypes";
import {initial, State} from "./assessmentStatisticsSiteReducer";
import Logger from "../../../helpers/logger";
import {AssessmentStatisticsFullBase} from "../../../types/assessmentStatistics";
import {initial as initialAssessmentStatistics} from './assessmentStatisticsSiteReducer'
import {PaginationHelper} from '../../../helpers/paginationHelper'


export interface GET_assessmentStatisticsAction
  extends ActionBase, ReduxPromiseAction<AssessmentStatisticsFullBase, GET_assessmentStatistics_PENDINGAction, GET_assessmentStatistics_FULFILLEDAction, GET_assessmentStatistics_REJECTEDAction> {
  readonly type: ActionType.GET_assessmentStatistics
  readonly payload: Promise<AssessmentStatisticsFullBase>
}

//automatically dispatched by redux promise middleware
export interface GET_assessmentStatistics_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_assessmentStatistics_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_assessmentStatistics_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<AssessmentStatisticsFullBase> {
  readonly type: ActionType.GET_assessmentStatistics_FULFILLED
  readonly payload: AssessmentStatisticsFullBase
}

//automatically dispatched by redux promise middleware
export interface GET_assessmentStatistics_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_assessmentStatistics_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_assessmentStatistics_PENDINGAction
  | GET_assessmentStatistics_FULFILLEDAction
  | GET_assessmentStatistics_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_assessmentStatistics_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_assessmentStatistics_FULFILLED:
      return {
        ...state,
        isLoading: false,
        assessmentStatistics: {
          firstName: action.payload.firstName,
          lastName: action.payload.lastName,
          userId: action.payload.userId,
          someResultsAreMissingBecauseOfPermissions: action.payload.someResultsAreMissingBecauseOfPermissions,
          assessmentStatistics: action.payload.assessmentStatistics.items
        },
        pagination: PaginationHelper.getPaginationStateSpreadObj(action.payload.assessmentStatistics)
      }
    case ActionType.GET_assessmentStatistics_REJECTED:
      return {
        ...state,
        isLoading: false,
        assessmentStatistics: null
      }

    default:
      notExhaustive(action)
      return state
  }
}

