/**
 * Created by janis dähne.
 */
import {ActionBase, ActionType} from "../dashboardSiteActionTypes";
import {initial, State} from "../dashboardSiteReducer";
import {LogicalUsageStatistic} from "../../../../types/dashboard";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {notExhaustive} from "../../_notExhausiveHelper";


export interface GET_logicalUsageAction
  extends ActionBase, ReduxPromiseAction<LogicalUsageStatistic, GET_logicalUsage_PENDINGAction, GET_logicalUsage_FULFILLEDAction, GET_logicalUsage_REJECTEDAction> {
  readonly type: ActionType.GET_logicalUsage
  readonly payload: Promise<LogicalUsageStatistic>
}

//automatically dispatched by redux promise middleware
export interface GET_logicalUsage_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_logicalUsage_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_logicalUsage_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<LogicalUsageStatistic> {
  readonly type: ActionType.GET_logicalUsage_FULFILLED
  readonly payload: LogicalUsageStatistic
}

//automatically dispatched by redux promise middleware
export interface GET_logicalUsage_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_logicalUsage_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_logicalUsage_PENDINGAction
  | GET_logicalUsage_FULFILLEDAction
  | GET_logicalUsage_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_logicalUsage_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_logicalUsage_FULFILLED:
      return {
        ...state,
        isLoading: false,
        logicalUsageStatistic: action.payload
      }
    case ActionType.GET_logicalUsage_REJECTED:
      return {
        ...state,
        isLoading: false,
        logicalUsageStatistic: null
      }

    default:
      notExhaustive(action)
      return state
  }
}

