
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../../index";
import {Action} from 'redux'
import {ActionType} from '../../doExerciseSiteActionTypes'
import {AfterSolutionDoExerciseFullBase} from '../../../../../types/exerciseAfterSolution'
import {State, initial} from '../../doExerciseSiteReducer'
import {notExhaustive} from '../../../_notExhausiveHelper'
import {SolutionFileDoExerciseFullBase} from '../../../../../types/exerciseSolution'


export interface ActionBase extends Action {
  readonly type: ActionType
}

export interface GET_doExercise_afterSolutionAction
  extends ActionBase, ReduxPromiseAction<AfterSolutionDoExerciseFullBase, GET_doExercise_afterSolution_PENDINGAction, GET_doExercise_afterSolution_FULFILLEDAction, GET_doExercise_afterSolution_REJECTEDAction> {
  readonly type: ActionType.GET_doExercise_afterSolution
  readonly payload: Promise<AfterSolutionDoExerciseFullBase>
}

//automatically dispatched by redux promise middleware
export interface GET_doExercise_afterSolution_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_doExercise_afterSolution_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_doExercise_afterSolution_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<AfterSolutionDoExerciseFullBase> {
  readonly type: ActionType.GET_doExercise_afterSolution_FULFILLED
  readonly payload: AfterSolutionDoExerciseFullBase
}

//automatically dispatched by redux promise middleware
export interface GET_doExercise_afterSolution_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_doExercise_afterSolution_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_doExercise_afterSolution_PENDINGAction
  | GET_doExercise_afterSolution_FULFILLEDAction
  | GET_doExercise_afterSolution_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_doExercise_afterSolution_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_doExercise_afterSolution_FULFILLED:

      // noinspection TsLint
      const afterSolutionFromBackend = {
        ...action.payload,
        //ensure same file display order (because display indices are not applied)
        //we cannot use ids because the backend saves files in any order
        //we do sorting in the editorPanel when we convert the files into editor tabs
        solutionFiles: (action.payload.solutionFiles as SolutionFileDoExerciseFullBase[])
      }

      return {
        ...state,
        isLoading: false,
        afterSolution: afterSolutionFromBackend
      }
    case ActionType.GET_doExercise_afterSolution_REJECTED:
      return {
        ...state,
        isLoading: false,
        afterSolution: null
      }

    default:
      notExhaustive(action)
      return state
  }
}

