import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../../index";
import {initial, State} from '../../doExerciseSiteReducer'
import {ActionType} from '../../doExerciseSiteActionTypes'
import {Action} from 'redux'
import {AssessedSubmitTestFromBackend} from '../../../../../types/testResults'
import {convertAssessmentFromBackendToFrontend} from '../../../../../helpers/convertersAndTransformers'
import {notExhaustive} from '../../../_notExhausiveHelper'

export interface ActionBase extends Action {
  readonly type: ActionType
}

export interface GET_afterSubmitTestResultsAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<AssessedSubmitTestFromBackend>, GET_afterSubmitTestResults_PENDINGAction, GET_afterSubmitTestResults_FULFILLEDAction, GET_afterSubmitTestResults_REJECTEDAction> {
  readonly type: ActionType.GET_afterSubmitTestResults
  readonly payload: Promise<ReadonlyArray<AssessedSubmitTestFromBackend>>
}

//automatically dispatched by redux promise middleware
export interface GET_afterSubmitTestResults_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_afterSubmitTestResults_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_afterSubmitTestResults_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<AssessedSubmitTestFromBackend>> {
  readonly type: ActionType.GET_afterSubmitTestResults_FULFILLED
  readonly payload: ReadonlyArray<AssessedSubmitTestFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_afterSubmitTestResults_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_afterSubmitTestResults_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_afterSubmitTestResults_PENDINGAction
  | GET_afterSubmitTestResults_FULFILLEDAction
  | GET_afterSubmitTestResults_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_afterSubmitTestResults_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_afterSubmitTestResults_FULFILLED:
      return {
        ...state,
        isLoading: false,
        afterSubmitTests: convertAssessmentFromBackendToFrontend(action.payload, true),
      }
    case ActionType.GET_afterSubmitTestResults_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

