
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../../index";
import {ActionType} from '../../doExerciseSiteActionTypes'
import {Action} from 'redux'
import {StoredTestResultTupleFromBackend} from '../../../../../types/testResults'
import {initial, State} from '../../doExerciseSiteReducer'
import {RunnableAfterCustomTestFrontendExclusive} from '../../../../../types/customTests'
import {_storedTestResultToTestResult} from '../../../../../helpers/convertersAndTransformers'
import {notExhaustive} from '../../../_notExhausiveHelper'
import {RunnableExerciseTestFrontendExclusive} from '../../../../../types/exerciseEditor'

export interface ActionBase extends Action {
  readonly type: ActionType
}

export interface GET_allAfterTestResultsAction
  extends ActionBase, ReduxPromiseAction<StoredTestResultTupleFromBackend, GET_allAfterTestResults_PENDINGAction, GET_allAfterTestResults_FULFILLEDAction, GET_allAfterTestResults_REJECTEDAction> {
  readonly type: ActionType.GET_allAfterTestResults
  readonly payload: Promise<StoredTestResultTupleFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_allAfterTestResults_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_allAfterTestResults_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_allAfterTestResults_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<StoredTestResultTupleFromBackend> {
  readonly type: ActionType.GET_allAfterTestResults_FULFILLED
  readonly payload: StoredTestResultTupleFromBackend
}

//automatically dispatched by redux promise middleware
export interface GET_allAfterTestResults_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_allAfterTestResults_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_allAfterTestResults_PENDINGAction
  | GET_allAfterTestResults_FULFILLEDAction
  | GET_allAfterTestResults_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_allAfterTestResults_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_allAfterTestResults_FULFILLED:
      return {
        ...state,
        isLoading: false,
        afterNormalTests: state.afterNormalTests.map<RunnableExerciseTestFrontendExclusive>(p => {
          const storedResult = action.payload.testResults.find(result => result.testId === p.id)
          return {
            ...p,
            result: _storedTestResultToTestResult(storedResult),
            uiIsAfterTest: true
          }
        }),
        afterCustomTests: state.afterCustomTests.map<RunnableAfterCustomTestFrontendExclusive>(p => {
          const storedResult = action.payload.customTestResults.find(result => result.testId === p.id)
          return {
            ...p,
            result: _storedTestResultToTestResult(storedResult),
            uiIsAfterTest: true
          }
        })
      }
    case ActionType.GET_allAfterTestResults_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

