
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../../index";
import {TestAnswerFromBackend} from '../../../../../types/testsAndTestProtocol'
import {ActionType} from '../../doExerciseSiteActionTypes'
import {Action} from 'redux'
import {initial, State} from '../../doExerciseSiteReducer'
import {notExhaustive} from '../../../_notExhausiveHelper'
import {
  mergeAfterCustomTestResults,
  mergeCustomTestResults,
  mergeNormalTestResults
} from '../../../../../helpers/convertersAndTransformers'


export interface ActionBase extends Action {
  readonly type: ActionType
}

export interface RUN_afterCustomTestAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<TestAnswerFromBackend>, RUN_afterCustomTest_PENDINGAction, RUN_afterCustomTest_FULFILLEDAction, RUN_afterCustomTest_REJECTEDAction> {
  readonly type: ActionType.RUN_afterCustomTest
  readonly payload: Promise<ReadonlyArray<TestAnswerFromBackend>>
}

//automatically dispatched by redux promise middleware
export interface RUN_afterCustomTest_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.RUN_afterCustomTest_PENDING
}

//automatically dispatched by redux promise middleware
export interface RUN_afterCustomTest_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<TestAnswerFromBackend>> {
  readonly type: ActionType.RUN_afterCustomTest_FULFILLED
  readonly payload: ReadonlyArray<TestAnswerFromBackend>
}

//automatically dispatched by redux promise middleware
export interface RUN_afterCustomTest_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.RUN_afterCustomTest_REJECTED
  readonly payload: any
}


export type AllActions =
  RUN_afterCustomTest_PENDINGAction
  | RUN_afterCustomTest_FULFILLEDAction
  | RUN_afterCustomTest_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.RUN_afterCustomTest_PENDING:
      return {
        ...state,
        isLoading: false
      }
    case ActionType.RUN_afterCustomTest_FULFILLED:
      return {
        ...state,
        isLoading: false,
        afterCustomTests: mergeAfterCustomTestResults(state.afterCustomTests, action.payload)
      }
    case ActionType.RUN_afterCustomTest_REJECTED:
      return {
        ...state,
        isLoading: false,
        afterCustomTests: mergeAfterCustomTestResults(state.afterCustomTests, [])
      }

    default:
      notExhaustive(action)
      return state
  }
}

