import {ActionBase, ActionType} from '../../doExerciseSiteActionTypes'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../../index'
import {initial, State} from '../../doExerciseSiteReducer'
import {notExhaustive} from '../../../_notExhausiveHelper'
import {FilePreviewFromBackend} from '../../../../../types/exerciseEditor'


interface Meta {
  readonly customTestId: number
}

export interface ATTACH_customTestAssetPreviewAction
  extends ActionBase, ReduxPromiseAction<FilePreviewFromBackend, ATTACH_customTestAssetPreview_PENDINGAction, ATTACH_customTestAssetPreview_FULFILLEDAction, ATTACH_customTestAssetPreview_REJECTEDAction, Meta> {
  readonly type: ActionType.ATTACH_customTestAssetPreview
  readonly payload: Promise<FilePreviewFromBackend>
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface ATTACH_customTestAssetPreview_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.ATTACH_customTestAssetPreview_PENDING
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface ATTACH_customTestAssetPreview_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<FilePreviewFromBackend, Meta> {
  readonly type: ActionType.ATTACH_customTestAssetPreview_FULFILLED
  readonly payload: FilePreviewFromBackend
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface ATTACH_customTestAssetPreview_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void, Meta> {
  readonly type: ActionType.ATTACH_customTestAssetPreview_REJECTED
  readonly payload: any
  readonly meta: Meta
}


export type AllActions =
  ATTACH_customTestAssetPreview_PENDINGAction
  | ATTACH_customTestAssetPreview_FULFILLEDAction
  | ATTACH_customTestAssetPreview_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.ATTACH_customTestAssetPreview_PENDING:
      return {
        ...state,
        isAddCustomTestAssetFilesModalBusy: true
      }
    case ActionType.ATTACH_customTestAssetPreview_FULFILLED:
      return {
        ...state,
        isAddCustomTestAssetFilesModalBusy: false,
        exercise: {
          ...state.exercise,
          customTests: state.exercise.customTests.map(p => p.id !== action.meta.customTestId
                                                           ? p
                                                           : {
              ...p,
              files: p.files.concat(action.payload)
            })
        }
      }
    case ActionType.ATTACH_customTestAssetPreview_REJECTED:
      return {
        ...state,
        isAddCustomTestAssetFilesModalBusy: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

