import {ActionBase, ActionType} from '../../doExerciseSiteActionTypes'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../../index'
import {initial, State} from '../../doExerciseSiteReducer'
import {notExhaustive} from '../../../_notExhausiveHelper'
import {FilePreviewFromBackend} from '../../../../../types/exerciseEditor'


export interface GET_allUserAssetPreviewsAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<FilePreviewFromBackend>, GET_allUserAssetPreviews_PENDINGAction, GET_allUserAssetPreviews_FULFILLEDAction, GET_callUserAssetPreviews_REJECTEDAction> {
  readonly type: ActionType.GET_allUserAssetPreviews
  readonly payload: Promise<ReadonlyArray<FilePreviewFromBackend>>
}

//automatically dispatched by redux promise middleware
export interface GET_allUserAssetPreviews_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_allUserAssetPreviews_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_allUserAssetPreviews_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<FilePreviewFromBackend>> {
  readonly type: ActionType.GET_allUserAssetPreviews_FULFILLED
  readonly payload: ReadonlyArray<FilePreviewFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_callUserAssetPreviews_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_allUserAssetPreviews_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_allUserAssetPreviews_PENDINGAction
  | GET_allUserAssetPreviews_FULFILLEDAction
  | GET_callUserAssetPreviews_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_allUserAssetPreviews_PENDING:
      return {
        ...state,
        isLoading: true,
        isAddCustomTestAssetFilesModalBusy: true
      }
    case ActionType.GET_allUserAssetPreviews_FULFILLED:
      return {
        ...state,
        isLoading: false,
        isAddCustomTestAssetFilesModalBusy: false,
        allCustomTestAssetFilePreviews: action.payload
      }
    case ActionType.GET_allUserAssetPreviews_REJECTED:
      return {
        ...state,
        isLoading: false,
        isAddCustomTestAssetFilesModalBusy: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

