/**
 * Created by janis dähne.
 */
import {ActionBase, ActionType} from "../doExerciseSiteActionTypes";
import {initial, State} from "../doExerciseSiteReducer";
import Logger from "../../../../helpers/logger";
import {TestAnswerFromBackend} from "../../../../types/testsAndTestProtocol";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {notExhaustive} from "../../_notExhausiveHelper";
import {RunnableCustomTestFrontendExclusive} from "../../../../types/customTests";
import {mergeCustomTestResults} from "../../../../helpers/convertersAndTransformers";


export interface RUN_customTestAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<TestAnswerFromBackend>, RUN_customTest_PENDINGAction, RUN_customTest_FULFILLEDAction, RUN_customTest_REJECTEDAction> {
  readonly type: ActionType.RUN_customTest
  readonly payload: Promise<ReadonlyArray<TestAnswerFromBackend>>
}

//automatically dispatched by redux promise middleware
export interface RUN_customTest_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.RUN_customTest_PENDING
}

//automatically dispatched by redux promise middleware
export interface RUN_customTest_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<TestAnswerFromBackend>> {
  readonly type: ActionType.RUN_customTest_FULFILLED
  readonly payload: ReadonlyArray<TestAnswerFromBackend>
}

//automatically dispatched by redux promise middleware
export interface RUN_customTest_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.RUN_customTest_REJECTED
  readonly payload: any
}


export type AllActions =
  RUN_customTest_PENDINGAction
  | RUN_customTest_FULFILLEDAction
  | RUN_customTest_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.RUN_customTest_PENDING:
      return {
        ...state,
        isLoading: false //we use the nice test number spinner for loading
      }
    case ActionType.RUN_customTest_FULFILLED:
      return {
        ...state,
        isLoading: false,
        exercise: {
          ...state.exercise,
          customTests: mergeCustomTestResults(state.exercise.customTests, action.payload)
        }
      }
    case ActionType.RUN_customTest_REJECTED:
      return {
        ...state,
        isLoading: false,
        exercise: {
          ...state.exercise,
          customTests: mergeCustomTestResults(state.exercise.customTests, [])
        }
      }

    default:
      notExhaustive(action)
      return state
  }
}

