import {FilePreviewFromBackend} from '../../../../../types/exerciseEditor'
import {ActionBase, ActionType} from '../../editCustomProjectSiteActionTypes'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../../index'
import {State, initial} from '../../editCustomProjectSiteReducer'
import {notExhaustive} from '../../../_notExhausiveHelper'


interface Meta {
  readonly customTestId: number
  readonly assetPreviewId: number
}

export interface DETACH_customProjectCustomTestAssetPreviewAction
  extends ActionBase, ReduxPromiseAction<void, DETACH_customProjectCustomTestAssetPreview_PENDINGAction, DETACH_customProjectCustomTestAssetPreview_FULFILLEDAction, DETACH_customProjectCustomTestAssetPreview_REJECTEDAction, Meta> {
  readonly type: ActionType.DETACH_customProjectCustomTestAssetPreview
  readonly payload: Promise<void>
}

//automatically dispatched by redux promise middleware
export interface DETACH_customProjectCustomTestAssetPreview_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.DETACH_customProjectCustomTestAssetPreview_PENDING
}

//automatically dispatched by redux promise middleware
export interface DETACH_customProjectCustomTestAssetPreview_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<void, Meta> {
  readonly type: ActionType.DETACH_customProjectCustomTestAssetPreview_FULFILLED
  readonly payload: void
}

//automatically dispatched by redux promise middleware
export interface DETACH_customProjectCustomTestAssetPreview_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void, Meta> {
  readonly type: ActionType.DETACH_customProjectCustomTestAssetPreview_REJECTED
  readonly payload: any
}


export type AllActions =
  DETACH_customProjectCustomTestAssetPreview_PENDINGAction
  | DETACH_customProjectCustomTestAssetPreview_FULFILLEDAction
  | DETACH_customProjectCustomTestAssetPreview_REJECTEDAction


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.DETACH_customProjectCustomTestAssetPreview_PENDING:
      return {
        ...state,
        isAddCustomProjectCustomTestAssetFilesModalBusy: true
      }
    case ActionType.DETACH_customProjectCustomTestAssetPreview_FULFILLED:
      return {
        ...state,
        isAddCustomProjectCustomTestAssetFilesModalBusy: false,
        customProject: {
          ...state.customProject,
          tests: state.customProject.tests.map(p => p.id !== action.meta.customTestId
                                                    ? p
                                                    : {
              ...p,
              files: p.files.filter(k => k.id !== action.meta.assetPreviewId)
            })
        }
      }
    case ActionType.DETACH_customProjectCustomTestAssetPreview_REJECTED:
      return {
        ...state,
        isAddCustomProjectCustomTestAssetFilesModalBusy: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

