import {FilePreviewFromBackend} from '../../../../../types/exerciseEditor'
import {ActionBase, ActionType} from '../../editCustomProjectSiteActionTypes'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../../index'
import {State, initial} from '../../editCustomProjectSiteReducer'
import {notExhaustive} from '../../../_notExhausiveHelper'
import {createAndRefreshAssetUrlsAndObjects} from '../../../../../helpers/assetHelper'

export interface Meta {
  readonly assetPreviewId: number
}


export interface DETACH_customProjectMarkdownAssetPreviewAction
  extends ActionBase, ReduxPromiseAction<void, DETACH_customProjectMarkdownAssetPreview_PENDINGAction, DETACH_customProjectMarkdownAssetPreview_FULFILLEDAction, DETACH_customProjectMarkdownAssetPreview_REJECTEDAction, Meta> {
  readonly type: ActionType.DETACH_customProjectMarkdownAssetPreview
  readonly payload: Promise<void>
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface DETACH_customProjectMarkdownAssetPreview_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.DETACH_customProjectMarkdownAssetPreview_PENDING
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface DETACH_customProjectMarkdownAssetPreview_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<void, Meta> {
  readonly type: ActionType.DETACH_customProjectMarkdownAssetPreview_FULFILLED
  readonly payload: void
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface DETACH_customProjectMarkdownAssetPreview_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void, Meta> {
  readonly type: ActionType.DETACH_customProjectMarkdownAssetPreview_REJECTED
  readonly payload: any
  readonly meta: Meta
}


export type AllActions =
  DETACH_customProjectMarkdownAssetPreview_PENDINGAction
  | DETACH_customProjectMarkdownAssetPreview_FULFILLEDAction
  | DETACH_customProjectMarkdownAssetPreview_REJECTEDAction


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.DETACH_customProjectMarkdownAssetPreview_PENDING:
      return {
        ...state,
        isAddCustomProjectAssetFilesModalBusy: true
      }
    case ActionType.DETACH_customProjectMarkdownAssetPreview_FULFILLED:
      //remove the asset we detached

      const newAssetList = state.customProject.description.assets.filter(p => p.id !== action.meta.assetPreviewId)

      createAndRefreshAssetUrlsAndObjects(newAssetList)

      return {
        ...state,
        isAddCustomProjectAssetFilesModalBusy: false,
        customProject: {
          ...state.customProject,
          description: {
            ...state.customProject.description,
            assets: newAssetList
          }
        }
      }
    case ActionType.DETACH_customProjectMarkdownAssetPreview_REJECTED:
      return {
        ...state,
        isAddCustomProjectAssetFilesModalBusy: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

