import {FilePreviewFromBackend, FileWithData} from '../../../../../types/exerciseEditor'
import {ActionBase, ActionType} from '../../editCustomProjectSiteActionTypes'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../../index'
import {State, initial} from '../../editCustomProjectSiteReducer'
import {notExhaustive} from '../../../_notExhausiveHelper'
import {createAndRefreshAssetUrlsAndObjects} from '../../../../../helpers/assetHelper'



export interface LOAD_customProjectMarkdownAssetAction
  extends ActionBase, ReduxPromiseAction<FileWithData, LOAD_customProjectMarkdownAsset_PENDINGAction, LOAD_customProjectMarkdownAsset_FULFILLEDAction, LOAD_customProjectMarkdownAsset_REJECTEDAction> {
  readonly type: ActionType.LOAD_customProjectMarkdownAsset
  readonly payload: Promise<FileWithData>
}

//automatically dispatched by redux promise middleware
export interface LOAD_customProjectMarkdownAsset_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.LOAD_customProjectMarkdownAsset_PENDING
}

//automatically dispatched by redux promise middleware
export interface LOAD_customProjectMarkdownAsset_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<FileWithData> {
  readonly type: ActionType.LOAD_customProjectMarkdownAsset_FULFILLED
  readonly payload: FileWithData
}

//automatically dispatched by redux promise middleware
export interface LOAD_customProjectMarkdownAsset_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.LOAD_customProjectMarkdownAsset_REJECTED
  readonly payload: any
}


export type AllActions =
  LOAD_customProjectMarkdownAsset_PENDINGAction
  | LOAD_customProjectMarkdownAsset_FULFILLEDAction
  | LOAD_customProjectMarkdownAsset_REJECTEDAction


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.LOAD_customProjectMarkdownAsset_PENDING:
      return {
        ...state,
        isAddCustomProjectAssetFilesModalBusy: true,
      }
    case ActionType.LOAD_customProjectMarkdownAsset_FULFILLED:
      //append the loaded asset file

      const newAssetsList = state.customProject.description.assets.concat(action.payload)

      createAndRefreshAssetUrlsAndObjects(newAssetsList)

      return {
        ...state,
        isAddCustomProjectAssetFilesModalBusy: false,
        customProject: {
          ...state.customProject,
          description: {
            ...state.customProject.description,
            assets: newAssetsList
          }
        }
      }
    case ActionType.LOAD_customProjectMarkdownAsset_REJECTED:
      return {
        ...state,
        isAddCustomProjectAssetFilesModalBusy: false,
      }

    default:
      notExhaustive(action)
      return state
  }
}

