/**
 * Created by janis dähne.
 */

import {ActionBase, ActionType} from "../editCustomProjectSiteActionTypes";
import {initial, State} from "../editCustomProjectSiteReducer";
import {CustomProjectSolutionFullBase} from "../../../../types/customProjects";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {notExhaustive} from "../../_notExhausiveHelper";
import {KnownLangs} from '../../../../../i18n/i18nRoot'


interface Meta {
  readonly langId: KnownLangs
}

export interface UPDATE_customProjectSolutionAction
  extends ActionBase, ReduxPromiseAction<CustomProjectSolutionFullBase, UPDATE_customProjectSolution_PENDINGAction, UPDATE_customProjectSolution_FULFILLEDAction, UPDATE_customProjectSolution_REJECTEDAction, Meta> {
  readonly type: ActionType.UPDATE_customProjectSolution
  readonly payload: Promise<CustomProjectSolutionFullBase>
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface UPDATE_customProjectSolution_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.UPDATE_customProjectSolution_PENDING
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface UPDATE_customProjectSolution_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<CustomProjectSolutionFullBase, Meta> {
  readonly type: ActionType.UPDATE_customProjectSolution_FULFILLED
  readonly payload: CustomProjectSolutionFullBase
}

//automatically dispatched by redux promise middleware
export interface UPDATE_customProjectSolution_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void, Meta> {
  readonly type: ActionType.UPDATE_customProjectSolution_REJECTED
  readonly payload: any
}


export type AllActions =
  UPDATE_customProjectSolution_PENDINGAction
  | UPDATE_customProjectSolution_FULFILLEDAction
  | UPDATE_customProjectSolution_REJECTEDAction


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.UPDATE_customProjectSolution_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.UPDATE_customProjectSolution_FULFILLED:

      const selectedFile = action.payload.solutionFiles.find(
        p => p.id === state.solutionEditorState.selectedTabIdDoEditor)
      const newSelectedFileId = selectedFile
                                ? selectedFile.id
                                : -1

      return {
        ...state,
        isLoading: false,
        solutionEditorState: {
          ...state.solutionEditorState,
          selectedTabIdDoEditor: newSelectedFileId,
        },
        initialSolutionFromBackend: {
          ...action.payload //this makes only a shallow copy but we use readonly so should be ok
        },
        customProject: {
          ...state.customProject,
          solution: action.payload
        },
      }
    case ActionType.UPDATE_customProjectSolution_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

