/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {ActionBase, ActionType} from "./editorSettingsDialogActionTypes";
import {State, initial} from "./editorSettingsDialogReducer";
import Logger from "../../../helpers/logger";
import {UserSettingsFullBase} from "../../../types/userData";

export interface UPDATE_editorSettingsAction
  extends ActionBase, ReduxPromiseAction<UserSettingsFullBase, UPDATE_editorSettings_PENDINGAction, UPDATE_editorSettings_FULFILLEDAction, UPDATE_editorSettings_REJECTEDAction> {
  readonly type: ActionType.UPDATE_editorSettings
  readonly payload: Promise<UserSettingsFullBase>
}

//automatically dispatched by redux promise middleware
export interface UPDATE_editorSettings_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.UPDATE_editorSettings_PENDING
}

//automatically dispatched by redux promise middleware
export interface UPDATE_editorSettings_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<UserSettingsFullBase> {
  readonly type: ActionType.UPDATE_editorSettings_FULFILLED
  readonly payload: UserSettingsFullBase
}

//automatically dispatched by redux promise middleware
export interface UPDATE_editorSettings_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.UPDATE_editorSettings_REJECTED
  readonly payload: any
}


export type AllActions =
  UPDATE_editorSettings_PENDINGAction
  | UPDATE_editorSettings_FULFILLEDAction
  | UPDATE_editorSettings_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.UPDATE_editorSettings_PENDING:
      return {
        ...state,
        isBusy: true
      }
    case ActionType.UPDATE_editorSettings_FULFILLED:
      return {
        ...state,
        isBusy: false,
        editorSettings: action.payload.codeEditorSettings
      }
    case ActionType.UPDATE_editorSettings_REJECTED:
      return {
        ...state,
        isBusy: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

