import {ActionBase, ActionType} from '../../editorExerciseSiteActionTypes'
import {State, initial} from '../../exerciseEditorSiteReducer'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../../index'
import {notExhaustive} from '../../../_notExhausiveHelper'


export interface ATTACH_markdownAssetAction
  extends ActionBase, ReduxPromiseAction<void, ATTACH_markdownAsset_PENDINGAction, ATTACH_markdownAsset_FULFILLEDAction, ATTACH_markdownAsset_REJECTEDAction> {
  readonly type: ActionType.ATTACH_markdownAsset
  readonly payload: Promise<void>
}

//automatically dispatched by redux promise middleware
export interface ATTACH_markdownAsset_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.ATTACH_markdownAsset_PENDING
}

//automatically dispatched by redux promise middleware
export interface ATTACH_markdownAsset_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<void> {
  readonly type: ActionType.ATTACH_markdownAsset_FULFILLED
  readonly payload: void
}

//automatically dispatched by redux promise middleware
export interface ATTACH_markdownAsset_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.ATTACH_markdownAsset_REJECTED
  readonly payload: any
}


export type AllActions =
  ATTACH_markdownAsset_PENDINGAction
  | ATTACH_markdownAsset_FULFILLEDAction
  | ATTACH_markdownAsset_REJECTEDAction


export function reducer(state: State = initial(), action: AllActions): State {

  //the data is not handled here because the action is a multi action and also triggers loadSingleMarkdownAssetReducer
  //which handles the data

  switch (action.type) {

    case ActionType.ATTACH_markdownAsset_PENDING:
      return {
        ...state,
        isAddAssetFilesModalBusy: true
      }
    case ActionType.ATTACH_markdownAsset_FULFILLED:
      return {
        ...state,
        isAddAssetFilesModalBusy: false
      }
    case ActionType.ATTACH_markdownAsset_REJECTED:
      return {
        ...state,
        isAddAssetFilesModalBusy: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

