import {ActionBase, ActionType} from '../../editorExerciseSiteActionTypes'
import {State, initial} from '../../exerciseEditorSiteReducer'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../../index'
import {notExhaustive} from '../../../_notExhausiveHelper'
import {createAndRefreshAssetUrlsAndObjects} from '../../../../../helpers/assetHelper'

export interface Meta {
  readonly testId: number
  readonly assetPreviewId: number
}

export interface DETACH_testAssetAction
  extends ActionBase, ReduxPromiseAction<void, DETACH_testAsset_PENDINGAction, DETACH_testAsset_FULFILLEDAction, DETACH_testAsset_REJECTEDAction, Meta> {
  readonly type: ActionType.DETACH_testAsset
  readonly payload: Promise<void>
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface DETACH_testAsset_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.DETACH_testAsset_PENDING
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface DETACH_testAsset_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<void, Meta> {
  readonly type: ActionType.DETACH_testAsset_FULFILLED
  readonly payload: void
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface DETACH_testAsset_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void, Meta> {
  readonly type: ActionType.DETACH_testAsset_REJECTED
  readonly payload: any
  readonly meta: Meta
}


export type AllActions =
  DETACH_testAsset_PENDINGAction
  | DETACH_testAsset_FULFILLEDAction
  | DETACH_testAsset_REJECTEDAction


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.DETACH_testAsset_PENDING:
      return {
        ...state,
        isAddTestAssetFilesModalBusy: true
      }
    case ActionType.DETACH_testAsset_FULFILLED:
      return {
        ...state,
        isAddTestAssetFilesModalBusy: false,
        editorExercise: {
          ...state.editorExercise,
          tests: state.editorExercise.tests.map(p =>
                                                  p.id !== action.meta.testId
                                                  ? p
                                                  : {
                                                      ...p,
                                                      files: p.files.filter(
                                                        file => file.id !== action.meta.assetPreviewId)
                                                    })
        }
      }
    case ActionType.DETACH_testAsset_REJECTED:
      return {
        ...state,
        isAddTestAssetFilesModalBusy: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

