import {ActionBase, ActionType} from '../../editorExerciseSiteActionTypes'
import {State, initial} from '../../exerciseEditorSiteReducer'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../../index'
import {notExhaustive} from '../../../_notExhausiveHelper'
import {FilePreviewFromBackend} from '../../../../../types/exerciseEditor'

export interface GET_testAssetPreviewsAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<FilePreviewFromBackend>, GET_testAssetPreviews_PENDINGAction, GET_testAssetPreviews_FULFILLEDAction, GET_testAssetPreviews_REJECTEDAction> {
  readonly type: ActionType.GET_testAssetPreviews
  readonly payload: Promise<ReadonlyArray<FilePreviewFromBackend>>
}

//automatically dispatched by redux promise middleware
export interface GET_testAssetPreviews_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_testAssetPreviews_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_testAssetPreviews_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<FilePreviewFromBackend>> {
  readonly type: ActionType.GET_testAssetPreviews_FULFILLED
  readonly payload: ReadonlyArray<FilePreviewFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_testAssetPreviews_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_testAssetPreviews_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_testAssetPreviews_PENDINGAction
  | GET_testAssetPreviews_FULFILLEDAction
  | GET_testAssetPreviews_REJECTEDAction


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_testAssetPreviews_PENDING:
      return {
        ...state,
        isLoading: true,
        isAddTestAssetFilesModalBusy: true,
      }
    case ActionType.GET_testAssetPreviews_FULFILLED:
      return {
        ...state,
        isLoading: false,
        isAddTestAssetFilesModalBusy: false,
        allTestAssetFilePreviews: action.payload
      }
    case ActionType.GET_testAssetPreviews_REJECTED:
      return {
        ...state,
        isLoading: false,
        isAddTestAssetFilesModalBusy: false,
        allTestAssetFilePreviews: [],
      }

    default:
      notExhaustive(action)
      return state
  }
}

