import {ActionBase, ActionType} from '../../editorExerciseSiteActionTypes'
import {State, initial} from '../../exerciseEditorSiteReducer'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../../index'
import {notExhaustive} from '../../../_notExhausiveHelper'
import {FileWithData} from '../../../../../types/exerciseEditor'
import {createAndRefreshAssetUrlsAndObjects} from '../../../../../helpers/assetHelper'

export interface LOAD_markdownAssetAction
  extends ActionBase, ReduxPromiseAction<FileWithData, LOAD_markdownAsset_PENDINGAction, LOAD_markdownAsset_FULFILLEDAction, LOAD_markdownAsset_REJECTEDAction> {
  readonly type: ActionType.LOAD_markdownAsset
  readonly payload: Promise<FileWithData>
}

//automatically dispatched by redux promise middleware
export interface LOAD_markdownAsset_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.LOAD_markdownAsset_PENDING
}

//automatically dispatched by redux promise middleware
export interface LOAD_markdownAsset_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<FileWithData> {
  readonly type: ActionType.LOAD_markdownAsset_FULFILLED
  readonly payload: FileWithData
}

//automatically dispatched by redux promise middleware
export interface LOAD_markdownAsset_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.LOAD_markdownAsset_REJECTED
  readonly payload: any
}


export type AllActions =
  LOAD_markdownAsset_PENDINGAction
  | LOAD_markdownAsset_FULFILLEDAction
  | LOAD_markdownAsset_REJECTEDAction


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.LOAD_markdownAsset_PENDING:
      return {
        ...state,
        isAddAssetFilesModalBusy: true
      }
    case ActionType.LOAD_markdownAsset_FULFILLED:
      //append the loaded asset file

      const newAssetsList = state.editorExercise.exerciseDescription.assets.concat(action.payload)

      createAndRefreshAssetUrlsAndObjects(newAssetsList)

      return {
        ...state,
        isAddAssetFilesModalBusy: false,
        editorExercise: {
          ...state.editorExercise,
          exerciseDescription: {
            ...state.editorExercise.exerciseDescription,
            assets: newAssetsList
          }
        }
      }
    case ActionType.LOAD_markdownAsset_REJECTED:
      return {
        ...state,
        isAddAssetFilesModalBusy: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

