import {ActionBase, ActionType} from '../editorExerciseSiteActionTypes'
import {State, initial} from '../exerciseEditorSiteReducer'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../index'
import {notExhaustive} from '../../_notExhausiveHelper'


export interface LOCK_exercisePermanentlyAction
  extends ActionBase, ReduxPromiseAction<void, LOCK_exercisePermanently_PENDINGAction, LOCK_exercisePermanently_FULFILLEDAction, LOCK_exercisePermanently_REJECTEDAction> {
  readonly type: ActionType.LOCK_exercisePermanently
  readonly payload: Promise<void>
}

//automatically dispatched by redux promise middleware
export interface LOCK_exercisePermanently_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.LOCK_exercisePermanently_PENDING
}

//automatically dispatched by redux promise middleware
export interface LOCK_exercisePermanently_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<void> {
  readonly type: ActionType.LOCK_exercisePermanently_FULFILLED
  readonly payload: void
}

//automatically dispatched by redux promise middleware
export interface LOCK_exercisePermanently_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.LOCK_exercisePermanently_REJECTED
  readonly payload: any
}


export type AllActions =
  LOCK_exercisePermanently_PENDINGAction
  | LOCK_exercisePermanently_FULFILLEDAction
  | LOCK_exercisePermanently_REJECTEDAction


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.LOCK_exercisePermanently_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.LOCK_exercisePermanently_FULFILLED:
      return {
        ...state,
        isLoading: false,
        editorExercise: {
          ...state.editorExercise,
          exerciseProperties: {
            ...state.editorExercise.exerciseProperties,
            isPermanentlyLocked: true,
            isOnlyVisibleToMe: false, //is also set in frontend
          }
        }
      }
    case ActionType.LOCK_exercisePermanently_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

