/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../../_notExhausiveHelper";
import {GroupRoleWithPermissionFullBase} from "../../../../types/groupRoles";

import {isValidDialog} from "../../../../validation/dialogValidations";
import {
  AllActions as SetActions,
  initial as preInitial,
  reducer as setReducer,
  validationRules
} from "./groupRoleReducer";
import {ActionBase, ActionType} from "./groupRolesDialogActionTypes";
import {AllActions as CreateActions, reducer as createReducer} from './createGroupRoleReducer'
import {AllActions as UpdateActions, reducer as updateReducer} from './updateGroupRoleReducer'
import {ActionTypeGlobal, RESET_GlobalAction} from '../../globalActions'

export type State = {
  readonly isValid: boolean

  /**
   * true: show loader,
   * false: not
   */
  readonly isBusy: boolean

  /**
   * true: create entity
   * false: change entity,
   */
  readonly isCreating: boolean

  readonly groupRole: GroupRoleWithPermissionFullBase

}

export const initial: State = {
  isCreating: false,
  isBusy: false,
  groupRole: preInitial,
  isValid: isValidDialog(preInitial, validationRules)
}


export interface SET_isCreatingAction extends ActionBase {
  readonly type: ActionType.SET_isCreating
  readonly isCreating: boolean
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_isCreatingAction
  | SetActions
  | CreateActions
  | UpdateActions

  | RESET_GlobalAction


export function reducer(state: State = initial, action: AllActions): State {

  // noinspection TsLint
  let temp: GroupRoleWithPermissionFullBase = null

  switch (action.type) {
    case ActionType.SET_groupRole:
    case ActionType.SET_id:
    case ActionType.SET_displayName:
    case ActionType.SET_canCreateExercises:
    case ActionType.SET_canChangeExercises:
    case ActionType.SET_canDeleteExercises:
    case ActionType.SET_canLockExercisesPermanently:
    case ActionType.SET_canManageExerciseReleases:
    case ActionType.SET_canAssessExercises:
    case ActionType.SET_canSeeExercisesFromOthersInGroup:
    case ActionType.SET_canAddUserToGroup:
    case ActionType.SET_canRemoveMemberFromGroup:
    case ActionType.SET_canChangeOtherMembersRole:
    case ActionType.SET_canSeeOtherMembers:
    case ActionType.SET_canChangeGroupData:
      temp = setReducer(state.groupRole, action)
      return {
        ...state,
        groupRole: temp,
        isValid: isValidDialog(temp, validationRules)
      }

    case ActionType.SET_isCreating:
      return {
        ...state,
        isCreating: action.isCreating
      }


    case ActionType.CREATE_groupRole_PENDING:
    case ActionType.CREATE_groupRole_FULFILLED:
    case ActionType.CREATE_groupRole_REJECTED:
      return createReducer(state, action)


    case ActionType.UPDATE_groupRole_PENDING:
    case ActionType.UPDATE_groupRole_FULFILLED:
    case ActionType.UPDATE_groupRole_REJECTED:
      return updateReducer(state, action)

    case ActionType.RESET:
    case ActionTypeGlobal.RESET_Global:
      return {
        ...initial
      }

    default:
      notExhaustive(action)
      return state
  }
}

