/**
 * Created by janis dähne.
 */

import {UserGroupBase, UserGroupFromBackend} from "../../../../types/group";
import {getValidationCollection, isEmailOrEmpty, isNotEmptyOrSpaces} from "../../../../validation/validations";
import {ActionBase, ActionType} from './actionTypes'
import {notExhaustive} from "../../_notExhausiveHelper";

export type State = UserGroupFromBackend


export const initial: State = {
  id: -1,
  email: '',
  displayName: '',
  groupRole: null
}


export const validationRules = getValidationCollection<UserGroupFromBackend>({
  id: [],
  displayName: [isNotEmptyOrSpaces],
  email: [isEmailOrEmpty],
  groupRole: [],
})


export interface SET_idAction extends ActionBase {
  readonly type: ActionType.SET_id
  readonly id: number
}

export interface SET_emailAction extends ActionBase {
  readonly type: ActionType.SET_email
  readonly email: string
}

export interface SET_displayNameAction extends ActionBase {
  readonly type: ActionType.SET_displayName
  readonly displayName: string
}


export type AllActions =
  SET_idAction
  | SET_emailAction
  | SET_displayNameAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.SET_id:
      return {
        ...state,
        id: action.id
      }
    case ActionType.SET_email:
      return {
        ...state,
        email: action.email
      }
    case ActionType.SET_displayName:
      return {
        ...state,
        displayName: action.displayName
      }


    default:
      notExhaustive(action)
      return state
  }
}

