/**
 * Created by janis dähne.
 */
import {ActionBase, ActionType} from "./actionTypes";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {UserGroupBase} from "../../../../types/group";
import {initial, State} from "./changeGroupReducer";
import {notExhaustive} from "../../_notExhausiveHelper";


export interface UPDATE_groupAction
  extends ActionBase, ReduxPromiseAction<UserGroupBase, UPDATE_group_PENDINGAction, UPDATE_group_FULFILLEDAction, UPDATE_group_REJECTEDAction> {
  readonly type: ActionType.UPDATE_group
  readonly payload: Promise<UserGroupBase>
}

//automatically dispatched by redux promise middleware
export interface UPDATE_group_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.UPDATE_group_PENDING
}

//automatically dispatched by redux promise middleware
export interface UPDATE_group_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<UserGroupBase> {
  readonly type: ActionType.UPDATE_group_FULFILLED
  readonly payload: UserGroupBase
}

//automatically dispatched by redux promise middleware
export interface UPDATE_group_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<UserGroupBase> {
  readonly type: ActionType.UPDATE_group_REJECTED
  readonly payload: any
}


export type AllActions =
  UPDATE_group_PENDINGAction
  | UPDATE_group_FULFILLEDAction
  | UPDATE_group_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.UPDATE_group_PENDING:
      return {
        ...state,
        isBusy: true
      }
    case ActionType.UPDATE_group_FULFILLED:
      return {
        ...state,
        isBusy: false,
      }
    case ActionType.UPDATE_group_REJECTED:
      return {
        ...state,
        isBusy: false
      }

    default:
      notExhaustive(action)
      return state
  }
}
