/**
 * Created by janis dähne.
 */
import {UserGroupBase, UserGroupWithExerciseCountFromBackend} from "../../../types/group";
import {notExhaustive} from "../_notExhausiveHelper";
import {ActionBase, ActionType} from "./groupsSiteActionTypes";
import {AllActions as GetActions, reducer as getReducer} from './getGroupsReducer'
import {AllActions as QuitActions, reducer as quitReducer} from './quitGroupReducer'
import {SortDirection} from "../../../types/sortDirection";
import {AllActions as DeleteGroupActions, reducer as deleteGroupReducer} from './deleteGroupReducer'
import {FrontendSettingsManager} from "../../../helpers/frontendSettingsManager";
import {PaginationStateExtend, PaginationStateData} from '../../../types/pagination'
import {getInitialPaginationData} from '../../../constants'
import {ActionTypeGlobal, RESET_GlobalAction} from '../globalActions'

export type State = {
  readonly groups: ReadonlyArray<UserGroupWithExerciseCountFromBackend>
  readonly isChangeDialogDisplayed: boolean
  readonly isLoading: boolean

} & PaginationStateExtend<UserGroupWithExerciseCountFromBackend>


export const initial: State = {
  groups: [],
  isLoading: false,
  isChangeDialogDisplayed: false,

  pagination: {
    ...getInitialPaginationData(),
    orderByProp: 'displayName'
  }
}

//---pagination
export interface SetSearchTextAction extends ActionBase {
  readonly type: ActionType.setSearchText
  readonly searchText: string
}

export interface SetActivePaginationPageAction extends ActionBase {
  readonly type: ActionType.setPaginationActivePage
  readonly activePage: number
}

export interface SetPaginationPageSizeAction extends ActionBase {
  readonly type: ActionType.setPaginationPageSize
  readonly pageSize: number
}

//---pagination end

export interface SET_isChangeDialogDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isChangeDialogDisplayed
  readonly isDisplayed: boolean
}


export interface SET_sortByKeyAction extends ActionBase {
  readonly type: ActionType.SET_sortByKey
  readonly sortByKey: keyof UserGroupBase
  readonly sortDirection: boolean | null
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_isChangeDialogDisplayedAction
  | SET_sortByKeyAction
  | GetActions
  | QuitActions
  | DeleteGroupActions
  | SetSearchTextAction
  | SetActivePaginationPageAction
  | SetPaginationPageSizeAction

  | RESET_GlobalAction


export function reducer(state: State = initial, action: AllActions): State {
  switch (action.type) {

    case ActionType.SET_isChangeDialogDisplayed:
      return {
        ...state,
        isChangeDialogDisplayed: action.isDisplayed
      }

    case ActionType.DELETE_userGroup_PENDING:
    case ActionType.DELETE_userGroup_FULFILLED:
    case ActionType.DELETE_userGroup_REJECTED:
      return deleteGroupReducer(state, action)

    case ActionType.GET_groups_PENDING:
    case ActionType.GET_groups_FULFILLED:
    case ActionType.GET_groups_REJECTED:
      return getReducer(state, action)

    case ActionType.QUIT_group_PENDING:
    case ActionType.QUIT_group_FULFILLED:
    case ActionType.QUIT_group_REJECTED:
      return quitReducer(state, action)

    case ActionType.SET_sortByKey:
      return {
        ...state,
        pagination: {
          ...state.pagination,
          orderByProp: action.sortByKey,
          isOrderByDesc: action.sortDirection
        }
      }

    case ActionType.setSearchText:
      return {
        ...state,
        pagination: {
          ...state.pagination,
          searchText: action.searchText
        }
      }
    case ActionType.setPaginationActivePage:
      return {
        ...state,
        pagination: {
          ...state.pagination,
          currentPage: action.activePage
        }
      }

    case ActionType.setPaginationPageSize:

      FrontendSettingsManager.set_groupsPageSize(action.pageSize)

      return {
        ...state,
        pagination: {
          ...state.pagination,
          pageSize: action.pageSize,
          currentPage: 1,
        }
      }

    case ActionType.RESET:
    case ActionTypeGlobal.RESET_Global:
      return {
        ...initial,
        pagination: {
          ...initial.pagination,
          pageSize: FrontendSettingsManager.getFrontendSettings().groupsPageSize,
        }
      }

    default:
      notExhaustive(action)
      return state
  }
}

