
import {ActionBase, ActionType} from '../langsViewActionTypes'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../index'
import {initial, State} from '../langsViewReducer'
import {notExhaustive} from '../../_notExhausiveHelper'
import {LangFullBase} from '../../../../types/languages'




export interface CREATE_langAction
  extends ActionBase, ReduxPromiseAction<LangFullBase, CREATE_lang_PENDINGAction, CREATE_lang_FULFILLEDAction, CREATE_lang_REJECTEDAction> {
  readonly type: ActionType.CREATE_lang
  readonly payload: Promise<LangFullBase>
}

//automatically dispatched by redux promise middleware
export interface CREATE_lang_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.CREATE_lang_PENDING
}

//automatically dispatched by redux promise middleware
export interface CREATE_lang_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<LangFullBase> {
  readonly type: ActionType.CREATE_lang_FULFILLED
  readonly payload: LangFullBase
}

//automatically dispatched by redux promise middleware
export interface CREATE_lang_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.CREATE_lang_REJECTED
  readonly payload: any
}


export type AllActions =
  CREATE_lang_PENDINGAction
  | CREATE_lang_FULFILLEDAction
  | CREATE_lang_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.CREATE_lang_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.CREATE_lang_FULFILLED:
      return {
        ...state,
        isLoading: false,
        langs: state.langs.concat([action.payload]),
        langCopy: null,
        editingLangId: null,
      }
    case ActionType.CREATE_lang_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

