
import {ActionBase, ActionType} from '../langsViewActionTypes'
import {initial, State} from '../langsViewReducer'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../index'
import {notExhaustive} from '../../_notExhausiveHelper'


interface Meta {
  readonly langId: number
}

export interface DELETE_langAction
  extends ActionBase, ReduxPromiseAction<void, DELETE_lang_PENDINGAction, DELETE_lang_FULFILLEDAction, DELETE_lang_REJECTEDAction, Meta> {
  readonly type: ActionType.DELETE_lang
  readonly payload: Promise<void>
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface DELETE_lang_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.DELETE_lang_PENDING
}

//automatically dispatched by redux promise middleware
export interface DELETE_lang_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<void, Meta> {
  readonly type: ActionType.DELETE_lang_FULFILLED
  readonly payload: void
}

//automatically dispatched by redux promise middleware
export interface DELETE_lang_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void, Meta> {
  readonly type: ActionType.DELETE_lang_REJECTED
  readonly payload: any
}


export type AllActions =
  DELETE_lang_PENDINGAction
  | DELETE_lang_FULFILLEDAction
  | DELETE_lang_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.DELETE_lang_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.DELETE_lang_FULFILLED:
      return {
        ...state,
        isLoading: false,
        langs: state.langs.filter(p => p.id !== action.meta.langId)
      }
    case ActionType.DELETE_lang_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

