import {ActionBase, ActionType} from '../langsViewActionTypes'
import {LangFullBase} from '../../../../types/languages'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../index'
import {initial, State} from '../langsViewReducer'
import {notExhaustive} from '../../_notExhausiveHelper'

export interface GET_languagesAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<LangFullBase>, GET_languages_PENDINGAction, GET_languages_FULFILLEDAction, GET_languages_REJECTEDAction> {
  readonly type: ActionType.GET_languages
  readonly payload: Promise<ReadonlyArray<LangFullBase>>
}

//automatically dispatched by redux promise middleware
export interface GET_languages_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_languages_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_languages_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<LangFullBase>> {
  readonly type: ActionType.GET_languages_FULFILLED
  readonly payload: ReadonlyArray<LangFullBase>
}

//automatically dispatched by redux promise middleware
export interface GET_languages_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_languages_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_languages_PENDINGAction
  | GET_languages_FULFILLEDAction
  | GET_languages_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_languages_PENDING:
      return {
        ...state,
        isLoading: true,
      }
    case ActionType.GET_languages_FULFILLED:
      return {
        ...state,
        isLoading: false,
        langs: action.payload
      }
    case ActionType.GET_languages_REJECTED:
      return {
        ...state,
        langs: []
      }

    default:
      notExhaustive(action)
      return state
  }
}

