/**
 * Created by janis dähne.
 */
import {BaseUserData, IdAndPasswordTuple, SystemRoleFullBase} from "../../../../types/userData";
import {getValidationCollection, isEmailAndNotEmpty, isNotEmptyOrSpaces} from "../../../../validation/validations";
import {ActionBase, ActionType} from "./changeActivatedUserDataDialogActionTypes";
import {notExhaustive} from "../../_notExhausiveHelper";
import {SelectableUserFrontendExclusive} from '../../../../types/users'
import moment from 'moment'


//TODO make 1 post for backend not two --> then make 1 type (BaseUserData + systemRole)

export type State = SelectableUserFrontendExclusive & IdAndPasswordTuple

export const initial: State = {
  targetUserId: -1,
  email: '',
  lastName: '',
  firstName: '',
  password: '',
  systemRole: null,

  uiIsSelected: false,
  token: '',
  systemRoleId: -1,
  lastLoginAt: moment(),
  isDisplayed: true,
  isCurrentUser: false,
  isActivated: true,
  id: -1,
  createdAt: moment(),
}

export const validationRules = getValidationCollection<State>({
                                                                targetUserId: [],
                                                                email: [isEmailAndNotEmpty],
                                                                lastName: [isNotEmptyOrSpaces],
                                                                firstName: [isNotEmptyOrSpaces],
                                                                password: [],
                                                                systemRole: [],

                                                                createdAt: [],
                                                                id: [],
                                                                isActivated: [],
                                                                isCurrentUser: [],
                                                                isDisplayed: [],
                                                                lastLoginAt: [],
                                                                systemRoleId: [],
                                                                token: [],
                                                                uiIsSelected: [],
                                                              })


export interface SET_targetUserAction extends ActionBase {
  readonly type: ActionType.SET_targetUser
  readonly targetUser: SelectableUserFrontendExclusive
}

export interface SET_emailAction extends ActionBase {
  readonly type: ActionType.SET_email
  readonly email: string
}

export interface SET_lastNameAction extends ActionBase {
  readonly type: ActionType.SET_lastName
  readonly lastName: string
}

export interface SET_firstNameAction extends ActionBase {
  readonly type: ActionType.SET_firstName
  readonly firstName: string
}


export interface SET_systemRoleAction extends ActionBase {
  readonly type: ActionType.SET_systemRole
  readonly systemRole: SystemRoleFullBase | null
}

export type AllActions =
  SET_targetUserAction
  | SET_emailAction
  | SET_lastNameAction
  | SET_firstNameAction
  | SET_systemRoleAction

export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.SET_targetUser:
      return {
        ...state,
        ...action.targetUser,
        targetUserId: action.targetUser.id
      }

    case ActionType.SET_email:
      return {
        ...state,
        email: action.email
      }
    case ActionType.SET_lastName:
      return {
        ...state,
        lastName: action.lastName
      }
    case ActionType.SET_firstName:
      return {
        ...state,
        firstName: action.firstName
      }
    case ActionType.SET_systemRole:
      return {
        ...state,
        systemRole: action.systemRole
      }

    default:
      notExhaustive(action)
      return state
  }
}

