/**
 * Created by janis dähne.
 */

import {ActionBase, ActionType} from "./manageCustomProjectsSiteActionTypes";
import {notExhaustive} from "../_notExhausiveHelper";
import {CustomProjectPreviewFullBase} from "../../../types/customProjects";

import {
  AllActions as GetCustomProjectOverviewActions,
  reducer as getCustomProjectOverviews
} from './getCustomProjectOverviewsReducer'
import {SortDirection} from "../../../types/sortDirection";

import {
  AllActions as DeleteCustomProjectActions,
  reducer as deleteCustomProjectReducer
} from './deleteCustomProjectReducer'
import {PaginationStateExtend} from '../../../types/pagination'
import {getInitialPaginationData} from '../../../constants'
import {FrontendSettingsManager} from '../../../helpers/frontendSettingsManager'
import {ActionTypeGlobal, RESET_GlobalAction} from '../globalActions'

export type State = {

  readonly isLoading: boolean
  readonly customProjectPreviews: ReadonlyArray<CustomProjectPreviewFullBase>

} & PaginationStateExtend<CustomProjectPreviewFullBase>

export const initial: State = {
  isLoading: false,
  customProjectPreviews: [],

  pagination: {
    ...getInitialPaginationData(),
    orderByProp: 'displayName'
  }
}

//---pagination
export interface SetSearchTextAction extends ActionBase {
  readonly type: ActionType.setSearchText
  readonly searchText: string
}

export interface SetActivePaginationPageAction extends ActionBase {
  readonly type: ActionType.setPaginationActivePage
  readonly activePage: number
}

export interface SetPaginationPageSizeAction extends ActionBase {
  readonly type: ActionType.setPaginationPageSize
  readonly pageSize: number
}

//---pagination end

export interface SET_sortByKeyAction extends ActionBase {
  readonly type: ActionType.SET_sortByKey
  readonly sortByKey: keyof CustomProjectPreviewFullBase
  readonly sortDirection: boolean | null
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_sortByKeyAction

  | GetCustomProjectOverviewActions
  | DeleteCustomProjectActions
  | SetSearchTextAction
  | SetActivePaginationPageAction
  | SetPaginationPageSizeAction

  | RESET_GlobalAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.DELETE_customProject_PENDING:
    case ActionType.DELETE_customProject_FULFILLED:
    case ActionType.DELETE_customProject_REJECTED:
      return deleteCustomProjectReducer(state, action)

    case ActionType.GET_customProjectOverviews_PENDING:
    case ActionType.GET_customProjectOverviews_FULFILLED:
    case ActionType.GET_customProjectOverviews_REJECTED:
      return getCustomProjectOverviews(state, action)

    case ActionType.SET_sortByKey:
      return {
        ...state,
        pagination: {
          ...state.pagination,
          orderByProp: action.sortByKey,
          isOrderByDesc: action.sortDirection
        }
      }

    case ActionType.setSearchText:
      return {
        ...state,
        pagination: {
          ...state.pagination,
          searchText: action.searchText
        }
      }
    case ActionType.setPaginationActivePage:
      return {
        ...state,
        pagination: {
          ...state.pagination,
          currentPage: action.activePage
        }
      }

    case ActionType.setPaginationPageSize:

      FrontendSettingsManager.set_myProjectsPageSize(action.pageSize)

      return {
        ...state,
        pagination: {
          ...state.pagination,
          pageSize: action.pageSize,
          currentPage: 1,
        }
      }

    case ActionType.RESET:
    case ActionTypeGlobal.RESET_Global:
      return {
        ...initial,
        pagination: {
          ...initial.pagination,
          pageSize: FrontendSettingsManager.getFrontendSettings().myProjectsPageSize,
        }
      }

    default:
      //notExhaustive(action)
      return state
  }
}

