/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../_notExhausiveHelper";
import {
  ReduxPromiseAction,
  ReduxPromiseFulfilledAction,
  ReduxPromisePendingAction,
  ReduxPromiseRejectedAction
} from "../index";
import {ActionBase, ActionType} from "./groupExercisesSiteActionTypes";
import {State, initial} from "./groupExercisesSiteReducer";
import Logger from "../../../helpers/logger";

//there is actually no difference in deleting an own exercise or group exercise...
//but maybe we need to do some other actions??

interface Meta {
  readonly groupId: number
}

export interface DELETE_groupExerciseAction
  extends ActionBase, ReduxPromiseAction<void, DELETE_groupExercise_PENDINGAction, DELETE_groupExercise_FULFILLEDAction, DELETE_groupExercise_REJECTEDAction, Meta> {
  readonly type: ActionType.DELETE_groupExercise
  readonly payload: Promise<void>
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface DELETE_groupExercise_PENDINGAction
  extends ActionBase, ReduxPromisePendingAction<Meta> {
  readonly type: ActionType.DELETE_groupExercise_PENDING
}

//automatically dispatched by redux promise middleware
export interface DELETE_groupExercise_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<void, Meta> {
  readonly type: ActionType.DELETE_groupExercise_FULFILLED
  readonly payload: void
}

//automatically dispatched by redux promise middleware
export interface DELETE_groupExercise_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void, Meta> {
  readonly type: ActionType.DELETE_groupExercise_REJECTED
  readonly payload: any
}


export type AllActions =
  DELETE_groupExercise_PENDINGAction
  | DELETE_groupExercise_FULFILLEDAction
  | DELETE_groupExercise_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.DELETE_groupExercise_PENDING:
      return {
        ...state,
        groupExercises: state.groupExercises.map(p =>
                                                   p.userGroup.id === action.meta.groupId
                                                   ? {
                                                       ...p,
                                                       isLoading: true
                                                     }
                                                   : p
        )
      }
    case ActionType.DELETE_groupExercise_FULFILLED:
      return {
        ...state,
        groupExercises: state.groupExercises.map(p =>
                                                   p.userGroup.id === action.meta.groupId
                                                   ? {
                                                       ...p,
                                                       isLoading: false
                                                     }
                                                   : p
        )
      }
    case ActionType.DELETE_groupExercise_REJECTED:
      return {
        ...state,
        groupExercises: state.groupExercises.map(p =>
                                                   p.userGroup.id === action.meta.groupId
                                                   ? {
                                                       ...p,
                                                       isLoading: false
                                                     }
                                                   : p
        )
      }

    default:
      notExhaustive(action)
      return state
  }
}

