import {notExhaustive} from "../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {ActionBase, ActionType} from './groupExercisesSiteActionTypes'
import {initial, State} from './groupExercisesSiteReducer'
import {UserGroupBase, UserGroupFromBackend} from '../../../types/group'
import {GroupPreviewExerciseTupleFrontendOnly} from '../../../types/exercisePreview'
import {getInitialPaginationData} from '../../../constants'
import {FrontendSettingsManager, frontendSettings_syncDisplaySortingItems} from '../../../helpers/frontendSettingsManager'
import orderBy from 'lodash-es/orderBy'


export interface GET_groupsForExercisesAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<UserGroupFromBackend>, GET_groupsForExercises_PENDINGAction, GET_groupsForExercises_FULFILLEDAction, GET_groupsForExercises_REJECTEDAction> {
  readonly type: ActionType.GET_groupsForExercises
  readonly payload: Promise<ReadonlyArray<UserGroupFromBackend>>
}

//automatically dispatched by redux promise middleware
export interface GET_groupsForExercises_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_groupsForExercises_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_groupsForExercises_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<UserGroupFromBackend>> {
  readonly type: ActionType.GET_groupsForExercises_FULFILLED
  readonly payload: ReadonlyArray<UserGroupFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_groupsForExercises_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_groupsForExercises_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_groupsForExercises_PENDINGAction
  | GET_groupsForExercises_FULFILLEDAction
  | GET_groupsForExercises_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_groupsForExercises_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_groupsForExercises_FULFILLED:

      const pageSizeSetting = FrontendSettingsManager.getFrontendSettings().groupExercisesPageSize

      //ensure our frontend group sorting is up-to-date


      const synResult = frontendSettings_syncDisplaySortingItems(FrontendSettingsManager.getFrontendSettings().groupExercisesGroupSorting, action.payload.map(p => p.id))

      if (synResult.anyChanges) {
        FrontendSettingsManager.set_groupExercisesGroupSorting(synResult.result)
      }

      const frontendSorting = FrontendSettingsManager.getFrontendSettings().groupExercisesGroupSorting

      let groupList = action.payload

      if (frontendSorting) {
        groupList = orderBy(action.payload, (val, index) => {
          const _index = frontendSorting.findIndex(p => p === val.id)
          return _index
        })
      }

      return {
        ...state,
        isLoading: false,
        groups: groupList,
        groupExercises: groupList.map<GroupPreviewExerciseTupleFrontendOnly>(p => {

          // noinspection TsLint
          let pageSize = pageSizeSetting[p.id]

          if (pageSize === undefined) {
            pageSize = getInitialPaginationData().pageSize
          }

          return {
            userGroup: p,
            pagination: {
              ...getInitialPaginationData(),
              orderByProp: 'displayName',
              pageSize: pageSize
            },
            exercisePreviews: [] as any,
            isLoading: true
          }
        })
      }
    case ActionType.GET_groupsForExercises_REJECTED:
      return {
        ...state,
        isLoading: false,
        groups: []
      }

    default:
      notExhaustive(action)
      return state
  }
}

