/**
 * Created by janis dähne.
 */
import {Action} from "redux";
import {notExhaustive} from "../_notExhausiveHelper";
import {ActionBase, ActionType} from "./manageNewUsersSiteActionTypes";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {UserPreviewFromBackend} from "../../../types/users";
import {State, initial} from './manageNewUsersSiteReducer'
import Logger from "../../../helpers/logger";
import {toSelectableUser} from "../../../communicationLayer/usersLayer";
import {PaginatedData} from '../../../types/pagination'
import {PaginationHelper} from '../../../helpers/paginationHelper'


export interface GET_newUsersAction
  extends ActionBase, ReduxPromiseAction<PaginatedData<UserPreviewFromBackend>, GET_newUsers_PENDINGAction, GET_newUsers_FULFILLEDAction, GET_newUsers_REJECTEDAction> {
  readonly type: ActionType.GET_newUsers
  readonly payload: Promise<PaginatedData<UserPreviewFromBackend>>
}

//automatically dispatched by redux promise middleware
export interface GET_newUsers_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_newUsers_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_newUsers_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<PaginatedData<UserPreviewFromBackend>> {
  readonly type: ActionType.GET_newUsers_FULFILLED
  readonly payload: PaginatedData<UserPreviewFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_newUsers_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<PaginatedData<UserPreviewFromBackend>> {
  readonly type: ActionType.GET_newUsers_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_newUsers_PENDINGAction
  | GET_newUsers_FULFILLEDAction
  | GET_newUsers_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_newUsers_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_newUsers_FULFILLED:
      return {
        ...state,
        newUsers: toSelectableUser(action.payload.items),
        pagination: PaginationHelper.getPaginationStateSpreadObj(action.payload),
        isLoading: false
      }
    case ActionType.GET_newUsers_REJECTED:
      return {
        ...state,
        newUsers: [],
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

