/**
 * Created by janis dähne.
 */

import {notExhaustive} from "../../../../_notExhausiveHelper";
import {ActionBase, ActionType} from "./changeExerciseReleaseDialogActionTypes";
import {State as PreInitState, initial as preInitial, validationRules} from './setExerciseReleaseReducer'
import {isValidDialog} from "../../../../../../validation/dialogValidations";
import {
  AllActions as SetExerciseReleaseActions,
  reducer as setExerciseReleaseReducer
} from './setExerciseReleaseReducer'
import {ExerciseReleaseForBackend} from "../../../../../../types/exerciseRelease";
import {
  AllActions as CreateExerciseReleaseActions,
  reducer as createExerciseReleaseReducer
} from './createExerciseReleaseReducer'
import {
  AllActions as UpdateExerciseReleaseActions,
  reducer as updateExerciseReleaseReducer
} from './updateExerciseReleaseReducer'
import {EditableExercisePreviewFromBackend} from '../../../../../../types/exercisePreview'
import {ActionTypeGlobal, RESET_GlobalAction} from '../../../../globalActions'

export type State = {
  readonly exercisePreview: EditableExercisePreviewFromBackend | null
  readonly exerciseRelease: ExerciseReleaseForBackend
  readonly isBusy: boolean
  readonly isValid: boolean
  readonly isCreatingExerciseRelease: boolean

  /**
   * the initial is released state for the dialog (used to determine if the user wants to unrelease or not
   * and to display different states/hints)
   *
   * only set through SET_exerciseReleaseAction
   */
  readonly initialIsReleased: boolean
}


export const initial: State = {
  exercisePreview: null,
  exerciseRelease: preInitial,
  isBusy: false,
  isCreatingExerciseRelease: false,
  initialIsReleased: false,
  isValid: isValidDialog(preInitial, validationRules),
}


export interface SET_exerciseReleaseAction extends ActionBase {
  readonly type: ActionType.SET_exerciseRelease
  readonly exerciseRelease: ExerciseReleaseForBackend | null
  readonly exercisePreview: EditableExercisePreviewFromBackend
}


export interface SET_isCreatingExerciseReleaseAction extends ActionBase {
  readonly type: ActionType.SET_isCreatingExerciseRelease
  readonly isCreating: boolean
}


export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_isCreatingExerciseReleaseAction
  | SetExerciseReleaseActions
  | CreateExerciseReleaseActions
  | UpdateExerciseReleaseActions

  | SET_exerciseReleaseAction

  | RESET_GlobalAction


export function reducer(state: State = initial, action: AllActions): State {

  // noinspection TsLint
  let temp = null

  switch (action.type) {

    case ActionType.SET_exerciseRelease:
      return {
        ...state,
        exerciseRelease: action.exerciseRelease ? {...action.exerciseRelease} : state.exerciseRelease, //make a copy to not change original from list
        initialIsReleased: action.exerciseRelease ? action.exerciseRelease.isReleased : state.initialIsReleased,
        exercisePreview: action.exercisePreview
      }

    case ActionType.SET_isCreatingExerciseRelease:
      return {
        ...state,
        isCreatingExerciseRelease: action.isCreating
      }

    case ActionType.CREATE_exerciseRelease_PENDING:
    case ActionType.CREATE_exerciseRelease_FULFILLED:
    case ActionType.CREATE_exerciseRelease_REJECTED:
      return createExerciseReleaseReducer(state, action)

    case ActionType.UPDATE_exerciseRelease_PENDING:
    case ActionType.UPDATE_exerciseRelease_FULFILLED:
    case ActionType.UPDATE_exerciseRelease_REJECTED:
      return updateExerciseReleaseReducer(state, action)


    case ActionType.SET_exerciseReleaseId:
    case ActionType.SET_isReleased:
    case ActionType.SET_generatedCode:
    case ActionType.SET_pLangId:
    case ActionType.SET_automaticStartAt:
    case ActionType.SET_automaticEndAt:
    case ActionType.SET_hasLimitedWorkingTime:
    case ActionType.SET_availableWorkingTimeInMinutes:
    case ActionType.SET_note:
    case ActionType.SET_isVisibleToAllAfterRelease:
    case ActionType.SET_shouldAutomaticAssessSubmissions:
    case ActionType.SET_runAlsoNormalTests:
    case ActionType.SET_releaseDurationType:
    case ActionType.SET_releaseStartType:
    case ActionType.SET_hideInOverviews:
    case ActionType.SET_shouldClearClipboard:
    case ActionType.SET_hideSiteHeaderBar:
    case ActionType.SET_hidePrintOptions:
    case ActionType.SET_hideExerciseLeaveActions:
    case ActionType.SET_showAdditionalLogButton:
    case ActionType.SET_maxManualPoints:

      temp = setExerciseReleaseReducer(state.exerciseRelease, action)

      return {
        ...state,
        exerciseRelease: temp,
        isValid: isValidDialog(temp, validationRules)
      }

    case ActionType.RESET:
    case ActionTypeGlobal.RESET_Global:
      return {
        ...initial
      }

    default:
      notExhaustive(action)
      return state
  }
}

