/**
 * Created by janis dähne.
 */
import {Action} from "redux";
import {notExhaustive} from "../../../../_notExhausiveHelper";
import {ExerciseReleaseFromBackend} from "../../../../../../types/exerciseRelease";
import {ActionBase, ActionType} from "./zoomedReleaseDialogActionTypes";
import {ReleaseDurationType} from "../../../../../../types/ReleaseDurationType";
import {ReleaseStartType} from "../../../../../../types/ReleaseStartType";
import {ActionTypeGlobal, RESET_GlobalAction} from '../../../../globalActions'

export type State = {
  readonly release: ExerciseReleaseFromBackend
}

export const initial: State = {
  release: {
    id: -1,
    isReleased: false,
    generatedCode: '',
    pLangId: -1,
    automaticEndAt: null,
    automaticStartAt: null,
    releasedAt: null,
    availableWorkingTimeInMinutes: 0,
    note: '',
    hasLimitedWorkingTime: false,
    solutionsCount: 0,
    isVisibleToAllAfterRelease: false,
    shouldAutomaticAssessSubmissions: false,
    runAlsoNormalTests: false,
    hasAutomaticAssessmentStarted: false,
    hasAutomaticAssessmentFinished: false,
    hadAutomaticAssessmentErrors: false,
    releaseDurationType: ReleaseDurationType.Infinitely,
    releaseStartType: ReleaseStartType.Automatic,

    hideInOverviews: false,
    shouldClearClipboard: false,
    hideSiteHeaderBar: false,
    hideExerciseLeaveActions: false,
    hidePrintOptions: false,
    showAdditionalLogButton: false,
    maxManualPoints: null,
  }
}

export interface SET_exerciseReleaseAction extends ActionBase {
  readonly type: ActionType.SET_exerciseRelease
  readonly exerciseRelease: ExerciseReleaseFromBackend
}


export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_exerciseReleaseAction

  | RESET_GlobalAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.SET_exerciseRelease:
      return {
        ...state,
        release: action.exerciseRelease

      }
    case ActionType.RESET:
    case ActionTypeGlobal.RESET_Global:
      return {
        ...initial
      }

    default:
      notExhaustive(action)
      return state
  }
}

