/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {ActionBase, ActionType} from "./releasesSiteActionTypes";
import {initial, State} from "./releaseSiteReducer";
import Logger from "../../../../helpers/logger";
import {ExerciseReleaseFromBackend} from "../../../../types/exerciseRelease";
import {PaginatedData} from '../../../../types/pagination'
import {PaginationHelper} from '../../../../helpers/paginationHelper'



export interface GET_exerciseReleasesAction
  extends ActionBase, ReduxPromiseAction<PaginatedData<ExerciseReleaseFromBackend>, GET_exerciseReleases_PENDINGAction, GET_exerciseReleases_FULFILLEDAction, GET_exerciseReleases_REJECTEDAction> {
  readonly type: ActionType.GET_exerciseReleases
  readonly payload: Promise<PaginatedData<ExerciseReleaseFromBackend>>
}

//automatically dispatched by redux promise middleware
export interface GET_exerciseReleases_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_exerciseReleases_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_exerciseReleases_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<PaginatedData<ExerciseReleaseFromBackend>> {
  readonly type: ActionType.GET_exerciseReleases_FULFILLED
  readonly payload: PaginatedData<ExerciseReleaseFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_exerciseReleases_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_exerciseReleases_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_exerciseReleases_PENDINGAction
  | GET_exerciseReleases_FULFILLEDAction
  | GET_exerciseReleases_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_exerciseReleases_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_exerciseReleases_FULFILLED:
      return {
        ...state,
        isLoading: false,
        releases: action.payload.items,
        pagination: PaginationHelper.getPaginationStateSpreadObj(action.payload)

      }
    case ActionType.GET_exerciseReleases_REJECTED:
      return {
        ...state,
        isLoading: false,
        releases: []
      }

    default:
      notExhaustive(action)
      return state
  }
}

