import {EditableExercisePreviewFromBackend} from '../../../../types/exercisePreview'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../index'
import {notExhaustive} from '../../_notExhausiveHelper'
import {ActionType} from './submissionsSiteActionTypes'
import {Action} from 'redux'
import {initial, State} from './submissionsSiteReducer'


export interface ActionBase extends Action {
  readonly type: ActionType
}

export interface GET_singleOwnExerciseAction
  extends ActionBase, ReduxPromiseAction<EditableExercisePreviewFromBackend, GET_singleOwnExercise_PENDINGAction, GET_singleOwnExercise_FULFILLEDAction, GET_singleOwnExercise_REJECTEDAction> {
  readonly type: ActionType.GET_singleOwnExercise
  readonly payload: Promise<EditableExercisePreviewFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_singleOwnExercise_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_singleOwnExercise_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_singleOwnExercise_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<EditableExercisePreviewFromBackend> {
  readonly type: ActionType.GET_singleOwnExercise_FULFILLED
  readonly payload: EditableExercisePreviewFromBackend
}

//automatically dispatched by redux promise middleware
export interface GET_singleOwnExercise_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_singleOwnExercise_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_singleOwnExercise_PENDINGAction
  | GET_singleOwnExercise_FULFILLEDAction
  | GET_singleOwnExercise_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_singleOwnExercise_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_singleOwnExercise_FULFILLED:
      return {
        ...state,
        isLoading: false,
        exercise: action.payload,
        group: null,
        groupRole: null
      }
    case ActionType.GET_singleOwnExercise_REJECTED:
      return {
        ...state,
        isLoading: false,
        exercise: null,
        group: null,
        groupRole: null
      }

    default:
      notExhaustive(action)
      return state
  }
}

