import {Action} from 'redux'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../index'
import {initial, State} from './submissionsSiteReducer'
import {notExhaustive} from '../../_notExhausiveHelper'
import {ExerciseReleaseFromBackend} from '../../../../types/exerciseRelease'
import {ActionType} from './submissionsSiteActionTypes'

export interface ActionBase extends Action {
  readonly type: ActionType
}

export interface GET_singleReleaseAction
  extends ActionBase, ReduxPromiseAction<ExerciseReleaseFromBackend, GET_singleRelease_PENDINGAction, GET_singleRelease_FULFILLEDAction, GET_singleRelease_REJECTEDAction> {
  readonly type: ActionType.GET_singleRelease
  readonly payload: Promise<ExerciseReleaseFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_singleRelease_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_singleRelease_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_singleRelease_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ExerciseReleaseFromBackend> {
  readonly type: ActionType.GET_singleRelease_FULFILLED
  readonly payload: ExerciseReleaseFromBackend
}

//automatically dispatched by redux promise middleware
export interface GET_singleRelease_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_singleRelease_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_singleRelease_PENDINGAction
  | GET_singleRelease_FULFILLEDAction
  | GET_singleRelease_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_singleRelease_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_singleRelease_FULFILLED:
      return {
        ...state,
        isLoading: false,
        release: action.payload
      }
    case ActionType.GET_singleRelease_REJECTED:
      return {
        ...state,
        isLoading: false,
        release: null
      }

    default:
      notExhaustive(action)
      return state
  }
}

