/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {ActionBase, ActionType} from "./submissionsSiteActionTypes";
import {initial, State} from "./submissionsSiteReducer";
import Logger from "../../../../helpers/logger";

export interface RERUN_automaticAssessmentAction
  extends ActionBase, ReduxPromiseAction<boolean, RERUN_automaticAssessment_PENDINGAction, RERUN_automaticAssessment_FULFILLEDAction, RERUN_automaticAssessment_REJECTEDAction> {
  readonly type: ActionType.RERUN_automaticAssessment
  readonly payload: Promise<boolean>
}

//automatically dispatched by redux promise middleware
export interface RERUN_automaticAssessment_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.RERUN_automaticAssessment_PENDING
}

//automatically dispatched by redux promise middleware
export interface RERUN_automaticAssessment_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<boolean> {
  readonly type: ActionType.RERUN_automaticAssessment_FULFILLED
  readonly payload: boolean
}

//automatically dispatched by redux promise middleware
export interface RERUN_automaticAssessment_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.RERUN_automaticAssessment_REJECTED
  readonly payload: any
}


export type AllActions =
  RERUN_automaticAssessment_PENDINGAction
  | RERUN_automaticAssessment_FULFILLEDAction
  | RERUN_automaticAssessment_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.RERUN_automaticAssessment_PENDING:
      return {
        ...state,
        isLoading: false, //this runs too long we just make the request but don't really care about the response
        release: {
          ...state.release,
          //this makes sure we disable the re-run assessment buttons
          hasAutomaticAssessmentStarted: true,
          hasAutomaticAssessmentFinished: false,
          hadAutomaticAssessmentErrors: false
        }
      }
    case ActionType.RERUN_automaticAssessment_FULFILLED:
      return {
        ...state,
        isLoading: false,
      }
    case ActionType.RERUN_automaticAssessment_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

