import {ActionBase, ActionType} from '../pLangsSettingsViewActionTypes'
import {initial, State} from '../pLangsReducer'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../index'
import {notExhaustive} from '../../_notExhausiveHelper'
import {PLangFullBase} from '../../../../types/pLangs'


export interface CREATE_pLangAction
  extends ActionBase, ReduxPromiseAction<PLangFullBase, CREATE_pLang_PENDINGAction, CREATE_pLang_FULFILLEDAction, CREATE_pLang_REJECTEDAction> {
  readonly type: ActionType.CREATE_pLang
  readonly payload: Promise<PLangFullBase>
}

//automatically dispatched by redux promise middleware
export interface CREATE_pLang_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.CREATE_pLang_PENDING
}

//automatically dispatched by redux promise middleware
export interface CREATE_pLang_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<PLangFullBase> {
  readonly type: ActionType.CREATE_pLang_FULFILLED
  readonly payload: PLangFullBase
}

//automatically dispatched by redux promise middleware
export interface CREATE_pLang_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.CREATE_pLang_REJECTED
  readonly payload: any
}


export type AllActions =
  CREATE_pLang_PENDINGAction
  | CREATE_pLang_FULFILLEDAction
  | CREATE_pLang_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.CREATE_pLang_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.CREATE_pLang_FULFILLED:
      return {
        ...state,
        isLoading: false,
        editingPLangId: null,
        pLangCopy: null,
        pLangsWithExtraData: state.pLangsWithExtraData.concat([{
          ...action.payload,
          canBeDeleted: true, //a newly created plang cannot have exercises
          cannotBeDeletedReason: null,
        }])
      }
    case ActionType.CREATE_pLang_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

