import {ActionBase, ActionType} from '../pLangsSettingsViewActionTypes'
import {initial, State} from '../pLangsReducer'
import {notExhaustive} from '../../_notExhausiveHelper'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../index'


interface Meta {
  readonly pLangId: number
}

export interface DELETE_pLangAction
  extends ActionBase, ReduxPromiseAction<void, DELETE_pLang_PENDINGAction, DELETE_pLang_FULFILLEDAction, DELETE_pLang_REJECTEDAction, Meta> {
  readonly type: ActionType.DELETE_pLang
  readonly payload: Promise<void>
  readonly meta: Meta

}

//automatically dispatched by redux promise middleware
export interface DELETE_pLang_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.DELETE_pLang_PENDING
}

//automatically dispatched by redux promise middleware
export interface DELETE_pLang_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<void, Meta> {
  readonly type: ActionType.DELETE_pLang_FULFILLED
  readonly payload: void
}

//automatically dispatched by redux promise middleware
export interface DELETE_pLang_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void, Meta> {
  readonly type: ActionType.DELETE_pLang_REJECTED
  readonly payload: any
}


export type AllActions =
  DELETE_pLang_PENDINGAction
  | DELETE_pLang_FULFILLEDAction
  | DELETE_pLang_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.DELETE_pLang_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.DELETE_pLang_FULFILLED:
      return {
        ...state,
        isLoading: false,
        pLangsWithExtraData: state.pLangsWithExtraData.filter(p => p.id !== action.meta.pLangId)
      }
    case ActionType.DELETE_pLang_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

