import {ActionBase, ActionType} from '../pLangsSettingsViewActionTypes'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../index'
import {initial, State} from '../pLangsReducer'
import {notExhaustive} from '../../_notExhausiveHelper'
import {PLangFullBase, PLangWithExtrasFromBackend} from '../../../../types/pLangs'


export interface UPDATE_pLangsAction
  extends ActionBase, ReduxPromiseAction<PLangWithExtrasFromBackend, UPDATE_pLangs_PENDINGAction, UPDATE_pLangs_FULFILLEDAction, UPDATE_pLangs_REJECTEDAction> {
  readonly type: ActionType.UPDATE_pLangs
  readonly payload: Promise<PLangWithExtrasFromBackend>
}

//automatically dispatched by redux promise middleware
export interface UPDATE_pLangs_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.UPDATE_pLangs_PENDING
}

//automatically dispatched by redux promise middleware
export interface UPDATE_pLangs_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<PLangWithExtrasFromBackend> {
  readonly type: ActionType.UPDATE_pLangs_FULFILLED
  readonly payload: PLangWithExtrasFromBackend
}

//automatically dispatched by redux promise middleware
export interface UPDATE_pLangs_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.UPDATE_pLangs_REJECTED
  readonly payload: any
}


export type AllActions =
  UPDATE_pLangs_PENDINGAction
  | UPDATE_pLangs_FULFILLEDAction
  | UPDATE_pLangs_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.UPDATE_pLangs_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.UPDATE_pLangs_FULFILLED:
      return {
        ...state,
        isLoading: false,
        editingPLangId: null,
        pLangCopy: null,
        pLangsWithExtraData: state.pLangsWithExtraData.map((value, index) => value.id !== action.payload.id
                                                                             ? value
                                                                             : action.payload)
      }
    case ActionType.UPDATE_pLangs_REJECTED:
      return {
        ...state,
        isLoading: false,
      }

    default:
      notExhaustive(action)
      return state
  }
}

