import {UserDataFullBase, UserSettingsFullBase} from "../../../types/userData";
import {notExhaustive} from "../_notExhausiveHelper";
import {ActionBase, ActionType} from "./settingsSiteActionTypes";
import {
  AllActions as UserDataAction, initial as preInitial, reducer as userDataCopyReducer, State as UserDataCopyState
} from './userDataCopyReducer'
import {AllActions as UpdateUserDataActions, reducer as updateUserDataReducer} from './updateUserDataReducer'
import {AllActions as GetActions, reducer as getReducer} from './getUserDataAndSettingsReducer'
import {AllActions as UpdateSettingsActions, reducer as updateUserSettingsReducer} from './updateUserSettingsReducer'
import {
  AllActions as UserSettingsCopyActions, defaultUserSettings as userSettingsInital, reducer as userSettingsCopyReducer
} from './userSettingsCopyReducer'
import {LoginTimeoutHelper} from "../../../helpers/loginTimeoutHelper";
import {ActionTypeGlobal, RESET_GlobalAction} from '../globalActions'


export enum ChangeState {
  notActive,
  active
}


export type State = {
  readonly isLoadingUserData: boolean
  readonly isLoadingMiscSettings: boolean
  readonly userDataChangeState: ChangeState
  readonly userSettingsChangeState: ChangeState
  /**
   * this is also set after login
   */
  readonly userData: UserDataFullBase

  //use copy data for form input

  readonly userDataCopy: UserDataCopyState
  readonly userSettingsCopy: UserSettingsFullBase
}

export const userDataInitial: UserDataFullBase = {
  id: -1,
  email: preInitial.email,
  firstName: '',
  lastName: '',
  systemRole: null,
  tempCanCreateExercise: false,
  settings: null
}

export const initial: State = {
  isLoadingUserData: false,
  isLoadingMiscSettings: false,

  userData: userDataInitial,
  userDataChangeState: ChangeState.notActive,
  userSettingsChangeState: ChangeState.notActive,

  userDataCopy: {
    ...preInitial
  },

  userSettingsCopy: userSettingsInital
}


export interface SET_userDataChangeStateAction extends ActionBase {
  readonly type: ActionType.SET_userDataChangeState
  readonly changeState: ChangeState
}

export interface SET_userSettingsChangeStateAction extends ActionBase {
  readonly type: ActionType.SET_userSettingsChangeState
  readonly changeState: ChangeState
}


//normally only used after login
export interface SET_userDataAction extends ActionBase {
  readonly type: ActionType.SET_userData
  readonly userData: UserDataFullBase
}


export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | UserDataAction
  | UpdateUserDataActions
  | GetActions
  | UpdateSettingsActions
  | SET_userDataChangeStateAction
  | SET_userSettingsChangeStateAction
  | UserSettingsCopyActions
  | SET_userDataAction

  | RESET_GlobalAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.SET_userData:


      if (LoginTimeoutHelper.isFirstLogin && action.userData.id !== initial.userData.id) {

        LoginTimeoutHelper.isReLogin = false
        LoginTimeoutHelper.isFirstLogin = false

      } else if (LoginTimeoutHelper.isFirstLogin === false && action.userData.id !== initial.userData.id) {

        //but could also be a reset (after timeout) or user switch

        //user switched?
        if (action.userData.id !== LoginTimeoutHelper.lastKnownUserId) {
          LoginTimeoutHelper.isReLogin = false
        }

        //reset?
        else if (action.userData.id === LoginTimeoutHelper.lastKnownUserId) {
          //user re logged in
          LoginTimeoutHelper.isReLogin = true
        }
        else {
          LoginTimeoutHelper.isReLogin = false
        }
      }

      //only set real user ids (not the reset data)
      if (action.userData.id !== initial.userData.id) {
        LoginTimeoutHelper.lastKnownUserId = action.userData.id
      }

      return {
        ...state,
        userData: action.userData,
        userDataCopy: {
          email: action.userData.email,
          oldPassword: '',
          newPassword: '',
          newPasswordCheck: '',
          isValid: false
        },
        userSettingsCopy: action.userData.settings
          ? {
            ...action.userData.settings
          }
          : userSettingsInital
      }

    case ActionType.SET_userDataChangeState:
      return {
        ...state,
        userDataChangeState: action.changeState
      }

    case ActionType.SET_userSettingsChangeState:
      return {
        ...state,
        userSettingsChangeState: action.changeState
      }


    case ActionType.GET_userData_PENDING:
    case ActionType.GET_userData_FULFILLED:
    case ActionType.GET_userData_REJECTED:
      return getReducer(state, action)

    case ActionType.UPDATE_userData_PENDING:
    case ActionType.UPDATE_userData_FULFILLED:
    case ActionType.UPDATE_userData_REJECTED:
      return updateUserDataReducer(state, action)

    case ActionType.UPDATE_userSettings_PENDING:
    case ActionType.UPDATE_userSettings_FULFILLED:
    case ActionType.UPDATE_userSettings_REJECTED:
      return updateUserSettingsReducer(state, action)


    case ActionType.SET_newPasswordCopy:
    case ActionType.SET_newPasswordCheckCopy:
    case ActionType.SET_oldPasswordCopy:
    case ActionType.SET_emailCopy:
    case ActionType.RESET_userData:
      return {
        ...state,
        userDataCopy: userDataCopyReducer(state.userDataCopy, action)
      }

    case ActionType.SET_langId:
    case ActionType.SET_theme:
    case ActionType.RESET_userSettings:
      return {
        ...state,
        userSettingsCopy: userSettingsCopyReducer(state.userSettingsCopy, action)
      }

    case ActionType.RESET:
    case ActionTypeGlobal.RESET_Global:
      return {
        ...initial
      }

    default:
      notExhaustive(action)
      return state
  }


}
