import {notExhaustive} from "../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {ActionBase, ActionType} from './singleGroupSiteActionTypes'
import {UserGroupBase, UserGroupFromBackend, UserGroupWithExerciseCountFromBackend} from '../../../types/group'
import {initial, State} from './singleGroupSiteReducer'


export interface GET_singleGroupAction
  extends ActionBase, ReduxPromiseAction<UserGroupWithExerciseCountFromBackend, GET_singleGroup_PENDINGAction, GET_singleGroup_FULFILLEDAction, GET_singleGroup_REJECTEDAction> {
  readonly type: ActionType.GET_singleGroup
  readonly payload: Promise<UserGroupWithExerciseCountFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_singleGroup_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_singleGroup_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_singleGroup_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<UserGroupWithExerciseCountFromBackend> {
  readonly type: ActionType.GET_singleGroup_FULFILLED
  readonly payload: UserGroupWithExerciseCountFromBackend
}

//automatically dispatched by redux promise middleware
export interface GET_singleGroup_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_singleGroup_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_singleGroup_PENDINGAction
  | GET_singleGroup_FULFILLEDAction
  | GET_singleGroup_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_singleGroup_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_singleGroup_FULFILLED:
      return {
        ...state,
        isLoading: false,
        group: action.payload
      }
    case ActionType.GET_singleGroup_REJECTED:
      return {
        ...state,
        isLoading: false,
        group: null
      }

    default:
      notExhaustive(action)
      return state
  }
}

