/**
 * Created by janis dähne.
 */
import {SelectableGroupUserFrontend, UserGroupWithExerciseCountFromBackend} from "../../../types/group";
import {notExhaustive} from "../_notExhausiveHelper";
import {ActionBase, ActionType} from "./singleGroupSiteActionTypes";
import {GroupRoleWithPermissionFullBase} from "../../../types/groupRoles";
import {
  AllActions as GetActions,
  AllActions as GetSingleGroupActions,
  reducer as getSingleGroupReducer
} from './getSingleGroupReducer'
import {AllActions as GetGroupRolesActions, reducer as getGroupRolesReducer} from './getGroupRolesReducer'
import {
  AllActions as SetMemberGroupRoleActions,
  reducer as setMemberGroupRoleReducer
} from './setMemberGroupRoleReducer'
import {AllActions as RemoveMemberActions, reducer as removeMemberReducer} from './removeMemberReducer'
import {ActionType as GroupRolesSiteActionTypes} from '../groupRolesSite/groupRolesSiteActionTypes'
import {FrontendSettingsManager} from "../../../helpers/frontendSettingsManager";
import {PaginationStateExtend} from '../../../types/pagination'
import {getInitialPaginationData} from '../../../constants'
import {AllActions as GetMembersActions, reducer as getMembersReducer} from './getMembersReducer'
import {ActionTypeGlobal, RESET_GlobalAction} from '../globalActions'


//TODO make only 1 request for members + roles + group data
export type State = {
  readonly selectedGroupId: number
  readonly members: ReadonlyArray<SelectableGroupUserFrontend>
  readonly groupRoles: ReadonlyArray<GroupRoleWithPermissionFullBase>
  readonly group: UserGroupWithExerciseCountFromBackend | null
  readonly isLoading: boolean
  readonly isAddUsersDialogDisplayed: boolean

} & PaginationStateExtend<SelectableGroupUserFrontend>


export const initial: State = {
  members: [],
  groupRoles: [],
  group: null,
  selectedGroupId: -1,
  isLoading: false,
  isAddUsersDialogDisplayed: false,

  pagination: {
    ...getInitialPaginationData(),
    orderByProp: 'lastName'
  }
}


//---pagination
export interface SetSearchTextAction extends ActionBase {
  readonly type: ActionType.setSearchText
  readonly searchText: string
}

export interface SetActivePaginationPageAction extends ActionBase {
  readonly type: ActionType.setPaginationActivePage
  readonly activePage: number
}

export interface SetPaginationPageSizeAction extends ActionBase {
  readonly type: ActionType.setPaginationPageSize
  readonly pageSize: number
}

//---pagination end

export interface SET_selectedGroupIdAction extends ActionBase {
  readonly type: ActionType.SET_selectedGroupId
  readonly groupId: number
}

export interface SET_isAddUsersDialogDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isAddUsersDialogDisplayed
  readonly isDisplayed: boolean
}

export interface SET_member_uiIsSelectedAction extends ActionBase {
  readonly type: ActionType.SET_member_uiIsSelected
  readonly memberId: number
  readonly isSelected: boolean
}

export interface SET_all_member_uiIsSelectedAction extends ActionBase {
  readonly type: ActionType.SET_all_member_uiIsSelected
  readonly isSelected: boolean
}


export interface SET_user_uiIsChangeGroupRolePopupDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_user_uiIsChangeGroupRolePopupDisplayed
  readonly isDisplayed: boolean
  readonly userId: number
}

export interface SET_sortByKeyAction extends ActionBase {
  readonly type: ActionType.SET_sortByKey
  readonly sortByKey: keyof SelectableGroupUserFrontend
  readonly sortDirection: boolean | null
}


export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_selectedGroupIdAction
  | GetActions
  | GetGroupRolesActions

  | SET_isAddUsersDialogDisplayedAction
  | SetMemberGroupRoleActions
  | SET_user_uiIsChangeGroupRolePopupDisplayedAction
  | RemoveMemberActions
  | SET_member_uiIsSelectedAction
  | SET_all_member_uiIsSelectedAction
  | SET_sortByKeyAction
  | GetSingleGroupActions
  | GetMembersActions
  | SetSearchTextAction
  | SetActivePaginationPageAction
  | SetPaginationPageSizeAction

  | RESET_GlobalAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.REMOVE_member_PENDING:
    case ActionType.REMOVE_member_FULFILLED:
    case ActionType.REMOVE_member_REJECTED:
      return removeMemberReducer(state, action)

    case ActionType.GET_singleGroup_PENDING:
    case ActionType.GET_singleGroup_FULFILLED:
    case ActionType.GET_singleGroup_REJECTED:
      return getSingleGroupReducer(state, action)

    case GroupRolesSiteActionTypes.GET_groupRoles_PENDING:
    case GroupRolesSiteActionTypes.GET_groupRoles_FULFILLED:
    case GroupRolesSiteActionTypes.GET_groupRoles_REJECTED:
      return getGroupRolesReducer(state, action)

    case ActionType.GET_members_PENDING:
    case ActionType.GET_members_FULFILLED:
    case ActionType.GET_members_REJECTED:
      return getMembersReducer(state, action)

    case ActionType.SET_memberGroupRole_PENDING:
    case ActionType.SET_memberGroupRole_FULFILLED:
    case ActionType.SET_memberGroupRole_REJECTED:
      return setMemberGroupRoleReducer(state, action)


    case ActionType.SET_member_uiIsSelected:
      return {
        ...state,
        members: state.members.map(member => {
          return member.id === action.memberId
                 ?
            {
              ...member,
              uiIsSelected: action.isSelected
            }
                 : member
        })
      }

    case ActionType.SET_all_member_uiIsSelected:
      return {
        ...state,
        members: state.members.map(member => {
          return {
            ...member,
            uiIsSelected: member.isCurrentUser
                          ? false
                          : action.isSelected
          }
        })
      }

    case ActionType.SET_user_uiIsChangeGroupRolePopupDisplayed:
      return {
        ...state,
        members: state.members.map(member => {
          return member.id === action.userId
                 ?
            {
              ...member,
              uiIsChangeGroupRolePopupDisplayed: action.isDisplayed
            }
                 : member
        })
      }

    case ActionType.SET_isAddUsersDialogDisplayed:
      return {
        ...state,
        isAddUsersDialogDisplayed: action.isDisplayed
      }

    case ActionType.SET_selectedGroupId:
      return {
        ...state,
        selectedGroupId: action.groupId
      }

    case ActionType.SET_sortByKey:
      return {
        ...state,
        pagination: {
          ...state.pagination,
          orderByProp: action.sortByKey,
          isOrderByDesc: action.sortDirection
        }
      }

    case ActionType.setSearchText:
      return {
        ...state,
        pagination: {
          ...state.pagination,
          searchText: action.searchText
        }
      }
    case ActionType.setPaginationActivePage:
      return {
        ...state,
        pagination: {
          ...state.pagination,
          currentPage: action.activePage
        }
      }

    case ActionType.setPaginationPageSize:

      FrontendSettingsManager.set_singleGroupPageSize(action.pageSize)

      return {
        ...state,
        pagination: {
          ...state.pagination,
          pageSize: action.pageSize,
          currentPage: 1,
        }
      }

    case ActionType.RESET:
    case ActionTypeGlobal.RESET_Global:
      return {
        ...initial,
        pagination: {
          ...initial.pagination,
          pageSize: FrontendSettingsManager.getFrontendSettings().singleGroupPageSize,
        }
      }

    default:
      notExhaustive(action)
      return state
  }
}

