import {
  SystemSettingsBase,
  SystemSettingsForBackend,
  SystemSettingsFromBackend,
  TestServerPublicSettings
} from "../../../types/systemSettings";
import {notExhaustive} from "../_notExhausiveHelper";
import {ActionBase, ActionType} from './systemSettingsSiteActionTypes'

import {
  AllActions as AllGetSystemSettingsActions,
  reducer as getSystemSettingsReducer
} from './crud/getSystemSettingsReducer'
import {
  AllActions as AllUpdateSystemSettingsActions,
  reducer as updateSystemSettingsReducer
} from './crud/updateSystemSettingsReducer'

import {
  AllActions as AllGetInitialSystemSettingsActions,
  reducer as getInitialSystemSettingsReducer
} from './crud/getInitialSystemSettingsReducer'
import {ActionTypeGlobal, RESET_GlobalAction} from '../globalActions'
import {GroupRoleFullBase} from '../../../types/groupRoles'
import {GroupPreviewFromBackend} from '../../../types/group'

export type State = {
  readonly isLoading: boolean
  readonly systemSettings: SystemSettingsFromBackend
  readonly initialSystemSettings: SystemSettingsFromBackend | null

  readonly allDefaultRoles: ReadonlyArray<GroupRoleFullBase>
  readonly allDefaultGroups: ReadonlyArray<GroupPreviewFromBackend>
  readonly submitTestServerPublicSettings: TestServerPublicSettings
}
import {
  AllActions as AllGetAllDefaultRolesReducer,
  reducer as getAllDefaultRolesReducer
} from './crud/getAllDefaultRolesReducer'
import {
  AllActions as AllGetAllDefaultGroupsReducer,
  reducer as getAllDefaultGroupsReducer
} from './crud/getAllDefaultGroupsReducer'
import {AllActions as AllGetSubmitTestServerPublicSettingsReducer,
  reducer as getSubmitTestServerPublicSettingsReducer} from './crud/getSubmitTestServerPublicSettingsReducer'


export const initial: State = {
  isLoading: false,
  systemSettings: {
    maxNumberOfTestsWithOneRequest: 1,
    maxCustomProjectsPerUser: -1,
    maxCustomTestsPerParticipation: -1,
    currentServerMessage: '',
    maxNumberOfTestsWithOneRequestSubmitTestServer: -1,
    submitTestServerTimeoutInMs: -1,
    submitTestServerUrl: '',
    testServerStatsUrl: '',
    submitTestServerStatsUrl: '',
    testServerTimeoutInMs: -1,
    testServerUrl: '',
    testServerConfigUiUrl: '',
    submitTestServerConfigUiUrl: '',
    justRunProgramMaxDiskSpaceInKb: -1,
    justRunProgramMemoryLimitInKb: -1,
    justRunProgramTimeoutInMs: -1,
    justRunProgramCompileTimeoutInMs: -1,
    customProjectTestTimeoutInMs: -1,
    customProjectTestMemoryLimitInKb: -1,
    customProjectTestMaxDiskSpaceInKb: -1,
    customProjectTestCompileTimeoutInMs: -1,
    defaultUserGroupId: -1,
    defaultGroupRoleId: -1,
    defaultGroupCreatorGroupRoleId: -1,
    testServerTraceLogApiUrl: '',
  },
  initialSystemSettings: null,
  allDefaultRoles: [],
  allDefaultGroups: [],

  submitTestServerPublicSettings: {
    hardCompileTimeoutInMs: -1,
    hardDiskSpaceLimitInKb: -1,
    hardMemoryLimitInKb: -1,
    hardTimeoutInMs: -1,
    maxNumberOfTestsWithOneRequest: -1,
  }

}


export interface SET_serverMessageAction extends ActionBase {
  readonly type: ActionType.SET_serverMessage
  readonly serverMessage: string
}

//--- set system settings props

export interface SET_maxCustomProjectsPerUserAction extends ActionBase {
  readonly type: ActionType.SET_maxCustomProjectsPerUser
  readonly maxCustomProjectsPerUser: number
}

export interface SET_maxCustomTestsPerParticipationAction extends ActionBase {
  readonly type: ActionType.SET_maxCustomTestsPerParticipation
  readonly maxCustomTestsPerParticipation: number
}

export interface SET_testServerUrlAction extends ActionBase {
  readonly type: ActionType.SET_testServerUrl
  readonly testServerUrl: string
}

export interface SET_testServerStatsUrlAction extends ActionBase {
  readonly type: ActionType.SET_testServerStatsUrl
  readonly testServerStatsUrl: string
}

export interface SET_submitTestServerStatsUrlAction extends ActionBase {
  readonly type: ActionType.SET_submitTestServerStatsUrl
  readonly submitTestServerStatsUrl: string
}

export interface SET_submitTestServerUrlAction extends ActionBase {
  readonly type: ActionType.SET_submitTestServerUrl
  readonly submitTestServerUrl: string
}

export interface SET_testServerTraceLogApiUrlAction extends ActionBase {
  readonly type: ActionType.SET_testServerTraceLogApiUrl
  readonly testServerTraceLogApiUrl: string
}

export interface SET_testServerConfigUiUrlAction extends ActionBase {
  readonly type: ActionType.SET_testServerConfigUiUrl
  readonly testServerConfigUiUrl: string
}

export interface SET_testServerTimeoutInMsAction extends ActionBase {
  readonly type: ActionType.SET_testServerTimeoutInMs
  readonly testServerTimeoutInMs: number
}

export interface SET_submitTestServerTimeoutInMsAction extends ActionBase {
  readonly type: ActionType.SET_submitTestServerTimeoutInMs
  readonly submitTestServerTimeoutInMs: number
}

export interface SET_maxNumberOfTestsWithOneRequestAction extends ActionBase {
  readonly type: ActionType.SET_maxNumberOfTestsWithOneRequest
  readonly maxNumberOfTestsWithOneRequest: number
}

export interface SET_maxNumberOfTestsWithOneRequestSubmitTestServerAction extends ActionBase {
  readonly type: ActionType.SET_maxNumberOfTestsWithOneRequestSubmitTestServer
  readonly maxNumberOfTestsWithOneRequestSubmitTestServer: number
}


export interface SET_justRunProgramTimeoutInMsAction extends ActionBase {
  readonly type: ActionType.SET_justRunProgramTimeoutInMs
  readonly justRunProgramTimeoutInMs: number
}

export interface SET_justRunProgramCompileTimeoutInMsAction extends ActionBase {
  readonly type: ActionType.SET_justRunProgramCompileTimeoutInMs
  readonly justRunProgramCompileTimeoutInMs: number
}

export interface SET_justRunProgramMemoryLimitInKbAction extends ActionBase {
  readonly type: ActionType.SET_justRunProgramMemoryLimitInKb
  readonly justRunProgramMemoryLimitInKb: number
}

export interface SET_justRunProgramMaxDiskSpaceInKbAction extends ActionBase {
  readonly type: ActionType.SET_justRunProgramMaxDiskSpaceInKb
  readonly justRunProgramMaxDiskSpaceInKb: number
}

export interface SET_customProjectTestTimeoutInMsAction extends ActionBase {
  readonly type: ActionType.SET_customProjectTestTimeoutInMs
  readonly customProjectTestTimeoutInMs: number
}

export interface SET_customProjectTestCompileTimeoutInMsAction extends ActionBase {
  readonly type: ActionType.SET_customProjectTestCompileTimeoutInMs
  readonly customProjectTestCompileTimeoutInMs: number
}

export interface SET_customProjectTestMemoryLimitInKbAction extends ActionBase {
  readonly type: ActionType.SET_customProjectTestMemoryLimitInKb
  readonly customProjectTestMemoryLimitInKb: number
}

export interface SET_customProjectTestMaxDiskSpaceInKbAction extends ActionBase {
  readonly type: ActionType.SET_customProjectTestMaxDiskSpaceInKb
  readonly customProjectTestMaxDiskSpaceInKb: number
}


export interface SET_defaultUserGroupIdAction extends ActionBase {
  readonly type: ActionType.SET_defaultUserGroupId
  readonly defaultUserGroupId: number
}

export interface SET_defaultGroupRoleIdAction extends ActionBase {
  readonly type: ActionType.SET_defaultGroupRoleId
  readonly defaultGroupRoleId: number
}

export interface SET_defaultGroupCreatorGroupRoleIdAction extends ActionBase {
  readonly type: ActionType.SET_defaultGroupCreatorGroupRoleId
  readonly defaultGroupCreatorGroupRoleId: number
}


export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | AllGetSystemSettingsActions
  | AllUpdateSystemSettingsActions
  | AllGetInitialSystemSettingsActions

  | AllGetAllDefaultRolesReducer
  | AllGetAllDefaultGroupsReducer
  | AllGetSubmitTestServerPublicSettingsReducer

  | SET_serverMessageAction

  | SET_maxCustomProjectsPerUserAction
  | SET_maxCustomTestsPerParticipationAction
  | SET_testServerUrlAction
  | SET_testServerStatsUrlAction
  | SET_submitTestServerStatsUrlAction
  | SET_submitTestServerUrlAction
  | SET_testServerTraceLogApiUrlAction
  | SET_testServerConfigUiUrlAction
  | SET_testServerTimeoutInMsAction
  | SET_submitTestServerTimeoutInMsAction
  | SET_maxNumberOfTestsWithOneRequestAction
  | SET_maxNumberOfTestsWithOneRequestSubmitTestServerAction
  | SET_justRunProgramTimeoutInMsAction
  | SET_justRunProgramCompileTimeoutInMsAction
  | SET_justRunProgramMemoryLimitInKbAction
  | SET_justRunProgramMaxDiskSpaceInKbAction
  | SET_customProjectTestTimeoutInMsAction
  | SET_customProjectTestCompileTimeoutInMsAction
  | SET_customProjectTestMemoryLimitInKbAction
  | SET_customProjectTestMaxDiskSpaceInKbAction
  | SET_defaultUserGroupIdAction
  | SET_defaultGroupRoleIdAction
  | SET_defaultGroupCreatorGroupRoleIdAction

  | RESET_GlobalAction




export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {


    case ActionType.GET_initialSystemSettings_PENDING:
    case ActionType.GET_initialSystemSettings_FULFILLED:
    case ActionType.GET_initialSystemSettings_REJECTED:
      return getInitialSystemSettingsReducer(state, action)

    case ActionType.GET_systemSettings_PENDING:
    case ActionType.GET_systemSettings_FULFILLED:
    case ActionType.GET_systemSettings_REJECTED:
      return getSystemSettingsReducer(state, action)

    case ActionType.UPDATE_systemSettings_PENDING:
    case ActionType.UPDATE_systemSettings_FULFILLED:
    case ActionType.UPDATE_systemSettings_REJECTED:
      return updateSystemSettingsReducer(state, action)

    case ActionType.GET_allDefaultRoles_PENDING:
    case ActionType.GET_allDefaultRoles_FULFILLED:
    case ActionType.GET_allDefaultRoles_REJECTED:
      return getAllDefaultRolesReducer(state, action)

    case ActionType.GET_allDefaultGroups_PENDING:
    case ActionType.GET_allDefaultGroups_FULFILLED:
    case ActionType.GET_allDefaultGroups_REJECTED:
      return getAllDefaultGroupsReducer(state, action)

    case ActionType.GET_submitTestServerPublicSettings_PENDING:
    case ActionType.GET_submitTestServerPublicSettings_FULFILLED:
    case ActionType.GET_submitTestServerPublicSettings_REJECTED:
      return getSubmitTestServerPublicSettingsReducer(state, action)

    case ActionType.SET_serverMessage:
      return {
        ...state,
        systemSettings: {
          ...state.systemSettings,
          currentServerMessage: action.serverMessage,
        }
      }

    case ActionType.SET_maxCustomProjectsPerUser:
      return {
        ...state,
        systemSettings: {
          ...state.systemSettings,
          maxCustomProjectsPerUser: action.maxCustomProjectsPerUser
        }
      }
    case ActionType.SET_maxCustomTestsPerParticipation:
      return {
        ...state,
        systemSettings: {
          ...state.systemSettings,
          maxCustomTestsPerParticipation: action.maxCustomTestsPerParticipation
        }
      }
    case ActionType.SET_testServerUrl:
      return {
        ...state,
        systemSettings: {
          ...state.systemSettings,
          testServerUrl: action.testServerUrl
        }
      }
    case ActionType.SET_testServerStatsUrl:
      return {
        ...state,
        systemSettings: {
          ...state.systemSettings,
          testServerStatsUrl: action.testServerStatsUrl
        }
      }
    case ActionType.SET_submitTestServerStatsUrl:
      return {
        ...state,
        systemSettings: {
          ...state.systemSettings,
          submitTestServerStatsUrl: action.submitTestServerStatsUrl
        }
      }
    case ActionType.SET_submitTestServerUrl:
      return {
        ...state,
        systemSettings: {
          ...state.systemSettings,
          submitTestServerUrl: action.submitTestServerUrl
        }
      }
    case ActionType.SET_testServerTraceLogApiUrl:
      return {
        ...state,
        systemSettings: {
          ...state.systemSettings,
          testServerTraceLogApiUrl: action.testServerTraceLogApiUrl
        }
      }
    case ActionType.SET_testServerConfigUiUrl:
      return {
        ...state,
        systemSettings: {
          ...state.systemSettings,
          testServerConfigUiUrl: action.testServerConfigUiUrl
        }
      }
    case ActionType.SET_testServerTimeoutInMs:
      return {
        ...state,
        systemSettings: {
          ...state.systemSettings,
          testServerTimeoutInMs: action.testServerTimeoutInMs
        }
      }
    case ActionType.SET_submitTestServerTimeoutInMs:
      return {
        ...state,
        systemSettings: {
          ...state.systemSettings,
          submitTestServerTimeoutInMs: action.submitTestServerTimeoutInMs
        }
      }
    case ActionType.SET_maxNumberOfTestsWithOneRequest:
      return {
        ...state,
        systemSettings: {
          ...state.systemSettings,
          maxNumberOfTestsWithOneRequest: action.maxNumberOfTestsWithOneRequest
        }
      }
    case ActionType.SET_maxNumberOfTestsWithOneRequestSubmitTestServer:
      return {
        ...state,
        systemSettings: {
          ...state.systemSettings,
          maxNumberOfTestsWithOneRequestSubmitTestServer: action.maxNumberOfTestsWithOneRequestSubmitTestServer
        }
      }

    case ActionType.SET_justRunProgramTimeoutInMs:
      return {
        ...state,
        systemSettings: {
          ...state.systemSettings,
          justRunProgramTimeoutInMs: action.justRunProgramTimeoutInMs
        }
      }
    case ActionType.SET_justRunProgramCompileTimeoutInMs:
      return {
        ...state,
        systemSettings: {
          ...state.systemSettings,
          justRunProgramCompileTimeoutInMs: action.justRunProgramCompileTimeoutInMs
        }
      }

    case ActionType.SET_justRunProgramMemoryLimitInKb:
      return {
        ...state,
        systemSettings: {
          ...state.systemSettings,
          justRunProgramMemoryLimitInKb: action.justRunProgramMemoryLimitInKb
        }
      }

    case ActionType.SET_justRunProgramMaxDiskSpaceInKb:
      return {
        ...state,
        systemSettings: {
          ...state.systemSettings,
          justRunProgramMaxDiskSpaceInKb: action.justRunProgramMaxDiskSpaceInKb
        }
      }

    case ActionType.SET_customProjectTestTimeoutInMs:
      return {
        ...state,
        systemSettings: {
          ...state.systemSettings,
          customProjectTestTimeoutInMs: action.customProjectTestTimeoutInMs
        }
      }

    case ActionType.SET_customProjectTestCompileTimeoutInMs:
      return {
        ...state,
        systemSettings: {
          ...state.systemSettings,
          customProjectTestCompileTimeoutInMs: action.customProjectTestCompileTimeoutInMs
        }
      }

    case ActionType.SET_customProjectTestMemoryLimitInKb:
      return {
        ...state,
        systemSettings: {
          ...state.systemSettings,
          customProjectTestMemoryLimitInKb: action.customProjectTestMemoryLimitInKb
        }
      }

    case ActionType.SET_customProjectTestMaxDiskSpaceInKb:
      return {
        ...state,
        systemSettings: {
          ...state.systemSettings,
          customProjectTestMaxDiskSpaceInKb: action.customProjectTestMaxDiskSpaceInKb
        }
      }

    case ActionType.SET_defaultUserGroupId:
      return {
        ...state,
        systemSettings: {
          ...state.systemSettings,
          defaultUserGroupId: action.defaultUserGroupId
        }
      }

    case ActionType.SET_defaultGroupRoleId:
      return {
        ...state,
        systemSettings: {
          ...state.systemSettings,
          defaultGroupRoleId: action.defaultGroupRoleId
        }
      }

    case ActionType.SET_defaultGroupCreatorGroupRoleId:
      return {
        ...state,
        systemSettings: {
          ...state.systemSettings,
          defaultGroupCreatorGroupRoleId: action.defaultGroupCreatorGroupRoleId
        }
      }

    case ActionType.RESET:
    case ActionTypeGlobal.RESET_Global:
      return {
        ...initial
      }

    default:
      notExhaustive(action)
      return state
  }
}

