import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../../index";
import {Action} from 'redux'
import {ActionType} from '../../tutorViewSiteActionTypes'
import {initial, State} from '../../tutorViewSiteReducer'
import {notExhaustive} from '../../../_notExhausiveHelper'
import {StoredTestResultTupleFromBackend} from '../../../../../types/testResults'
import {RunnableExerciseTestFrontendExclusive} from '../../../../../types/exerciseEditor'
import {_storedTestResultToTestResult} from '../../../../../helpers/convertersAndTransformers'

export interface ActionBase extends Action {
  readonly type: ActionType
}

export interface GET_afterTestResultsAction
  extends ActionBase, ReduxPromiseAction<StoredTestResultTupleFromBackend, GET_afterTestResults_PENDINGAction, GET_afterTestResults_FULFILLEDAction, GET_afterTestResults_REJECTEDAction> {
  readonly type: ActionType.GET_afterTestResults
  readonly payload: Promise<StoredTestResultTupleFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_afterTestResults_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_afterTestResults_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_afterTestResults_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<StoredTestResultTupleFromBackend> {
  readonly type: ActionType.GET_afterTestResults_FULFILLED
  readonly payload: StoredTestResultTupleFromBackend
}

//automatically dispatched by redux promise middleware
export interface GET_afterTestResults_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_afterTestResults_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_afterTestResults_PENDINGAction
  | GET_afterTestResults_FULFILLEDAction
  | GET_afterTestResults_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_afterTestResults_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_afterTestResults_FULFILLED:
      return {
        ...state,
        isLoading: false,
        afterNormalTests: state.exercise.tests.map<RunnableExerciseTestFrontendExclusive>(p => {
          const testResult = action.payload.testResults.find(test => test.testId === p.id)
          return {
            ...p,
            uiIsRunning: false,
            result: _storedTestResultToTestResult(testResult),
            uiIsAfterTest: true
          }
        })
        //custom tests are currently not displayed... also not needed
      }
    case ActionType.GET_afterTestResults_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

