/**
 * Created by janis dähne.
 */
import {ActionBase, ActionType} from "../tutorViewSiteActionTypes";
import {initial, State} from "../tutorViewSiteReducer";
import Logger from "../../../../helpers/logger";
import {AssessmentFullBase} from "../../../../types/submissions";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {notExhaustive} from "../../_notExhausiveHelper";
import {AceEditorsStorageHelper} from '../../../../components/codeEditors/aceEditors/aceEditorsStorageHelper'
import {tutorViewTutorFeedbackForStudentEditor} from '../../../../constants'
import {EditorTab} from '../../../../components/codeEditors/multiTabCodeEditorProps'

export interface GET_singleAssessmentAction
  extends ActionBase, ReduxPromiseAction<AssessmentFullBase, GET_singleAssessment_PENDINGAction, GET_singleAssessment_FULFILLEDAction, GET_singleAssessment_REJECTEDAction> {
  readonly type: ActionType.GET_singleAssessment
  readonly payload: Promise<AssessmentFullBase>
}

//automatically dispatched by redux promise middleware
export interface GET_singleAssessment_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_singleAssessment_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_singleAssessment_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<AssessmentFullBase> {
  readonly type: ActionType.GET_singleAssessment_FULFILLED
  readonly payload: AssessmentFullBase
}

//automatically dispatched by redux promise middleware
export interface GET_singleAssessment_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_singleAssessment_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_singleAssessment_PENDINGAction
  | GET_singleAssessment_FULFILLEDAction
  | GET_singleAssessment_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_singleAssessment_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_singleAssessment_FULFILLED:

      if (state.exercise.id === -1) {
        Logger.error('[getUserSolutionReducer] state.exercise must be loaded before loading the assessment!')
      }

      let releaseFinishedAndAutomaticAssessmentHasFinished =  state.exercise.exerciseRelease.isReleased &&
        state.exercise.exerciseRelease.hasAutomaticAssessmentFinished

      const tempEditorTab: EditorTab = {
        id: 0, //not -1 else we cannot distinct between this and the default tab/session
        content: (action.payload.hasAssessment && action.payload.feedbackForStudent !== null)
                 ? action.payload.feedbackForStudent
                 : '',
        language: 'markdown',
        isReadonly: releaseFinishedAndAutomaticAssessmentHasFinished === false || action.payload.hasAssessment === false,
        fileNameWithExtensions: '',
        displayIndex: 1,
        isMainFile: true,
        isContentVisibleForUser: true,
      }
      AceEditorsStorageHelper.addEditor(tutorViewTutorFeedbackForStudentEditor, tempEditorTab)

      return {
        ...state,
        isLoading: false,
        assessment: action.payload
      }
    case ActionType.GET_singleAssessment_REJECTED:
      return {
        ...state,
        isLoading: false,
        assessment: null
      }

    default:
      notExhaustive(action)
      return state
  }
}

