/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {ActionBase, ActionType} from "./tutorViewSiteActionTypes";
import {initial, State} from "./tutorViewSiteReducer";
import {DoExerciseFromBackend} from "../../../types/doExercise";
import Logger from "../../../helpers/logger";
import {
  convertAssessmentFromBackendToFrontend,
  convertDoExerciseFromBackendToFrontend, convertTestFromBackendToFrontend
} from "../../../helpers/convertersAndTransformers";


export interface GET_exerciseAction
  extends ActionBase, ReduxPromiseAction<DoExerciseFromBackend, GET_exercise_PENDINGAction, GET_exercise_FULFILLEDAction, GET_exercise_REJECTEDAction> {
  readonly type: ActionType.GET_exercise
  readonly payload: Promise<DoExerciseFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_exercise_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_exercise_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_exercise_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<DoExerciseFromBackend> {
  readonly type: ActionType.GET_exercise_FULFILLED
  readonly payload: DoExerciseFromBackend
}

//automatically dispatched by redux promise middleware
export interface GET_exercise_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_exercise_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_exercise_PENDINGAction
  | GET_exercise_FULFILLEDAction
  | GET_exercise_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_exercise_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_exercise_FULFILLED:
      return {
        ...state,
        isLoading: false,
        exercise: convertDoExerciseFromBackendToFrontend(action.payload),
        normalTestsForTutor: convertTestFromBackendToFrontend(action.payload.tests, false),
        allCodeTemplates: [...action.payload.codeTemplates] //because all is readonly we can use the ref here... (only a flat copy)
      }
    case ActionType.GET_exercise_REJECTED:
      return {
        ...state,
        isLoading: false,
      }

    default:
      notExhaustive(action)
      return state
  }
}

