/**
 * Created by janis dähne.
 */

import {AssessmentFullBase} from "../../../../types/submissions";
import {ActionBase, ActionType} from "../tutorViewSiteActionTypes";
import {notExhaustive} from "../../_notExhausiveHelper";

export type State = AssessmentFullBase


export const initial: State = {
  hasAssessment: false,
  token: '',
  firstName: '',
  lastName: '',
  userId: -1,
  pLangId: -1,
  exerciseReleaseId: -1,
  submitTestPoints: null,
  maxSubmitTestPoints: null,
  normalTestPoints: null,
  maxNormalTestPoints: null,
  manualPoints: null,
  noteForOtherTutors: '',
  feedbackForStudent: '',
  shouldNotCount: false,
  participationCount: 0,
  lastAssessmentErrorMessage: null,
  lastEditingIpAddress: null,
  maxManualPoints: null,
}

export interface SET_manualAssessmentPointsAction extends ActionBase {
  readonly type: ActionType.SET_manualAssessmentPoints
  readonly manualPoints: number | null
}

export interface SET_manualAssessmentNoteAction extends ActionBase {
  readonly type: ActionType.SET_manualAssessmentNote
  readonly note: string
}

export interface SET_feedbackForStudentAction extends ActionBase {
  readonly type: ActionType.SET_feedbackForStudent
  readonly feedbackForStudent: string
}

export type AllActions =
  SET_manualAssessmentPointsAction
  | SET_manualAssessmentNoteAction
  | SET_feedbackForStudentAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.SET_manualAssessmentPoints:
      return {
        ...state,
        manualPoints: action.manualPoints
      }
    case ActionType.SET_manualAssessmentNote:
      return {
        ...state,
        noteForOtherTutors: action.note
      }
    case ActionType.SET_feedbackForStudent:
      return {
        ...state,
        feedbackForStudent: action.feedbackForStudent
      }

    default:
      notExhaustive(action)
      return state
  }
}

