/**
 * Created by janis dähne.
 */

import {ActionBase, ActionType} from "./tutorViewSiteActionTypes";
import {notExhaustive} from "../_notExhausiveHelper";
import {PanelViewSize} from "../../../components/panelSystem/panelGrid/panelGrid";
import {DoExerciseFrontendExclusive} from "../../../types/doExercise";
import {initial as initalDoExercise} from '../doExerciseSite/doExerciseReducer'
import {AllActions as GetExerciseActions, reducer as getExerciseReducer} from './getExerciseReducer'
import {TestAnswerFromBackend} from "../../../types/testsAndTestProtocol";
import {AssessedSubmitTestFrontendExclusive} from "../../../types/testResults";
import {CustomTestFromBackend, RunnableCustomTestFrontendExclusive} from "../../../types/customTests";
import {
  ExerciseCodeTemplateFileFullBase,
  ExerciseCodeTemplateFullBase,
  RunnableExerciseTestFrontendExclusive, TestAssetFromBackendWithData
} from "../../../types/exerciseEditor";
import {SolutionDoExerciseFullBase} from "../../../types/exerciseSolution";
import {AllActions as GetUserSolutionActions, reducer as getUserSolutionReducer} from './crud/getUserSolutionReducer'
import {initial as initialSolution,} from '../doExerciseSite/subSets/setDoExerciseSolutionReducer'
import {DoEditorState} from "../doExerciseSite/editorReducers/doEditorReducer";
import {
  AllActions as AllUserSolutionEditorActions,
  initial as initialUserSolutionEditorState,
  reducer as userSolutionEditorReducer
} from './editorReducer/userSolutionTutorViewEditorReducer'

import {
  AllActions as TutorEditorActions,
  initial as initialTutorEditorState,
  reducer as tutorEditorReducer
} from './editorReducer/tutorEditorReducer'

import {
  AllActions as CodeTemplateEditorActions,
  initial as initialCodeTemplateEditorState,
  reducer as codeTemplateTutorEditorReducer
} from './editorReducer/codeTemplateTutorViewEditorReducer'

import {
  AllActions as GetAllUserTestResultActions,
  reducer as getAllUserTestResults
} from './crud/getAllUserTestResultsReducer'
import {
  AllActions as GetSubmitTestResultActions,
  reducer as getUserSubmitTestResults
} from './crud/getUserSubmitTestResults'

import {AllActions as SetUserSolutionActions, reducer as setUserSolutionReducer} from './subSets/setUserSolutionReducer'

import {
  AllActions as SetTutorEditorSolutionActions,
  initial as initialTutorEditorSolution,
  reducer as setTutorEditorSolutionReducer
} from './subSets/setTutorEditorSolutionReducer'


import {
  AllActions as RunNormalTestActions,
  reducer as runNormalTestAsTutorReducer
} from './crud/runNormalTestAsTutorReducer'
import {
  AllActions as RunSubmitTestActions,
  reducer as runSubmitTestAsTutorReducer
} from './crud/runSubmitTestAsTutorReducer'
import {AssessmentFullBase} from "../../../types/submissions";
import {
  AllActions as SetAssessmentActions,
  initial as initialAssessment,
  reducer as setAssessmentReducer
} from './subSets/setAssessmentReducer'
import {
  AllActions as GetSingleAssessmentActions,
  reducer as getSingleAssessmentReducer
} from './crud/getSingleAssessmentReducer'

import {
  AllActions as UpdateSingleAssessmentActions,
  reducer as updateSingleAssessmentReducer
} from './crud/updateSingleAssessmentReducer'
import {FrontendSettingsManager} from "../../../helpers/frontendSettingsManager";
import {AfterSolutionDoExerciseFullBase} from '../../../types/exerciseAfterSolution'

import {
  AllActions as GetUserAfterSolutionActions,
  reducer as getUserAfterSolutionReducer
} from './crud/afterSolution/getUserAfterSolutionReducer'
import {
  AllActions as SetUserAfterSolutionActions,
  reducer as setUserAfterSolutionReducer
} from './subSets/setUserAfterSolutionReducer'
import {
  AllActions as UserAfterSolutionTutorViewEditorActions,
  reducer as afterSolutionEditorReducer,
  initial as initialUserAfterSolutionEditorState
} from './editorReducer/userAfterSolutionTutorViewEditorReducer'
import {
  AllActions as GetAllUserAfterTestResultsActions,
  reducer as getAllUserAfterTestResultsReducer
} from './crud/afterSolution/getAllUserAfterTestResultsReducer'
import {
  AllActions as GetAfterSubmitTestResultsActions,
  reducer as getAfterSubmitTestResultsReducer
} from './crud/afterSolution/getAfterSubmitTestResultsReducer'
import {
  convertDoExerciseFromBackendToFrontend,
  convertTestFromBackendToFrontend
} from '../../../helpers/convertersAndTransformers'

import {
  AllActions as AllJustRunProgramReducerActions,
  reducer as justRunProgramReducer
} from './crud/justRunProgramReducer'

import {
  AllActions as AllRunCompileSingleFileReducerActions,
  reducer as runCompileSingleFileReducer
} from './crud/runCompileSingleFileReducer'
import {ActionTypeGlobal, RESET_GlobalAction} from '../globalActions'

export type State = {

  /**
   * null on error
   * set via CRUD
   * managed via setAssessmentReducer
   *
   * this is never null because backend returns some basic data in case there is no assessment yet
   *   if the assessment in the backend was null then the property {@link AssessmentFullBase.hasAssessment} is set to false (else true)
   */
  readonly assessment: AssessmentFullBase

  /**
   * true: go back to own exercises, false: back to group exercises
   * null: just go back
   */
  readonly comesFromOwnExercisesSite: boolean | null

  /**
   * set via getExerciseReducer
   * managed via getExerciseReducer
   */
  readonly exercise: DoExerciseFrontendExclusive


  /**
   * normal tests for the tutor to run (results won't be saved to backend)
   */
  readonly normalTestsForTutor: ReadonlyArray<RunnableExerciseTestFrontendExclusive>
  /**
   * submit tests for the tutor to run (results won't be saved to backend)
   */
  readonly submitTestsForTutor: ReadonlyArray<AssessedSubmitTestFrontendExclusive>


  /**
   * the submit test results, only available if the release has ended
   *
   * set via CRUD
   */
  readonly submitTestResults: ReadonlyArray<AssessedSubmitTestFrontendExclusive>

  /**
   * the after submit test results, only available if the release has ended
   *
   * set via CRUD
   */
  readonly afterSubmitTests: ReadonlyArray<AssessedSubmitTestFrontendExclusive>

  /**
   * the normal after tests from the user
   */
  readonly afterNormalTests: ReadonlyArray<RunnableExerciseTestFrontendExclusive>

  /**
   * the user solution
   * set via CRUD
   */
  readonly userSolution: SolutionDoExerciseFullBase

  /**
   * the user after solution or null if not created
   * set via crud
   */
  readonly userAfterSolution: AfterSolutionDoExerciseFullBase | null

  /**
   * the tutor solution to exeriment with the users solution
   */
  readonly tutorEditorSolution: SolutionDoExerciseFullBase

  /**
   * normally the test results are attached to the tests... but for this command we have no test...
   * we only check if the CURRENT (not saved in db) state of the current file compiles
   *
   * this is set via async crud action callback
   */
  readonly lastCompileSingleFileResult: TestAnswerFromBackend | null

  /**
   * the currently selected test to display in the console view
   *
   * no id because it could be RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive
   */
  readonly selectedConsoleViewTest: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null


  /**
   * the editor state for the user solution ... should be read only
   */
  readonly userSolutionEditorState: DoEditorState

  readonly userAfterSolutionEditorState: DoEditorState

  /**
   * display the code template readonly to compare the solution against the default code template
   */
  readonly codeTemplateEditorState: DoEditorState

  /**
   * the code templates one for every plang
   * use an additional array for performance... when we hide an file we need to change this array
   * if we use the one in the exercise we would copy the exercise over and over again (which might update the ui depending on the exercise)
   *
   * THIS is set via CRUD only when we get the exercise
   */
  readonly allCodeTemplates: ReadonlyArray<ExerciseCodeTemplateFullBase>

  /**
   * editor to test / experiment with the users solution
   */
  readonly tutorEditorState: DoEditorState

  readonly isLoading: boolean

  readonly isViewTestDialogDisplayed: boolean
  readonly isCodeEditorSettingsDialogDisplayed: boolean

  readonly topRightSelectedTabIndex: number
  readonly topLeftSelectedTabIndex: number
  readonly bottomRightSelectedTabIndex: number
  readonly bottomLeftSelectedTabIndex: number

  readonly topRightPanelViewSize: PanelViewSize
  readonly topLeftPanelViewSize: PanelViewSize
  readonly botRightPanelViewSize: PanelViewSize
  readonly botLeftPanelViewSize: PanelViewSize

  readonly hDividerPosOffset: number
  readonly vDividerPosOffset: number

  /**
   * the args for the just run operation
   *
   * must be <=
   * @see justRunProgramArgsMaxLength
   */
  readonly justRunProgramArgs: string

  readonly isJustRunProgramArgsInputDisplayed: boolean

  /**
   * the last just run program result (like lastCompileSingleFileResult)
   *
   * this is set via async crud action callback
   */
  readonly lastJustRunProgramResult: TestAnswerFromBackend | null


  /**
   * toggle this to force a re render of hte preview panel (when assets are finished creating)
   */
  readonly dummy: boolean

  /**
   * 0 is the editor
   * 1 is the preview
   */
  readonly feedbackForStudentPanelSelectedSubTabIndex: number
}

export const initial: State = {

  assessment: initialAssessment,
  comesFromOwnExercisesSite: true,
  exercise: initalDoExercise,

  normalTestsForTutor: [],
  submitTestsForTutor: [],


  submitTestResults: [],
  afterSubmitTests: [],
  afterNormalTests: [],

  allCodeTemplates: [],

  userSolution: initialSolution,
  userAfterSolution: null,

  tutorEditorSolution: initialTutorEditorSolution,

  lastCompileSingleFileResult: null,
  selectedConsoleViewTest: null,

  userSolutionEditorState: initialUserSolutionEditorState,
  userAfterSolutionEditorState: initialUserAfterSolutionEditorState,
  tutorEditorState: initialTutorEditorState,
  codeTemplateEditorState: initialCodeTemplateEditorState,

  isLoading: false,

  isViewTestDialogDisplayed: false,
  isCodeEditorSettingsDialogDisplayed: false,

  topRightSelectedTabIndex: 1,
  topLeftSelectedTabIndex: 0,
  bottomRightSelectedTabIndex: 0,
  bottomLeftSelectedTabIndex: 0,

  topRightPanelViewSize: PanelViewSize.normal,
  topLeftPanelViewSize: PanelViewSize.normal,
  botRightPanelViewSize: PanelViewSize.normal,
  botLeftPanelViewSize: PanelViewSize.normal,

  hDividerPosOffset: 0,
  vDividerPosOffset: 150,

  justRunProgramArgs: '',
  isJustRunProgramArgsInputDisplayed: false,
  lastJustRunProgramResult: null,
  feedbackForStudentPanelSelectedSubTabIndex: 0,

  dummy: false
}


export interface SET_isLoadingAction extends ActionBase {
  readonly type: ActionType.SET_isLoading
  readonly isLoading: boolean
}

export interface SET_comesFromOwnExercisesSiteAction extends ActionBase {
  readonly type: ActionType.SET_comesFromOwnExercisesSite
  readonly comesFromOwnExercisesSite: boolean
}

export interface SET_normalTestsForTutorAction extends ActionBase {
  readonly type: ActionType.SET_normalTestsForTutor
  readonly testsForTutor: ReadonlyArray<RunnableExerciseTestFrontendExclusive>
}

export interface SET_submitTestsForTutorAction extends ActionBase {
  readonly type: ActionType.SET_submitTestsForTutor
  readonly submitTestsForTutor: ReadonlyArray<AssessedSubmitTestFrontendExclusive>
}

export interface SET_selectedConsoleViewTestAction extends ActionBase {
  readonly type: ActionType.SET_selectedConsoleViewTest
  readonly selectedConsoleViewTest: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null
}

export interface SET_dummyAction extends ActionBase {
  readonly type: ActionType.SET_dummy
  readonly dummy: boolean
}


export interface SET_justRunProgramArgsAction extends ActionBase {
  readonly type: ActionType.SET_justRunProgramArgs
  readonly justRunProgramArgs: string
}

export interface SET_isJustRunProgramArgsInputDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isJustRunProgramArgsInputDisplayed
  readonly isJustRunProgramArgsInputDisplayed: boolean
}

//--- console view options are now global in do exercise site

//--- dialogs

export interface SET_isCodeEditorSettingsDialogDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isCodeEditorSettingsDialogDisplayed
  readonly isDisplayed: boolean
}

export interface SET_isViewTestDialogDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isViewTestDialogDisplayed
  readonly isViewTestDialogDisplayed: boolean
}

// --- panel view sizes

export interface SET_topRightPanelViewSizeAction extends ActionBase {
  readonly type: ActionType.SET_topRightPanelViewSize
  readonly topRightPanelViewSize: PanelViewSize
}

export interface SET_topLeftPanelViewSizeAction extends ActionBase {
  readonly type: ActionType.SET_topLeftPanelViewSize
  readonly topLeftPanelViewSize: PanelViewSize
}

export interface SET_botRightPanelViewSizeAction extends ActionBase {
  readonly type: ActionType.SET_botRightPanelViewSize
  readonly botRightPanelViewSize: PanelViewSize
}

export interface SET_botLeftPanelViewSizeAction extends ActionBase {
  readonly type: ActionType.SET_botLeftPanelViewSize
  readonly botLeftPanelViewSize: PanelViewSize
}

//--- panel selected indices

export interface SET_topRightSelectedTabIndexAction extends ActionBase {
  readonly type: ActionType.SET_topRightSelectedTabIndex
  readonly topRightSelectedTabIndex: number
}

export interface SET_topLeftSelectedTabIndexAction extends ActionBase {
  readonly type: ActionType.SET_topLeftSelectedTabIndex
  readonly topLeftSelectedTabIndex: number
}

export interface SET_bottomRightSelectedTabIndexAction extends ActionBase {
  readonly type: ActionType.SET_bottomRightSelectedTabIndex
  readonly bottomRightSelectedTabIndex: number
}

export interface SET_bottomLeftSelectedTabIndexAction extends ActionBase {
  readonly type: ActionType.SET_bottomLeftSelectedTabIndex
  readonly bottomLeftSelectedTabIndex: number
}

export interface SET_feedbackForStudentPanelSelectedSubTabIndexAction extends ActionBase {
  readonly type: ActionType.SET_feedbackForStudentPanelSelectedSubTabIndex
  readonly feedbackForStudentPanelSelectedSubTabIndex: number
}


//--- panel grip

export interface SET_hDividerPosOffsetAction extends ActionBase {
  readonly type: ActionType.SET_hDividerPosOffset
  readonly hDividerPosOffset: number
}

export interface SET_vDividerPosOffsetAction extends ActionBase {
  readonly type: ActionType.SET_vDividerPosOffset
  readonly vDividerPosOffset: number
}

//used to hide files (setting display index < 0)
export interface SET_codeTemplate_templateFilesAction extends ActionBase {
  readonly type: ActionType.SET_codeTemplate_templateFiles
  readonly templateFiles: ReadonlyArray<ExerciseCodeTemplateFileFullBase>
  readonly codeTemplateId: number
}

export interface RESET_userSpecificDataAction extends ActionBase {
  readonly type: ActionType.RESET_userSpecificData
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | RESET_userSpecificDataAction

  | SET_isLoadingAction
  | SET_topRightSelectedTabIndexAction
  | SET_topLeftSelectedTabIndexAction
  | SET_bottomRightSelectedTabIndexAction
  | SET_bottomLeftSelectedTabIndexAction

  | SET_topRightPanelViewSizeAction
  | SET_topLeftPanelViewSizeAction
  | SET_botRightPanelViewSizeAction
  | SET_botLeftPanelViewSizeAction
  | SET_comesFromOwnExercisesSiteAction
  | SET_feedbackForStudentPanelSelectedSubTabIndexAction

  | SET_hDividerPosOffsetAction
  | SET_vDividerPosOffsetAction
  | SET_dummyAction

  | SetUserSolutionActions

  | SET_normalTestsForTutorAction
  | SET_submitTestsForTutorAction

  | TutorEditorActions
  | CodeTemplateEditorActions
  | UserAfterSolutionTutorViewEditorActions
  | SetTutorEditorSolutionActions
  | SetUserAfterSolutionActions
  | SetAssessmentActions

  | RunNormalTestActions
  | RunSubmitTestActions

  | SET_selectedConsoleViewTestAction
  | SET_isViewTestDialogDisplayedAction
  | SET_isCodeEditorSettingsDialogDisplayedAction

  | AllUserSolutionEditorActions

  | GetExerciseActions
  | GetUserSolutionActions
  | GetAllUserTestResultActions
  | GetSubmitTestResultActions
  | GetSingleAssessmentActions
  | GetUserAfterSolutionActions
  | GetAllUserAfterTestResultsActions
  | GetAfterSubmitTestResultsActions

  | UpdateSingleAssessmentActions
  | SET_codeTemplate_templateFilesAction

  | SET_justRunProgramArgsAction
  | SET_isJustRunProgramArgsInputDisplayedAction
  | AllJustRunProgramReducerActions

  | AllRunCompileSingleFileReducerActions

  | RESET_GlobalAction


export function reducer(state: State = initial, action: AllActions): State {


  switch (action.type) {


    case ActionType.RUN_compileSingleFile_PENDING:
    case ActionType.RUN_compileSingleFile_FULFILLED:
    case ActionType.RUN_compileSingleFile_REJECTED:
      return runCompileSingleFileReducer(state, action)

    case ActionType.JUST_RUN_program_PENDING:
    case ActionType.JUST_RUN_program_FULFILLED:
    case ActionType.JUST_RUN_program_REJECTED:
      return justRunProgramReducer(state, action)

    case ActionType.GET_afterSubmitTestResults_PENDING:
    case ActionType.GET_afterSubmitTestResults_FULFILLED:
    case ActionType.GET_afterSubmitTestResults_REJECTED:
      return getAfterSubmitTestResultsReducer(state, action)

    case ActionType.GET_afterTestResults_PENDING:
    case ActionType.GET_afterTestResults_FULFILLED:
    case ActionType.GET_afterTestResults_REJECTED:
      return getAllUserAfterTestResultsReducer(state, action)

    case ActionType.GET_user_afterSolution_PENDING:
    case ActionType.GET_user_afterSolution_FULFILLED:
    case ActionType.GET_user_afterSolution_REJECTED:
      return getUserAfterSolutionReducer(state, action)

    case ActionType.UPDATE_singleAssessment_PENDING:
    case ActionType.UPDATE_singleAssessment_FULFILLED:
    case ActionType.UPDATE_singleAssessment_REJECTED:
      return updateSingleAssessmentReducer(state, action)

    case ActionType.SET_feedbackForStudent:
    case ActionType.SET_manualAssessmentPoints:
    case ActionType.SET_manualAssessmentNote:
      return {
        ...state,
        assessment: setAssessmentReducer(state.assessment, action)
      }

    case ActionType.GET_singleAssessment_PENDING:
    case ActionType.GET_singleAssessment_FULFILLED:
    case ActionType.GET_singleAssessment_REJECTED:
      return getSingleAssessmentReducer(state, action)

    case ActionType.RUN_normalTestAsTutor_PENDING:
    case ActionType.RUN_normalTestAsTutor_FULFILLED:
    case ActionType.RUN_normalTestAsTutor_REJECTED:
      return runNormalTestAsTutorReducer(state, action)

    case ActionType.RUN_submitTestAsTutor_PENDING:
    case ActionType.RUN_submitTestAsTutor_FULFILLED:
    case ActionType.RUN_submitTestAsTutor_REJECTED:
      return runSubmitTestAsTutorReducer(state, action)

    case ActionType.SET_tutorEditor_hasFileChangedMap:
    case ActionType.SET_tutorEditor_hasUnsavedChanges:
    case ActionType.SET_tutorEditor_selectedTabIdDoEditor:
    case ActionType.SET_tutorEditor_isFileExplorerDisplayedDoEditor:
    case ActionType.SET_tutorEditor_newFileNameDoEditor:
    case ActionType.SET_tutorEditor_renamingFileIdDoEditor:
    case ActionType.SET_tutorEditor_isRenamingFileDoEditor:
    case ActionType.SET_tutorEditor_renamingFileNameDoEditor:
      return {
        ...state,
        tutorEditorState: tutorEditorReducer(state.tutorEditorState, action)
      }

    case ActionType.SET_afterSolutionEditor_selectedTabIdDoEditor:
    case ActionType.SET_afterSolutionEditor_isFileExplorerDisplayedDoEditor:
      return {
        ...state,
        userAfterSolutionEditorState: afterSolutionEditorReducer(state.userAfterSolutionEditorState, action)
      }

    case ActionType.SET_codeTemplateEditor_selectedTabIdDoEditor:
    case ActionType.SET_codeTemplateEditor_isFileExplorerDisplayedDoEditor:
      return {
        ...state,
        codeTemplateEditorState: codeTemplateTutorEditorReducer(state.codeTemplateEditorState, action)
      }

    case ActionType.GET_userSubmitTestResults_PENDING:
    case ActionType.GET_userSubmitTestResults_FULFILLED:
    case ActionType.GET_userSubmitTestResults_REJECTED:
      return getUserSubmitTestResults(state, action)

    case ActionType.GET_allUserTestResults_PENDING:
    case ActionType.GET_allUserTestResults_FULFILLED:
    case ActionType.GET_allUserTestResults_REJECTED:
      return getAllUserTestResults(state, action)

    case ActionType.GET_user_solutionVersion_PENDING:
    case ActionType.GET_user_solutionVersion_FULFILLED:
    case ActionType.GET_user_solutionVersion_REJECTED:
      return getUserSolutionReducer(state, action)

    case ActionType.GET_exercise_PENDING:
    case ActionType.GET_exercise_FULFILLED:
    case ActionType.GET_exercise_REJECTED:
      return getExerciseReducer(state, action)


    case ActionType.SET_normalTestsForTutor:
      return {
        ...state,
        normalTestsForTutor: action.testsForTutor
      }

    case ActionType.SET_submitTestsForTutor:
      return {
        ...state,
        submitTestsForTutor: action.submitTestsForTutor
      }

    case ActionType.SET_tutorEditorSolution_solutionFiles:
    case ActionType.SET_tutorEditorSolution_mainFileId:
      return {
        ...state,
        tutorEditorSolution: setTutorEditorSolutionReducer(state.tutorEditorSolution, action)
      }

    case ActionType.SET_codeTemplate_templateFiles:
      return {
        ...state,
        allCodeTemplates: state.allCodeTemplates.map((p, index) => {
          return p.id !== action.codeTemplateId
                 ? p
                 : {
              ...p,
              templateFiles: action.templateFiles
            }
        })
      }

    case ActionType.SET_userSolution_solutionFiles:
    case ActionType.SET_userSolution_mainFileId:
      return {
        ...state,
        userSolution: setUserSolutionReducer(state.userSolution, action)
      }

    case ActionType.SET_userAfterSolution_solutionFiles:
    case ActionType.SET_userAfterSolution_mainFileId:
      return {
        ...state,
        userAfterSolution: setUserAfterSolutionReducer(state.userAfterSolution, action)
      }

    case ActionType.SET_userSolutionEditor_selectedTabIdDoEditor:
    case ActionType.SET_userSolutionEditor_isFileExplorerDisplayedDoEditor:
      return {
        ...state,
        userSolutionEditorState: userSolutionEditorReducer(state.userSolutionEditorState, action)
      }

    case ActionType.SET_isCodeEditorSettingsDialogDisplayed:
      return {
        ...state,
        isCodeEditorSettingsDialogDisplayed: action.isDisplayed
      }

    case ActionType.SET_isViewTestDialogDisplayed:
      return {
        ...state,
        isViewTestDialogDisplayed: action.isViewTestDialogDisplayed
      }

    case ActionType.SET_selectedConsoleViewTest:
      return {
        ...state,
        selectedConsoleViewTest: action.selectedConsoleViewTest
      }

    case ActionType.SET_dummy:
      return {
        ...state,
        dummy: action.dummy
      }
    case ActionType.SET_comesFromOwnExercisesSite:
      return {
        ...state,
        comesFromOwnExercisesSite: action.comesFromOwnExercisesSite
      }

    case ActionType.SET_topRightSelectedTabIndex:
      return {
        ...state,
        topRightSelectedTabIndex: action.topRightSelectedTabIndex
      }

    case ActionType.SET_topLeftSelectedTabIndex:
      return {
        ...state,
        topLeftSelectedTabIndex: action.topLeftSelectedTabIndex
      }

    case ActionType.SET_bottomRightSelectedTabIndex:
      return {
        ...state,
        bottomRightSelectedTabIndex: action.bottomRightSelectedTabIndex
      }

    case ActionType.SET_bottomLeftSelectedTabIndex:
      return {
        ...state,
        bottomLeftSelectedTabIndex: action.bottomLeftSelectedTabIndex
      }

    case ActionType.SET_feedbackForStudentPanelSelectedSubTabIndex:
      return {
        ...state,
        feedbackForStudentPanelSelectedSubTabIndex: action.feedbackForStudentPanelSelectedSubTabIndex
      }

    case ActionType.SET_hDividerPosOffset:
      return {
        ...state,
        hDividerPosOffset: action.hDividerPosOffset
      }
    case ActionType.SET_vDividerPosOffset:
      return {
        ...state,
        vDividerPosOffset: action.vDividerPosOffset
      }

    case ActionType.SET_topRightPanelViewSize:
      return {
        ...state,
        topRightPanelViewSize: action.topRightPanelViewSize
      }

    case ActionType.SET_topLeftPanelViewSize:
      return {
        ...state,
        topLeftPanelViewSize: action.topLeftPanelViewSize
      }

    case ActionType.SET_botRightPanelViewSize:
      return {
        ...state,
        botRightPanelViewSize: action.botRightPanelViewSize
      }

    case ActionType.SET_botLeftPanelViewSize:
      return {
        ...state,
        botLeftPanelViewSize: action.botLeftPanelViewSize
      }


    case ActionType.SET_isLoading:
      return {
        ...state,
        isLoading: action.isLoading
      }

    case ActionType.SET_justRunProgramArgs:
      return {
        ...state,
        justRunProgramArgs: action.justRunProgramArgs
      }
    case ActionType.SET_isJustRunProgramArgsInputDisplayed:
      return {
        ...state,
        isJustRunProgramArgsInputDisplayed: action.isJustRunProgramArgsInputDisplayed
      }



    //this is the data a normal page load would also load we just reset some data (e.g. test results)
    case ActionType.RESET_userSpecificData:
      return {
        ...state,
        exercise: convertDoExerciseFromBackendToFrontend({
                                                           ...state.exercise,
                                                           customTests: state.exercise.customTests.map<CustomTestFromBackend>(
                                                             p => {
                                                               return {
                                                                 ...p,
                                                                 files: p.files
                                                               }
                                                             })
                                                         }),
        normalTestsForTutor: convertTestFromBackendToFrontend(state.normalTestsForTutor, false),
        userAfterSolution: null, //we need to reset this because this might not be loaded and thus not overwritten
        topLeftSelectedTabIndex: state.topLeftSelectedTabIndex > 1
                                 ? 0
                                 : state.topLeftSelectedTabIndex, //we don't know if the next use has an after solution...
        selectedConsoleViewTest: null
      }
    case ActionType.RESET:
    case ActionTypeGlobal.RESET_Global:
      return {
        ...initial,
      }

    default:
      notExhaustive(action)
      return state
  }
}

