import {AssessmentFullBase} from './submissions'

export interface CsvExportAssessmentOption {

  readonly kind: 'normal'

  readonly displayIndex: number | null //null to hide (not exportable)

  /**
   * true: the column will be exported in the backend
   * false: not
   */
  readonly willExport: boolean
  /**
   * if we want to replace the content of the cells...
   */
  readonly replaceRulesObj: CsvExportReplaceRuleObj
}


export type CsvExportAssessmentOptionsMap = {
  /**
   * the key is the lowercase name (mostly) of the column
   */
  [key in keyof AssessmentFullBase | string]: CsvExportAssessmentOption | AdditionalCsvColumn
}


export interface CsvExportReplaceRuleObj {

  /**
   * the rules to apply, the first matching rule is applied then be break
   */
  readonly replaceRules: ReadonlyArray<CsvExportReplaceRule>

  /**
   * if no rule matched then we check this
   * if null do nothing
   * if this is a string replace the cell value with this string
   */
  readonly elseReplace: string | null

}

export interface CsvExportReplaceRule {
  /**
   * we need to check if this is really a regex e.g. on submit
   */
  readonly regex: string
  /**
   * only i or/and m allowed
   */
  readonly regexFlags: string
  readonly replaceWith: string
}

export interface AdditionalCsvColumn {
  readonly kind: 'additional'

  /**
   * an artificial key so when we change the name of the col
   * we don't have to add/remove the entry in
   * make sure this is unique across browser restarts (e.g. when we save this in local storage)
   * @see CsvExportAssessmentOptionsMap
   */
  readonly artificialKey: string
  /**
   * the name of the column
   */
  readonly name: string
  /**
   * the value for the new column (for all rows)
   */
  readonly value: string

  /**
   * the display index
   */
  readonly displayIndex: number
}


/**
 * for backend
 */
export interface CsvExportAssessmentOptionsObj {
  readonly options: CsvExportAssessmentOptionsMap
  readonly additionalCsvColumns: ReadonlyArray<AdditionalCsvColumn>

}