/**
 * Created by janis dähne.
 */


import {FilePreviewFromBackend} from "./exerciseEditor";
import {TestAnswerFromBackend} from "./testsAndTestProtocol";


/**
 * a tuple with the test id and the new display index
 * used when the user changes the display index of one or all custom tests
 */
export interface UpdateCustomTestIndicesTuple {
  /**
   * the custom test id
   */
  readonly id: number
  /**
   * the new display index for this test
   */
  readonly newDisplayIndex: number
}

/**
 * a custom test
 */
export interface CustomTestFromBackend {
  readonly id:number
  /**
   * the display na,e
   */
  readonly displayName:string
  /**
   * the content
   */
  readonly content:string
  /**
   * the index for the sorting
   */
  readonly displayIndex:number
  /**
   * the test case type id
   */
  readonly testTypeId:number

  /**
   * the test assets (files)
   */
  readonly files:ReadonlyArray<FilePreviewFromBackend>
}

export interface RunnableCustomTestFrontendExclusive extends CustomTestForBackend {
  /**
   * true: test is running, false: not
   * used to display a spinner
   */
  readonly uiIsRunning: boolean
  /**
   * true: this is an after test, false: a normal (not after/subsequent solution) test
   */
  readonly uiIsAfterTest: boolean
  /**
   * the result for this test
   */
  readonly result: TestAnswerFromBackend | null
}

/**
 * only used to store some ui props and the result... for
 * all other props use the original custom test
 */
export interface RunnableAfterCustomTestFrontendExclusive {

  readonly id:number
  /**
   * true: test is running, false: not
   * used to display a spinner
   */
  readonly uiIsRunning: boolean
  /**
   * true: this is an after test, false: a normal (not after/subsequent solution) test
   */
  readonly uiIsAfterTest: boolean
  /**
   * the result for this test
   */
  readonly result: TestAnswerFromBackend | null
}

/**
 * a custom test (IGNORE TestCaseSettings because we use the defaultTestCase settings from the exercise)
 */
export interface CustomTestForBackend {
  readonly id:number
  /**
   * the display name
   */
  readonly displayName:string
  /**
   * the content
   */
  readonly content:string
  /**
   * the index for the sorting
   */
  readonly displayIndex:number
  /**
   * the test case type id
   */
  readonly testTypeId:number
  /**
   * the test assets (files)
   */
  readonly files:ReadonlyArray<FilePreviewFromBackend>
}
