/**
 * Created by janis dähne.
 */


import {ExerciseCodeTemplateFileFullBase} from "./exerciseEditor";
import {Moment} from "moment";


/**
 *  a preview of a solution  (includes just some basic data so that the user knows about the  (just the id is not so informative))
 */
export interface SolutionPreview {
  /**
   * the id of the solution
   */
  readonly id: number
  /**
   * a note about the
   */
  readonly note: string
  /**
   * when the  was created
   */
  readonly createdAt: Moment
}

/**
 * a solution tuple for the backend (includes one solution  and a list of all known solutions from the user)
 */
export interface SolutionTupleFromBackend {
  /**
   * the solution for the frontend
   */
  readonly solution: SolutionDoExerciseFullBase

  /**
   * if we change/create a new file in the frontend then we send back the saved  (backend ) to the frontend
   * because the frontend can replace the file (else the undo history would be lost/corrupted) it needs to set the
   * file content directly. for this we need to know which old/frontend file had which content
   */
  readonly fileIdAssociationMap: ReadonlyArray<FileIdAssociation>
}

export interface FileIdAssociation {
  readonly frontendFileId: number
  readonly backendFileId: number
}

/**
 * a solution  backend sends Dates as strings to the frontend so we need a pre type to convert his to a Date
 */
export interface SolutionDoExerciseFullBase {
  /**
   * the main file id (where the entry point is declared, e.g. public static void main...)
   */
  readonly mainFileId: number
  /**
   * the plang the solution  is for
   */
  readonly pLangId: number

  readonly userId: number

  readonly releaseId: number

  /**
   * a note about the
   */
  readonly note: string

  /**
   * the datetime when the  was ?
   */
  readonly lastUpdatedAt: Moment
  /**
   * the files for this solution
   */
  readonly solutionFiles: ReadonlyArray<SolutionFileDoExerciseFullBase>

  /**
   * this is taken from the users solution
   * true: the student wants that his/her solution is not counted (for whatever), false: not
   * can only be set once by the student
   * NOT SET through this prop (set through backend)
   */
  readonly shouldNotCount: boolean

  /**
   * true: backend has an after solution, false: not or the normal solution is still active (editable) --> time is not used up
   * only important/set on initial site load (not when resetting the user solution)
   */
  readonly shouldLoadAfterSolution: boolean

  /**
   * true: user already created one, false: not
   * only important/set on initial site load (not when resetting the user solution)
   */
  readonly hasLoadAfterSolution: boolean

}

/**
 * a solution file (part of the solution verion)
 */
export interface SolutionFileDoExerciseFullBase extends ExerciseCodeTemplateFileFullBase {

  /**
   * ui property, true: is displayed as tab, false: not (only displayed in project explorer)
   * set to true if displayIndex is >= 0
   */
  readonly uiIsDisplayed: boolean
  /**
   * null or the template file id from which the file was created
   */
  readonly templateFileId: number | null


}
