/**
 * Created by janis dähne.
 */


/*+
 represents a user group
 */
import {GroupRoleWithPermissionFullBase} from "./groupRoles";

/**
 * a group (just name and id)
 */
export interface GroupPreviewFromBackend {
  readonly id: number
  /**
   * the display name
   */
  readonly displayName: string
}


export interface UserGroupBase {

  readonly id: number
  /**
   * the display name
   */
  readonly displayName: string
  /**
   * the email for the group
   */
  readonly email: string
}

export interface UserGroupForBackend  extends  UserGroupBase{

}

export interface UserGroupFromBackend extends UserGroupBase{

  /**
   * the group role of the current user for the user group
   */
  readonly groupRole: GroupRoleWithPermissionFullBase | null //can be null if not yet initialized (we get roles and users separately

}

export interface UserGroupWithExerciseCountFromBackend extends UserGroupFromBackend {
  /**
   * number of exercises connected to the group
   */
  readonly numExercisesInGroup: number
}

/**
 * describes a group member
 */
export interface GroupUserBase {
  readonly id: number

  /**
   * the token of the user (id)
   */
  readonly token: string
  /**
   * the first name
   */
  readonly firstName: string
  /**
   * the last name
   */
  readonly lastName: string
  /**
   * the email
   */
  readonly email: string
  /**
   * true: the user is activated, false: not
   */
  readonly isActivated: boolean

  /**
   * true: this member is the current user, false: not
   */
  readonly isCurrentUser: boolean
}

/**
 * describes a group member, from backend (no group role set)
 */
export interface GroupUserFromBackend extends GroupUserBase {
  /**
   * the related group role id
   */
  readonly groupRoleId: number
}

/**
 * describes a group member in the frontend
 */
export interface SelectableGroupUserFrontend extends GroupUserBase {

  /**
   * true: member is selected, false: not
   */
  readonly uiIsSelected: boolean

  /**
   * true: is is displayed (in some list), false: not
   */
  readonly uiIsChangeGroupRolePopupDisplayed: boolean

  /**
   * true: is is displayed (in some list), false: not
   */
  readonly isDisplayed: boolean
  /**
   * the group role for this member
   */
  readonly groupRole: GroupRoleWithPermissionFullBase
}


/**
 * used to add users to a user group
 */
export interface AddUsersToGroupTuplesForBackend {
  readonly groupId: number
  readonly userIds: ReadonlyArray<number>
}

/**
 * used to change the role of a member
 */
export interface ChangeGroupRoleTupleForBackend {
  readonly userId: number
  readonly groupId: number
  readonly groupRoleId: number
}


/**
 * used to remove a member
 */
export interface RemoveUsersFromGroupTuple {
  readonly groupId: number
  readonly userId?: number
  readonly useCurrentUser: boolean
}
