/**
 * Created by janis dähne.
 */


/**
 * checkboxes onChange usage:
 *
 * <Checkbox onChange={(event: any, data: CheckboxOnChangeEventData) => ...} />
 *
 */
import {HTMLAttributes, SyntheticEvent, KeyboardEvent} from "react";


/**
 * just a better name for DetailedHTMLProps
 * this is useful when we want to extend the props of a custom component by some existing comp
 * e.g. extend Tag with html div props
 *
 */
export type PropsOf<T> = HTMLAttributes<T>

export type CheckboxOnChangeEventData = {
  readonly checked: boolean,
  readonly indeterminate: boolean
  readonly radio: boolean
  readonly value: number
  //... some others
}
export type RadioOnChangeEventData = CheckboxOnChangeEventData

/**
 * usage:
 *
 * <select
 * value={this.props.test.testTypeId}
 * onChange={(e: SelectOnChangeEventData) => {
 *   const id = parseInt(e.currentTarget.value, 10)
 * }}
 * >
 *
 *
 */
export type SelectOnChangeEventData = SyntheticEvent<HTMLSelectElement>

export type InputOnChangeEvent = SyntheticEvent<HTMLInputElement>
export type TextAreaOnChangeEvent = SyntheticEvent<HTMLTextAreaElement>

/**
 *
 * usage:
 *
 *
 * if (e.key === 'Enter') {...}
 *
 */
export type OnKeyPressEvent = KeyboardEvent<HTMLInputElement>